Type: Package
Package: traumar
Title: Calculate Metrics for Trauma System Performance
Version: 1.2.6
Authors@R: c(
    person("Nicolas", "Foss", , "nicolas.foss@hhs.iowa.gov", role = c("aut", "cre")),
    person("Iowa Department of Health and Human Services", role = "cph")
  )
Description: Hospitals, hospital systems, and even trauma systems that
    provide care to injured patients may not be aware of robust metrics
    that can help gauge the efficacy of their programs in saving the lives
    of injured patients.  'traumar' provides robust functions driven by
    the academic literature to automate the calculation of relevant
    metrics to individuals desiring to measure the performance of their
    trauma center or even a trauma system.  'traumar' also provides some
    helper functions for the data analysis journey. Users can refer to the
    following publications for descriptions of the methods used in
    'traumar'.  TRISS methodology, including probability of survival, and
    the W, M, and Z Scores - Flora (1978)
    <doi:10.1097/00005373-197810000-00003>, Boyd et al. (1987,
    PMID:3106646), Llullaku et al. (2009) <doi:10.1186/1749-7922-4-2>,
    Singh et al. (2011) <doi:10.4103/0974-2700.86626>, Baker et al. (1974,
    PMID:4814394), and Champion et al. (1989)
    <doi:10.1097/00005373-198905000-00017>. For the Relative Mortality
    Metric, see Napoli et al. (2017) <doi:10.1080/24725579.2017.1325948>,
    Schroeder et al. (2019) <doi:10.1080/10903127.2018.1489021>, and
    Kassar et al. (2016) <doi:10.1177/00031348221093563>. For more
    information about methods to calculate over- and under-triage in
    trauma hospital populations and samples, please see the following
    publications - Peng & Xiang (2016) <doi:10.1016/j.ajem.2016.08.061>,
    Beam et al. (2022) <doi:10.23937/2474-3674/1510136>, Roden-Foreman et
    al. (2017) <doi:10.1097/JTN.0000000000000283>.
License: MIT + file LICENSE
URL: https://bemts-hhs.github.io/traumar/,
        https://github.com/bemts-hhs/traumar
BugReports: https://github.com/bemts-hhs/traumar/issues
Depends: R (>= 4.1)
Imports: cli, dplyr (>= 1.2.1), ggplot2 (>= 4.0.3), glue, hms (>=
        1.1.4), infer (>= 1.1.0), lifecycle, lubridate (>= 1.9.5),
        nemsqar (>= 1.1.2), nortest (>= 1.0-4), patchwork (>= 1.3.2),
        purrr (>= 1.2.2), rlang, stats, stringr (>= 1.6.0), tibble (>=
        3.3.1), tidyr (>= 1.3.2), tidyselect (>= 1.2.1), utils
Suggests: broom, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-06 19:34:06 UTC; nfoss0
Author: Nicolas Foss [aut, cre],
  Iowa Department of Health and Human Services [cph]
Maintainer: Nicolas Foss <nicolas.foss@hhs.iowa.gov>
Repository: CRAN
Date/Publication: 2026-05-07 09:30:02 UTC
Built: R 4.4.3; ; 2026-05-07 10:05:35 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%not_in%")
export(impute)
export(is_it_normal)
export(nonlinear_bins)
export(normalize)
export(pretty_number)
export(pretty_percent)
export(probability_of_survival)
export(rm_bin_summary)
export(rmm)
export(season)
export(seqic_indicator_1)
export(seqic_indicator_10)
export(seqic_indicator_11)
export(seqic_indicator_12)
export(seqic_indicator_13)
export(seqic_indicator_2)
export(seqic_indicator_3)
export(seqic_indicator_4)
export(seqic_indicator_5)
export(seqic_indicator_6)
export(seqic_indicator_7)
export(seqic_indicator_8)
export(seqic_indicator_9)
export(small_count_label)
export(stat_sig)
export(theme_cleaner)
export(trauma_case_mix)
export(trauma_performance)
export(weekend)
import(rlang)
importFrom(glue,glue)
importFrom(infer,generate)
importFrom(lifecycle,badge)
importFrom(lifecycle,deprecated)
importFrom(nemsqar,nemsqa_binomial_confint)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(utils,head)
importFrom(utils,tail)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check](https://github.com/bemts-hhs/traumar/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/bemts-hhs/traumar/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/traumar)](https://CRAN.R-project.org/package=traumar)
![CRAN total
downloads](https://cranlogs.r-pkg.org/badges/grand-total/traumar) ![CRAN
monthly downloads](https://cranlogs.r-pkg.org/badges/last-month/traumar)
![CRAN weekly
downloads](https://cranlogs.r-pkg.org/badges/last-week/traumar)
[![Codecov test
coverage](https://codecov.io/gh/bemts-hhs/traumar/graph/badge.svg)](https://app.codecov.io/gh/bemts-hhs/traumar)
<!-- badges: end -->

# traumar <a href="https://bemts-hhs.github.io/traumar/"><img src="man/figures/logo.png" align="right" height="137" alt="traumar website" /></a>

Continuous Quality Improvement (CQI) and Process Improvement (PI) are
essential pillars of healthcare, particularly in the care of injured
patients. However, hospitals, trauma systems, and their trauma program
managers (TPMs) often lack access to standardized quality measures
derived from academic literature. The {traumar} package addresses this
gap by providing tools to calculate quality measures from trauma center
or trauma system effectiveness and efficiency, to relative mortality
efficiently and accurately. By automating these calculations, {traumar}
empowers hospital systems, trauma networks, and TPMs to focus their
efforts on analyzing outcomes and driving meaningful improvements in
patient care. Whether you’re seeking to enhance PI initiatives or
streamline CQI processes, {traumar} serves as a valuable resource for
advancing trauma care quality.

## Installation

You can install the development version of `traumar` from
[GitHub](https://github.com/bemts-hhs/traumar) with:

``` r
# install.packages("remotes")
remotes::install_github("bemts-hhs/traumar")
```

Additionally, you can install the CRAN version of `traumar` via:

``` r
install.packages("traumar")
```

## Helper Functions

{traumar} has many functions to help you in your data analysis journey!
In particular, if you do not presently have access to probability of
survival data, {traumar} provides the `probability_of_survival()`
function to do just that using the TRISS method. Check out the
additional package documentation at
<https://bemts-hhs.github.io/traumar/> where you can find examples of
each function the package has to offer.

## Trauma System Evaluation and Quality Improvement

{traumar} includes functions to allow users to calculate the efficiency
and effectiveness of one or more trauma centers, or even a trauma
system. The `seqic_indicator_*` family of functions affords users the
logic and formulas to assess their center or system on topics of
importance such as:

- Surgeon/physician/mid-level provider response times
- Data Quality
- Rate of autopsy among deceased trauma patients
- Blood alcohol measurements / drug testing
- Analysis of mortality among risk groups
- Time from patient arrival to a) physical discharge, b) decision, c)
  time from decision to physical discharge
- Trauma team activations
- Under and over triage
- …among many others. Please check the {traumar}
  <https://bemts-hhs.github.io/traumar/> documentation to explore all
  these analytical opportunities

## Calculating the W-Score

The W-Score tells us how many survivals (or deaths) on average out of
every 100 cases seen in a trauma center. Using R, we can do this with
the {traumar} package.

### First, we will create the data for these examples

``` r

# Generate example data
set.seed(123)

# Parameters
n_patients <- 5000 # Total number of patients

groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE) # Arbitrary group labels

trauma_type_values <- sample(
  x = c("Blunt", "Penetrating"),
  size = n_patients,
  replace = TRUE
) # Trauma types

rts_values <- sample(
  x = seq(from = 0, to = 7.8408, by = 0.005),
  size = n_patients,
  replace = TRUE
) # RTS values

ages <- sample(
  x = seq(from = 0, to = 100, by = 1),
  size = n_patients,
  replace = TRUE
) # patient ages

iss_scores <- sample(
  x = seq(from = 0, to = 75, by = 1),
  size = n_patients,
  replace = TRUE
) # ISS scores

# Generate survival probabilities (Ps)
Ps <- traumar::probability_of_survival(
  trauma_type = trauma_type_values,
  age = ages,
  rts = rts_values,
  iss = iss_scores
)

# Simulate survival outcomes based on Ps
survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

# Create data frame
data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
```

### The W-Score!

``` r

# Calculate trauma performance (W, M, Z scores)
trauma_performance(data, Ps_col = Ps, outcome_col = death)
#> # A tibble: 1 × 9
#>   N_Patients N_Survivors N_Deaths Predicted_Survivors Predicted_Deaths
#>        <int>       <int>    <int>               <dbl>            <dbl>
#> 1       5000        1701     3299               1711.            3289.
#> # ℹ 4 more variables: Patient_Estimate <dbl>, W_Score <dbl>, M_Score <dbl>,
#> #   Z_Score <dbl>
```

## Comparing the Probability of Survival Distribution of your Patient Mix to the Major Trauma Outcomes Study

The M and Z scores are calculated using methods defined in the
literature (Champion et al., 1990, Flora, 1978) may not be meaningful if
the distribution of the probability of survival measure is not similar
enough to the Major Trauma Outcomes Study distribution. {traumar}
provides a way to check this in your data analysis script, or even from
the console. The `trauma_performance()` function does this under the
hood for you, so you can get a read out of how much confidence you can
put into the Z score.

``` r

# Compare the current case mix with the MTOS case mix
trauma_case_mix(data, Ps_col = Ps, outcome_col = death)
#> # A tibble: 6 × 8
#>   Ps_range    current_fraction MTOS_distribution survivals predicted_survivals
#>   <chr>                  <dbl>             <dbl>     <int>               <dbl>
#> 1 0.00 - 0.25           0.541              0.01       2534                188.
#> 2 0.26 - 0.50           0.143              0.043       468                270.
#> 3 0.51 - 0.75           0.128              0           217                406.
#> 4 0.76 - 0.90           0.087              0.052        58                366.
#> 5 0.91 - 0.95           0.0508             0.053        18                238.
#> 6 0.96 - 1.00           0.0498             0.842         4                243.
#> # ℹ 3 more variables: deaths <int>, predicted_deaths <dbl>, count <int>
```

## The Relative Mortality Metric

Napoli et al.(2017) published methods for calculating a measure of
trauma center (or system) performance while overcoming a problem with
the W-Score and the TRISS methodology. Given that the majority of
patients seen at trauma centers will have a probability of survival over
90%, estimating performance based on the W-Score may only indicate how
well a center performed with lower acuity patients. Using Napoli et
al. (2017), it is possible to calculate a score that is similar to the
W-Score in its interpretability, but deals with the negatively skewed
probability of survival problem by creating non-linear bins of score
ranges, and then weighting a score based on the nature of those bins.
The Relative Mortality Metric (RMM) has a scale from -1 to 1.

- An RMM of 0 indicates that the observed mortality aligns with the
  expected national benchmark across all acuity levels.
- An RMM greater than 0 indicates better-than-expected performance,
  where the center is outperforming the national benchmark.
- An RMM less than 0 indicates under-performance, where the center’s
  observed mortality is higher than the expected benchmark.

## Non-Linear Binning Algorithm

An important part of the approach Napoli et al. (2017) took was to
modify the M-Score approach of looking at linear bins of the probability
of survival distribution, and make it non-linear. The {traumar} package
does this for you using Dr. Napoli’s method:

``` r

# Apply the nonlinear_bins function
results <- nonlinear_bins(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  divisor1 = 4,
  divisor2 = 4,
  threshold_1 = 0.9,
  threshold_2 = 0.99
)

# View intervals created by the algorithm
results$intervals
#> [1] 0.0002015449 0.0256191282 0.1455317587 0.4842820556 0.9003870455
#> [6] 0.9285354475 0.9518925450 0.9722272703 0.9968989233

# View the bin statistics
results$bin_stats
#> # A tibble: 8 × 13
#>   bin_number bin_start bin_end    mean      sd Pred_Survivors_b Pred_Deaths_b
#>        <int>     <dbl>   <dbl>   <dbl>   <dbl>            <dbl>         <dbl>
#> 1          1  0.000202  0.0256 0.00935 0.00722             10.4       1106.  
#> 2          2  0.0256    0.146  0.0732  0.0345              81.7       1033.  
#> 3          3  0.146     0.484  0.293   0.0959             327.         788.  
#> 4          4  0.484     0.900  0.697   0.124              777.         337.  
#> 5          5  0.900     0.929  0.916   0.00790            114.          10.5 
#> 6          6  0.929     0.952  0.940   0.00680            117.           7.50
#> 7          7  0.952     0.972  0.963   0.00564            120.           4.65
#> 8          8  0.972     0.997  0.984   0.00686            162.           2.67
#> # ℹ 6 more variables: AntiS_b <dbl>, AntiM_b <dbl>, alive <int>, dead <int>,
#> #   count <int>, percent <dbl>
```

## The RMM function

The RMM is sensitive to higher acuity patients, meaning that if a trauma
center struggles with these patients, it will be reflected in the RMM.
In contrast, the W-Score may mask declines in performance due to the
influence of lower acuity patients via the MTOS Distribution. The
{traumar} package automates RMM calculation as a single score using the
nonlinear binning method from Napoli et al. (2017). The `rmm()` and
`rm_bin_summary()` functions internally call `nonlinear_bins()` to
generate the non-linear binning process. The function uses bootstrap
sampling with `n_samples` to simulate an RMM distribution and estimate
95% confidence intervals. The RMM, along with corresponding confidence
intervals, are provided for the population in `data`, as well.

``` r

# Example usage of the `rmm()` function
rmm(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  n_samples = 250,
  Divisor1 = 4,
  Divisor2 = 4
)
#> # A tibble: 1 × 8
#>   population_RMM_LL population_RMM population_RMM_UL population_CI
#>               <dbl>          <dbl>             <dbl>         <dbl>
#> 1            -0.106        0.00247             0.111         0.108
#> # ℹ 4 more variables: bootstrap_RMM_LL <dbl>, bootstrap_RMM <dbl>,
#> #   bootstrap_RMM_UL <dbl>, bootstrap_CI <dbl>

# Pivoting can be helpful at times
rmm(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  n_samples = 250,
  Divisor1 = 4,
  Divisor2 = 4,
  pivot = TRUE
)
#> # A tibble: 8 × 2
#>   stat                  value
#>   <chr>                 <dbl>
#> 1 population_RMM_LL -0.106   
#> 2 population_RMM     0.00247 
#> 3 population_RMM_UL  0.111   
#> 4 population_CI      0.108   
#> 5 bootstrap_RMM_LL   0.000492
#> 6 bootstrap_RMM      0.00218 
#> 7 bootstrap_RMM_UL   0.00386 
#> 8 bootstrap_CI       0.00168

# RMM calculated by non-linear bin range
# `rm_bin_summary()` function
rm_bin_summary(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  Divisor1 = 4,
  Divisor2 = 4,
  n_samples = 250
)
#> # A tibble: 8 × 19
#>   bin_number  TA_b  TD_b   N_b  EM_b AntiS_b AntiM_b bin_start bin_end midpoint
#>        <int> <int> <int> <int> <dbl>   <dbl>   <dbl>     <dbl>   <dbl>    <dbl>
#> 1          1     9  1107  1116 0.992 0.00935  0.991   0.000202  0.0256   0.0129
#> 2          2    72  1043  1115 0.935 0.0732   0.927   0.0256    0.146    0.0856
#> 3          3   300   815  1115 0.731 0.293    0.707   0.146     0.484    0.315 
#> 4          4   805   309  1114 0.277 0.697    0.303   0.484     0.900    0.692 
#> 5          5   115    10   125 0.08  0.916    0.0844  0.900     0.929    0.914 
#> 6          6   116     9   125 0.072 0.940    0.0600  0.929     0.952    0.940 
#> 7          7   119     6   125 0.048 0.963    0.0372  0.952     0.972    0.962 
#> 8          8   165     0   165 0     0.984    0.0162  0.972     0.997    0.985 
#> # ℹ 9 more variables: R_b <dbl>, population_RMM_LL <dbl>, population_RMM <dbl>,
#> #   population_RMM_UL <dbl>, population_CI <dbl>, bootstrap_RMM_LL <dbl>,
#> #   bootstrap_RMM <dbl>, bootstrap_RMM_UL <dbl>, bootstrap_CI <dbl>
```

## Code of Conduct

Please note that the traumar project is released with a [Contributor
Code of
Conduct](https://bemts-hhs.github.io/traumar/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

#' @title SEQIC Indicator 9 - Emergency Department Transfer Timeliness
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Calculates the proportion of EMS-transferred trauma patients who experienced
#' delayed transfer from the emergency department (ED) based on disposition and
#' decision-to-transfer time frames. This includes both overall rates and
#' stratified results by trauma team activation status, with optional confidence
#' intervals.
#'
#' @inheritParams seqic_indicator_1
#' @inheritParams seqic_indicator_4
#' @inheritParams seqic_indicator_5
#' @inheritParams seqic_indicator_6
#' @inheritParams seqic_indicator_8
#'
#' @param transport_method Column identifying the EMS transport method (e.g.,
#'   ambulance, private vehicle). Used to exclude non-qualified modes of
#'   arrival.
#' @param trauma_team_activated Column indicating whether the trauma team was
#'   activated (character, factor, or logical).
#' @param ed_decision_LOS Numeric column representing minutes from ED arrival to
#'   decision to transfer.
#' @param ed_decision_discharge_LOS Numeric column representing minutes from ED
#'   decision to discharge to physical discharge.
#'
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details
#'
#'  This function:
#' \itemize{
#'   \item Filters the dataset to include only transfers out from trauma centers
#'   designated Level I through IV.
#'   \item Deduplicates records using `unique_incident_id`.
#'   \item Flags records where emergency department decision to discharge
#'   occurred more than 60 or 120 minutes after ED arrival.
#'   \item Flags records where physical departure from the ED occurred more than
#'   120 or 180 minutes after ED arrival.
#'   \item Flags records where physical discharge occurred more than 60 or 120
#'   minutes after ED decision to discharge.
#'   \item Stratifies results by trauma team activation status and
#'   one or more grouping variables.
#'   \item Stratifies results by risk groups and one or more grouping variables.
#'   \item Returns a summarized tibble with the number of delayed cases
#'   (numerator), eligible records (denominator), and the proportion for each
#'   delay threshold.
#'   \item Optionally includes 95% confidence intervals if `calculate_ci =
#'   TRUE`.
#' }
#'
#' @note
#'
#' This function calculates discharge timeliness outcomes for patients
#' transported to trauma centers, stratified by risk of mortality. Risk
#' groups—low, moderate, and high— are defined by the Iowa System Evaluation and
#' Quality Improvement Committee (SEQIC) as described below. Users may also
#' apply alternative risk stratification methods if preferred.
#'
#' \itemize{
#'   \item Abnormal Physiology Criteria: GCS 3–5; Respirations <5 or >30 per
#'   minute; Systolic BP <60 mm Hg
#'   \item High Risk: Probability of Survival < 0.2; ISS > 41; ISS > 24 with
#'   abnormal physiology
#'   \item Moderate Risk: Probability of Survival 0.2–0.5; ISS 16–41
#'   \item Low Risk: Probability of Survival > 0.5; ISS < 16; Normal physiology
#' }
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @returns
#' A list of four tibbles, with optional 95% confidence intervals:
#' \itemize{
#'   \item{`seqic_9_all`}: Proportion of transferred trauma patients with ED
#'   discharge or decision delays >2 or >3 hours, grouped by optional
#'   variables.
#'   \item{`seqic_9_activations`}: Same proportions as above, further stratified
#'   by trauma team activation status.
#'   \item{`seqic_9_risk`}: Same proportions as above, further stratified by
#'   risk groups.
#'   \item{`seqic_9_activations_risk`}: Same proportions as above, further
#'   stratified by risk groups and trauma team activation status.
#' }
#'
#' Each tibble includes numerators, denominators, proportions, and (optionally)
#' confidence intervals for:
#' \itemize{
#'   \item{9a}: Delayed discharge >2 hours
#'   \item{9b}: Delayed discharge >3 hours
#'   \item{9c}: Delayed decision >1 hours
#'   \item{9d}: Delayed decision >2 hours
#'   \item{9e}: Delayed decision to discharge >1 hour
#'   \item{9f}: Delayed decision to discharge >2 hours
#' }
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Simulated dataset for SEQIC Indicator 9
#' test_data <- tibble::tibble(
#'   id = as.character(1:10),
#'   trauma_level = c("I", "II", "III", "IV", "V", "II", "III", "IV", "I",
#'   "II"),
#'   transport = c("Ambulance", "Ambulance", "Private Vehicle", "Ambulance",
#'   "Helicopter", "Ambulance", "Ambulance", "Ambulance", "Ambulance",
#'   "Ambulance"),
#'   activated = c(TRUE, FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, FALSE,
#'   FALSE),
#'   ed_LOS = c(120, 180, 90, 60, 200, 130, 110, 160, 95, 220),
#'   ed_decision = c(55, 125, 65, 30, 190, 80, 70, 45, 61, 130),
#'   ed_discharge = c(130, 185, 110, 65, 150, 160, 95, 180, 70, 210),
#'   transfer_out = c(TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, FALSE, TRUE,
#'   TRUE),
#'   risk = c("High", "High", "Moderate", "Low", "Moderate", "Low",
#'            "High", "Low", "Moderate", "High")
#' )
#'
#' # Run the function, and store as a list object
#' seqic_9_result <- traumar::seqic_indicator_9(
#'   data = test_data,
#'   level = trauma_level,
#'   included_levels = c("I", "II", "III", "IV"),
#'   unique_incident_id = id,
#'   transport_method = transport,
#'   transfer_out_indicator = transfer_out,
#'   ed_LOS = ed_LOS,
#'   ed_decision_LOS = ed_decision,
#'   ed_decision_discharge_LOS = ed_discharge,
#'   trauma_team_activated = activated,
#'   risk_group = risk
#' )
#'
#' # Take a look at the overall output of the function
#' seqic_9_result$overall |>
#' tidyr::pivot_longer(cols = -1,
#'                     names_to = "Indicator",
#'                     values_to = "Values"
#'                     )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_9 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  transfer_out_indicator,
  transport_method,
  unique_incident_id,
  trauma_team_activated,
  risk_group,
  ed_LOS,
  ed_decision_LOS,
  ed_decision_discharge_LOS,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure that `transfer_out_indicator` can be validated  ----
  transfer_out_indicator_check <- validate_data_pull(
    input = data,
    col = {{ transfer_out_indicator }},
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Validate `transfer_out_indicator` ----
  validate_class(
    input = transfer_out_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Ensure that `transport_method` can be validated ----
  transport_method_check <- validate_data_pull(
    input = data,
    col = {{ transport_method }},
    type = "error",
    var_name = "transport_method"
  )

  # Validate that `transport_method` is character or factor ----
  validate_character_factor(
    input = transport_method_check,
    type = "error",
    var_name = "transport_method"
  )

  # Ensure that `ed_LOS` can be validated ----
  ed_los_check <- validate_data_pull(
    input = data,
    col = {{ ed_LOS }},
    type = "error",
    var_name = "ed_LOS"
  )

  # Validate `ed_LOS` ----
  validate_numeric(input = ed_los_check, type = "error", var_name = "ed_LOS")

  # Ensure that `ed_decision_LOS` can be validated ----
  ed_decision_los_check <- validate_data_pull(
    input = data,
    col = {{ ed_decision_LOS }},
    type = "error",
    var_name = "ed_decision_LOS"
  )

  # Validate `ed_decision_LOS` ----
  validate_numeric(
    input = ed_decision_los_check,
    type = "error",
    var_name = "ed_decision_LOS"
  )

  # Ensure that `ed_decision_discharge_LOS` can be validated ----
  ed_decision_discharge_los_check <- validate_data_pull(
    input = data,
    col = {{ ed_decision_discharge_LOS }},
    type = "error",
    var_name = "ed_decision_discharge_LOS"
  )

  # Validate `ed_decision_discharge_LOS`
  validate_numeric(
    input = ed_decision_discharge_los_check,
    type = "error",
    var_name = "ed_decision_discharge_LOS"
  )

  #
  trauma_team_activated_check <- validate_data_pull(
    input = data,
    col = {{ trauma_team_activated }},
    type = "error",
    var_name = "trauma_team_activated"
  )

  # Validate that `trauma_team_activated` is character, factor, or logical ----
  validate_class(
    input = trauma_team_activated_check,
    class_type = c("character", "factor", "logical"),
    type = "error",
    var_name = "trauma_team_activated"
  )

  # Ensure that `risk_group` can be validated
  risk_group_check <- validate_data_pull(
    input = data,
    col = {{ risk_group }},
    type = "error",
    var_name = "risk_group"
  )

  # Validate the `risk_group` column
  validate_character_factor(
    input = risk_group_check,
    type = "error",
    var_name = "risk_group"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data`.
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  # Get vector of transport methods as strings that will not be included ----
  excluded_transport_methods <- paste(
    "private vehicle",
    "public vehicle",
    "walk[\\s-]in",
    "not\\s(?:known|recorded|applicable)",
    "other",
    sep = "|"
  )

  # Create a regex from `excluded_transport_methods` ----
  excluded_transport_methods_regex <- paste0(
    "(?:",
    excluded_transport_methods,
    ")"
  )

  # Initiate the output list
  seqic_9 <- list()

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Get `data` with manipulations ----
  data_prep <- data |>
    dplyr::filter(
      {{ level }} %in% included_levels,
      {{ transfer_out_indicator }} %in% c("Yes", TRUE),
      !grepl(
        pattern = excluded_transport_methods_regex,
        x = {{ transport_method }},
        ignore.case = TRUE
      )
    ) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::mutate(
      Delayed_DC_2hr = {{ ed_LOS }} > 120,
      Delayed_DC_3hr = {{ ed_LOS }} > 180,
      Delayed_Decision_1hr = {{ ed_decision_LOS }} > 60,
      Delayed_Decision_2hr = {{ ed_decision_LOS }} > 120,
      Delayed_Decision_Discharge_1hr = {{ ed_decision_discharge_LOS }} > 60,
      Delayed_Decision_Discharge_2hr = {{ ed_decision_discharge_LOS }} > 120
    )

  ###___________________________________________________________________________
  ### Overall ----
  ###___________________________________________________________________________

  # 9a-f overall ----
  seqic_9_all <- data_prep |>
    dplyr::summarize(
      numerator_9a_all = sum(Delayed_DC_2hr == TRUE, na.rm = TRUE),
      denominator_9a_all = dplyr::n(),
      seqic_9a_all = dplyr::if_else(
        denominator_9a_all > 0,
        numerator_9a_all / denominator_9a_all,
        NA_real_
      ),
      numerator_9b_all = sum(Delayed_DC_3hr == TRUE, na.rm = TRUE),
      denominator_9b_all = dplyr::n(),
      seqic_9b_all = dplyr::if_else(
        denominator_9b_all > 0,
        numerator_9b_all / denominator_9b_all,
        NA_real_
      ),
      numerator_9c_all = sum(Delayed_Decision_1hr == TRUE, na.rm = TRUE),
      denominator_9c_all = dplyr::n(),
      seqic_9c_all = dplyr::if_else(
        denominator_9c_all > 0,
        numerator_9c_all / denominator_9c_all,
        NA_real_
      ),
      numerator_9d_all = sum(Delayed_Decision_2hr == TRUE, na.rm = TRUE),
      denominator_9d_all = dplyr::n(),
      seqic_9d_all = dplyr::if_else(
        denominator_9d_all > 0,
        numerator_9d_all / denominator_9d_all,
        NA_real_
      ),
      numerator_9e_all = sum(
        Delayed_Decision_Discharge_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9e_all = dplyr::n(),
      seqic_9e_all = dplyr::if_else(
        denominator_9e_all > 0,
        numerator_9e_all / denominator_9e_all,
        NA_real_
      ),
      numerator_9f_all = sum(
        Delayed_Decision_Discharge_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9f_all = dplyr::n(),
      seqic_9f_all = dplyr::if_else(
        denominator_9f_all > 0,
        numerator_9f_all / denominator_9f_all,
        NA_real_
      ),
      .by = {{ groups }}
    )

  ###___________________________________________________________________________
  ### Activations ----
  ###___________________________________________________________________________

  # 9a-f for activations ----
  seqic_9_activations <- data_prep |>
    dplyr::summarize(
      numerator_9a_activations = sum(Delayed_DC_2hr == TRUE, na.rm = TRUE),
      denominator_9a_activations = dplyr::n(),
      seqic_9a_activations = dplyr::if_else(
        denominator_9a_activations > 0,
        numerator_9a_activations / denominator_9a_activations,
        NA_real_
      ),
      numerator_9b_activations = sum(Delayed_DC_3hr == TRUE, na.rm = TRUE),
      denominator_9b_activations = dplyr::n(),
      seqic_9b_activations = dplyr::if_else(
        denominator_9b_activations > 0,
        numerator_9b_activations / denominator_9b_activations,
        NA_real_
      ),
      numerator_9c_activations = sum(
        Delayed_Decision_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9c_activations = dplyr::n(),
      seqic_9c_activations = dplyr::if_else(
        denominator_9c_activations > 0,
        numerator_9c_activations / denominator_9c_activations,
        NA_real_
      ),
      numerator_9d_activations = sum(
        Delayed_Decision_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9d_activations = dplyr::n(),
      seqic_9d_activations = dplyr::if_else(
        denominator_9d_activations > 0,
        numerator_9d_activations / denominator_9d_activations,
        NA_real_
      ),
      numerator_9e_activations = sum(
        Delayed_Decision_Discharge_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9e_activations = dplyr::n(),
      seqic_9e_activations = dplyr::if_else(
        denominator_9e_activations > 0,
        numerator_9e_activations / denominator_9e_activations,
        NA_real_
      ),
      numerator_9f_activations = sum(
        Delayed_Decision_Discharge_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9f_activations = dplyr::n(),
      seqic_9f_activations = dplyr::if_else(
        denominator_9f_activations > 0,
        numerator_9f_activations / denominator_9f_activations,
        NA_real_
      ),
      .by = c({{ groups }}, {{ trauma_team_activated }})
    )

  ###___________________________________________________________________________
  ### Risk Groups ----
  ###___________________________________________________________________________

  # 9a-f for risk groups ----
  seqic_9_risk <- data_prep |>
    dplyr::summarize(
      numerator_9a_risk = sum(Delayed_DC_2hr == TRUE, na.rm = TRUE),
      denominator_9a_risk = dplyr::n(),
      seqic_9a_risk = dplyr::if_else(
        denominator_9a_risk > 0,
        numerator_9a_risk / denominator_9a_risk,
        NA_real_
      ),
      numerator_9b_risk = sum(Delayed_DC_3hr == TRUE, na.rm = TRUE),
      denominator_9b_risk = dplyr::n(),
      seqic_9b_risk = dplyr::if_else(
        denominator_9b_risk > 0,
        numerator_9b_risk / denominator_9b_risk,
        NA_real_
      ),
      numerator_9c_risk = sum(
        Delayed_Decision_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9c_risk = dplyr::n(),
      seqic_9c_risk = dplyr::if_else(
        denominator_9c_risk > 0,
        numerator_9c_risk / denominator_9c_risk,
        NA_real_
      ),
      numerator_9d_risk = sum(
        Delayed_Decision_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9d_risk = dplyr::n(),
      seqic_9d_risk = dplyr::if_else(
        denominator_9d_risk > 0,
        numerator_9d_risk / denominator_9d_risk,
        NA_real_
      ),
      numerator_9e_risk = sum(
        Delayed_Decision_Discharge_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9e_risk = dplyr::n(),
      seqic_9e_risk = dplyr::if_else(
        denominator_9e_risk > 0,
        numerator_9e_risk / denominator_9e_risk,
        NA_real_
      ),
      numerator_9f_risk = sum(
        Delayed_Decision_Discharge_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9f_risk = dplyr::n(),
      seqic_9f_risk = dplyr::if_else(
        denominator_9f_risk > 0,
        numerator_9f_risk / denominator_9f_risk,
        NA_real_
      ),
      .by = c({{ groups }}, {{ risk_group }})
    )

  ###___________________________________________________________________________
  ### Activations and Risk Groups ----
  ###___________________________________________________________________________

  # 9a-f for risk groups and trauma team activations ----
  seqic_9_activations_risk <- data_prep |>
    dplyr::summarize(
      numerator_9a_activations_risk = sum(Delayed_DC_2hr == TRUE, na.rm = TRUE),
      denominator_9a_activations_risk = dplyr::n(),
      seqic_9a_activations_risk = dplyr::if_else(
        denominator_9a_activations_risk > 0,
        numerator_9a_activations_risk / denominator_9a_activations_risk,
        NA_real_
      ),
      numerator_9b_activations_risk = sum(Delayed_DC_3hr == TRUE, na.rm = TRUE),
      denominator_9b_activations_risk = dplyr::n(),
      seqic_9b_activations_risk = dplyr::if_else(
        denominator_9b_activations_risk > 0,
        numerator_9b_activations_risk / denominator_9b_activations_risk,
        NA_real_
      ),
      numerator_9c_activations_risk = sum(
        Delayed_Decision_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9c_activations_risk = dplyr::n(),
      seqic_9c_activations_risk = dplyr::if_else(
        denominator_9c_activations_risk > 0,
        numerator_9c_activations_risk / denominator_9c_activations_risk,
        NA_real_
      ),
      numerator_9d_activations_risk = sum(
        Delayed_Decision_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9d_activations_risk = dplyr::n(),
      seqic_9d_activations_risk = dplyr::if_else(
        denominator_9d_activations_risk > 0,
        numerator_9d_activations_risk / denominator_9d_activations_risk,
        NA_real_
      ),
      numerator_9e_activations_risk = sum(
        Delayed_Decision_Discharge_1hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9e_activations_risk = dplyr::n(),
      seqic_9e_activations_risk = dplyr::if_else(
        denominator_9e_activations_risk > 0,
        numerator_9e_activations_risk / denominator_9e_activations_risk,
        NA_real_
      ),
      numerator_9f_activations_risk = sum(
        Delayed_Decision_Discharge_2hr == TRUE,
        na.rm = TRUE
      ),
      denominator_9f_activations_risk = dplyr::n(),
      seqic_9f_activations_risk = dplyr::if_else(
        denominator_9f_activations_risk > 0,
        numerator_9f_activations_risk / denominator_9f_activations_risk,
        NA_real_
      ),
      .by = c({{ groups }}, {{ trauma_team_activated }}, {{ risk_group }})
    )

  ###___________________________________________________________________________
  ### Optional 95% CIs ----
  ###___________________________________________________________________________

  # Compute confidence intervals if requested ----
  if (!is.null(calculate_ci)) {
    # Overall CIs
    seqic_9_all <- seqic_9_all |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_all,
          x = numerator_9a_all,
          n = denominator_9a_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_9a_all = lower_ci, upper_ci_9a_all = upper_ci),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_all,
          x = numerator_9b_all,
          n = denominator_9b_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9b_all = lower_ci,
            upper_ci_9b_all = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_all,
          x = numerator_9c_all,
          n = denominator_9c_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9c_all = lower_ci,
            upper_ci_9c_all = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_all,
          x = numerator_9d_all,
          n = denominator_9d_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9d_all = lower_ci,
            upper_ci_9d_all = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_all,
          x = numerator_9e_all,
          n = denominator_9e_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9e_all = lower_ci,
            upper_ci_9e_all = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_all,
          x = numerator_9f_all,
          n = denominator_9f_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9f_all = lower_ci,
            upper_ci_9f_all = upper_ci
          )
      ) |>
      dplyr::relocate(lower_ci_9a_all, .after = seqic_9a_all) |>
      dplyr::relocate(upper_ci_9a_all, .after = lower_ci_9a_all) |>
      dplyr::relocate(lower_ci_9b_all, .after = seqic_9b_all) |>
      dplyr::relocate(upper_ci_9b_all, .after = lower_ci_9b_all) |>
      dplyr::relocate(lower_ci_9c_all, .after = seqic_9c_all) |>
      dplyr::relocate(upper_ci_9c_all, .after = lower_ci_9c_all) |>
      dplyr::relocate(lower_ci_9d_all, .after = seqic_9d_all) |>
      dplyr::relocate(upper_ci_9d_all, .after = lower_ci_9d_all) |>
      dplyr::relocate(lower_ci_9e_all, .after = seqic_9e_all) |>
      dplyr::relocate(upper_ci_9e_all, .after = lower_ci_9e_all) |>
      dplyr::relocate(lower_ci_9f_all, .after = seqic_9f_all) |>
      dplyr::relocate(upper_ci_9f_all, .after = lower_ci_9f_all)

    # Activations CIs ----
    seqic_9_activations <- seqic_9_activations |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations,
          x = numerator_9a_activations,
          n = denominator_9a_activations,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9a_activations = lower_ci,
            upper_ci_9a_activations = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations,
          x = numerator_9b_activations,
          n = denominator_9b_activations,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9b_activations = lower_ci,
            upper_ci_9b_activations = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations,
          x = numerator_9c_activations,
          n = denominator_9c_activations,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9c_activations = lower_ci,
            upper_ci_9c_activations = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations,
          x = numerator_9d_activations,
          n = denominator_9d_activations,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9d_activations = lower_ci,
            upper_ci_9d_activations = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations,
          x = numerator_9e_activations,
          n = denominator_9e_activations,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9e_activations = lower_ci,
            upper_ci_9e_activations = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations,
          x = numerator_9f_activations,
          n = denominator_9f_activations,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9f_activations = lower_ci,
            upper_ci_9f_activations = upper_ci
          )
      ) |>
      dplyr::relocate(
        lower_ci_9a_activations,
        .after = seqic_9a_activations
      ) |>
      dplyr::relocate(
        upper_ci_9a_activations,
        .after = lower_ci_9a_activations
      ) |>
      dplyr::relocate(
        lower_ci_9b_activations,
        .after = seqic_9b_activations
      ) |>
      dplyr::relocate(
        upper_ci_9b_activations,
        .after = lower_ci_9b_activations
      ) |>
      dplyr::relocate(
        lower_ci_9c_activations,
        .after = seqic_9c_activations
      ) |>
      dplyr::relocate(
        upper_ci_9c_activations,
        .after = lower_ci_9c_activations
      ) |>
      dplyr::relocate(
        lower_ci_9d_activations,
        .after = seqic_9d_activations
      ) |>
      dplyr::relocate(
        upper_ci_9d_activations,
        .after = lower_ci_9d_activations
      ) |>
      dplyr::relocate(
        lower_ci_9e_activations,
        .after = seqic_9e_activations
      ) |>
      dplyr::relocate(
        upper_ci_9e_activations,
        .after = lower_ci_9e_activations
      ) |>
      dplyr::relocate(
        lower_ci_9f_activations,
        .after = seqic_9f_activations
      ) |>
      dplyr::relocate(
        upper_ci_9f_activations,
        .after = lower_ci_9f_activations
      )

    # Risk Groups CIs ----
    seqic_9_risk <- seqic_9_risk |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_risk,
          x = numerator_9a_risk,
          n = denominator_9a_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9a_risk = lower_ci,
            upper_ci_9a_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_risk,
          x = numerator_9b_risk,
          n = denominator_9b_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9b_risk = lower_ci,
            upper_ci_9b_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_risk,
          x = numerator_9c_risk,
          n = denominator_9c_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9c_risk = lower_ci,
            upper_ci_9c_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_risk,
          x = numerator_9d_risk,
          n = denominator_9d_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9d_risk = lower_ci,
            upper_ci_9d_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_risk,
          x = numerator_9e_risk,
          n = denominator_9e_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9e_risk = lower_ci,
            upper_ci_9e_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_risk,
          x = numerator_9f_risk,
          n = denominator_9f_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9f_risk = lower_ci,
            upper_ci_9f_risk = upper_ci
          )
      ) |>
      dplyr::relocate(
        lower_ci_9a_risk,
        .after = seqic_9a_risk
      ) |>
      dplyr::relocate(
        upper_ci_9a_risk,
        .after = lower_ci_9a_risk
      ) |>
      dplyr::relocate(
        lower_ci_9b_risk,
        .after = seqic_9b_risk
      ) |>
      dplyr::relocate(
        upper_ci_9b_risk,
        .after = lower_ci_9b_risk
      ) |>
      dplyr::relocate(
        lower_ci_9c_risk,
        .after = seqic_9c_risk
      ) |>
      dplyr::relocate(
        upper_ci_9c_risk,
        .after = lower_ci_9c_risk
      ) |>
      dplyr::relocate(
        lower_ci_9d_risk,
        .after = seqic_9d_risk
      ) |>
      dplyr::relocate(
        upper_ci_9d_risk,
        .after = lower_ci_9d_risk
      ) |>
      dplyr::relocate(
        lower_ci_9e_risk,
        .after = seqic_9e_risk
      ) |>
      dplyr::relocate(
        upper_ci_9e_risk,
        .after = lower_ci_9e_risk
      ) |>
      dplyr::relocate(
        lower_ci_9f_risk,
        .after = seqic_9f_risk
      ) |>
      dplyr::relocate(
        upper_ci_9f_risk,
        .after = lower_ci_9f_risk
      )

    # Activations and Risk Groups CIs ----
    seqic_9_activations_risk <- seqic_9_activations_risk |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations_risk,
          x = numerator_9a_activations_risk,
          n = denominator_9a_activations_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9a_activations_risk = lower_ci,
            upper_ci_9a_activations_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations_risk,
          x = numerator_9b_activations_risk,
          n = denominator_9b_activations_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9b_activations_risk = lower_ci,
            upper_ci_9b_activations_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations_risk,
          x = numerator_9c_activations_risk,
          n = denominator_9c_activations_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9c_activations_risk = lower_ci,
            upper_ci_9c_activations_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations_risk,
          x = numerator_9d_activations_risk,
          n = denominator_9d_activations_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9d_activations_risk = lower_ci,
            upper_ci_9d_activations_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations_risk,
          x = numerator_9e_activations_risk,
          n = denominator_9e_activations_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9e_activations_risk = lower_ci,
            upper_ci_9e_activations_risk = upper_ci
          ),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_9_activations_risk,
          x = numerator_9f_activations_risk,
          n = denominator_9f_activations_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(
            lower_ci_9f_activations_risk = lower_ci,
            upper_ci_9f_activations_risk = upper_ci
          )
      ) |>
      dplyr::relocate(
        lower_ci_9a_activations_risk,
        .after = seqic_9a_activations_risk
      ) |>
      dplyr::relocate(
        upper_ci_9a_activations_risk,
        .after = lower_ci_9a_activations_risk
      ) |>
      dplyr::relocate(
        lower_ci_9b_activations_risk,
        .after = seqic_9b_activations_risk
      ) |>
      dplyr::relocate(
        upper_ci_9b_activations_risk,
        .after = lower_ci_9b_activations_risk
      ) |>
      dplyr::relocate(
        lower_ci_9c_activations_risk,
        .after = seqic_9c_activations_risk
      ) |>
      dplyr::relocate(
        upper_ci_9c_activations_risk,
        .after = lower_ci_9c_activations_risk
      ) |>
      dplyr::relocate(
        lower_ci_9d_activations_risk,
        .after = seqic_9d_activations_risk
      ) |>
      dplyr::relocate(
        upper_ci_9d_activations_risk,
        .after = lower_ci_9d_activations_risk
      ) |>
      dplyr::relocate(
        lower_ci_9e_activations_risk,
        .after = seqic_9e_activations_risk
      ) |>
      dplyr::relocate(
        upper_ci_9e_activations_risk,
        .after = lower_ci_9e_activations_risk
      ) |>
      dplyr::relocate(
        lower_ci_9f_activations_risk,
        .after = seqic_9f_activations_risk
      ) |>
      dplyr::relocate(
        upper_ci_9f_activations_risk,
        .after = lower_ci_9f_activations_risk
      )
  }

  # Label output or arrange by grouping vars <-
  if (is.null(groups)) {
    seqic_9$overall <- seqic_9_all |>
      tibble::add_column(data = "population/sample", .before = 1)
    seqic_9$activations <- seqic_9_activations |>
      tibble::add_column(data = "population/sample TTA groups", .before = 1) |>
      dplyr::arrange({{ trauma_team_activated }})
    seqic_9$risk_groups <- seqic_9_risk |>
      tibble::add_column(data = "population/sample risk groups", .before = 1) |>
      dplyr::arrange({{ risk_group }})
    seqic_9$activations_risk_groups <- seqic_9_activations_risk |>
      tibble::add_column(
        data = "population/sample TTA and risk groups",
        .before = 1
      ) |>
      dplyr::arrange(
        {{ trauma_team_activated }},
        {{ risk_group }}
      )
  } else {
    seqic_9$overall <- seqic_9_all |>
      dplyr::arrange(!!!rlang::syms(groups))
    seqic_9$activations <- seqic_9_activations |>
      dplyr::arrange(!!!rlang::syms(groups), {{ trauma_team_activated }})
    seqic_9$risk_groups <- seqic_9_risk |>
      dplyr::arrange(!!!rlang::syms(groups), {{ risk_group }})
    seqic_9$activations_risk_groups <- seqic_9_activations_risk |>
      dplyr::arrange(
        !!!rlang::syms(groups),
        {{ trauma_team_activated }},
        {{ risk_group }}
      )
  }

  # Return the output as a list ----
  return(seqic_9)
}

# tests/testthat/test-seqic_indicator_9.R

testthat::test_that("seqic_indicator_9() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  test_data <- dplyr::tibble(
    id = as.character(1:6),
    trauma_level = c("I", "II", "V", "III", "II", "IV"),
    transport = c(
      "Ambulance",
      "Private Vehicle",
      "Ambulance",
      "walk-in",
      "Ambulance",
      "Other"
    ),
    transfer_out = c(TRUE, TRUE, TRUE, TRUE, FALSE, TRUE),
    activated = c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE),
    ed_LOS = c(121, 90, 200, 100, 80, 130),
    ed_decision = c(61, 55, 130, 50, 30, 125),
    ed_discharge = c(130, 110, 140, 70, 60, 160),
    risk = c("High", "Moderate", "Low", "High", "Moderate", "Low")
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = false,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = not_a_column,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  bad_method <- test_data |> dplyr::mutate(transport_method = 1:6)
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = bad_method,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport_method,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "transport_method.*must be of class.*character.*or.*factor"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = fake,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_hospitals,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = length_of_stay,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = decision_time,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = a_fake,
      trauma_team_activated = activated,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = uninterested,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transport_method = transport,
      transfer_out_indicator = transfer_out,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge,
      trauma_team_activated = activated,
      risk_group = TRUE
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_9() validates input types correctly", {
  data <- tibble::tibble(
    id = 1:3,
    trauma_level = c("I", "II", "III"),
    transport = c("Ambulance", "Ambulance", "Private"),
    activated = c(TRUE, FALSE, TRUE),
    ed_LOS = c(100, 120, 80),
    ed_decision = c(60, 70, 50),
    ed_discharge = c(130, 150, 110),
    transfer_out = c(TRUE, FALSE, TRUE),
    risk = c("High", "Moderate", "Low")
  )

  # data is not a data.frame or tibble
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = as.matrix(data),
      level = trauma_level,
      included_levels = c("I", "II", "III"),
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      transport_method = transport,
      trauma_team_activated = activated,
      risk_group = risk,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge
    ),
    "data.*must be of class.*data.frame, tbl, tbl_df"
  )

  # level column invalid type
  bad_data <- dplyr::mutate(data, trauma_level = as.numeric(1:3))
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III"),
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      transport_method = transport,
      trauma_team_activated = activated,
      risk_group = risk,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge
    ),
    "level.*must be of class.*character.*factor"
  )

  # unique_incident_id wrong class
  bad_data <- dplyr::mutate(data, id = list(1, 2, 3)) # list column
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III"),
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      transport_method = transport,
      trauma_team_activated = activated,
      risk_group = risk,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge
    ),
    "unique_incident_id.*must be of class.*numeric, integer, character, factor"
  )

  # transfer_out_indicator wrong class
  bad_data <- dplyr::mutate(data, transfer_out = as.Date("2024-01-01") + 0:2)
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III"),
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      transport_method = transport,
      trauma_team_activated = activated,
      risk_group = risk,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge
    ),
    "transfer_out_indicator.*must be of class"
  )

  # ed_LOS not numeric
  bad_data <- dplyr::mutate(data, ed_LOS = as.character(ed_LOS))
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III"),
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      transport_method = transport,
      trauma_team_activated = activated,
      risk_group = risk,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision,
      ed_decision_discharge_LOS = ed_discharge
    ),
    "ed_LOS.*must be.*numeric"
  )
})

testthat::test_that("seqic_indicator_9() input validation: remaining checks", {
  data <- dplyr::tibble(
    incident_id = "G",
    probability_of_survival = 0.95,
    trauma_center_level = "I",
    transfer_out = TRUE,
    transport_method = "Ambulance",
    ed_LOS = 3,
    ed_decision_LOS = "2 hours",
    ed_decision_discharge_LOS = 1.5,
    trauma_team_activated = 1L,
    risk_group = 3L
  )

  # ed_decision_LOS must be numeric
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS
    ),
    "ed_decision_LOS.*must be.*numeric"
  )

  # ed_decision_discharge_LOS must be numeric
  data$ed_decision_LOS <- 2.0
  data$ed_decision_discharge_LOS <- "1.5 hours"
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS
    ),
    "ed_decision_discharge_LOS.*must be.*numeric"
  )

  # trauma_team_activated must be character, factor, or logical
  data$ed_decision_discharge_LOS <- 1.5
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS
    ),
    "trauma_team_activated.*must be of class.*character, factor, logical"
  )

  # risk_group must be character or factor
  data$trauma_team_activated <- "Yes"
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS
    ),
    "risk_group.*must be of class.*character.*factor"
  )

  # groups must be character vector
  data$risk_group <- "Low"
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS,
      groups = 123
    ),
    "groups.*must be of class.*character.*factor"
  )

  # groups must exist in data
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS,
      groups = c("not_a_column")
    ),
    "groups.*contains invalid column names.*not_a_column.*Valid column names are.*incident_id, probability_of_survival, trauma_center_level, transfer_out, transport_method, ed_LOS, ed_decision_LOS, ed_decision_discharge_LOS, trauma_team_activated, risk_group"
  )

  # calculate_ci must be "wilson" or "clopper-pearson"
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS,
      calculate_ci = "bootstrap"
    ),
    "calculate_ci.*contains invalid values.*bootstrap.*Valid values are.*wilson, clopper-pearson"
  )

  # included_levels must be character, numeric, or factor
  testthat::expect_error(
    traumar::seqic_indicator_9(
      data,
      level = trauma_center_level,
      transfer_out_indicator = transfer_out,
      transport_method = transport_method,
      unique_incident_id = incident_id,
      trauma_team_activated = trauma_team_activated,
      risk_group = risk_group,
      ed_LOS = ed_LOS,
      ed_decision_LOS = ed_decision_LOS,
      ed_decision_discharge_LOS = ed_decision_discharge_LOS,
      included_levels = as.Date("2024-01-01")
    ),
    "included_levels.*must be of class.*numeric, character, factor, integer"
  )
})

testthat::test_that("seqic_indicator_9 filtering and flags behave as expected", {
  test_data <- dplyr::tibble(
    id = as.character(1:6),
    trauma_level = c("I", "II", "V", "III", "II", "IV"),
    transport = c(
      "Ambulance",
      "Private Vehicle",
      "Ambulance",
      "walk-in",
      "Ambulance",
      "Other"
    ),
    transfer_out = c(TRUE, TRUE, TRUE, TRUE, FALSE, TRUE),
    activated = c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE),
    ed_LOS = c(121, 90, 200, 100, 80, 130),
    ed_decision = c(61, 55, 130, 50, 30, 125),
    ed_discharge = c(130, 110, 140, 70, 60, 160),
    risk = c("High", "Moderate", "Low", "High", "Moderate", "Low")
  )

  result <- traumar::seqic_indicator_9(
    data = test_data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id = id,
    transport_method = transport,
    transfer_out_indicator = transfer_out,
    ed_LOS = ed_LOS,
    ed_decision_LOS = ed_decision,
    ed_decision_discharge_LOS = ed_discharge,
    trauma_team_activated = activated,
    risk_group = risk
  )

  prep_data <- result$overall

  # Expect only records with trauma level I-IV and valid transport
  testthat::expect_equal(nrow(prep_data), 1)

  # Validate numerator counts
  testthat::expect_equal(prep_data$numerator_9a_all, 1) # ed_LOS > 120
  testthat::expect_equal(prep_data$numerator_9b_all, 0) # ed_LOS > 180
  testthat::expect_equal(prep_data$numerator_9c_all, 1) # ed_decision > 60
  testthat::expect_equal(prep_data$numerator_9d_all, 0) # ed_decision > 120
  testthat::expect_equal(prep_data$numerator_9e_all, 1) # ed_discharge > 60
  testthat::expect_equal(prep_data$numerator_9f_all, 1) # ed_discharge > 120

  # Validate denominators match count of eligible filtered records
  testthat::expect_equal(prep_data$denominator_9a_all, 1)
  testthat::expect_equal(prep_data$denominator_9b_all, 1)
})

testthat::test_that("regex transport exclusion works", {
  excluded_transports <- c(
    "Private Vehicle",
    "public vehicle",
    "walk-in",
    "walk in",
    "Not Known",
    "Not Recorded",
    "Not Applicable",
    "Other"
  )

  test_data <- dplyr::tibble(
    id = as.character(seq_along(excluded_transports)),
    trauma_level = rep("I", length(excluded_transports)),
    transport = excluded_transports,
    transfer_out = rep(TRUE, length(excluded_transports)),
    activated = rep(TRUE, length(excluded_transports)),
    ed_LOS = rep(130, length(excluded_transports)),
    ed_decision = rep(70, length(excluded_transports)),
    ed_discharge = rep(125, length(excluded_transports)),
    risk = rep("High", length(excluded_transports))
  )

  result <- traumar::seqic_indicator_9(
    data = test_data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id = id,
    transport_method = transport,
    transfer_out_indicator = transfer_out,
    ed_LOS = ed_LOS,
    ed_decision_LOS = ed_decision,
    ed_decision_discharge_LOS = ed_discharge,
    trauma_team_activated = activated,
    risk_group = risk
  )

  # Expect no records passed filtering due to transport exclusion
  testthat::expect_equal(nrow(result$overall), 1)
})

testthat::test_that("CI computation for seqic_indicator_9() returns expected columns and structure", {
  # Simulated data
  test_data <- tibble::tibble(
    id = as.character(1:6),
    trauma_level = c("I", "II", "III", "IV", "I", "II"),
    transport = c(
      "Ambulance",
      "Ambulance",
      "Ambulance",
      "Ambulance",
      "Ambulance",
      "Ambulance"
    ),
    activated = c(TRUE, FALSE, TRUE, FALSE, TRUE, FALSE),
    ed_LOS = c(130, 190, 110, 65, 150, 220),
    ed_decision = c(65, 125, 70, 30, 160, 140),
    ed_discharge = c(180, 200, 130, 90, 210, 250),
    transfer_out = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    risk = c("High", "Moderate", "Low", "High", "Low", "Moderate")
  )

  # Run function with CI enabled
  result <- traumar::seqic_indicator_9(
    data = test_data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id = id,
    transport_method = transport,
    transfer_out_indicator = transfer_out,
    ed_LOS = ed_LOS,
    ed_decision_LOS = ed_decision,
    ed_decision_discharge_LOS = ed_discharge,
    trauma_team_activated = activated,
    risk_group = risk,
    calculate_ci = "wilson"
  )

  # Confirm all expected CI columns exist in overall output
  expected_ci_cols <- c(
    "lower_ci_9a_all",
    "upper_ci_9a_all",
    "lower_ci_9b_all",
    "upper_ci_9b_all",
    "lower_ci_9c_all",
    "upper_ci_9c_all",
    "lower_ci_9d_all",
    "upper_ci_9d_all",
    "lower_ci_9e_all",
    "upper_ci_9e_all",
    "lower_ci_9f_all",
    "upper_ci_9f_all"
  )

  testthat::expect_true(all(expected_ci_cols %in% colnames(result$overall)))

  # Confirm all CI columns are numeric
  for (col in expected_ci_cols) {
    testthat::expect_type(result$overall[[col]], "double")
  }

  # Confirm no NA CIs when denominators are non-zero
  numerators <- result$overall |>
    dplyr::select(dplyr::starts_with("numerator_")) |>
    purrr::map_lgl(~ all(.x >= 0))

  denominators <- result$overall |>
    dplyr::select(dplyr::starts_with("denominator_")) |>
    purrr::map_lgl(~ all(.x > 0))

  testthat::expect_true(all(numerators))
  testthat::expect_true(all(denominators))
})

#' @title Validate Error Type
#'
#' @description
#' This function displays an error, warning, or message based on the specified
#' type.
#'
#' @inheritParams validate_numeric
#' @param message The message to be displayed.
#' @inheritParams validate_data_pull
#'
#' @return NULL. The function is used for its side effects.
#'
#' @author
#' Nicolas Foss, Ed.D., MS
#'
#' @keywords internal
#' @noRd
#'
validate_error_type <- function(
  input,
  message,
  type = c("error", "warning", "message"),
  calls = NULL
) {
  # Validate the type argument
  type <- match.arg(type, choices = c("error", "warning", "message"))

  # Create the dynamic message using glue
  dynamic_message <- glue::glue("{cli::col_cyan(input)}: {message}")

  # Define number of callers to go back
  calls <- ifelse(is.null(calls), 2, calls)

  # Depending on the type, use the appropriate cli function
  if (type == "error") {
    cli::cli_abort(
      message = dynamic_message,
      call = rlang::caller_call(n = calls)
    )
  } else if (type == "warning") {
    cli::cli_warn(
      message = dynamic_message,
      call = rlang::caller_call(n = calls)
    )
  } else if (type == "message") {
    cli::cli_inform(
      message = dynamic_message,
      call = rlang::caller_call(n = calls)
    )
  }
}

#' @title Get Season Based on a Date
#'
#' @description
#'
#' This function determines the season (Winter, Spring, Summer, or Fall) based
#' on an input date.
#'
#' The seasons are assigned based on geographic regions similar to how seasons
#' occur in the United States.
#'
#' The seasons are determined using the month of the year and
#' the traditional meteorological definition of seasons (Winter: December,
#' January, February; Spring: March, April, May; Summer: June, July, August;
#' Fall: September, October, November).
#'
#' @param input_date A Date or POSIXct object representing the date to determine
#'   the season for. The input must be of class `Date` or `POSIXct`.
#'
#' @returns A factor indicating the season corresponding to the input date. The
#'   factor levels are:
#'   \itemize{
#'   \item "Winter" for December, January, and February.
#'   \item "Spring" for March, April, and May.
#'   \item "Summer" for June, July, and August.
#'   \item "Fall" for September, October, and November.
#'   \item "Undetermined" if the input is not a valid Date or POSIXct object or
#'   if the month is missing.
#'}
#'
#' @export
#'
#' @examples
#' # Example usage of the season function
#' season(as.Date("2025-01-15"))
#' season(as.POSIXct("2025-07-01 12:00:00"))
#'
#' @author Nicolas Foss, Ed.D., MS
#'
season <- function(input_date) {
  # Check if the value supplied is in fact Date or POSIXct ----
  validate_class(
    input = input_date,
    class_type = c("date", "date-time"),
    logic = "or",
    type = "error",
    null_ok = TRUE
  )

  # Create the month boundaries of the season based on ----
  # https://www.weather.gov/dvn/Climate_Astronomical_Seasons
  winter_months <- c(12, 1, 2)
  spring_months <- c(3, 4, 5)
  summer_months <- c(6, 7, 8)
  fall_months <- c(9, 10, 11)

  # Format the input_date as a month number ----
  month_num <- suppressWarnings(
    # in case input_date is NA or NULL for any given row, supress warning
    # record will be classified as "Undetermined"
    as.numeric(format(input_date, "%m"))
  )

  # Assign the season based on the month number ----
  factor_result <- ifelse(
    month_num %in% winter_months,
    "Winter",
    ifelse(
      month_num %in% spring_months,
      "Spring",
      ifelse(
        month_num %in% summer_months,
        "Summer",
        ifelse(
          month_num %in% fall_months,
          "Fall",
          ifelse(
            is.na(month_num),
            "Undetermined",
            "Undetermined"
          )
        )
      )
    )
  )

  # Convert result to a factor ----
  factor_result <- factor(factor_result)

  # Return result ----
  return(factor_result)
}

test_that("season returns correct season for valid Date input", {
  expect_equal(season(as.Date("2025-01-15")), factor("Winter"))
  expect_equal(season(as.Date("2025-05-10")), factor("Spring"))
  expect_equal(season(as.Date("2025-07-01")), factor("Summer"))
  expect_equal(season(as.Date("2025-10-15")), factor("Fall"))
})

test_that("season returns correct season for valid POSIXct input", {
  expect_equal(season(as.POSIXct("2025-01-15 12:00:00")), factor("Winter"))
  expect_equal(season(as.POSIXct("2025-04-25 08:00:00")), factor("Spring"))
  expect_equal(season(as.POSIXct("2025-08-01 18:00:00")), factor("Summer"))
  expect_equal(season(as.POSIXct("2025-09-30 00:00:00")), factor("Fall"))
})

test_that("season returns errors for non-dates and Undetermined for NA/NULL input", {
  expect_error(
    season("2025-01-15"),
    regexp = "input_date.*must be of class.*date, date-time"
  )
  expect_error(
    season(20250115),
    regexp = "input_date.*must be of class.*date, date-time"
  )
  expect_equal(
    season(NULL),
    expected = factor("Undetermined")
  )
})

test_that("season handles NA input correctly", {
  expect_error(season(NA))
})

test_that("season handles edge cases for months", {
  expect_equal(season(as.Date("2025-12-01")), factor("Winter"))
  expect_equal(season(as.POSIXct("2025-03-01 00:00:00")), factor("Spring"))
})

#' @title Check if Elements Are Not in a Vector
#'
#' @description This function returns a logical vector indicating whether each
#'   element of `x` is not in `y`.
#'
#' @param x A vector of values to be checked.
#'
#' @param y A vector of values to check against.
#'
#' @returns A logical vector of the same length as `x`, where `TRUE` indicates
#'   the corresponding element in `x` is not found in `y`, and `FALSE` indicates
#'   it is found in `y`.
#'
#' @examples
#'
#' # Example vectors
#' x <- c("apple", "banana", "cherry")
#' y <- c("banana", "grape")
#'
#' # Check which elements in `x` are not in `y`
#' x %not_in% y
#'
#' # Example with numeric values
#' a <- c(1, 2, 3, 4, 5)
#' b <- c(2, 4, 6)
#'
#' a %not_in% b
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
`%not_in%` <- function(x, y) {
  !(x %in% y)

}

#' @title SEQIC Indicator 6 - Delayed Arrival Following Low GCS
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Computes SEQIC Indicator 6 for trauma system quality monitoring. This
#' indicator measures the proportion of patients presenting with a Glasgow Coma
#' Scale (GCS) score < 9 who arrive at a trauma level I–IV center more than 180
#' minutes after injury. It excludes patients transferred out of the facility
#' and focuses on those transferred into a facility.
#'
#' @inheritParams seqic_indicator_1
#' @param transfer_out_indicator Column name indicating whether the patient was
#'   transferred out of the initial trauma center to definitive care. Logical,
#'   character, or factor type. Values representing "No" (e.g., FALSE, "No")
#'   indicate no transfer out.
#' @param receiving_indicator Column name indicating whether the
#'   patient was transferred into the trauma center. Logical, character, or
#'   factor type. Values representing "Yes" (e.g., TRUE, "Yes") indicate
#'   transfer in.
#' @param low_GCS_indicator Column name for identifying patients with a
#'   Glasgow Coma Scale score less than 9. Logical, character, or factor type.
#' @param time_from_injury_to_arrival Column name representing the time
#'   in minutes from injury occurrence to arrival at the trauma center. Numeric
#'   type.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function:
#' \itemize{
#'   \item Filters to trauma center records from facilities at trauma levels
#'   I–IV.
#'   \item Deduplicates records using `unique_incident_id`.
#'   \item Calculates:
#'     \itemize{
#'       \item {Numerator:} Patients with low GCS (< 9) who arrived more than
#'         180 minutes after injury, were transferred in, and not transferred
#'         out.
#'       \item {Denominator:} All patients with low GCS (< 9) who were
#'         transferred in and not transferred out.
#'     }
#'   \item Optionally calculates Wilson or Clopper-Pearson confidence intervals
#'     for the resulting proportion if `calculate_ci` is specified.
#' }
#'
#' @note
#'
#' Users must ensure input columns are appropriately coded and standardized.
#' Transfer and GCS indicators should use consistent logical or textual
#' representations.
#'
#' @return A tibble summarizing SEQIC Indicator 6 results. Includes numerator,
#'   denominator, calculated proportion, and optionally 95% confidence
#'   intervals.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Create test data for Indicator 6
#' test_data <- tibble::tibble(
#'   id = as.character(1:10),
#'   trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
#'   transfer_out = c("No", "No", "Yes", "No", "No", "No", "No", "No", "No",
#'   "No"),
#'   transfer_in = c("Yes", "Yes", "No", "Yes", "No", "Yes", "Yes", "Yes",
#'   "Yes", "Yes"),
#'   gcs_low = c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
#'   time_to_arrival = c(200, 100, 300, 190, 400, 181, 100, 179, 240, 178)
#' )
#'
#' # Run the indicator function
#' traumar::seqic_indicator_6(
#'   data = test_data,
#'   level = trauma_level,
#'   unique_incident_id = id,
#'   transfer_out_indicator = transfer_out,
#'   receiving_indicator = transfer_in,
#'   low_GCS_indicator = gcs_low,
#'   time_from_injury_to_arrival = time_to_arrival
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_6 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  transfer_out_indicator,
  receiving_indicator,
  low_GCS_indicator,
  time_from_injury_to_arrival,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure that `transfer_out_indicator` can be validated  ----
  transfer_out_indicator_check <- validate_data_pull(
    input = data,
    col = {{ transfer_out_indicator }},
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Validate `transfer_out_indicator` ----
  validate_class(
    input = transfer_out_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Ensure that `receiving_indicator` can be validated
  receiving_indicator_check <- validate_data_pull(
    input = data,
    col = {{ receiving_indicator }},
    type = "error",
    var_name = "receiving_indicator"
  )

  # Validate that `receiving_indicator` is character, factor, or logical. ----
  validate_class(
    input = receiving_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "receiving_indicator"
  )

  # Ensure that `low_GCS_indicator` can be validated ----
  low_GCS_indicator_check <- validate_data_pull(
    input = data,
    col = {{ low_GCS_indicator }},
    type = "error",
    var_name = "low_GCS_indicator"
  )

  # Validate that `low_GCS_indicator` is character, factor, or logical ----
  validate_class(
    input = low_GCS_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "low_GCS_indicator"
  )

  # Ensure that `time_from_injury_to_arrival` can be validated
  time_from_injury_to_arrival_check <- validate_data_pull(
    input = data,
    col = {{ time_from_injury_to_arrival }},
    type = "error",
    var_name = "time_from_injury_to_arrival"
  )

  # Validate that `time_from_injury_to_arrival` is numeric.
  validate_numeric(
    input = time_from_injury_to_arrival_check,
    type = "error",
    var_name = "time_from_injury_to_arrival"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data`.
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Filter for trauma levels I-IV, deduplicate by `unique_incident_id` ----
  # then summarize.
  seqic_6 <- data |>
    dplyr::filter({{ level }} %in% included_levels) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::summarize(
      numerator_6 = sum(
        {{ low_GCS_indicator }} == TRUE & # GCS < 9
          {{ time_from_injury_to_arrival }} > 180 & # Time to arrival > 180 min
          {{ receiving_indicator }} %in% c(TRUE, "Yes") & # Hospital received
          {{ transfer_out_indicator }} %in% c(FALSE, "No"), # No transfer out
        na.rm = TRUE
      ),
      denominator_6 = sum(
        {{ low_GCS_indicator }} == TRUE &
          {{ receiving_indicator }} %in% c(TRUE, "Yes") &
          {{ transfer_out_indicator }} %in% c(FALSE, "No"),
        na.rm = TRUE
      ),
      seqic_6 = dplyr::if_else(
        denominator_6 > 0,
        numerator_6 / denominator_6,
        NA_real_
      ),
      .by = {{ groups }} # Grouping if applicable
    )

  # Optional: Add binomial confidence intervals if requested. ----
  if (!is.null(calculate_ci)) {
    seqic_6 <- seqic_6 |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_6,
          x = numerator_6, # Number of positive outcomes.
          n = denominator_6, # Number of eligible patients.
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |> # Drop intermediate columns.
          dplyr::rename(lower_ci_6 = lower_ci, upper_ci_6 = upper_ci) # Rename CIs for clarity.
      )
  }

  # Add a `Data` label if not grouped ----
  # or arrange output by grouping variables.
  if (is.null(groups)) {
    seqic_6 <- seqic_6 |>
      tibble::add_column(data = "population/sample", .before = "numerator_6")
  } else if (!is.null(groups)) {
    seqic_6 <- seqic_6 |>
      dplyr::arrange(!!!rlang::syms(groups)) # Order rows based on group vars.
  }

  # Return the final summarized dataset. ----
  return(seqic_6)
}

# tests/testthat/test-seqic_indicator_6.R

testthat::test_that("seqic_indicator_6() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  test_data <- tibble::tibble(
    id = as.character(1:10),
    trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
    transfer_out = c(
      "No",
      "No",
      "Yes",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No"
    ),
    transfer_in = c(
      "Yes",
      "Yes",
      "No",
      "Yes",
      "No",
      "Yes",
      "Yes",
      "Yes",
      "Yes",
      "Yes"
    ),
    gcs_low = c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    time_to_arrival = c(200, 100, 300, 190, 400, 181, 100, 179, 240, 178)
  )

  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = wrong_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = FALSE,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = "another thing",
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = sample,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_high,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = arrival_time
    ),
    regexp = "It was not possible to validate"
  )
})

test_that("Data validation for seqic_indicator_6 works correctly", {
  # Create a simple test data frame for validation
  test_data <- tibble::tibble(
    id = as.character(1:10),
    trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
    transfer_out = c(
      "No",
      "No",
      "Yes",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No"
    ),
    transfer_in = c(
      "Yes",
      "Yes",
      "No",
      "Yes",
      "No",
      "Yes",
      "Yes",
      "Yes",
      "Yes",
      "Yes"
    ),
    gcs_low = c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    time_to_arrival = c(200, 100, 300, 190, 400, 181, 100, 179, 240, 178)
  )

  # Test: Valid inputs should pass without errors
  testthat::expect_silent(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    )
  )

  # Test: Invalid `data` (not a data frame or tibble)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = list(),
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "data.*must be of class.*data.frame, tbl, tbl_df"
  )

  # Test: Invalid `level` (not character or factor)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = time_to_arrival, # Using `id` which is character, should be `trauma_level`
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "must be of class.*character.*factor"
  )

  # Test: Invalid `unique_incident_id` (not character, factor, or numeric)
  bad_data <- test_data |> dplyr::mutate(id = logical(10))
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = bad_data,
      level = trauma_level,
      unique_incident_id = id, # Using `trauma_level`, which is factor, should be `id`
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "unique_incident_id.*must be of class.*numeric, integer, character, factor"
  )

  # Test: Invalid `transfer_out_indicator` (not character, factor, or logical)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = time_to_arrival, # Using `gcs_low`, which is logical, should be `transfer_out`
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "transfer_out_indicator.*must be of class.*logical, character, factor"
  )

  # Test: Invalid `receiving_indicator` (not character, factor, or logical)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = time_to_arrival, # Using `gcs_low`, which is logical, should be `transfer_in`
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "receiving_indicator.*must be of class.*logical, character, factor"
  )

  # Test: Invalid `low_GCS_indicator` (not character, factor, or logical)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = time_to_arrival, # Using `time_to_arrival`, which is numeric, should be `gcs_low`
      time_from_injury_to_arrival = time_to_arrival
    ),
    regexp = "low_GCS_indicator.*must be of class.*logical, character, factor"
  )

  # Test: Invalid `time_from_injury_to_arrival` (not numeric)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = gcs_low # Using `gcs_low`, which is logical, should be numeric
    ),
    regexp = "time_from_injury_to_arrival.*must be.*numeric"
  )

  # Test: Invalid `groups` (not a character vector)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival,
      groups = 123 # Invalid: groups must be a character vector
    ),
    regexp = "groups.*must be of class.*character.*factor"
  )

  # Test: Invalid `groups` (columns not present in `data`)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival,
      groups = c("nonexistent_column") # Invalid: `nonexistent_column` does not exist in the data
    ),
    regexp = "groups.*contains invalid column names.*nonexistent_column.*Valid column names are.*id, trauma_level, transfer_out, transfer_in, gcs_low, time_to_arrival"
  )

  # Test: Invalid `calculate_ci` (not NULL, "wilson", or "clopper-pearson")
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival,
      calculate_ci = "invalid_ci" # Invalid: must be NULL, "wilson", or "clopper-pearson"
    ),
    regexp = "calculate_ci.*contains invalid values.*invalid_ci.*Valid values are.*wilson, clopper-pearson"
  )

  # Test: Valid `included_levels` (should pass)
  testthat::expect_silent(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival,
      included_levels = c("I", "II")
    )
  )

  # Test: Invalid `included_levels` (not character, factor, or numeric)
  testthat::expect_error(
    traumar::seqic_indicator_6(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      transfer_out_indicator = transfer_out,
      receiving_indicator = transfer_in,
      low_GCS_indicator = gcs_low,
      time_from_injury_to_arrival = time_to_arrival,
      included_levels = TRUE # Invalid: should be character, factor, or numeric
    ),
    regexp = "included_levels.*must be of class.*numeric, character, factor, integer"
  )
})

testthat::test_that("seqic_indicator_6 calculates correct values without CI", {
  test_data <- tibble::tibble(
    id = as.character(1:10),
    trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
    transfer_out = c(
      "No",
      "No",
      "Yes",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No"
    ),
    transfer_in = c(
      "Yes",
      "Yes",
      "No",
      "Yes",
      "No",
      "Yes",
      "Yes",
      "Yes",
      "Yes",
      "Yes"
    ),
    gcs_low = c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    time_to_arrival = c(200, 100, 300, 190, 400, 181, 100, 179, 240, 178)
  )

  result <- traumar::seqic_indicator_6(
    data = test_data,
    level = trauma_level,
    unique_incident_id = id,
    transfer_out_indicator = transfer_out,
    receiving_indicator = transfer_in,
    low_GCS_indicator = gcs_low,
    time_from_injury_to_arrival = time_to_arrival
  )

  testthat::expect_equal(result$numerator_6, 3)
  testthat::expect_equal(result$denominator_6, 6)
  testthat::expect_equal(result$seqic_6, 3 / 6)
})

testthat::test_that("seqic_indicator_6 computes CI and supports grouping", {
  test_data <- tibble::tibble(
    id = as.character(1:10),
    region = rep(c("East", "West"), each = 5),
    trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
    transfer_out = c(
      "No",
      "No",
      "Yes",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No",
      "No"
    ),
    transfer_in = c(
      "Yes",
      "Yes",
      "No",
      "Yes",
      "No",
      "Yes",
      "Yes",
      "Yes",
      "Yes",
      "Yes"
    ),
    gcs_low = c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    time_to_arrival = c(200, 100, 300, 190, 400, 181, 100, 179, 240, 178)
  )

  result_grouped <- traumar::seqic_indicator_6(
    data = test_data,
    level = trauma_level,
    unique_incident_id = id,
    transfer_out_indicator = transfer_out,
    receiving_indicator = transfer_in,
    low_GCS_indicator = gcs_low,
    time_from_injury_to_arrival = time_to_arrival,
    groups = "region",
    calculate_ci = "wilson"
  )

  # Check presence of CI columns
  testthat::expect_true(all(
    c("lower_ci_6", "upper_ci_6") %in% names(result_grouped)
  ))

  # Check output structure
  testthat::expect_true("region" %in% names(result_grouped))
  testthat::expect_equal(nrow(result_grouped), 2)

  # Validate CI bounds are within [0,1] and lower ≤ upper
  testthat::expect_true(all(dplyr::between(result_grouped$lower_ci_6, 0, 1)))
  testthat::expect_true(all(dplyr::between(result_grouped$upper_ci_6, 0, 1)))
  testthat::expect_true(all(
    result_grouped$lower_ci_6 <= result_grouped$upper_ci_6
  ))
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#' @importFrom glue glue
#' @importFrom infer generate
#' @importFrom lifecycle badge
#' @importFrom lifecycle deprecated
#' @importFrom stats ks.test
#' @importFrom stats median
#' @importFrom stats na.omit
#' @importFrom stats quantile
#' @importFrom stats sd
#' @importFrom stats shapiro.test
#' @importFrom utils head
#' @importFrom utils tail
#' @importFrom nemsqar nemsqa_binomial_confint
## usethis namespace: end
NULL

#' @title Exploratory Data Analysis, Normality Testing, and
#'   Visualization
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' `is_it_normal()` calculates descriptive statistics and conducts univariate
#' normality testing on one or more numeric variables in a dataset using a
#' selected statistical test. Optional plots are included for one variable at a
#' time, only. Results are returned as a named list containing summaries and,
#' optionally, normality tests and/or diagnostic plots.
#'
#' @param df A `data.frame` or `tibble` containing the variables to assess.
#' @param ... One or more unquoted column names from `df` to be analyzed.
#' @param group_vars Optional. A character vector of column names in `df` to
#'   group results by (e.g., `c("year", "hospital_level")`). If `NULL`, no
#'   grouping is applied. Grouped summaries and normality tests are computed
#'   within each unique combination of values across these variables.
#' @param seed A numeric value passed to `set.seed()` to ensure reproducibility.
#'   Default is `10232015`.
#' @param normality_test A character string specifying the statistical test to
#'   use. Must be one of: `"shapiro-wilk" or "shapiro" or "sw"`,
#'   `"kolmogorov-smirnov" or "ks"`, `"anderson-darling" or "ad"`, `"lilliefors"
#'   or "lilli"`, `"cramer-von-mises" or "cvm"`, `"pearson" or "p"`, or
#'   `"shapiro-francia" or "sf"`. If `NULL`, no normality test is performed,
#'   which is the default.
#' @param include_plots Logical. If `TRUE`, plots are generated for a single
#'   variable. Plotting is disabled if multiple variables are passed.
#' @param plot_theme A `ggplot2::theme` function to apply to all plots. Default
#'   is `traumar::theme_cleaner`.
#'
#' @return A named list with the following elements:
#' \describe{
#'   \item{descriptive_statistics}{A `tibble` of summary statistics for each
#'   variable.}
#'   \item{normality_test}{A `tibble` of test statistics and p-values
#'   (if `normality_test == TRUE`).}
#'   \item{plots}{A patchwork object containing four plots (if `include_plots =
#'   TRUE` and one variable supplied).}
#' }
#'
#' @details
#' \itemize{
#' \item If the data do not meet the test requirements for a chosen test of
#' normality, `is_it_normal()` will not run the tests.
#' \item Normality tests may yield differing results. Each test has distinct
#' assumptions and sensitivity. Users should verify assumptions and consult
#' test-specific guidance to ensure appropriate use.
#' \item The function will abort with helpful CLI messages if input types or
#' structures are incorrect.
#' \item If plotting is enabled, and `nrow(df) > 10000`, a warning is issued
#' as plotting may become computationally expensive.
#'}
#'
#' @note Supported normality tests are below. Please check the specifications of
#'   these tests in the corresponding documentation.
#' \itemize{
#' \item Shapiro-Wilk (`stats::shapiro.test()`)
#' \item Kolmogorov-Smirnov (`stats::ks.test()`)
#' \item Anderson-Darling (`nortest::ad.test()`)
#' \item Lilliefors (`nortest::lillie.test()`)
#' \item Cramer-von Mises (`nortest::cvm.test()`)
#' \item Pearson (`norest::pearson.test()`)
#' \item Shapiro-Francia (`nortest::sf.test()`)
#'}
#'
#' Please note that if grouped plotting is enabled, each group will generate its
#' own set of plots. This may flood your IDE or console. Plan your use of this
#' functionality with care to avoid lags or unwanted outputs.
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
is_it_normal <- function(
  df,
  ...,
  group_vars = NULL,
  seed = 10232015,
  normality_test = NULL,
  include_plots = FALSE,
  plot_theme = traumar::theme_cleaner
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate the `df` argument ----
  validate_data_structure(
    input = df,
    structure_type = c("data.frame", "tibble"),
    logic = "or",
    type = "error",
    na_ok = TRUE,
    null_ok = FALSE
  )

  # Validate group_vars ----
  if (!is.null(group_vars)) {
    # Enforce requirement that group_vars is a character vector
    validate_character_factor(input = group_vars, type = "error")

    # Enforce requirement that group_vars are always column names in df
    validate_names(
      input = df,
      check_names = group_vars,
      type = "error",
      var_name = "group_vars",
      null_ok = TRUE
    )
  }

  # validate the `normality_test` ----
  if (!is.null(normality_test)) {
    # Normalize aliases
    normality_test <- tolower(normality_test)
    normality_test <- switch(
      normality_test,
      "shapiro" = "shapiro-wilk",
      "sw" = "shapiro-wilk",
      "sf" = "shapiro-francia",
      "ks" = "kolmogorov-smirnov",
      "ad" = "anderson-darling",
      "lilli" = "lilliefors",
      "cvm" = "cramer-von mises",
      "p" = "pearson",
      normality_test # if not an alias, keep original
    )

    # Use match.arg to check normality_test ----
    attempt <- try(
      match.arg(
        normality_test,
        choices = c(
          "shapiro-wilk",
          "kolmogorov-smirnov",
          "anderson-darling",
          "lilliefors",
          "cramer-von mises",
          "pearson",
          "shapiro-francia"
        )
      ),
      silent = TRUE
    )

    if (inherits(attempt, "try-error")) {
      cli::cli_abort(
        c(
          "If {.var normality_test} is not {cli::col_blue('NULL')}, it must be one of {.val shapiro-wilk}, {.val kolmogorov-smirnov}, {.val anderson-darling}, {.val lilliefors}, {.val cramer-von mises}, {.val pearson}, or {.val shapiro-francia}.",
          "i" = "{.var normality_test} was {.val {normality_test}}."
        )
      )
    }

    normality_test <- attempt
  }

  # Check if plot_theme is a function ----
  if (include_plots && !is.function(plot_theme)) {
    cli::cli_abort(c(
      "Input to argument {.strong {.var plot_theme}} was not of the correct class.",
      "x" = "You supplied an object of class {.cls {class(plot_theme)}} to {.var plot_theme}.",
      "i" = "Please use an object of class {.cls function}, usually a {.fn ggplot2::theme} object."
    ))
  } else {
    chosen_theme <- as.function(plot_theme)
  }

  # Validate the seed argument ----
  validate_numeric(input = seed, type = "error")

  # Set up variables to use in purrr::map() ----
  vars <- rlang::enquos(...)

  # Dynamically generate the variable names ----
  var_names <- purrr::map_chr(vars, ~ rlang::as_label(.))

  # Check if multiple columns were provided ----
  multiple_columns <- if (length(vars) > 1) TRUE else FALSE

  # Callout for the variables passed ----
  cli::cli_h1("Output Summary")
  cli::cli_h3("Descriptive Statistics")
  cli::cli_inform(c(
    "*" = paste0(
      "Exploratory data analysis of the variable(s): ",
      cli::col_green(cli::style_bold(paste(var_names, collapse = ", ")))
    )
  ))

  # Messaging related to normality testing ----
  cli::cli_h3("Normality Tests")

  # Issue a warning if data violates any normality test choice assumptions ----
  if (
    !is.null(normality_test) &&
      normality_test %in%
        c(
          "shapiro-wilk",
          "shapiro-francia",
          "anderson-darling",
          "cramer-von mises",
          "lilliefors"
        )
  ) {
    # Default values for tests ----
    test_info <- list(
      "shapiro-wilk" = list(
        min_n = 3,
        max_n = 5000,
        fn = "stats::shapiro.test"
      ),
      "shapiro-francia" = list(
        min_n = 5,
        max_n = 5000,
        fn = "nortest::sf.test"
      ),
      "anderson-darling" = list(
        min_n = 8,
        max_n = Inf,
        fn = "nortest::ad.test"
      ),
      "cramer-von mises" = list(
        min_n = 8,
        max_n = Inf,
        fn = "nortest::cvm.test"
      ),
      "lilliefors" = list(
        min_n = 5,
        max_n = Inf,
        fn = "nortest::lillie.test"
      )
    )[[normality_test]]

    # Perform the test within each group ----
    group_status <- df |>
      dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) |>
      dplyr::group_modify(
        ~ {
          group_data <- .x
          group_keys <- .y

          var_status <- purrr::map_chr(vars, function(var) {
            var_name <- rlang::as_label(var)
            vec <- dplyr::pull(group_data, !!var)
            n_complete <- sum(!is.na(vec))

            # control structure for no group ----
            if (!is.null(group_vars)) {
              group_label <- paste(
                purrr::map2_chr(
                  names(group_keys),
                  group_keys,
                  ~ glue::glue("{.x} = {.y}")
                ),
                collapse = ", "
              )
            } else {
              group_label <- "full dataset"
            }

            if (n_complete < test_info$min_n) {
              glue::glue(
                "{var_name} ({group_label}): n_complete = {n_complete}, problem: n_complete < {test_info$min_n}"
              )
            } else if (n_complete > test_info$max_n) {
              glue::glue(
                "{var_name} ({group_label}): n_complete = {n_complete}, problem: n_complete > {test_info$max_n}"
              )
            } else {
              NA_character_
            }
          })

          tibble::tibble(var_status = var_status)
        }
      )

    # Explicit check for any actual messages ----
    violations_exist <- any(!is.na(group_status$var_status))

    if (violations_exist) {
      cli::cli_alert_warning(c(
        "One or more variables do not meet the sample size requirements for {.fn {test_info$fn}}.\n",
        "The test will not run for overall data or any groups that fail to meet the requirements.\n",
        "The test will be performed for groups that meet the sample size requirements.\n",
        "Please select another test if desired."
      ))
      cli::cli_alert_info("Problematic variables:")
      purrr::walk(stats::na.omit(group_status$var_status), cli::cli_li)
    } else {
      # When all is well, let the user know.
      cli::cli_alert_info(
        "No issues reported with the univariate tests of normality. Proceeding."
      )
    }
  } else if (is.null(normality_test)) {
    # When no normality test procedure is requested, return a message ----
    cli::cli_alert_info("No normality test requested.")
  } else {
    # Data limits do not apply to other tests in these ways. ----
    cli::cli_alert_info(
      "No issues reported with the univariate tests of normality. Proceeding."
    )
  }

  # Summary on grouping behavior of the function ----
  cli::cli_h3("Strata")
  # Callout for the grouping applied (or no grouping if NULL) ----
  if (is.null(group_vars) || length(group_vars) == 0) {
    cli::cli_inform(
      c(
        "*" = "No grouping applied. Descriptive statistics and normality tests (as applicable) are computed on the entire dataset."
      )
    )
  } else {
    cli::cli_inform(
      c(
        "*" = paste0(
          "Grouping by: ",
          cli::col_blue(cli::style_bold(paste(group_vars, collapse = ", ")))
        )
      )
    )
  }

  ###___________________________________________________________________________
  ### Create some convention functions for plotting ----
  ###___________________________________________________________________________

  # A utility function to check the length of a vector ----
  is_valid_for_plotting <- function(vec, min_n = 2) {
    vec <- vec[!is.na(vec)]
    length(vec) >= min_n && all(is.finite(vec))
  }

  # Utility function for a density plot ----
  make_density_plot <- function(data, vec, var, var_name, group_label = NULL) {
    if (!is_valid_for_plotting(vec)) {
      cli::cli_alert_danger(
        "Skipping density plot for {.var {var_name}}{if (!is.null(group_label)) paste0(' (', group_label, ')')}: insufficient valid data."
      )
      return(NULL)
    }

    ggplot2::ggplot(data, ggplot2::aes(x = !!var)) +
      ggplot2::geom_density(
        fill = "#F27026",
        color = "black",
        alpha = 0.5,
        na.rm = TRUE
      ) +
      ggplot2::geom_segment(
        x = median(vec, na.rm = TRUE),
        xend = median(vec, na.rm = TRUE),
        y = 0,
        yend = Inf,
        linetype = "dashed",
        color = "darkslategray",
        alpha = 0.25,
        linewidth = 1.5
      ) +
      ggplot2::ggtitle(
        if (is.null(group_label)) {
          paste0(
            "Kernel Density Plot of ",
            var_name,
            " with horizontal line at the median"
          )
        } else {
          paste0("Density Plot with Median (", group_label, ")")
        }
      ) +
      ggplot2::labs(
        x = var_name,
        caption = paste0("n = ", sum(!is.na(vec)), " non-missings")
      ) +
      chosen_theme()
  }

  # Utility function for a histogram plot ----
  make_hist_plot <- function(data, vec, var, var_name, group_label = NULL) {
    if (!is_valid_for_plotting(vec)) {
      cli::cli_alert_danger(
        "Skipping histogram for {.var {var_name}}{if (!is.null(group_label)) paste0(' (', group_label, ')')}: insufficient valid data."
      )
      return(NULL)
    }

    # Deftaul bin width ----
    bin_width <- (max(vec, na.rm = TRUE) - min(vec, na.rm = TRUE)) / 15

    # Histogram ----
    ggplot2::ggplot(data, ggplot2::aes(x = !!var)) +
      ggplot2::geom_histogram(
        binwidth = bin_width,
        fill = "#C6D667",
        color = "black",
        na.rm = TRUE
      ) +
      ggplot2::ggtitle(
        if (is.null(group_label)) {
          paste0("Histogram of ", var_name)
        } else {
          paste0("Histogram (", group_label, ")")
        }
      ) +
      ggplot2::labs(x = var_name) +
      chosen_theme()
  }

  # Utility function for a boxplot ----
  make_boxplot <- function(data, vec, var, var_name, group_label = NULL) {
    if (!is_valid_for_plotting(vec)) {
      cli::cli_alert_danger(
        "Skipping boxplot for {.var {var_name}}{if (!is.null(group_label)) paste0(' (', group_label, ')')}: insufficient valid data."
      )
      return(NULL)
    }

    # Boxplot ----
    ggplot2::ggplot(data, ggplot2::aes(x = !!var, y = "")) +
      ggplot2::geom_jitter(
        color = "#03617A",
        alpha = 0.25,
        na.rm = TRUE,
        height = 0.35
      ) +
      ggplot2::geom_boxplot(
        fill = "#B9E1DA",
        color = "black",
        alpha = 0.2,
        na.rm = TRUE,
        orientation = "y"
      ) +
      ggplot2::stat_boxplot(geom = "errorbar", width = 0.5) +
      ggplot2::ggtitle(
        if (is.null(group_label)) {
          paste0("Boxplot with scatterplot of ", var_name)
        } else {
          paste0("Boxplot (", group_label, ")")
        }
      ) +
      ggplot2::labs(x = var_name, y = "") +
      chosen_theme()
  }

  # Utility function for a Q-Q plot ----
  make_qq_plot <- function(data, vec, var, var_name, group_label = NULL) {
    if (!is_valid_for_plotting(vec)) {
      cli::cli_alert_danger(
        "Skipping Q-Q plot for {.var {var_name}}{if (!is.null(group_label)) paste0(' (', group_label, ')')}: insufficient valid data."
      )
      return(NULL)
    }

    ggplot2::ggplot(data, ggplot2::aes(sample = !!var)) +
      ggplot2::geom_qq_line(
        linewidth = 1.25,
        color = "#70C8B8",
        alpha = 0.7,
        na.rm = TRUE
      ) +
      ggplot2::stat_qq(
        color = "#19405B",
        alpha = 0.4,
        size = 2,
        na.rm = TRUE
      ) +
      ggplot2::ggtitle(
        if (is.null(group_label)) {
          paste0("Normal Q-Q Plot of ", var_name)
        } else {
          paste0("Normal Q-Q Plot (", group_label, ")")
        }
      ) +
      chosen_theme()
  }

  ###___________________________________________________________________________
  ### Initialize the output list ----
  ###___________________________________________________________________________

  output_list <- list()

  ###___________________________________________________________________________
  ### Generate descriptive statistics ----
  ###___________________________________________________________________________

  # Iterate over the provided columns ----
  results <- purrr::map(vars, function(var) {
    var_name <- rlang::as_label(var)

    df_summarized <- df |>
      dplyr::select(dplyr::all_of(c(group_vars, var_name))) |>
      dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) |>
      dplyr::summarize(
        variable = var_name,
        mean = round(mean(.data[[var_name]], na.rm = TRUE), 3),
        std_dev = round(sd(.data[[var_name]], na.rm = TRUE), 3),
        min = min(.data[[var_name]], na.rm = TRUE),
        quantile_25 = quantile(.data[[var_name]], 0.25, na.rm = TRUE),
        median = quantile(.data[[var_name]], 0.5, na.rm = TRUE),
        quantile_75 = quantile(.data[[var_name]], 0.75, na.rm = TRUE),
        max = max(.data[[var_name]], na.rm = TRUE),
        non_missings = sum(!is.na(.data[[var_name]])),
        missings = sum(is.na(.data[[var_name]])),
        p_complete = round(mean(!is.na(.data[[var_name]])), 3),
        p_missing = round(mean(is.na(.data[[var_name]])), 3),
        n_obs = dplyr::n(),
        .groups = "drop"
      )

    df_summarized
  })

  # set names ----
  names(results) <- purrr::map_chr(vars, rlang::as_label)

  # Pivot results wider ----
  results <- results |>
    purrr::list_rbind()

  # Assign results to the output_list ----
  output_list$descriptive_statistics <- results

  ###___________________________________________________________________________
  ### Optional: Perform the test of normality on variables ----
  ###___________________________________________________________________________

  if (!is.null(normality_test)) {
    # Only set the random seed if normality test is done ----
    set.seed(seed)

    # Grouping (if any) applied before normality test ----
    if (!is.null(group_vars) && length(group_vars) > 0) {
      # Group data by the specified group_vars ----
      df <- df |>
        dplyr::group_by(dplyr::across(dplyr::all_of(group_vars)))
    }

    # Iterate over the variables of interest to perform the requested normality
    # test within each group (or overall if no grouping)
    test_results <- purrr::map(vars, function(var) {
      var_name <- rlang::as_label(var)

      # Perform normality test within each group ----
      test_result <- df |>
        dplyr::group_modify(function(sub_df, group) {
          vec <- dplyr::pull(sub_df, !!var)

          # get complete observations ----
          n_complete <- sum(!is.na(vec))

          # Try to perform the normality test and handle any errors ----
          test_result <- tryCatch(
            {
              # Shapiro-Wilk ----
              if (normality_test == "shapiro-wilk") {
                test_result <- shapiro.test(vec) |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )

                # Kolmogorov-Smirnov ----
              } else if (normality_test == "kolmogorov-smirnov") {
                test_result <- ks.test(vec, y = "pnorm") |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )

                # Anderson-Darling ----
              } else if (normality_test == "anderson-darling") {
                test_result <- nortest::ad.test(vec) |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )

                # Lilliefors ----
              } else if (normality_test == "lilliefors") {
                test_result <- nortest::lillie.test(vec) |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )

                # Cramer-von Mises ----
              } else if (normality_test == "cramer-von mises") {
                test_result <- nortest::cvm.test(vec) |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )
                # Pearson ----
              } else if (normality_test == "pearson") {
                test_result <- nortest::pearson.test(vec) |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )
                # Shapiro-Francia ----
              } else if (normality_test == "shapiro-francia") {
                test_result <- nortest::sf.test(vec) |>
                  broom::tidy() |>
                  dplyr::mutate(
                    result = dplyr::if_else(
                      p.value < 0.05,
                      "Non-normal distribution",
                      "Normal distribution"
                    )
                  )
              }
            },
            error = function(e) {
              # Return a tibble indicating the test was skipped ----
              tibble::tibble(
                variable = var_name,
                test = "Test Skipped",
                statistic = NA_real_,
                p_value = NA_real_,
                result = "Test Skipped"
              )
            }
          )

          # Return the results in a tibble, including the group ----
          suppressWarnings(
            tibble::tibble(
              variable = var_name,
              test = test_result$method,
              statistic = test_result$statistic,
              p_value = test_result$p.value,
              result = test_result$result
            )
          )
        })

      # Return the test results for the variable ----
      test_result
    })

    # Combine the individual variable results into a single data frame ----
    test_results <- purrr::list_rbind(test_results)

    # Store the final results in the output list ----
    output_list$normality_test <- test_results
  } else {
    output_list$normality_test <- NULL
  }

  ###___________________________________________________________________________
  ### Optional plots  ----
  ### only available if one variable is passed to `variables`
  ###___________________________________________________________________________

  if (include_plots && !multiple_columns) {
    # Extract data programmatically ----
    var <- vars[[1]]
    var_name <- rlang::as_label(var)

    # Header for the plotting section ----
    cli::cli_h3("Visualizations")

    if (nrow(df) > 5000) {
      # Issue a warning
      cli::cli_alert_warning(
        c(
          "Large Data Warning! The number of observations within {.var df} is large, plotting may become computationally expensive."
        )
      )

      # Issue an alternative
      cli::cli_alert_success(
        "Consider downsampling your data using {.fn base::sample}, {.fn dplyr::slice_sample}, or a function with a similar purpose."
      )
    }

    # Grouped plotting can be computationally expensive ----
    # Ensure the user is aware and provide them a way out
    if (!is.null(group_vars)) {
      cli::cli_alert_warning(
        c(
          "Grouped plotting is enabled. Each group will generate its own set of plots. This may flood your IDE or console."
        )
      )
      cli::cli_alert_info(
        "To avoid this, assign the function result to an object and inspect plots selectively from the list output."
      )

      ###_______________________________________________________________________
      ### Plotting functionality ----
      ###_______________________________________________________________________

      # Group the data <-
      grouped_data <- df |>
        dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) |>
        dplyr::group_split()

      # Get labels for outside the loop ----
      plot_list_labels <- grouped_data |>
        purrr::map_chr(
          ~ {
            keys <- dplyr::distinct(
              .x,
              dplyr::across(dplyr::all_of(group_vars))
            )
            paste(
              purrr::map2_chr(names(keys), keys, \(n, v) glue::glue("{n}={v}")),
              collapse = ", "
            )
          }
        )

      # Create the plots with dynamic names ----
      output_list$plots <- grouped_data |>
        purrr::map(function(group_data) {
          group_keys <- dplyr::distinct(
            group_data,
            dplyr::across(dplyr::all_of(group_vars))
          )
          group_label <- paste(
            purrr::map2_chr(
              names(group_keys),
              group_keys,
              ~ glue::glue("{.x} = {.y}")
            ),
            collapse = ", "
          )

          # Get the data as a vector ----
          vec <- dplyr::pull(group_data, !!var)

          # QQ Plot ----
          qqplot <- make_qq_plot(group_data, vec, var, var_name, group_label)

          # Histogram ----
          hist_plot <- make_hist_plot(
            group_data,
            vec,
            var,
            var_name,
            group_label
          )

          # Kernel Density Plot with line at the median ----
          density_plot <- make_density_plot(
            group_data,
            vec,
            var,
            var_name,
            group_label
          )

          # Boxplot with scatterplot ----
          boxplot_plot <- make_boxplot(
            group_data,
            vec,
            var,
            var_name,
            group_label
          )

          # Set up list of plots, remove NULL objects ----
          valid_plots <- purrr::compact(
            list(
              qqplot,
              hist_plot,
              density_plot,
              boxplot_plot
            )
          )

          # Set up the patchwork ----
          patchwork::wrap_plots(valid_plots) +
            patchwork::plot_annotation(
              title = paste0(
                "Normality Test of the variable '",
                var_name,
                "'"
              ),
              theme = chosen_theme()
            )
        }) |>
        purrr::set_names(plot_list_labels) |>
        purrr::compact()

      # Plot a single variable without grouped output ----
    } else {
      # Get the data of interest as a vector ----
      vec <- df |> dplyr::pull(!!var)

      # QQ Plot ----
      qqplot <- make_qq_plot(df, vec, var, var_name)

      # Histogram ----
      hist_plot <- make_hist_plot(df, vec, var, var_name)

      # Kernel Density Plot ----
      density_plot <- make_density_plot(df, vec, var, var_name)

      # Boxplot with scatterplot ----
      boxplot_plot <- make_boxplot(df, vec, var, var_name)

      # Combine, remove NULL objects ----
      plot_list <- purrr::compact(list(
        qqplot,
        hist_plot,
        density_plot,
        boxplot_plot
      ))

      # Patchwork ----
      combined_plots <- patchwork::wrap_plots(plot_list) +
        patchwork::plot_annotation(
          title = paste0(
            "Normality Test of the variable '",
            var_name,
            "'"
          ),
          theme = chosen_theme()
        )

      # Add to the list ----
      output_list$plots <- combined_plots
    }
  } else if (include_plots && multiple_columns) {
    # Header for the plotting section ----
    cli::cli_h3("Visualizations")

    # If multiple_columns, then do not include plots ----
    include_plots <- FALSE

    cli::cli_alert_info(
      "More than one column was passed to {.fn is_it_normal}, and plotting is only available for one column at a time. {.var include_plots} is now set to {.val FALSE}."
    )
  }

  # Line to separate output for cleaner presentation ----
  cli::cli_rule()

  # Return outputs as a list ----
  return(output_list)
}

###_____________________________________________________________________________
### Data Validation Testing
###_____________________________________________________________________________

testthat::test_that("is_it_normal() input validation works correctly", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  # df must be data.frame or tibble
  testthat::expect_error(
    traumar::is_it_normal(df = "not_a_df", var1),
    class = "rlang_error"
  )

  # group_vars must be character vector or NULL
  testthat::expect_error(
    traumar::is_it_normal(df = test_data, var1, group_vars = 123),
    class = "rlang_error"
  )

  # group_vars must exist in df
  testthat::expect_error(
    traumar::is_it_normal(df = test_data, var1, group_vars = "nonexistent_col"),
    class = "rlang_error"
  )

  # Invalid normality_test should error
  testthat::expect_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      normality_test = "invalid_test"
    ),
    class = "rlang_error"
  )

  # plot_theme must be a function if include_plots is TRUE
  testthat::expect_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      include_plots = TRUE,
      plot_theme = "not_a_function"
    ),
    class = "rlang_error"
  )

  # seed must be numeric
  testthat::expect_error(
    traumar::is_it_normal(df = test_data, var1, seed = "not_numeric"),
    class = "rlang_error"
  )

  # Should succeed with valid input
  testthat::expect_no_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      group_vars = c("group1", "group2"),
      normality_test = "sw"
    )
  )
})

###_____________________________________________________________________________
### Calculation testing
###_____________________________________________________________________________

# Test 1: Ensure an error is raised if plot_theme is not a function when include_plots = TRUE
testthat::test_that("plot_theme must be a function if include_plots is TRUE", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  testthat::expect_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      include_plots = TRUE,
      plot_theme = "not_a_function",
      normality_test = NULL
    ),
    "not of the correct class",
  )
})

# Test 2: Ensure no error if plot_theme is a valid function (e.g., theme from ggplot2)
testthat::test_that("plot_theme can be a valid function if include_plots is TRUE", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  testthat::expect_no_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      include_plots = TRUE,
      plot_theme = ggplot2::theme_bw,
      normality_test = NULL
    )
  )
})

# Test 3: Ensure seed is numeric
testthat::test_that("seed must be numeric", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  testthat::expect_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      seed = "not_a_number",
      normality_test = NULL
    ),
    regexp = "seed.*must be.*numeric"
  )

  testthat::expect_no_error(
    traumar::is_it_normal(
      df = test_data,
      var1,
      seed = 123,
      normality_test = NULL
    )
  )
})

# Test 4: Ensure correct handling of grouping behavior
testthat::test_that("grouping behavior is handled correctly", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  # Test with no grouping
  testthat::expect_message(
    traumar::is_it_normal(
      df = test_data,
      var1,
      normality_test = NULL
    ),
    "No grouping applied"
  )

  # Test with grouping
  testthat::expect_message(
    traumar::is_it_normal(
      df = test_data,
      var1,
      group_vars = "group1",
      normality_test = NULL
    ),
    "Grouping by.*group1"
  )
})

# Test 5: Ensure that proper warning is shown for normality test size violations
testthat::test_that("warning is shown for normality test size violations", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(5100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = test_data,
      var1,
      include_plots = TRUE,
      plot_theme = ggplot2::theme_bw,
      normality_test = "shapiro-wilk"
    ),
    "One or more variables do not meet the sample size requirements"
  )
})

# Test 6: Ensure that info is displayed if no normality test is requested
testthat::test_that("info is displayed if no normality test is requested", {
  # Create minimal test data
  test_data <- data.frame(
    var1 = rnorm(100),
    group1 = rep(c("A", "B"), each = 50),
    group2 = rep(c("X", "Y"), 50)
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = test_data,
      var1,
      normality_test = NULL,
      include_plots = FALSE
    ),
    "No normality test requested"
  )
})

testthat::test_that("is_it_normal() calculates descriptive stats correctly", {
  df <- tibble::tibble(
    group = rep(c("A", "B"), each = 5),
    x = c(1, 2, 3, 4, 5, 6, NA, 8, 9, 10)
  )

  result <- traumar::is_it_normal(
    df,
    x,
    group_vars = "group"
  )

  stats <- result$descriptive_statistics

  testthat::expect_s3_class(stats, "tbl_df")
  testthat::expect_true(all(c("mean", "std_dev", "n_obs") %in% colnames(stats)))
  testthat::expect_equal(unique(stats$variable), "x")
  testthat::expect_equal(nrow(stats), 2) # 2 groups
})

testthat::test_that("is_it_normal() runs all supported normality tests without error", {
  df <- tibble::tibble(
    g = rep(letters[1:2], each = 50),
    y = c(rnorm(50), rnorm(50))
  )

  test_options <- c(
    "shapiro-wilk",
    "shapiro",
    "sw",
    "kolmogorov-smirnov",
    "ks",
    "anderson-darling",
    "ad",
    "lilliefors",
    "lilli",
    "cramer-von mises",
    "cvm",
    "pearson",
    "p",
    "shapiro-francia",
    "sf"
  )

  for (test_name in test_options) {
    result <- traumar::is_it_normal(
      df,
      y,
      group_vars = "g",
      normality_test = test_name
    )

    nt <- result$normality_test

    testthat::expect_s3_class(nt, "tbl_df")
    testthat::expect_true(
      all(
        c("variable", "test", "statistic", "p_value", "result") %in%
          colnames(nt)
      )
    )
    testthat::expect_equal(
      unique(nt$variable),
      "y"
    )
    testthat::expect_equal(nrow(nt), 2)
  }
})


testthat::test_that("is_it_normal() gracefully skips test on NA-only data", {
  df <- tibble::tibble(
    z = rep(NA_real_, 10)
  )

  result <- suppressWarnings(
    traumar::is_it_normal(
      df,
      z,
      normality_test = "shapiro-wilk"
    )
  )

  nt <- result$normality_test

  testthat::expect_s3_class(nt, "tbl_df")
  testthat::expect_true("Test Skipped" %in% nt$result)
  testthat::expect_true(is.na(nt$statistic))
})

testthat::test_that("is_it_normal() returns NULL for normality_test when not specified", {
  df <- tibble::tibble(a = rnorm(10))

  result <- traumar::is_it_normal(df, a)

  testthat::expect_null(result$normality_test)
})

testthat::test_that("Grouped plotting works with user confirmation", {
  df <- dplyr::tibble(
    group = rep(c("A", "B"), each = 100),
    value = c(
      stats::rnorm(100, mean = 5),
      stats::rnorm(100, mean = 10)
    )
  )

  result <- traumar::is_it_normal(
    df = df,
    value,
    group_vars = "group",
    include_plots = TRUE
  )

  testthat::expect_true("plots" %in% names(result))
  testthat::expect_s3_class(result$plots[[1]], "patchwork")
  testthat::expect_length(result$plots, 2)

  testthat::expect_message(
    traumar::is_it_normal(
      df = df,
      value,
      group_vars = "group",
      include_plots = TRUE
    ),
    "Grouped plotting is enabled"
  )
})

testthat::test_that("Grouped plotting is skipped when user passes more than one variable", {
  df <- dplyr::tibble(
    group = rep(c("A", "B"), each = 100),
    value1 = stats::rnorm(200),
    value2 = stats::rnorm(200, mean = 10, sd = 5)
  )

  result <- traumar::is_it_normal(
    df = df,
    value1,
    value2,
    group_vars = "group",
    include_plots = TRUE
  )

  testthat::expect_null(result$plots)
})

testthat::test_that("Large data warning is issued for plotting", {
  df <- dplyr::tibble(
    group = rep("A", 10001),
    value = stats::rnorm(10001)
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = df,
      value,
      group_vars = "group",
      include_plots = TRUE
    ),
    "Large Data Warning"
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = df,
      value,
      group_vars = "group",
      include_plots = TRUE
    ),
    "Consider downsampling"
  )
})

testthat::test_that("Grouped plotting: plots generated, warnings issued", {
  # Synthetic grouped dataset
  set.seed(123)
  df <- dplyr::tibble(
    group = rep(c("A", "B"), each = 60),
    x = c(rnorm(60, mean = 10), rnorm(60, mean = 20))
  )

  # Capture CLI output
  result <- traumar::is_it_normal(
    df = df,
    x,
    group_vars = "group",
    include_plots = TRUE
  )

  # Basic structure checks
  testthat::expect_true("plots" %in% names(result))
  testthat::expect_identical(class(result$plots), "list")
  testthat::expect_length(result$plots, 2) # Two groups: A and B

  # Check that each plot object is a patchwork
  purrr::walk(result$plots, ~ testthat::expect_s3_class(.x, "patchwork"))

  # CLI warnings/messages for grouped plotting
  testthat::expect_message(
    traumar::is_it_normal(
      df = df,
      x,
      group_vars = "group",
      include_plots = TRUE
    ),
    "Grouped plotting is enabled"
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = df,
      x,
      group_vars = "group",
      include_plots = TRUE
    ),
    "assign the function result to an object"
  )
})

testthat::test_that("Large data warning issued for nrow > 10000", {
  df_large <- dplyr::tibble(
    group = rep(c("A", "B"), each = 6000),
    x = rnorm(12000)
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = df_large,
      x,
      group_vars = "group",
      include_plots = TRUE
    ),
    "Large Data Warning"
  )

  testthat::expect_message(
    traumar::is_it_normal(
      df = df_large,
      x,
      group_vars = "group",
      include_plots = TRUE
    ),
    "Consider downsampling your data"
  )
})

#' @title Customizable Minimalistic ggplot2 Theme
#'
#' @description
#'
#' A flexible and customizable theme function for creating polished and
#' minimalistic plots using `ggplot2`. The `theme_cleaner` function provides
#' various options to control the appearance of plot elements, including font
#' styles, sizes, colors, axis lines, grid lines, legend, title, subtitle,
#' captions, and facet appearance. The theme is highly customizable, allowing
#' for the creation of visually appealing and clean plots.
#'
#' @param base_size Numeric. Default font size for plot elements. Defaults to
#'   `12`.
#' @param base_family Character. Font family used for text in the plot. Defaults
#'   to `"Work Sans"`.
#' @param base_color Character. Hex color code for primary plot elements (e.g.,
#'   axis text, legend text). Defaults to `"#70C8B8"`.
#' @param base_color_title Character. Hex color code for plot title and legend
#'   title text. Defaults to `"#03617A"`.
#' @param title_text_size Numeric. Font size for plot title text. Defaults to
#'   `base_size * 1.1`.
#' @param subtitle_text_size Numeric. Font size for plot subtitle text. Defaults
#'   to `base_size * 1.05`.
#' @param caption_color Character. Hex color code for plot caption text.
#'   Defaults to `"#19405B"`.
#' @param legend_position Character. Legend position on the plot. Accepts "top",
#'   "bottom", "left", or "right". Defaults to `"top"`.
#' @param vjust_title Numeric. Vertical justification of the plot title.
#'   Defaults to `0`.
#' @param vjust_subtitle Numeric. Vertical justification of the plot subtitle.
#'   Defaults to `0`.
#' @param hjust_title Numeric. Horizontal justification of the plot title.
#'   Defaults to `0`.
#' @param hjust_subtitle Numeric. Horizontal justification of the plot subtitle.
#'   Defaults to `0`.
#' @param axis_lines Logical. If `TRUE`, axis lines are drawn in `base_color`;
#'   otherwise, they are hidden. Defaults to `FALSE`.
#' @param facets Logical. If `TRUE`, additional formatting for facet plots is
#'   applied. Defaults to `FALSE`.
#' @param facet_text_size Numeric. If `facets = TRUE`, size formatting for facet
#'   text (`strip.text`) is applied. Defaults to `base_size`.
#' @param draw_panel_border Logical. If `TRUE`, a border is drawn around panels
#'   in facet plots. Defaults to `FALSE`.
#' @param ... Additional arguments passed to `ggplot2::theme` for further
#'   customization.
#'
#' @returns A `ggplot2` theme object that can be applied to plots.
#'
#' @details The function customizes common plot elements like axis text, titles,
#' subtitles, captions, legend, and facet labels. It is designed to work with
#' `ggplot2` plots, providing a clean and professional look with minimal
#' styling. You can adjust various aesthetic features such as font size, color,
#' and legend position for maximum control over the appearance.
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
#' @examples
#' # Create a ggplot2 plot with the theme_cleaner theme
#' library(ggplot2)
#' ggplot(mtcars, aes(x = mpg, y = wt)) +
#'   geom_point() +
#'   theme_cleaner(
#'     base_size = 14,
#'     title_text_size = 16,
#'     legend_position = "bottom"
#'   )
#'
#' # Customize facet plots with theme_cleaner
#' ggplot(mtcars, aes(x = mpg, y = wt)) +
#'   geom_point() +
#'   facet_wrap(~cyl) +
#'   theme_cleaner(facets = TRUE,
#'   facet_text_size = 12,
#'   draw_panel_border = TRUE
#'   )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
theme_cleaner <-
  function(
    base_size = 12,
    base_family = "sans",
    base_color = "#70C8B8",
    base_color_title = "#03617A",
    title_text_size = ceiling(base_size * 1.1),
    subtitle_text_size = ceiling(base_size * 1.05),
    caption_color = "#19405B",
    legend_position = "top",
    vjust_title = 0,
    vjust_subtitle = 0,
    hjust_title = 0,
    hjust_subtitle = 0,
    axis_lines = FALSE,
    facets = FALSE,
    facet_text_size = base_size,
    draw_panel_border = FALSE,
    ...
  ) {
    # If facets are not enabled, apply the following theme settings
    if (!facets) {
      ggplot2::theme(
        # Remove axis ticks
        axis.ticks = ggplot2::element_blank(),
        # Remove background for the legend
        legend.background = ggplot2::element_blank(),
        # Remove background for the legend keys
        legend.key = ggplot2::element_blank(),
        # Remove background for the panel
        panel.background = ggplot2::element_blank(),
        # Remove border for the panel
        panel.border = ggplot2::element_blank(),
        # Remove background for the strip (facet labels)
        strip.background = ggplot2::element_blank(),
        # Remove background for the plot
        plot.background = ggplot2::element_blank(),
        # Indicate that this theme is complete and should override other themes
        complete = TRUE,
        # Set text properties for the plot
        text = ggplot2::element_text(
          family = base_family, # Font family
          face = "bold", # Font face (bold)
          color = base_color # Font color
        ),

        # Adjust font color for various plot elements

        # Set font color and size for axis text
        axis.text = ggplot2::element_text(
          color = base_color, # Color for axis text
          size = base_size # Font size for axis text
        ),

        # Set font color and size for axis titles
        axis.title = ggplot2::element_text(
          color = base_color_title, # Color for axis titles
          size = base_size # Font size for axis titles
        ),

        # Set font color and size for legend titles
        legend.title = ggplot2::element_text(
          color = base_color_title, # Color for legend titles
          size = base_size # Font size for legend titles
        ),

        # Set font color and size for legend text
        legend.text = ggplot2::element_text(
          color = base_color, # Color for legend text
          size = base_size # Font size for legend text
        ),

        # Set font color, size, and alignment for plot title
        plot.title = ggplot2::element_text(
          color = base_color_title, # Color for plot title
          size = title_text_size, # Font size for plot title
          face = "bold", # Font face (bold) for plot title
          hjust = hjust_title, # Horizontal justification for plot title
          vjust = vjust_title # Vertical justification for plot title
        ),

        # Set font color, size, and alignment for plot subtitle
        plot.subtitle = ggplot2::element_text(
          color = base_color, # Color for plot subtitle
          size = subtitle_text_size, # Font size for plot subtitle
          hjust = hjust_subtitle, # Horizontal justification for plot subtitle
          vjust = vjust_subtitle # Vertical justification for plot subtitle
        ),

        # Set font color, size, and alignment for plot caption
        plot.caption = ggplot2::element_text(
          size = base_size, # Font size for plot caption
          color = caption_color, # Color for plot caption
          hjust = 0 # Horizontal justification for plot caption
        ),
        # Set the position of the legend
        legend.position = legend_position,

        # Center the legend within its position
        legend.justification = "center",

        # Remove all grid lines from the panel
        panel.grid = ggplot2::element_blank(),

        # Remove major grid lines from the panel
        panel.grid.major = ggplot2::element_blank(),

        # Remove minor grid lines from the panel
        panel.grid.minor = ggplot2::element_blank(),

        # Set the color of the axis lines
        axis.line = ggplot2::element_line(
          color = dplyr::if_else(
            axis_lines == TRUE, # Check if axis lines should be drawn
            base_color, # Use base_color if axis lines are enabled
            "transparent" # Otherwise, make axis lines transparent
          )
        ),

        # Additional arguments passed to ggplot2::theme for customization
        ...
      )

      # If facets are enabled, apply the following theme settings
    } else if (facets) {
      ggplot2::theme(
        # Remove axis ticks
        axis.ticks = ggplot2::element_blank(),
        # Remove background for the legend
        legend.background = ggplot2::element_blank(),
        # Remove background for the legend keys
        legend.key = ggplot2::element_blank(),
        # Remove background for the panel
        panel.background = ggplot2::element_blank(),
        # Set border for the panel based on draw_panel_border parameter
        panel.border = ggplot2::element_rect(
          color = dplyr::if_else(
            draw_panel_border == FALSE, # Check if panel border should be drawn
            "transparent", # Make border transparent if not drawn
            base_color # Use base_color if border is drawn
          ),
          fill = NA # No fill for the panel border
        ),
        # Remove background for the plot
        plot.background = ggplot2::element_blank(),
        # Indicate that this theme is complete and should override other themes
        complete = TRUE,
        # Set text properties for the plot
        text = ggplot2::element_text(
          family = base_family, # Font family
          face = "bold", # Font face (bold)
          color = base_color # Font color
        ),
        # Adjust font color for various plot elements

        # Set font color and size for axis text
        axis.text = ggplot2::element_text(
          color = base_color, # Color for axis text
          size = base_size # Font size for axis text
        ),

        # Set font color and size for axis titles
        axis.title = ggplot2::element_text(
          color = base_color_title, # Color for axis titles
          size = base_size # Font size for axis titles
        ),

        # Set font color and size for legend titles
        legend.title = ggplot2::element_text(
          color = base_color_title, # Color for legend titles
          size = base_size # Font size for legend titles
        ),

        # Set font color and size for legend text
        legend.text = ggplot2::element_text(
          color = base_color, # Color for legend text
          size = base_size # Font size for legend text
        ),

        # Set font color, size, and alignment for plot title
        plot.title = ggplot2::element_text(
          color = base_color_title, # Color for plot title
          size = title_text_size, # Font size for plot title
          face = "bold", # Font face (bold) for plot title
          hjust = hjust_title, # Horizontal justification for plot title
          vjust = vjust_title # Vertical justification for plot title
        ),

        # Set font color, size, and alignment for plot subtitle
        plot.subtitle = ggplot2::element_text(
          color = base_color, # Color for plot subtitle
          size = subtitle_text_size, # Font size for plot subtitle
          hjust = hjust_subtitle, # Horizontal justification for plot subtitle
          vjust = vjust_subtitle # Vertical justification for plot subtitle
        ),

        # Set font color, size, and alignment for plot caption
        plot.caption = ggplot2::element_text(
          size = base_size, # Font size for plot caption
          color = caption_color, # Color for plot caption
          hjust = 0 # Horizontal justification for plot caption
        ),
        # Set the position of the legend
        legend.position = legend_position,

        # Center the legend within its position
        legend.justification = "center",

        # Remove all grid lines from the panel
        panel.grid = ggplot2::element_blank(),

        # Remove major grid lines from the panel
        panel.grid.major = ggplot2::element_blank(),

        # Remove minor grid lines from the panel
        panel.grid.minor = ggplot2::element_blank(),

        # Set the color of the axis lines
        axis.line = ggplot2::element_line(
          color = dplyr::if_else(
            axis_lines == TRUE, # Check if axis lines should be drawn
            base_color, # Use base_color if axis lines are enabled
            "transparent" # Otherwise, make axis lines transparent
          )
        ),

        # Set the background color for facet strips
        strip.background = ggplot2::element_rect(
          fill = "#2A6357", # Background color for facet strips
          color = "#B9E1DA" # Border color for facet strips
        ),

        # Set the text properties for facet strips
        strip.text = ggplot2::element_text(
          color = "ghostwhite", # Text color for facet strips
          size = facet_text_size # Font size for facet strips
        ),

        # Additional arguments passed to ggplot2::theme for customization
        ...
      )
    }
  }

test_that("theme_cleaner applies the correct default theme settings", {
  # Create a basic ggplot
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner()

  # Check default values in the theme
  expect_s3_class(p$theme, "theme")
  expect_equal(p$theme$axis.text$colour, "#70C8B8")  # Default base colour
  expect_equal(p$theme$plot.title$size, 14)  # Default title text size
  expect_equal(p$theme$legend.position, "top")  # Default legend position
})

test_that("theme_cleaner customizes font family and sizes", {
  # Create a ggplot with custom base font size and family
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(base_size = 14, base_family = "Arial")

  # Check if custom base font size and family are applied
  expect_equal(p$theme$text$family, "Arial")
  expect_equal(p$theme$axis.text$size, 14)
})

test_that("theme_cleaner adjusts title and subtitle sizes", {
  # Create a ggplot with custom title and subtitle sizes
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(title_text_size = 16, subtitle_text_size = 14)

  # Check if custom title and subtitle sizes are applied
  expect_equal(p$theme$plot.title$size, 16)
  expect_equal(p$theme$plot.subtitle$size, 14)
})

test_that("theme_cleaner works with facet plots", {
  # Create a ggplot with facet wrapping
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~cyl) +
    theme_cleaner(facets = TRUE, facet_text_size = 12, draw_panel_border = TRUE)

  # Check if facet text size and panel border are applied
  expect_equal(p$theme$strip.text$size, 12)
  expect_equal(p$theme$panel.border$colour, "#70C8B8")  # Default base colour for panel border
})

test_that("theme_cleaner handles axis line visibility", {
  # Create a ggplot with axis lines visible
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(axis_lines = TRUE)

  # Check if axis lines are visible
  expect_equal(p$theme$axis.line$colour, "#70C8B8")  # Default base colour
})

test_that("theme_cleaner handles empty facet panels", {
  # Create a ggplot with facet_wrap but no data for faceting
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~gear) +
    theme_cleaner(facets = TRUE)

  # Check that no errors occur, and the facets are applied
  expect_s3_class(p$theme, "theme")
})

test_that("theme_cleaner applies custom caption colour", {
  # Create a ggplot with custom caption colour
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(caption_color = "#FF5733")

  # Check if custom caption colour is applied
  expect_equal(p$theme$plot.caption$colour, "#FF5733")
})

test_that("theme_cleaner handles additional ggplot theme arguments", {
  # Create a ggplot with additional theme arguments
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(base_size = 14, plot.margin = c(1, 1, 1, 1))

  # Check if the custom margin is applied
  expect_equal(p$theme$plot.margin, c(1, 1, 1, 1))
})

test_that("theme_cleaner works with different legend positions", {
  # Create ggplots with different legend positions
  p1 <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(legend_position = "bottom")
  p2 <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) +
    ggplot2::geom_point() +
    theme_cleaner(legend_position = "right")

  # Check if the legend positions are applied correctly
  expect_equal(p1$theme$legend.position, "bottom")
  expect_equal(p2$theme$legend.position, "right")
})

#' @title Validate Column Names
#'
#' @description
#' This function checks if all column names of a data frame or tibble are within
#' a specified set of valid values. Depending on the specified type, it will
#' either throw an error, issue a warning, or send a message.
#'
#' @inheritParams validate_numeric
#' @param input A data.frame or tibble. validate_names() will run
#' colnames(input) to get the expected column names.
#' @param check_names A vector of column names as strings to check against
#' input.
#' @inheritParams validate_data_pull
#'
#' @return NULL. The function is used for its side effects.
#'
#' @author
#' Nicolas Foss, Ed.D., MS
#'
#' @keywords internal
#' @noRd
#'
validate_names <- function(
  input,
  check_names,
  type = c("error", "warning", "message"),
  na_ok = TRUE,
  null_ok = TRUE,
  var_name = NULL,
  calls = NULL
) {
  # Validate the type argument
  type <- match.arg(arg = type, choices = c("error", "warning", "message"))

  # Define number of callers to go back
  calls <- ifelse(is.null(calls), 2, calls)

  # Get the input name, optionally using var_name
  if (is.null(var_name)) {
    input_name <- rlang::as_name(rlang::enquo(check_names))
  } else {
    # Validate var_name
    validate_character_factor(input = var_name, type = "error", calls = 1)

    # Initialize input_name using var_name
    input_name <- var_name
  }

  # Check if the input is NULL
  if (is.null(check_names)) {
    if (!null_ok) {
      validate_error_type(
        input = input_name,
        message = "must not be NULL.",
        type = "error",
        calls = calls
      )
    }
    return(NULL)
  }

  # Check for NA values if na_ok is FALSE
  if (!na_ok && any(is.na(input))) {
    validate_error_type(
      input = input_name,
      message = "must not be a missing value.",
      type = "error",
      calls = calls
    )
  }

  # Validate the valid_set input
  validate_data_structure(
    input = input,
    structure_type = c("data.frame", "tibble"),
    logic = "or",
    type = "error",
    calls = calls
  )

  # Validate check_names, ensure it has class character
  validate_character_factor(input = check_names, type = "error", calls = 1)

  # Get the column names of the target data
  valid_set <- colnames(input)

  # Check if all column names are within the valid set
  invalid_values <- setdiff(x = check_names, y = valid_set)

  # Check to ensure the invalid_values are not empty
  if (length(invalid_values) > 0) {
    if (length(valid_set) <= 10) {
      # Clip invalid_values down to a length of <= 10
      invalid_values <- head(invalid_values, n = 10)

      # Call the validate_error_type function to handle the message display
      # For small valid_set
      validate_error_type(
        input = input_name,
        message = glue::glue(
          "contains invalid column names such as {cli::col_grey(paste0('(', paste0(invalid_values, collapse = ', '), ')'))}. Valid column names are: {cli::col_blue(paste0('(', paste0(valid_set, collapse = ', '), ')'))}."
        ),
        type = type,
        calls = calls
      )
    } else {
      # Clip invalid_values down to a length of <= 10
      invalid_values <- head(invalid_values, n = 10)

      # Call the validate_error_type function to handle the message display
      # For large valid_set

      # Clip valid_set down to a length of <= 10
      valid_set <- head(valid_set, 10)

      # Modified messaging
      validate_error_type(
        input = input_name,
        message = glue::glue(
          "contains invalid column names such as {cli::col_grey(paste0('(', paste0(invalid_values, collapse = ', '), ')'))}. Some examples of valid column names are: {cli::col_blue(paste0('(', paste0(valid_set, collapse = ', '), ',...', ')'))}."
        ),
        type = type,
        calls = calls
      )
    }
  }
}

#' @title Impute Numeric Column Values
#'
#' @description Cleans numeric columns by handling extreme values or imputing
#'   missing values. The function supports two main focuses: handling skewed
#'   distributions or imputing missing data.
#'
#' @param x A numeric vector to be cleaned.
#' @param focus A character string indicating the focus. Options are:
#'   - `"skew"`: Handle extreme values using percentile or
#'   IQR methods (default).
#'   - `"missing"`: Impute missing values.
#' @param method A character string specifying the method:
#'   - For `focus = "skew"`:
#'     - `"winsorize"`: Replace values outside specified percentiles (default).
#'     - `"iqr"`: Use IQR to limit extreme values.
#'   - For `focus = "missing"`:
#'     - `"mean"`: Replace missing values with the mean.
#'     - `"median"`: Replace missing values with the median.
#' @param percentile A numeric value (percentile > 0) for winsorization.
#'   If not provided, defaults to 0.01 and 0.99.
#'
#' @return A numeric vector with cleaned or imputed values.
#'
#' @examples
#' x <- c(1, 2, 3, 100, 200, NA)
#' # Winsorize to 1% and 99%
#' impute(x, focus = "skew", method = "winsorize")
#'
#' # Replace missing values with the mean
#' impute(x, focus = "missing", method = "mean")
#'
#' @export
#'
impute <- function(
  x,
  focus = c("skew", "missing"),
  method = c("winsorize", "iqr", "mean", "median"),
  percentile = NULL
) {
  # Check if x is numeric ----
  validate_numeric(input = x, type = "e")

  # Validate focus ----
  focus <- match.arg(focus, choices = c("skew", "missing"))

  # Ensure that focus is used correctly ----
  validate_length(input = focus, exact_length = 1, type = "error")

  # Ensure that method is used correctly ----
  validate_length(input = method, exact_length = 1, type = "error")

  # Dynamic definition of valid method argument input ----
  valid_methods <- if (focus == "skew") {
    c("winsorize", "iqr")
  } else if (focus == "missing") {
    c("mean", "median")
  }

  # Validate method ----
  validate_set(input = method, valid_set = valid_methods, type = "error")

  # Validate percentile for skew focus and winsorize method ----
  if (focus == "skew" && method == "winsorize" && !is.null(percentile)) {
    validate_numeric(
      input = percentile,
      min = 0,
      max = 1,
      na_ok = FALSE,
      null_ok = TRUE,
      type = "error"
    )
  }

  # Check if the focus is on skewness ----
  if (focus == "skew") {
    # Check if the method is winsorization and the percentile is not provided
    if (method == "winsorize" && is.null(percentile)) {
      # Calculate the upper limit as the 99th percentile of the data ----
      upper_limit <- as.numeric(stats::quantile(
        {{ x }},
        na.rm = TRUE, # Remove NA values before calculation
        probs = 0.99 # 99th percentile
      ))

      # Calculate the lower limit as the 1st percentile of the data ----
      lower_limit <- as.numeric(stats::quantile(
        {{ x }},
        na.rm = TRUE, # Remove NA values before calculation
        probs = 0.01 # 1st percentile
      ))

      # Winsorize the data:  ----
      # replace values above the upper limit with the upper
      # limit, and values below the lower limit with the lower limit
      imputed_x <- dplyr::if_else(
        {{ x }} > upper_limit,
        upper_limit,
        dplyr::if_else({{ x }} < lower_limit, lower_limit, {{ x }})
      )

      # Check if the method is winsorization and the percentile is provided ----
    } else if (method == "winsorize" && !is.null(percentile)) {
      # Calculate the upper limit as the specified percentile of the data
      upper_limit <- as.numeric(stats::quantile(
        {{ x }},
        na.rm = TRUE, # Remove NA values before calculation
        probs = percentile # Specified percentile
      ))

      # Calculate the lower limit  ----
      # as the complement of the specified percentile
      lower_limit <- as.numeric(stats::quantile(
        {{ x }},
        na.rm = TRUE, # Remove NA values before calculation
        probs = 1 - percentile # Complement of the specified percentile
      ))

      # Winsorize the data: ----
      # replace values above the upper limit with the upper
      # limit, and values below the lower limit with the lower limit
      imputed_x <- dplyr::if_else(
        {{ x }} > upper_limit,
        upper_limit,
        dplyr::if_else({{ x }} < lower_limit, lower_limit, {{ x }})
      )

      # Check if the method is interquartile range (IQR) for extreme values ----
    } else if (method == "iqr") {
      # Calculate the IQR of the data
      iqr_x <- stats::IQR({{ x }}, na.rm = TRUE)

      # Calculate the upper quantile (75th percentile) of the data ----
      upper_quantile <- as.numeric(stats::quantile(
        {{ x }},
        probs = 0.75,
        na.rm = TRUE
      ))

      # Calculate the lower quantile (25th percentile) of the data ----
      lower_quantile <- as.numeric(stats::quantile(
        {{ x }},
        probs = 0.25,
        na.rm = TRUE
      ))

      # Calculate the upper limit as the upper quantile plus 1.5 times the IQR
      upper_limit <- upper_quantile + (1.5 * iqr_x)
      # Calculate the lower limit as the lower quantile minus 1.5 times the IQR
      lower_limit <- lower_quantile - (1.5 * iqr_x)

      # Clip the data: replace values above the upper limit with the upper ----
      # limit, and values below the lower limit with the lower limit
      imputed_x <- dplyr::if_else(
        {{ x }} > upper_limit,
        upper_limit,
        dplyr::if_else({{ x }} < lower_limit, lower_limit, {{ x }})
      )
    }
  } else if (focus == "missing") {
    # Check if the focus is on missing values ----

    if (method == "mean") {
      # If the method is mean imputation for missing values ----

      # Replace missing values with the mean of the data ----
      imputed_x <- dplyr::if_else(
        is.na({{ x }}),
        base::mean({{ x }}, na.rm = TRUE),
        {{ x }}
      )
    } else if (method == "median") {
      # If the method is median imputation for missing values ----

      # Replace missing values with the median of the data ----
      imputed_x <- dplyr::if_else(
        is.na({{ x }}),
        stats::median({{ x }}, na.rm = TRUE),
        {{ x }}
      )
    }
  }

  # Return the imputed data ----
  return(imputed_x)
}

test_that("impute() handles non-numeric input", {
  expect_error(impute(c("a", "b", "c")), "x: must be.*numeric")
})

test_that("impute() defaults focus and method correctly", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  expect_error(
    impute(x, focus = c("skew", "missing")),
    "must have an exact length of 1"
  )
  expect_error(
    impute(x, focus = "skew", method = c("winsorize", "iqr")),
    "must have an exact length of 1"
  )
  expect_error(
    impute(x, focus = "missing", method = c("mean", "median")),
    "must have an exact length of 1"
  )
})

test_that("impute() validates method for skew focus", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  expect_error(
    impute(x, focus = "skew", method = "mean")
  )
})


test_that("impute() validates percentile for winsorization", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  expect_error(impute(
    x,
    focus = "skew",
    method = "winsorize",
    percentile = -0.1
  ))
  expect_no_error(impute(
    x,
    focus = "skew",
    method = "winsorize",
    percentile = 0.1
  ))
})

test_that("impute() correctly winsorizes with default percentiles", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  result <- impute(x, focus = "skew", method = "winsorize")
  expect_equal(result, c(10.4, 20.0, 30.0, 9680.0, 2000.0, NA)) # Default 1% and 99%
})

test_that("impute() correctly winsorizes with custom percentiles", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  result <- impute(x, focus = "skew", method = "winsorize", percentile = 0.05)
  expect_equal(result, c(8400.0, 12.0, 12.0, 12.0, 12.0, NA)) # Custom 5% and 95%
})

test_that("impute() handles IQR for extreme values", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  result <- impute(x, focus = "skew", method = "iqr")
  expect_equal(result, c(10, 20, 30, 4970, 2000, NA)) # IQR-based limits
})

test_that("impute() imputes missing values with mean", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  result <- impute(x, focus = "missing", method = "mean")
  expect_equal(result, c(10, 20, 30, 10000, 2000, 2412)) # Mean = 2412
})

test_that("impute() imputes missing values with median", {
  x <- c(10, 20, 30, 10000, 2000, NA)
  result <- impute(x, focus = "missing", method = "median")
  expect_equal(result, c(10, 20, 30, 10000, 2000, 30)) # Median = 30
})

#' @title SEQIC Indicator 10 – Trauma Team Activation Appropriateness
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Calculates three trauma system quality indicators related to trauma team
#' activations where the patient was kept at the facility:
#' \itemize{
#'   \item 10a: Proportion of patients meeting triage criteria (based on Injury
#'   Severity Score or Need For Trauma Intervention) who received low-level
#'   or no activation (undertriage).
#'   \item 10b: Proportion of patients not meeting triage criteria who received
#'   highest-level trauma activation (overtriage).
#'   \item 10c: Proportion of major trauma patients receiving a full activation
#'   (undertriage via Peng & Xiang, 2019).
#'
#'   (10a, 10b, 10c can be based on Injury Severity Score or Need For Trauma
#'   Intervention based on user choice)
#' }
#'
#' Users may stratify results by one or more grouping variables and optionally
#' compute confidence intervals.
#'
#' @inheritParams seqic_indicator_1
#' @inheritParams seqic_indicator_6
#' @inheritParams seqic_indicator_9
#'
#' @param trauma_team_activation_level Column indicating the trauma team
#'   activation level (e.g., `"Level 1"`, `"Level 2"`, `"Level 3"`,
#'   `"Consultation"`). Must be character or factor.
#' @param iss Optional numeric column representing the Injury Severity Score.
#' @param nfti Optional column indicating Need For Trauma Intervention
#'   classification of positive or negative. Should be character, factor, or
#'   logical.
#' @param groups Optional character vector of column names used for grouping
#'   results.
#' @param calculate_ci Optional; if not `NULL`, must be `"wilson"` or
#'   `"clopper-pearson"` to compute confidence intervals.
#'
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details
#' This function:
#' \itemize{
#'   \item Restricts analysis to Level I–IV trauma centers.
#'   \item Removes duplicate incidents using `unique_incident_id`.
#'   \item Classifies each record as meeting or not meeting triage criteria
#'   based on ISS or NFTI logic.
#'   \item Optionally computes 95% confidence intervals for each indicator.
#' }
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @returns A list of two tibbles with counts and proportions for SEQIC
#'   Indicators 10a, 10b, and 10c, along with model diagnostics for the Cribari
#'   or NFTI ouputs.  The proportions in 10a, 10b, and 10c will optionally
#'   include 95% confidence intervals.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Simulated data for SEQIC Indicator 10
#' test_data <- tibble::tibble(
#'   id = as.character(1:12),
#'   trauma_level = c("I", "II", "III", "IV", "II", "I", "IV", "III", "II", "I",
#'   "III", "IV"),
#'   activation = c("Level 1", "Level 2", "None", "Consultation", "Level 1",
#'   "Level 1", "None", "Level 3", "Level 1", "Consultation", "None", "Level
#'   2"),
#'   acute_transfer = rep("No", 12),
#'   iss = c(25, 10, 16, 8, 30, 45, 12, 9, 28, 6, 17, 14),
#'   nfti = c(TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, FALSE, FALSE, TRUE, FALSE,
#'   TRUE, TRUE),
#'   region = rep(c("East", "West"), each = 6)
#' )
#'
#' # Run the function, this will succeed
#' traumar::seqic_indicator_10(
#'   data = test_data,
#'   level = trauma_level,
#'   included_levels = c("I", "II", "III", "IV"),
#'   unique_incident_id = id,
#'   transfer_out_indicator = acute_transfer,
#'   trauma_team_activation_level = activation,
#'   iss = iss,
#'   nfti = NULL,
#'   groups = "region",
#'   calculate_ci = "wilson"
#' )
#'
#' # Run the function, this will fail
#' # Only one of `iss` or `nfti` arguments can be passed, not both
#' try(
#'   traumar::seqic_indicator_10(
#'   data = test_data,
#'   level = trauma_level,
#'   included_levels = c("I", "II", "III", "IV"),
#'   unique_incident_id = id,
#'   transfer_out_indicator = acute_transfer,
#'   trauma_team_activation_level = activation,
#'   iss = iss,
#'   nfti = nfti,
#'   groups = "region",
#'   calculate_ci = "wilson"
#' ))
#'
#' @references
#'
#' Beam G, Gorman K, Nannapaneni S, Zipf J, Simunich T, et al. (2022) Need for
#' Trauma Intervention and Improving Under-Triaging in Geriatric Trauma
#' Patients: undertriaged or Misclassified. Int J Crit Care Emerg Med 8:136.
#' doi.org/10.23937/2474-3674/1510136
#'
#' Peng J, Xiang H. Trauma undertriage and overtriage rates: are we using the
#' wrong formulas? Am J Emerg Med. 2016 Nov;34(11):2191-2192. doi:
#' 10.1016/j.ajem.2016.08.061. Epub 2016 Aug 31. PMID: 27615156; PMCID:
#' PMC6469681.
#'
#' Roden-Foreman JW, Rapier NR, Yelverton L, Foreman ML. Asking a Better
#' Question: Development and Evaluation of the Need For Trauma Intervention
#' (NFTI) Metric as a Novel Indicator of Major Trauma. J Trauma Nurs. 2017
#' May/Jun;24(3):150-157. doi: 10.1097/JTN.0000000000000283. PMID: 28486318.
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_10 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  transfer_out_indicator,
  trauma_team_activation_level,
  iss,
  nfti,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure that `transfer_out_indicator` can be validated  ----
  transfer_out_indicator_check <- validate_data_pull(
    input = data,
    col = {{ transfer_out_indicator }},
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Validate `transfer_out_indicator` ----
  validate_class(
    input = transfer_out_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Ensure that `trauma_team_activation_level` can be validated ----
  trauma_team_activation_level_check <- validate_data_pull(
    input = data,
    col = {{ trauma_team_activation_level }},
    type = "error",
    var_name = "trauma_team_activation_level"
  )

  # Validate that `trauma_team_activation_level` ----
  validate_character_factor(
    input = trauma_team_activation_level_check,
    type = "error",
    var_name = "trauma_team_activation_level"
  )

  # Validate that `iss` is numeric. ----
  if (!rlang::quo_is_null(rlang::enquo(iss))) {
    # Ensure that `iss` can be validated ----
    iss_check <- validate_data_pull(
      input = data,
      col = {{ iss }},
      type = "error",
      var_name = "iss"
    )

    # Validation of `iss` ----
    validate_numeric(
      input = iss_check,
      min = 0,
      max = 75,
      type = "error",
      var_name = "iss"
    )
  }

  # Validate that `nfti` is character, factor, or logical.
  if (!rlang::quo_is_null(rlang::enquo(nfti))) {
    # Ensure that `nfti` can be validated
    nfti_check <- validate_data_pull(
      input = data,
      col = {{ nfti }},
      type = "error",
      var_name = "nfti"
    )

    # Validate `nfti`
    validate_class(
      input = nfti_check,
      class_type = c("character", "factor", "logical"),
      logic = "or",
      type = "error",
      var_name = "nfti"
    )
  }

  # Check if all elements in groups are strings (i.e., character vectors)
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all groups exist in the `data`
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Data preparation ----
  ###___________________________________________________________________________

  # Preprocess the input dataset: ----
  # - Remove duplicate incidents to avoid double-counting (based on unique
  # incident ID)
  # - Restrict analysis to Level I-IV trauma centers
  # - Exclude transfers out (e.g., for whom definitive triage data may be
  # incomplete)

  data_prep <- data |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::filter(
      {{ level }} %in% included_levels,
      {{ transfer_out_indicator }} %in% c("No", FALSE)
    ) |>
    dplyr::mutate(
      # Define cases with highest-level activation (Level 1)
      full_activation = grepl(
        pattern = "level 1",
        x = {{ trauma_team_activation_level }},
        ignore.case = TRUE
      ),

      # Define low activation: ----
      # - Activation not recorded; assume no activation called
      # - Any level other than "Level 1"
      limited_no_activation = is.na({{ trauma_team_activation_level }}) |
        !grepl(
          pattern = "level 1",
          x = {{ trauma_team_activation_level }},
          ignore.case = TRUE
        )
    )

  # Dynamically classify patients using either ISS or NFTI logic ----
  if (
    !rlang::quo_is_null(rlang::enquo(iss)) &&
      rlang::quo_is_null(rlang::enquo(nfti))
  ) {
    data_prep <- data_prep |>
      dplyr::mutate(
        # Patients who should have had activation based on Cribari ISS > 15
        major_trauma = {{ iss }} > 15,
        # Patients who clearly did not require activation (ISS < 9)
        minor_trauma = {{ iss }} < 9,
        # Over triage = full activation and minor trauma
        overtriage = full_activation & minor_trauma,
        # Under triage = limited-to-no activation and major trauma
        undertriage = limited_no_activation & major_trauma
      )
  } else if (
    rlang::quo_is_null(rlang::enquo(iss)) &&
      !rlang::quo_is_null(rlang::enquo(nfti))
  ) {
    data_prep <- data_prep |>
      dplyr::mutate(
        # Patients flagged by NFTI as needing activation
        major_trauma = {{ nfti }} %in% c("Positive", TRUE, "Yes"),
        # Patients flagged by NFTI as NOT needing activation
        minor_trauma = {{ nfti }} %in% c("Negative", FALSE, "No"),
        # Over triage = full activation and minor trauma
        overtriage = full_activation & minor_trauma,
        # Under triage = limited-to-no activation and major trauma
        undertriage = limited_no_activation & major_trauma
      )
  } else {
    # Fail clearly if both or neither triage criteria are supplied
    cli::cli_abort(
      "Please supply exactly one of {.var iss} or {.var nfti}."
    )
  }

  # Get an identifier of how the triage classification was performed ----
  # Determine triage logic source as a scalar
  triage_logic_source <- if (
    !rlang::quo_is_null(rlang::enquo(nfti)) &&
      rlang::quo_is_null(rlang::enquo(iss))
  ) {
    "nfti"
  } else if (
    rlang::quo_is_null(rlang::enquo(nfti)) &&
      !rlang::quo_is_null(rlang::enquo(iss))
  ) {
    "cribari"
  } else if (
    !rlang::quo_is_null(rlang::enquo(nfti)) &&
      !rlang::quo_is_null(rlang::enquo(iss))
  ) {
    cli::cli_abort(
      "Please supply exactly one of {.var iss} or {.var nfti}."
    )
  }

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Initiate the list for output ----
  seqic_10 <- list()

  # Measure 10a: undertriage ----
  # Patients who met triage criteria (positive) but received low activation
  # Denominator: all limited-to-no trauma team activation cases
  # Numerator: major_trauma AND limited_no_activation
  seqic_10a <- data_prep |>
    dplyr::summarize(
      numerator_10a = sum(
        undertriage,
        na.rm = TRUE
      ),

      # Patients who had a limited or no trauma team activation
      denominator_10a = sum(limited_no_activation, na.rm = TRUE),
      seqic_10a = dplyr::if_else(
        denominator_10a > 0,
        numerator_10a / denominator_10a,
        NA_real_ # Return NA if no denominator
      ),
      .by = {{ groups }}
    )

  # Measure 10b: overtriage ----
  # Patients who did NOT meet triage criteria (negative) but received highest
  # activation
  # Denominator: all full trauma team activations
  # Numerator: minor_trauma AND full_activation
  seqic_10b <- data_prep |>
    dplyr::summarize(
      numerator_10b = sum(
        overtriage,
        na.rm = TRUE
      ),
      denominator_10b = sum(full_activation, na.rm = TRUE),
      seqic_10b = dplyr::if_else(
        denominator_10b > 0,
        numerator_10b / denominator_10b,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # Measure 10c: undertriage ----
  # Patients who met triage criteria (positive) but received low activation
  # Denominator: all major trauma cases
  # Numerator: major_trauma AND limited_no_activation
  # This is Peng & Xiang's (2016) update to the Cribari method of calculating
  # under triage
  seqic_10c <- data_prep |>
    dplyr::summarize(
      numerator_10c = sum(
        undertriage,
        na.rm = TRUE
      ),

      # All major trauma patients as denominator
      denominator_10c = sum(major_trauma, na.rm = TRUE),
      seqic_10c = dplyr::if_else(
        denominator_10c > 0,
        numerator_10c / denominator_10c,
        NA_real_ # Return NA if no denominator
      ),
      .by = {{ groups }}
    )

  # Model Diagnostic Testing ----
  # Cribari 2x2 matrix to produce model diagnostic tests
  # Based on methods in Peng & Xiang (2016)

  # The following is from Table 1 in Peng & Xiang (2016)
  # The Cribari matrix: Injury severity and trauma team activation.
  #
  #                            | Minor Trauma | Major Trauma |   Total
  # ---------------------------|--------------|--------------|---------
  # Full Trauma Team Activation|       a      |      b       |   a + b
  # Limited/No Activation      |       c      |      d       |   c + d
  # ---------------------------|--------------|--------------|---------
  # Total                      |     a + c    |    b + d     |     N
  #
  # Common statistical terms used in diagnostic testing:
  #
  # Sensitivity              = b / (b + d)
  # Specificity              = c / (a + c)
  #
  # False Negative Rate (FNR) = d / (b + d)      # 1 - Sensitivity
  # False Positive Rate (FPR) = a / (a + c)      # 1 - Specificity
  #
  # Positive Predictive Value (PPV) = b / (a + b)
  # Negative Predictive Value (NPV) = c / (c + d)
  #
  # False Discovery Rate (FDR) = a / (a + b)     # 1 - PPV
  # False Omission Rate (FOR)  = d / (c + d)     # 1 - NPV

  diagnostics <- data_prep |>
    dplyr::summarize(
      # Calculate the key confusion matrix values
      full_minor = sum(full_activation & minor_trauma, na.rm = TRUE), # False Positive
      full_major = sum(full_activation & major_trauma, na.rm = TRUE), # True Positive
      limited_minor = sum(limited_no_activation & minor_trauma, na.rm = TRUE), # True Negative
      limited_major = sum(limited_no_activation & major_trauma, na.rm = TRUE), # False Negative
      .by = {{ groups }}
    ) |>
    dplyr::mutate(
      # Total number of classified records ----
      # N here is total records not missing classification information
      N = full_minor + full_major + limited_minor + limited_major,

      # Sensitivity = b / (b + d) ----
      sensitivity = dplyr::if_else(
        (full_major + limited_major) > 0,
        full_major / (full_major + limited_major),
        NA_real_
      ),

      # Specificity = c / (a + c) ----
      specificity = dplyr::if_else(
        (full_minor + limited_minor) > 0,
        limited_minor / (full_minor + limited_minor),
        NA_real_
      ),

      # Positive Predictive Value (PPV) = b / (a + b) <-
      positive_predictive_value = dplyr::if_else(
        (full_minor + full_major) > 0,
        full_major / (full_minor + full_major),
        NA_real_
      ),

      # Negative Predictive Value (NPV) = c / (c + d) ----
      negative_predictive_value = dplyr::if_else(
        (limited_minor + limited_major) > 0,
        limited_minor / (limited_minor + limited_major),
        NA_real_
      ),

      # False Negative Rate (FNR) = d / (b + d); 1 - Sensitivity ----
      false_negative_rate = dplyr::if_else(
        (full_major + limited_major) > 0,
        limited_major / (full_major + limited_major),
        NA_real_
      ),

      # False Positive Rate (FPR) = a / (a + c); 1 - Specificity ----
      false_positive_rate = dplyr::if_else(
        (full_minor + limited_minor) > 0,
        full_minor / (full_minor + limited_minor),
        NA_real_
      ),

      # False Discovery Rate (FDR) = a / (a + b); ----
      # 1 - Positive Predictive Value
      false_discovery_rate = dplyr::if_else(
        (full_minor + full_major) > 0,
        full_minor / (full_minor + full_major),
        NA_real_
      ),

      # False Omission Rate (FOR) = d / (c + d); ----
      # 1 - Negative Predictive Value
      false_omission_rate = dplyr::if_else(
        (limited_minor + limited_major) > 0,
        limited_major / (limited_minor + limited_major),
        NA_real_
      )
    )

  # Optionally compute confidence intervals ----
  if (!is.null(calculate_ci)) {
    # Apply 95% confidence interval function

    # 10a CIs ----
    seqic_10a <- seqic_10a |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_10a,
          x = numerator_10a,
          n = denominator_10a,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_10a = lower_ci, upper_ci_10a = upper_ci)
      )

    # 10b CIs ----
    seqic_10b <- seqic_10b |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_10b,
          x = numerator_10b,
          n = denominator_10b,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_10b = lower_ci, upper_ci_10b = upper_ci)
      )

    # 10c CIs ----
    seqic_10c <- seqic_10c |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_10c,
          x = numerator_10c,
          n = denominator_10c,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_10c = lower_ci, upper_ci_10c = upper_ci)
      )
  }

  # Add label if ungrouped ----
  if (is.null(groups)) {
    seqic_10$seqic_10 <-
      tibble::tibble(
        data = "population/sample",
        triage_logic = triage_logic_source
      ) |>
      dplyr::bind_cols(seqic_10a, seqic_10b, seqic_10c)

    seqic_10$diagnostics <- tibble::tibble(
      data = "population/sample",
      triage_logic = triage_logic_source
    ) |>
      dplyr::bind_cols(diagnostics)
  } else {
    # Arrange by grouping variables ----
    seqic_10$seqic_10 <- tibble::tibble(
      triage_logic = triage_logic_source
    ) |>
      dplyr::bind_cols(seqic_10a) |>
      dplyr::full_join(
        seqic_10b,
        by = dplyr::join_by(!!!rlang::syms(groups))
      ) |>
      dplyr::full_join(
        seqic_10c,
        by = dplyr::join_by(!!!rlang::syms(groups))
      ) |>
      dplyr::arrange(!!!rlang::syms(groups))

    seqic_10$diagnostics <- tibble::tibble(
      triage_logic = triage_logic_source
    ) |>
      dplyr::bind_cols(diagnostics) |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  # Return measures in a tibble ----
  return(seqic_10)
}

# tests/testthat/test-seqic_indicator_10.R

testthat::test_that("seqic_indicator_10() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  test_data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "II", "III", "IV", "II"),
    acute_transfer = rep("No", 6),
    activation = c("Level 1", "None", "Level 2", "Level 1", NA, "Consultation"),
    iss = c(20, 10, 25, 12, 18, 9),
    nfti = c(TRUE, FALSE, TRUE, FALSE, TRUE, FALSE),
    region = c("East", "West", "East", "West", "East", "West")
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = false,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = NULL,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = other,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = NULL,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = minor_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = NULL,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = TRUE,
      iss = iss,
      nfti = NULL,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = "faked",
      nfti = NULL,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = NULL,
      nfti = FALSE,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti,
      groups = "region",
      calculate_ci = NULL
    ),
    regexp = "Please supply exactly one of"
  )
})

test_that("data validation fails appropriately", {
  data <- tibble::tibble(
    id = as.character(1:3),
    trauma_level = c("I", "II", "III"),
    acute_transfer = rep("No", 3),
    activation = c("Level 1", "Level 2", "None"),
    iss = c(15, 22, 10),
    nfti = c(TRUE, FALSE, TRUE),
    region = c("East", "West", "East")
  )

  # Not a data frame
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = list(),
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "data.*must be of class.*data.frame, tbl, tbl_df"
  )

  # Level not character or factor
  bad_data <- data |> dplyr::mutate(trauma_level = as.numeric(1:3))
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "level.*must be of class.*character.*factor"
  )

  # unique_incident_id wrong class
  bad_data <- data |> dplyr::mutate(id = as.Date("2023-01-01") + 0:2)
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "must be of class"
  )

  # trauma_team_activation_level not character/factor
  bad_data <- data |> dplyr::mutate(activation = as.Date("2023-01-01") + 0:2)
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "trauma_team_activation_level: must be of class.*character.*factor"
  )

  # trauma_team_activation_level not character/factor
  bad_data <- data |> dplyr::mutate(acute_transfer = numeric(length = 3))
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "transfer_out_indicator: must be of class.*logical, character, factor"
  )

  # iss is not numeric
  bad_data <- data |> dplyr::mutate(iss = as.character(iss))
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "iss.*must be.*numeric"
  )

  # nfti wrong class
  bad_data <- data |> dplyr::mutate(nfti = as.Date("2023-01-01") + 0:2)
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = bad_data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "nfti.* must be of class.*character, factor, logical"
  )

  # groups not character
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti,
      groups = 1:2
    ),
    "groups.* must be of class.*character.*factor"
  )

  # groups has invalid column names
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti,
      groups = c("region", "nonexistent_col")
    ),
    "groups.*contains invalid column names.*nonexistent_col.*Valid column names are.*id, trauma_level, acute_transfer, activation, iss, nfti, region"
  )

  # calculate_ci invalid
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti,
      calculate_ci = "banana"
    ),
    "calculate_ci.* contains invalid values.*banana.*Valid values are.*wilson.*clopper-pearson"
  )

  # included_levels wrong type
  testthat::expect_error(
    traumar::seqic_indicator_10(
      data = data,
      level = trauma_level,
      included_levels = list("I", "II"),
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      trauma_team_activation_level = activation,
      iss = iss,
      nfti = nfti
    ),
    "must be of class"
  )
})

test_that("SEQIC Indicator 10a, 10b, 10c calculate correctly", {
  test_data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "II", "III", "IV", "II"),
    acute_transfer = rep("No", 6),
    activation = c("Level 1", "None", "Level 2", "Level 1", NA, "Consultation"),
    iss = c(20, 10, 25, 12, 18, 9),
    region = c("East", "West", "East", "West", "East", "West")
  )

  result <- traumar::seqic_indicator_10(
    data = test_data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id = id,
    transfer_out_indicator = acute_transfer,
    trauma_team_activation_level = activation,
    iss = iss,
    nfti = NULL,
    groups = "region",
    calculate_ci = NULL
  )

  # Check 10a: under-triage (major trauma but limited/no activation)
  testthat::expect_named(
    result,
    c("seqic_10", "diagnostics")
  )
  testthat::expect_true(all(
    c(
      "numerator_10a",
      "denominator_10a",
      "seqic_10a"
    ) %in%
      names(result$seqic_10)
  ))

  testthat::expect_true(all(
    result$seqic_10$seqic_10a >= 0 & result$seqic_10$seqic_10a <= 1,
    na.rm = TRUE
  ))

  # Check 10b: over-triage (minor trauma with full activation)
  testthat::expect_true(all(
    c("numerator_10b", "denominator_10b", "seqic_10b") %in%
      names(result$seqic_10)
  ))
  testthat::expect_true(all(
    result$seqic_10$seqic_10b >= 0 & result$seqic_10$seqic_10b <= 1,
    na.rm = TRUE
  ))

  # Check 10c: under-triage (major trauma cases missed, Peng & Xiang)
  testthat::expect_true(all(
    c("numerator_10c", "denominator_10c", "seqic_10c") %in%
      names(result$seqic_10)
  ))
  testthat::expect_true(all(
    result$seqic_10$seqic_10c >= 0 & result$seqic_10$seqic_10c <= 1,
    na.rm = TRUE
  ))
})

test_that("Model diagnostic statistics are calculated correctly", {
  test_data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("I", "II", "II", "IV"),
    acute_transfer = rep("No", 4),
    activation = c("Level 1", "None", "Level 1", "Consultation"),
    iss = c(20, 10, 8, 18),
    region = c("East", "East", "West", "West")
  )

  result <- traumar::seqic_indicator_10(
    data = test_data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id = id,
    transfer_out_indicator = acute_transfer,
    trauma_team_activation_level = activation,
    iss = iss,
    nfti = NULL,
    groups = "region",
    calculate_ci = NULL
  )

  diag <- result$diagnostics
  testthat::expect_true(all(
    c(
      "full_minor",
      "full_major",
      "limited_minor",
      "limited_major",
      "sensitivity",
      "specificity",
      "positive_predictive_value",
      "negative_predictive_value",
      "false_negative_rate",
      "false_positive_rate",
      "false_discovery_rate",
      "false_omission_rate"
    ) %in%
      names(diag)
  ))

  testthat::expect_true(all(
    diag$sensitivity >= 0 & diag$sensitivity <= 1,
    na.rm = TRUE
  ))
})

test_that("Function includes triage logic indicator", {
  test_data <- tibble::tibble(
    id = 1:2,
    trauma_level = c("I", "II"),
    acute_transfer = rep("No", 2),
    activation = c("Level 1", "None"),
    iss = c(20, 5)
  )

  res <- traumar::seqic_indicator_10(
    data = test_data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id = id,
    transfer_out_indicator = acute_transfer,
    trauma_team_activation_level = activation,
    iss = iss,
    nfti = NULL
  )

  testthat::expect_equal(res$seqic_10$triage_logic, "cribari")
})

test_that("Confidence intervals are returned when requested", {
  test_data <- tibble::tibble(
    id = 1:12,
    trauma_level = rep("I", 12),
    acute_transfer = rep("No", 12),
    activation = rep(c("Level 1", "Level 2", "None"), each = 4),
    iss = rep(c(20, 10, 25, 15), 3)
  )

  res <- traumar::seqic_indicator_10(
    data = test_data,
    level = trauma_level,
    included_levels = c("I"),
    unique_incident_id = id,
    transfer_out_indicator = acute_transfer,
    trauma_team_activation_level = activation,
    iss = iss,
    nfti = NULL,
    calculate_ci = "wilson"
  )

  testthat::expect_true("lower_ci_10a" %in% names(res$seqic_10))
  testthat::expect_true("upper_ci_10a" %in% names(res$seqic_10))

  testthat::expect_true("lower_ci_10b" %in% names(res$seqic_10))
  testthat::expect_true("upper_ci_10b" %in% names(res$seqic_10))

  testthat::expect_true("lower_ci_10c" %in% names(res$seqic_10))
  testthat::expect_true("upper_ci_10c" %in% names(res$seqic_10))
})

testthat::test_that("classification works with ISS only", {
  test_data <- tibble::tibble(
    id = 1:12,
    trauma_level = rep("I", 12),
    acute_transfer = rep("No", 12),
    activation = rep(c("Level 1", "Level 2", "None"), each = 4),
    iss = rep(c(20, 10, 25, 15), 3)
  )

  res <- traumar::seqic_indicator_10(
    data = test_data,
    unique_incident_id = id,
    level = trauma_level,
    transfer_out_indicator = acute_transfer,
    iss = iss,
    trauma_team_activation_level = activation,
    nfti = NULL
  )

  expected_cols <- c(
    "data",
    "triage_logic",
    "full_minor",
    "full_major",
    "limited_minor",
    "limited_major",
    "N",
    "sensitivity",
    "specificity",
    "positive_predictive_value",
    "negative_predictive_value",
    "false_negative_rate",
    "false_positive_rate",
    "false_discovery_rate",
    "false_omission_rate"
  )

  testthat::expect_true(all(expected_cols %in% names(res$diagnostic)))

  expect_true(all(res$diagnostic$triage_logic == "cribari"))
})

testthat::test_that("classification works with NFTI only", {
  set.seed(123)
  test_data <- tibble::tibble(
    id = 1:12,
    trauma_level = rep("I", 12),
    acute_transfer = rep("No", 12),
    activation = rep(c("Level 1", "Level 2", "None"), each = 4),
    NFTI = sample(x = c(TRUE, FALSE), size = 12, replace = TRUE)
  )

  res <- traumar::seqic_indicator_10(
    data = test_data,
    unique_incident_id = id,
    level = trauma_level,
    transfer_out_indicator = acute_transfer,
    iss = NULL,
    trauma_team_activation_level = activation,
    nfti = NFTI
  )

  expected_cols <- c(
    "data",
    "triage_logic",
    "full_minor",
    "full_major",
    "limited_minor",
    "limited_major",
    "N",
    "sensitivity",
    "specificity",
    "positive_predictive_value",
    "negative_predictive_value",
    "false_negative_rate",
    "false_positive_rate",
    "false_discovery_rate",
    "false_omission_rate"
  )

  testthat::expect_true(all(expected_cols %in% names(res$diagnostic)))

  expect_true(all(res$diagnostic$triage_logic == "nfti"))
})

testthat::test_that("error is thrown when both ISS and NFTI are supplied", {
  set.seed(123)
  test_data <- tibble::tibble(
    id = 1:12,
    trauma_level = rep("I", 12),
    acute_transfer = rep("No", 12),
    activation = rep(c("Level 1", "Level 2", "None"), each = 4),
    NFTI = sample(x = c(TRUE, FALSE), size = 12, replace = TRUE),
    iss = rep(c(20, 10, 25, 15), 3)
  )

  expect_error(
    traumar::seqic_indicator_10(
      data = test_data,
      unique_incident_id = id,
      transfer_out_indicator = acute_transfer,
      level = trauma_level,
      iss = iss,
      trauma_team_activation_level = activation,
      nfti = NFTI
    ),
    "Please supply exactly one of"
  )
})

#' @title SEQIC Indicator 7 - Delayed Arrival to Definitive Care
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Computes SEQIC Indicator 7, which measures the proportion of trauma patients
#' arriving at the definitive care facility trauma centers (level I–IV) more
#' than 180 minutes after injury. This indicator identifies delays in definitive
#' care.
#'
#' @inheritParams seqic_indicator_1
#' @inheritParams seqic_indicator_6
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function:
#' \itemize{
#'   \item Filters the dataset to trauma center levels I through IV.
#'   \item Deduplicates the dataset by `unique_incident_id`.
#'   \item Creates a logical flag for arrivals occurring more than 180 minutes
#'   after injury.
#'   \item Identifies definitive care records where the patient arrived greater
#'   than 180 minutes after the time of injury.
#'   \item Returns a summarized tibble with the number of such cases
#'   (numerator), total eligible records (denominator), and the proportion.
#'   \item Optionally includes 95% confidence intervals if `calculate_ci` is
#'   specified.
#' }
#'
#' @note
#'
#' The user must ensure all columns are correctly passed and that time values
#' are numeric and measured in minutes.
#'
#' @return A tibble summarizing SEQIC Indicator 7 results. Includes numerator,
#'   denominator, and proportion. 95% confidence intervals are included if
#'   requested.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Create test data for Indicator 7
#' test_data <- tibble::tibble(
#'   id = as.character(1:10),
#'   trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
#'   time_to_arrival = c(200, 100, 220, 150, 400, 181, 90, 179, 240, 178),
#'   transfer_out = c("No", "No", "No", "No", "Yes", "No", "No", "No", "No",
#'   "No")
#' )
#'
#' # Run the indicator function
#' traumar::seqic_indicator_7(
#'   data = test_data,
#'   level = trauma_level,
#'   unique_incident_id = id,
#'   time_from_injury_to_arrival = time_to_arrival,
#'   transfer_out_indicator = transfer_out
#' )
#'
#' @author Nicolas Foss Ed.D., MS
#'
#' @export
seqic_indicator_7 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  time_from_injury_to_arrival,
  transfer_out_indicator,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure that `transfer_out_indicator` can be validated  ----
  transfer_out_indicator_check <- validate_data_pull(
    input = data,
    col = {{ transfer_out_indicator }},
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Validate `transfer_out_indicator` ----
  validate_class(
    input = transfer_out_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Ensure that `time_from_injury_to_arrival` can be validated
  time_from_injury_to_arrival_check <- validate_data_pull(
    input = data,
    col = {{ time_from_injury_to_arrival }},
    type = "error",
    var_name = "time_from_injury_to_arrival"
  )

  # Validate that `time_from_injury_to_arrival` is numeric.
  validate_numeric(
    input = time_from_injury_to_arrival_check,
    type = "error",
    var_name = "time_from_injury_to_arrival"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data`.
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Measure Calculation ----
  ###___________________________________________________________________________

  # Filter only trauma center levels I–IV ----
  seqic_7 <- data |>
    dplyr::filter({{ level }} %in% included_levels) |>

    # Deduplicate records by unique incident ID ----
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>

    # Create flag for arrivals >180 minutes after injury ----
    dplyr::mutate(
      arrive_greater_than_180 = {{ time_from_injury_to_arrival }} > 180
    ) |>

    # Summarize: count patients meeting the criteria (numerator) and total ----
    # (denominator)
    dplyr::summarize(
      numerator_7 = sum(
        {{ transfer_out_indicator }} %in%
          c(FALSE, "No") &
          arrive_greater_than_180 == TRUE,
        na.rm = TRUE
      ),
      denominator_7 = sum(
        {{ transfer_out_indicator }} %in% c(FALSE, "No"),
        na.rm = TRUE
      ),
      seqic_7 = dplyr::if_else(
        denominator_7 > 0,
        numerator_7 / denominator_7,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # Optionally compute confidence intervals ----
  if (!is.null(calculate_ci)) {
    # Apply binomial confidence interval function
    seqic_7 <- seqic_7 |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_7,
          x = numerator_7,
          n = denominator_7,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_7 = lower_ci, upper_ci_7 = upper_ci)
      )
  }

  # Add label if ungrouped ----
  if (is.null(groups)) {
    seqic_7 <- seqic_7 |>
      tibble::add_column(data = "population/sample", .before = "numerator_7")
  } else {
    # Arrange by grouping variables ----
    seqic_7 <- seqic_7 |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  # Return the final summarized dataset. ----
  return(seqic_7)
}

# tests/testthat/test-seqic_indicator_7.R

testthat::test_that("seqic_indicator_7() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  data <- tibble::tibble(
    id = as.character(1:5),
    trauma_level = c("I", "II", "III", "IV", "V"),
    time_to_arrival = c(200, 100, 220, 150, 400),
    transfer_out = c("No", "No", "No", "No", "Yes")
  )

  testthat::expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = fake,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = not_a_column,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = "false",
      transfer_out_indicator = transfer_out
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = not_real
    ),
    regexp = "It was not possible to validate"
  )
})

test_that("Valid input returns expected structure and values", {
  data <- tibble::tibble(
    id = as.character(1:5),
    trauma_level = c("I", "II", "III", "IV", "V"),
    time_to_arrival = c(200, 100, 220, 150, 400),
    transfer_out = c("No", "No", "No", "No", "Yes")
  )

  res <- traumar::seqic_indicator_7(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    time_from_injury_to_arrival = time_to_arrival,
    transfer_out_indicator = transfer_out
  )

  expect_s3_class(res, "tbl_df")
  expect_named(res, c("data", "numerator_7", "denominator_7", "seqic_7"))
  expect_equal(res$numerator_7, 2)
  expect_equal(res$denominator_7, 4)
  expect_equal(res$seqic_7, 0.5)
})

test_that("Grouping works correctly", {
  data <- tibble::tibble(
    id = c(1, 2, 3, 4),
    trauma_level = c("I", "I", "II", "II"),
    time_to_arrival = c(200, 100, 300, 90),
    transfer_out = c("No", "No", "No", "No"),
    region = c("North", "North", "South", "South")
  )

  res <- traumar::seqic_indicator_7(
    data,
    level = trauma_level,
    unique_incident_id = id,
    time_from_injury_to_arrival = time_to_arrival,
    transfer_out_indicator = transfer_out,
    groups = "region"
  )

  expect_equal(nrow(res), 2)
  expect_true(all(
    c("region", "numerator_7", "denominator_7", "seqic_7") %in% names(res)
  ))
})

test_that("Confidence intervals are added correctly", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = rep("I", 6),
    time_to_arrival = c(200, 210, 90, 80, 185, 195),
    transfer_out = rep("No", 6)
  )

  res <- traumar::seqic_indicator_7(
    data,
    level = trauma_level,
    unique_incident_id = id,
    time_from_injury_to_arrival = time_to_arrival,
    transfer_out_indicator = transfer_out,
    calculate_ci = "wilson"
  )

  expect_true(all(c("lower_ci_7", "upper_ci_7") %in% names(res)))
  expect_true(res$lower_ci_7 < res$seqic_7 && res$upper_ci_7 > res$seqic_7)
})

#-------------------------------
# Input Validation Tests
#-------------------------------

test_that("Errors are thrown for invalid `data` input", {
  expect_error(
    traumar::seqic_indicator_7(
      "not_a_data",
      trauma_level,
      id,
      time_to_arrival,
      transfer_out
    ),
    "data"
  )
})

test_that("Errors are thrown for invalid `level` column type", {
  data <- tibble::tibble(
    id = 1,
    trauma_level = 1L,
    time_to_arrival = 10,
    transfer_out = "No"
  )
  expect_error(
    traumar::seqic_indicator_7(
      data,
      trauma_level,
      id,
      time_to_arrival,
      transfer_out
    ),
    "level"
  )
})

test_that("Errors are thrown for invalid `unique_incident_id` type", {
  data <- tibble::tibble(
    id = list("a"),
    trauma_level = "I",
    time_to_arrival = 10,
    transfer_out = "No"
  )
  expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out
    ),
    "unique_incident_id"
  )
})

test_that("Errors are thrown for invalid `transfer_out_indicator` type", {
  data <- tibble::tibble(
    id = 1,
    trauma_level = "I",
    time_to_arrival = 10,
    transfer_out = list(TRUE)
  )
  expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out
    ),
    "transfer_out_indicator"
  )
})

test_that("Errors are thrown for non-numeric `time_from_injury_to_arrival`", {
  data <- tibble::tibble(
    id = 1,
    trauma_level = "I",
    time_to_arrival = "late",
    transfer_out = "No"
  )
  expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out
    ),
    "time_from_injury_to_arrival"
  )
})

test_that("Invalid group column throws error", {
  data <- tibble::tibble(
    id = 1,
    trauma_level = "I",
    time_to_arrival = 10,
    transfer_out = "No"
  )
  expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out,
      groups = "not_a_column"
    ),
    "groups.*contains invalid column names.*not_a_column.*Valid column names are.*id, trauma_level, time_to_arrival, transfer_out"
  )
})

test_that("Invalid calculate_ci argument throws error", {
  data <- tibble::tibble(
    id = 1,
    trauma_level = "I",
    time_to_arrival = 10,
    transfer_out = "No"
  )
  expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out,
      calculate_ci = "bad_method"
    ),
    "calculate_ci"
  )
})

test_that("Invalid included_levels throws error", {
  data <- tibble::tibble(
    id = 1,
    trauma_level = "I",
    time_to_arrival = 10,
    transfer_out = "No"
  )
  expect_error(
    traumar::seqic_indicator_7(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      time_from_injury_to_arrival = time_to_arrival,
      transfer_out_indicator = transfer_out,
      included_levels = list("I")
    ),
    "included_levels"
  )
})

utils::globalVariables(c(
  "MTOS_distribution",
  "valid_trauma_types",
  "type",
  "column_name",
  "b0",
  "b1",
  "b2",
  "b3",
  "age_points",
  ":=",
  "bin_number",
  "bin_start",
  "bin_end",
  "TD_b",
  "TA_b",
  "midpoint",
  "AntiM_b",
  "N_b",
  "R_b",
  "EM_b",
  "numerator",
  "denominator",
  "RMM",
  "Ps_range",
  "survivors",
  "current_fraction",
  "smallest_val",
  "prob_death",
  "scale_factor",
  "bins",
  "bins_temp",
  "alive",
  "dead",
  "count",
  "sd",
  "bootstrap_RMM",
  "z_score",
  "AntiS_b",
  "ISS_Calculated",
  "Initial_Assessment_Calculated_RTS",
  "Patient_Age_Years",
  "bootstrap_CI",
  "bootstrap_RMM_LL",
  "bootstrap_RMM_UL",
  "population_CI",
  "population_RMM",
  "population_RMM_LL",
  "population_RMM_UL",
  "sd_bootstrap_RMM",
  "se_bootstrap_RMM",
  "stat_sig",
  "group_vars",
  "grouped_data",
  "grouped_stats",
  "Pred_Survivors_b",
  "Pred_Deaths_b",
  ".data",
  "Delayed_DC_2hr",
  "Delayed_DC_3hr",
  "Delayed_Decision_1hr",
  "Delayed_Decision_2hr",
  "Delayed_Decision_Discharge_1hr",
  "Delayed_Decision_Discharge_2hr",
  "arrive_greater_than_180",
  "full_minor",
  "full_major",
  "limited_minor",
  "limited_major",
  "denominator_10a",
  "denominator_10b",
  "denominator_10c",
  "denominator_11",
  "denominator_12",
  "denominator_13",
  "denominator_1a",
  "denominator_1b",
  "denominator_1c",
  "denominator_1d",
  "denominator_1e",
  "denominator_1f",
  "denominator_2",
  "denominator_3",
  "denominator_4a",
  "denominator_4b",
  "denominator_5a",
  "denominator_5b",
  "denominator_5c",
  "denominator_5d",
  "denominator_6",
  "denominator_7",
  "denominator_8_all",
  "denominator_8_risk",
  "denominator_9a_activations",
  "denominator_9a_activations_risk",
  "denominator_9a_all",
  "denominator_9a_risk",
  "denominator_9b_activations",
  "denominator_9b_activations_risk",
  "denominator_9b_all",
  "denominator_9b_risk",
  "denominator_9c_activations",
  "denominator_9c_activations_risk",
  "denominator_9c_all",
  "denominator_9c_risk",
  "denominator_9d_activations",
  "denominator_9d_activations_risk",
  "denominator_9d_all",
  "denominator_9d_risk",
  "denominator_9e_activations",
  "denominator_9e_activations_risk",
  "denominator_9e_all",
  "denominator_9e_risk",
  "denominator_9f_activations",
  "denominator_9f_activations_risk",
  "denominator_9f_all",
  "denominator_9f_risk",
  "full_activation",
  "ks.test",
  "limited_no_activation",
  "los_less_24_hrs",
  "lower_ci",
  "lower_ci_1d",
  "lower_ci_4a",
  "lower_ci_4b",
  "lower_ci_5a",
  "lower_ci_5b",
  "lower_ci_5c",
  "lower_ci_5d",
  "lower_ci_9a_activations",
  "lower_ci_9a_activations_risk",
  "lower_ci_9a_all",
  "lower_ci_9a_risk",
  "lower_ci_9b_activations",
  "lower_ci_9b_activations_risk",
  "lower_ci_9b_all",
  "lower_ci_9b_risk",
  "lower_ci_9c_activations",
  "lower_ci_9c_activations_risk",
  "lower_ci_9c_all",
  "lower_ci_9c_risk",
  "lower_ci_9d_activations",
  "lower_ci_9d_activations_risk",
  "lower_ci_9d_all",
  "lower_ci_9d_risk",
  "lower_ci_9e_activations",
  "lower_ci_9e_activations_risk",
  "lower_ci_9e_all",
  "lower_ci_9e_risk",
  "lower_ci_9f_activations",
  "lower_ci_9f_activations_risk",
  "lower_ci_9f_all",
  "lower_ci_9f_risk",
  "major_trauma",
  "median",
  "minor_trauma",
  "numerator_10a",
  "numerator_10b",
  "numerator_10c",
  "numerator_11",
  "numerator_12",
  "numerator_13",
  "numerator_1a",
  "numerator_1b",
  "numerator_1c",
  "numerator_1d",
  "numerator_1e",
  "numerator_1f",
  "numerator_2",
  "numerator_3",
  "numerator_4a",
  "numerator_4b",
  "numerator_5a",
  "numerator_5b",
  "numerator_5c",
  "numerator_5d",
  "numerator_6",
  "numerator_7",
  "numerator_8_all",
  "numerator_8_risk",
  "numerator_9a_activations",
  "numerator_9a_activations_risk",
  "numerator_9a_all",
  "numerator_9a_risk",
  "numerator_9b_activations",
  "numerator_9b_activations_risk",
  "numerator_9b_all",
  "numerator_9b_risk",
  "numerator_9c_activations",
  "numerator_9c_activations_risk",
  "numerator_9c_all",
  "numerator_9c_risk",
  "numerator_9d_activations",
  "numerator_9d_activations_risk",
  "numerator_9d_all",
  "numerator_9d_risk",
  "numerator_9e_activations",
  "numerator_9e_activations_risk",
  "numerator_9e_all",
  "numerator_9e_risk",
  "numerator_9f_activations",
  "numerator_9f_activations_risk",
  "numerator_9f_all",
  "numerator_9f_risk",
  "overtriage",
  "p.value",
  "quantile",
  "seqic_1d",
  "seqic_5a",
  "seqic_5b",
  "seqic_5c",
  "seqic_5d",
  "seqic_9a_activations",
  "seqic_9a_activations_risk",
  "seqic_9a_all",
  "seqic_9a_risk",
  "seqic_9b_activations",
  "seqic_9b_activations_risk",
  "seqic_9b_all",
  "seqic_9b_risk",
  "seqic_9c_activations",
  "seqic_9c_activations_risk",
  "seqic_9c_all",
  "seqic_9c_risk",
  "seqic_9d_activations",
  "seqic_9d_activations_risk",
  "seqic_9d_all",
  "seqic_9d_risk",
  "seqic_9e_activations",
  "seqic_9e_activations_risk",
  "seqic_9e_all",
  "seqic_9e_risk",
  "seqic_9f_activations",
  "seqic_9f_activations_risk",
  "seqic_9f_all",
  "seqic_9f_risk",
  "shapiro.test",
  "undertriage",
  "upper_ci",
  "upper_ci_1d",
  "upper_ci_4a",
  "upper_ci_4b",
  "upper_ci_5a",
  "upper_ci_5b",
  "upper_ci_5c",
  "upper_ci_5d",
  "upper_ci_9a_activations",
  "upper_ci_9a_activations_risk",
  "upper_ci_9a_all",
  "upper_ci_9a_risk",
  "upper_ci_9b_activations",
  "upper_ci_9b_activations_risk",
  "upper_ci_9b_all",
  "upper_ci_9b_risk",
  "upper_ci_9c_activations",
  "upper_ci_9c_activations_risk",
  "upper_ci_9c_all",
  "upper_ci_9c_risk",
  "upper_ci_9d_activations",
  "upper_ci_9d_activations_risk",
  "upper_ci_9d_all",
  "upper_ci_9d_risk",
  "upper_ci_9e_activations",
  "upper_ci_9e_activations_risk",
  "upper_ci_9e_all",
  "upper_ci_9e_risk",
  "upper_ci_9f_activations",
  "upper_ci_9f_activations_risk",
  "upper_ci_9f_all",
  "upper_ci_9f_risk",
  "validity_threshold",
  "validity",
  "ed_LOS"
))

#' @title Format Numeric Variables as Percentages
#'
#' @description
#'
#' This function formats numeric variables as percentages with a specified
#' number of decimal places. It refines the output by removing unnecessary
#' trailing zeros after the decimal point and ensures the percentage sign is
#' correctly applied without extraneous characters, resulting in a polished,
#' human-readable percentage representation.
#'
#' @param variable A numeric vector representing proportions to format as
#'   percentages. The values are on a scale from 0 to 1.
#' @param n_decimal A numeric value specifying the number of decimal places.
#'   Defaults to `1`.
#'
#' @returns A character vector containing the formatted percentages.
#'
#' @export
#'
#' @examples
#' # Example usage:
#' pretty_percent(0.12345)  # Default decimal places
#' pretty_percent(0.12345, n_decimal = 2)  # Two decimal places
#' pretty_percent(c(0.1, 0.25, 0.3333), n_decimal = 1)  # Vector input
#'
#' @author Nicolas Foss, Ed.D., MS
#'
pretty_percent <- function(variable, n_decimal = 1) {
  # Save the current options settings to restore them later ----
  old <- options()

  # Ensure that the original options are restored when the function exits, ----
  # even if an error occurs
  on.exit(options(old))

  # Set the 'scipen' option to a high value to prevent scientific notation ----
  # in the output. This ensures that large numbers are displayed in their
  # full numeric form rather than in scientific notation.
  options(scipen = 9999)

  # Ensure the input is numeric ----
  validate_numeric(input = variable, type = "error")

  # Ensure n_decimal is valid ----
  validate_numeric(input = n_decimal, min = 0, type = "error")

  # Format percentages with NA handling ----
  formatted_percent <- ifelse(
    is.na(variable),
    NA_character_,
    paste0(round(variable * 100, digits = n_decimal), "%")
  )

  # Remove trailing zeros after decimal point and the period if unnecessary ----
  formatted_percent <- sub("(\\.\\d*?)0+%$", "\\1%", formatted_percent) # Remove trailing zeros
  formatted_percent <- sub("\\.%$", "%", formatted_percent) # Remove ending "."

  # Complete the transformation ----
  return(formatted_percent)
}

test_that("pretty_percent handles numeric input", {
  expect_equal(pretty_percent(0.12345), "12.3%")
  expect_equal(pretty_percent(0.12345, n_decimal = 2), "12.35%")
  expect_equal(pretty_percent(0.12345, n_decimal = 0), "12%")
  expect_equal(
    pretty_percent(c(0.1, 0.25, 0.3333), n_decimal = 1),
    c("10%", "25%", "33.3%")
  )
})

test_that("pretty_percent handles non-numeric input", {
  expect_error(pretty_percent("text"), "variable.*must be.*numeric")
  expect_error(
    pretty_percent(list(1, 2, 3)),
    "variable.*must be.*numeric"
  )
})

test_that("pretty_percent handles n_decimal input correctly", {
  expect_error(
    pretty_percent(0.5, n_decimal = -1),
    "values must be greater than or equal to 0"
  )
  expect_equal(pretty_percent(0.5, n_decimal = 0), "50%")
})

test_that("pretty_percent removes unnecessary trailing zeros and periods", {
  expect_equal(pretty_percent(0.12345, n_decimal = 3), "12.345%")
  expect_equal(pretty_percent(0.5, n_decimal = 3), "50%")
  expect_equal(pretty_percent(0.33333, n_decimal = 3), "33.333%")
  expect_equal(pretty_percent(0.123, n_decimal = 2), "12.3%")
  expect_equal(pretty_percent(0.01, n_decimal = 1), "1%")
  expect_equal(pretty_percent(1, n_decimal = 0), "100%")
})

test_that("pretty_percent handles edge cases correctly", {
  expect_equal(pretty_percent(0), "0%")
  expect_equal(pretty_percent(1), "100%")
  expect_equal(pretty_percent(0.9999, n_decimal = 2), "99.99%")
  expect_equal(pretty_percent(-0.1), "-10%")
  expect_equal(pretty_percent(-0.12345, n_decimal = 3), "-12.345%")
})

test_that("pretty_percent handles vector input correctly", {
  input <- c(0.1, 0.25, 0.3333, 0.5)
  expected <- c("10%", "25%", "33.3%", "50%")
  expect_equal(pretty_percent(input, n_decimal = 1), expected)
})

test_that("pretty_percent handles NA values gracefully", {
  expect_equal(
    pretty_percent(c(0.1, 0.2, NA, NA)),
    c("10%", "20%", NA_character_, NA_character_)
  )
  expect_equal(
    pretty_percent(c(0.1, NA, 0.5), n_decimal = 1),
    c("10%", NA, "50%")
  )
})

#' @title Label Small Counts Based on a Cutoff
#'
#' @description
#'
#' This function labels values in a vector as a replacement string if they are
#' smaller than a specified cutoff. The input can be numeric, and the function
#' will return either a modified version of the input vector with small values
#' replaced by a given label, or it will keep the original values otherwise.
#'
#' @param var A numeric vector. This represents the variable to be checked
#'   against the cutoff.
#' @param cutoff A numeric value representing the threshold. Values in `var`
#'   smaller than this value will be replaced.
#' @param replacement A string or a numeric value. If the value in `var` is
#'   smaller than the `cutoff`, this value will replace it. If a string is
#'   provided, it will replace the numeric values with the string. If a numeric
#'   value is provided, the replacement will also be numeric.
#'
#' @returns A vector with values from `var`. Values smaller than the `cutoff`
#'   will be replaced by the `replacement`. If `replacement` is a string, the
#'   return type will be character, otherwise, it will remain numeric.
#'
#' @export
#'
#' @examples
#' # Example usage of the small_count_label function
#' small_count_label(c(1, 5, 10), 5, "Below Cutoff")
#' small_count_label(c(1, 5, 10), 5, 0)
#'
#' @author Nicolas Foss, Ed.D., MS
#'
small_count_label <- function(var, cutoff, replacement) {
  # Check if var is numeric ----
  validate_numeric(input = var, type = "error")

  # Check if cutoff is numeric ----
  validate_numeric(input = cutoff, type = "error")

  # Ensure cutoff has length >= 1 ----
  validate_length(input = cutoff, exact_length = 1, type = "error")

  # Check if replacement is of a valid type ----
  validate_class(
    input = replacement,
    class_type = c("numeric", "character", "logical"),
    logic = "or",
    type = "error",
    na_ok = TRUE
  )

  # Perform the replacement based on the cutoff ----
  # Check if all values in 'var' are greater than or equal to the `cutoff`
  if (all(var >= cutoff, na.rm = TRUE)) {
    # If true, keep the original `var` as `output`
    output <- var
  } else {
    # Otherwise, replace values in `var` that are less than `cutoff` with
    # `replacement`
    output <- ifelse(var < cutoff, replacement, var)
  }

  # End function ----
  return(output)
}

test_that("small_count_label returns correct output for numeric replacement", {
  expect_equal(small_count_label(c(1, 5, 10), 5, 0), c(0, 5, 10))
  expect_equal(small_count_label(c(1, 4, 10), 5, 0), c(0, 0, 10))
  expect_equal(small_count_label(c(1, 2, 3), 3, 0), c(0, 0, 3))
  expect_equal(small_count_label(c(1, 5, 10), 10, 0), c(0, 0, 10))
})

test_that("small_count_label returns correct output for character replacement", {
  expect_equal(
    small_count_label(c(1, 5, 10), 5, "Below Cutoff"),
    c("Below Cutoff", "5", "10")
  )
  expect_equal(
    small_count_label(c(1, 4, 10), 5, "Below Cutoff"),
    c("Below Cutoff", "Below Cutoff", "10")
  )
  expect_equal(
    small_count_label(c(1, 2, 3), 3, "Below Cutoff"),
    c("Below Cutoff", "Below Cutoff", "3")
  )
  expect_equal(
    small_count_label(c(1, 5, 10), 10, "Below Cutoff"),
    c("Below Cutoff", "Below Cutoff", "10")
  )
})

test_that("small_count_label handles invalid var input", {
  expect_error(
    small_count_label("text", 5, "Below Cutoff"),
    regexp = "var.*must be.*numeric"
  )
  expect_error(
    small_count_label(list(1, 2, 3), 5, "Below Cutoff"),
    regexp = "var.*must be.*numeric"
  )
})

test_that("small_count_label handles invalid cutoff input", {
  expect_error(
    small_count_label(c(1, 2, 3), "five", "Below Cutoff"),
    regexp = "cutoff.*must be.*numeric"
  )
  expect_error(
    small_count_label(c(1, 2, 3), c(5, 6), "Below Cutoff"),
    regexp = "cutoff.*must have an exact length of 1"
  )
})

test_that("small_count_label handles invalid replacement input", {
  expect_error(
    small_count_label(c(1, 2, 3), 5, list("Below Cutoff")),
    regexp = "replacement.*must be of class.*numeric, character, logical"
  )
  expect_error(
    small_count_label(c(1, 2, 3), 5, data.frame(x = "Below Cutoff")),
    regexp = "replacement.*must be of class.*numeric, character, logical"
  )
})

test_that("small_count_label handles edge cases", {
  expect_equal(small_count_label(c(5, 5, 5), 5, "At Cutoff"), c(5, 5, 5)) # Values equal to cutoff remain unchanged
  expect_equal(
    small_count_label(c(0, -1, 5), 0, "Below Zero"),
    c("0", "Below Zero", "5")
  ) # Edge case for negative and zero values
})

#' @title Create Nonlinear Probability of Survival Bins
#'
#' @description
#'
#' This function generates nonlinear bins for probability of survival data based
#' on specified thresholds and divisors as specified in Napoli et al. (2017),
#' Schroeder et al. (2019), and Kassar et al. (2016). This function calculates
#' bin statistics, including mean, standard deviation, total alive, total dead,
#' count, and percentage for each bin.
#'
#' @param data A `data.frame` or `tibble` containing the probability of survival
#'   data for a set of patients.
#' @param Ps_col The name of the column containing the survival probabilities
#'   (Ps). Should be numeric on a scale from 0 to 1.
#' @param outcome_col The name of the column containing the outcome data. It
#'   should be binary, with values indicating patient survival. A value of `1`
#'   should represent "alive" (survived), while `0` should represent "dead" (did
#'   not survive).  `TRUE/FALSE` are accepted as well. Ensure the column
#'   contains only these possible values.
#' @param group_vars Optional grouping variables for bin statistics
#'   calculations. These should be specified as quoted column names.
#' @param divisor1 A positive numeric value controlling the coarseness of bins
#'   for Ps values below `threshold_1`. It scales the number of steps from the
#'   start of the dataset up to the `threshold_1` cut point. Larger values
#'   produce fewer, broader bins; smaller values produce more, narrower bins.
#'   Defaults to `5`.
#' @param divisor2 A positive numeric value controlling the coarseness of bins
#'   for Ps values between `threshold_1` and `threshold_2`. Larger values yield
#'   wider bins, and smaller values yield narrower bins in this range. Defaults
#'   to `5`.
#' @param threshold_1 A numeric value that defines the lower bound of the
#'   high-survival probability range in `Ps_col`. The function identifies the
#'   first index where `Ps_col` exceeds this value and begins applying smaller
#'   bin widths from that point onward. Defaults to `0.9`, meaning binning
#'   changes once Ps > 0.90.
#' @param threshold_2 A numeric value that defines the upper bound of the
#'   high-survival probability range in `Ps_col`. The function identifies the
#'   first index where `Ps_col` exceeds this value. Between `threshold_1` and
#'   `threshold_2`, finer binning is applied; above `threshold_2`, binning may
#'   again change. Defaults to `0.99`, meaning the special binning range is
#'   between Ps values of 0.90 and 0.99.
#'
#' @details
#' Like other statistical computing functions, `nonlinear_bins()` is happiest
#' without missing data.  It is best to pass complete probability of survival
#' and outcome data to the function for optimal performance. With smaller
#' datasets, this is especially helpful.  However, `nonlinear_bins()` will throw
#' a warning about missing values, if any exist in `Ps_col` and/or
#' `outcome_col`.
#'
#' `nonlinear_bins()` assumes `Ps_col` contains probabilities derived from
#' real-world inputs for the Trauma Injury Severity Score (TRISS) model.
#' Synthetic or low-variability data (especially with small sample sizes) may
#' not reflect the distribution of TRISS-derived survival probabilities. This
#' can result in unstable estimates or function failure due to insufficient
#' dispersion. With small sample sizes, it may be important to use smaller
#' values with the divisor arguments and adjust the thresholds (based on the
#' distribution of the `Ps_col` values) to create bins that better accommodate
#' the data.
#'
#' By default, `nonlinear_bins()` derives bin cut points from the full dataset’s
#' distribution. This ensures comparability across groups when `group_vars` is
#' used. To tailor binning to a specific group (e.g., a single hospital), filter
#' the dataset to that subgroup before calling `nonlinear_bins()`. The function
#' will then compute bins and related statistics using only that subset’s
#' `Ps_col` distribution. When `group_vars` is used, and ff a group lacks
#' observations within one or more bins, `rm_bin_summary()` will compute
#' statistics only for the bins that contain data. Bins with no observations are
#' excluded from the summary for that group.
#'
#' The `threshold_1` and `threshold_2` arguments set probability cut points
#' that define the start and end of a high-survival range where bin widths are
#' adjusted for finer resolution. The `divisor1` and `divisor2` arguments are
#' scaling factors that determine how many bins are created before and within
#' this high-survival range, respectively. Lower divisors yield narrower bins,
#' capturing more detail, while higher divisors yield broader bins, smoothing
#' the distribution.
#'
#' @returns A list with two elements:
#'   - `intervals`: A vector defining bin boundaries for probability of
#'        survival.
#'   - `bin_stats`: A `tibble` containing:
#'     - `bin_number`: Bin index.
#'     - `bin_start`, `bin_end`: Bin range.
#'     - `mean`, `sd`: Mean and standard deviation of `Ps_col` within the bin.
#'     - `Pred_Survivors_b`, `Pred_Deaths_b`: Predicted counts of survivors and
#'        decedents, respectively.
#'     - `AntiS_b`, `AntiM_b`: Anticipated proportion survived, and deceased,
#'        respectively.
#'     - `alive`, `dead`: Count of observed survivors and non-survivors.
#'     - `count`: Total records in the bin.
#'     - `percent`: Percentage of total records within each bin.
#'
#' @note
#'
#' This function will produce the most reliable and interpretable results when
#' using a dataset that has one row per patient, with each column being a
#' feature.
#'
#' The `mean` and `AntiS_b` are approximately equivalent in this context.  They
#' are kept in the output for clarity.
#'
#' @export
#'
#' @references
#'
#' Kassar, O.M., Eklund, E.A., Barnhardt, W.F., Napoli, N.J., Barnes, L.E.,
#' Young, J.S. (2016). Trauma survival margin analysis: A dissection of trauma
#' center performance through initial lactate. The American Surgeon, 82(7),
#' 649-653. <doi:10.1177/000313481608200733>
#'
#' Napoli, N. J., Barnhardt, W., Kotoriy, M. E., Young, J. S., & Barnes, L. E.
#' (2017). Relative mortality analysis: A new tool to evaluate clinical
#' performance in trauma centers. IISE Transactions on Healthcare Systems
#' Engineering, 7(3), 181–191. <doi:10.1080/24725579.2017.1325948>
#'
#' Schroeder, P. H., Napoli, N. J., Barnhardt, W. F., Barnes, L. E., & Young, J.
#' S. (2018). Relative mortality analysis of the “golden hour”: A comprehensive
#' acuity stratification approach to address disagreement in current literature.
#' Prehospital Emergency Care, 23(2), 254–262.
#' <doi:10.1080/10903127.2018.1489021>
#'
#' @seealso [probability_of_survival()], [rmm()], and [rm_bin_summary()]
#'
#' @examples
#' # Generate example data
#' set.seed(123)
#'
#' # Parameters
#' # Total number of patients
#' n_patients <- 5000
#'
#' # Arbitrary group labels
#' groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)
#'
#' # Trauma types
#' trauma_type_values <- sample(
#'   x = c("Blunt", "Penetrating"),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # RTS values
#' rts_values <- sample(
#'   x = seq(from = 0, to = 7.8408, by = 0.005),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # patient ages
#' ages <- sample(
#'   x = seq(from = 0, to = 100, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # ISS scores
#' iss_scores <- sample(
#'   x = seq(from = 0, to = 75, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # Generate survival probabilities (Ps)
#' Ps <- traumar::probability_of_survival(
#'   trauma_type = trauma_type_values,
#'   age = ages,
#'   rts = rts_values,
#'   iss = iss_scores
#' )
#'
#' # Simulate survival outcomes based on Ps
#' survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)
#'
#' # Create data frame
#' data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Apply the nonlinear_bins function
#' results <- nonlinear_bins(
#'   data = data,
#'   Ps_col = Ps,
#'   outcome_col = survival,
#'   divisor1 = 4,
#'   divisor2 = 4,
#'   threshold_1 = 0.9,
#'   threshold_2 = 0.99
#' )
#'
#' # View results
#' results$intervals
#' results$bin_stats
#'
#' # Example with grouping by a categorical variable
#'
#' # Run the function using a single grouping variable
#' results_grouped <- nonlinear_bins(
#'   data,
#'   Ps_col = Ps,
#'   outcome_col = survival,
#'   group_vars = "groups"
#' )
#'
#' # View grouped results
#' results_grouped$bin_stats
#'
#' @author Nicolas Foss, Ed.D, MS, original implementation in MATLAB by Nicholas
#'   J. Napoli, Ph.D., MS
#'
nonlinear_bins <- function(
  data,
  Ps_col,
  outcome_col,
  group_vars = NULL,
  divisor1 = 5,
  divisor2 = 5,
  threshold_1 = 0.9,
  threshold_2 = 0.99
) {
  # Validation checks using `cli` for robust error messaging: ----
  # Ensures the input data is a data frame or tibble.
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error"
  )

  # Ensure Ps_col and outcome_col arguments are provided ----
  # with tailored error messages
  if (missing(Ps_col) && missing(outcome_col)) {
    cli::cli_abort(
      "Both {.var Ps_col} and {.var outcome_col} arguments must be provided."
    )
  } else if (missing(Ps_col)) {
    cli::cli_abort("The {.var Ps_col} argument must be provided.")
  } else if (missing(outcome_col)) {
    cli::cli_abort("The {.var outcome_col} argument must be provided.")
  }

  # dplyr::pull the Ps data ----
  Ps_check <- validate_data_pull(
    input = data,
    col = {{ Ps_col }},
    var_name = "Ps_col",
    calls = 5
  )

  # check the Ps_check remains continuous ----
  # Check if Ps column is continuous (values between 0 and 1)
  validate_numeric(
    input = Ps_check,
    min = 0,
    max = 1,
    type = "error",
    var_name = "Ps_col"
  )

  # Check for missingness in the probability of survival data ----
  validate_complete(input = Ps_check, type = "warning", var_name = "Ps_col")

  # Pull and check the outcome column ----
  binary_data <- validate_data_pull(
    input = data,
    col = {{ outcome_col }},
    var_name = "outcome_col",
    calls = 5
  )

  # Ensure the column is either logical or numeric ----
  validate_class(
    input = binary_data,
    class_type = c("logical", "numeric", "integer"),
    logic = "or",
    type = "error",
    var_name = "outcome_col"
  )

  # Get unique non-missing values ----
  non_missing <- stats::na.omit(binary_data)

  # Provide a warning about missing values ----
  validate_complete(
    input = binary_data,
    type = "warning",
    var_name = "outcome_col"
  )

  # Validate type and values ----
  if (is.logical(binary_data)) {
    validate_set(
      input = non_missing,
      valid_set = c(TRUE, FALSE),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.numeric(binary_data)) {
    # Numeric vector: ensure strictly 0 or 1
    validate_set(
      input = non_missing,
      valid_set = c(0, 1),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.integer(binary_data)) {
    # Integer vector: ensure strictly 0 or 1
    validate_set(
      input = non_missing,
      valid_set = c(0L, 1L),
      type = "error",
      var_name = "outcome_col"
    )
  }

  # Check if all elements in group_vars are strings ----
  # (i.e., character vectors)
  if (!is.null(group_vars)) {
    validate_character_factor(input = group_vars, type = "error")
  }

  # Check if all group_vars exist in the data ----
  validate_names(
    input = data,
    check_names = group_vars,
    type = "error",
    var_name = "group_vars",
    null_ok = TRUE
  )

  # Treat the column-names-as-strings as symbols ----
  if (!is.null(group_vars)) {
    group_vars <- rlang::syms(group_vars)
  }

  # Select and sort the Ps_col column ----
  # Assumes that the user has dealt with missing values, if they exist
  survival_data <- Ps_check |> sort()

  # length of non-missing `Ps_col` must be >= 2 ----
  # It is possible for this check to succeed if missings exist
  validate_length(
    input = survival_data,
    min_length = 2,
    max_length = Inf,
    type = "error",
    na_ok = TRUE,
    null_ok = TRUE,
    var_name = "Ps_col"
  )

  # Step 1: Find indices for level thresholds ----
  loc_9A <- which(survival_data > threshold_1) # Everything above 0.9 or other threshold
  loc_9B <- which(survival_data > threshold_2) # Everything above 0.99 or other threshold
  loc_9C <- which(survival_data > threshold_1 & survival_data <= threshold_2) # Between 0.9 and 0.99 or other thresholds

  # Step 2: Define step sizes based on the data ----
  step1 <- round(suppressWarnings(min(loc_9A, na.rm = TRUE)) / divisor1)
  step2 <- round(length(loc_9C) / divisor2)

  # Step 3: Define intervals ----
  # Check that loc_9A and loc_9B are finite before using them in seq()
  if (
    suppressWarnings(
      is.finite(min(loc_9A)) && is.finite(min(loc_9B))
    )
  ) {
    # Defensive approach to control for potential error when `Ps_col`
    # is not sufficiently dispersed and/or has a low sample size.
    attempt <- try(
      unique(
        c(
          seq(
            from = 1,
            to = suppressWarnings(min(loc_9A, na.rm = TRUE)),
            by = step1
          ),
          seq(
            from = suppressWarnings(min(loc_9A, na.rm = TRUE)),
            to = suppressWarnings(min(loc_9B, na.rm = TRUE)),
            by = step2
          ),
          suppressWarnings(max(loc_9B, na.rm = TRUE))
        )
      ),
      silent = TRUE
    )

    if (inherits(attempt, "try-error")) {
      cli::cli_abort(c(
        "Unable to calculate valid step sizes for defining survival probability intervals due to non-finite values.",
        "i" = "This typically occurs when {.var Ps_col} has low variability or the sample size is too small to support the requested binning.",
        "v" = "Try reducing {.var divisor1} and {.var divisor2} to create fewer, broader bins.",
        "v" = "Examine the distribution of {.var Ps_col} and consider adjusting {.var threshold_1} and {.var threshold_2} to better match the data.",
        "v" = "Increasing the sample size may also improve dispersion and allow for more stable interval boundaries."
      ))
    } else {
      # If `Ps_col` meets distributional assumptions, proceed
      len <- attempt
    }
  } else {
    # If either of the parts of the interval calculation are not finite throw an
    # error
    cli::cli_abort(
      c(
        "Unable to calculate valid step sizes for defining survival probability intervals due to non-finite values.",
        "i" = "This typically occurs when {.var Ps_col} has low variability or the sample size is too small to support the requested binning.",
        "v" = "Try reducing {.var divisor1} and {.var divisor2} to create fewer, broader bins.",
        "v" = "Examine the distribution of {.var Ps_col} and consider adjusting {.var threshold_1} and {.var threshold_2} to better match the data.",
        "v" = "Increasing the sample size may also improve dispersion and allow for more stable interval boundaries."
      )
    )
  }

  # Generate intervals based on these positions ----
  intervals <- unique(survival_data[len])

  # Step 4: Bin statistics ----

  # Apply binning to each group separately ----
  data <- data |>
    dplyr::mutate(
      bin_number = .bincode(
        {{ Ps_col }},
        breaks = intervals,
        include.lowest = TRUE
      ),
      bin_start = intervals[bin_number], # Start of the bin
      bin_end = intervals[bin_number + 1] # End of the bin
    )

  # Optionally group data by dynamic group_vars ----
  # Or run the bin statistics on the whole dataset
  if (!is.null(group_vars)) {
    # Group the data by the specified group_vars and bin-related columns
    grouped_stats <- data |>
      dplyr::group_by(!!!group_vars, bin_number, bin_start, bin_end) |>
      dplyr::summarize(
        # Calculate the mean of the probability of survival (Ps_col)
        mean = mean({{ Ps_col }}, na.rm = TRUE),
        # Calculate the standard deviation of the probability of survival (Ps_col)
        sd = stats::sd({{ Ps_col }}, na.rm = TRUE),
        # Sum the predicted survivors based on the probability of survival (Ps_col)
        Pred_Survivors_b = sum({{ Ps_col }}, na.rm = TRUE),
        # Sum the predicted deaths based on the probability of survival (Ps_col)
        Pred_Deaths_b = sum(1 - {{ Ps_col }}, na.rm = TRUE),
        # Calculate the anticipated survival rate
        AntiS_b = dplyr::if_else(
          dplyr::n() > 0,
          Pred_Survivors_b / dplyr::n(),
          NA_real_
        ),
        # Calculate the anticipated mortality rate
        AntiM_b = dplyr::if_else(
          dplyr::n() > 0,
          Pred_Deaths_b / dplyr::n(),
          NA_real_
        ),
        # Count the number of alive patients
        alive = sum({{ outcome_col }} == 1, na.rm = TRUE),
        # Count the number of dead patients
        dead = sum({{ outcome_col }} == 0, na.rm = TRUE),
        # Count the total number of observations
        count = dplyr::n(),
        .groups = "drop"
      ) |>
      dplyr::ungroup() |>
      # Group the data again by the specified group_vars
      dplyr::group_by(!!!group_vars) |>
      dplyr::mutate(
        # Calculate the percentage of each group
        percent = count / sum(count, na.rm = TRUE)
      ) |>
      dplyr::ungroup()
  } else {
    # Group the data by bin-related columns only
    grouped_stats <- data |>
      dplyr::group_by(bin_number, bin_start, bin_end) |>
      dplyr::summarize(
        # Calculate the mean of the probability of survival (Ps_col)
        mean = mean({{ Ps_col }}, na.rm = TRUE),
        # Calculate the standard deviation of the probability of survival (Ps_col)
        sd = stats::sd({{ Ps_col }}, na.rm = TRUE),
        # Sum the predicted survivors based on the probability of survival (Ps_col)
        Pred_Survivors_b = sum({{ Ps_col }}, na.rm = TRUE),
        # Sum the predicted deaths based on the probability of survival (Ps_col)
        Pred_Deaths_b = sum(1 - {{ Ps_col }}, na.rm = TRUE),
        # Calculate the anticipated survival; rate
        AntiS_b = dplyr::if_else(
          dplyr::n() > 0,
          Pred_Survivors_b / dplyr::n(),
          NA_real_
        ),
        # Calculate the anticipated mortality rate
        AntiM_b = dplyr::if_else(
          dplyr::n() > 0,
          Pred_Deaths_b / dplyr::n(),
          NA_real_
        ),
        # Count the number of alive patients
        alive = sum({{ outcome_col }} == 1, na.rm = TRUE),
        # Count the number of dead patients
        dead = sum({{ outcome_col }} == 0, na.rm = TRUE),
        # Count the total number of observations
        count = dplyr::n(),
        .groups = "drop"
      ) |>
      dplyr::mutate(
        # Calculate the percentage of each group
        percent = count / sum(count, na.rm = TRUE)
      )
  }

  # Return a list with intervals and the bin statistics ----
  return(list(intervals = intervals, bin_stats = grouped_stats))
}

# Tests for nonlinear_bins function

testthat::test_that("nonlinear_bins handles missing values and invalid data types", {
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Inject missing values into Ps
  data_na_Ps <- data
  data_na_Ps$Ps[1:5] <- NA
  testthat::expect_warning(
    nonlinear_bins(data = data_na_Ps, Ps_col = Ps, outcome_col = survival),
    regexp = "missing values detected"
  )

  # Inject missing values into outcome_col
  data_na_surv <- data
  data_na_surv$survival[1:5] <- NA
  testthat::expect_warning(
    nonlinear_bins(data = data_na_surv, Ps_col = Ps, outcome_col = survival),
    regexp = "missing values detected"
  )

  # Invalid logical values
  data_logical_invalid <- data |>
    dplyr::mutate(survival = as.logical(survival)) |>
    dplyr::mutate(survival = replace(survival, 1:3, NA)) # Valid NAs only
  testthat::expect_warning(
    nonlinear_bins(
      data = data_logical_invalid,
      Ps_col = Ps,
      outcome_col = survival
    ),
    regexp = "missing values detected"
  )
})

testthat::test_that("nonlinear_bins handles error messaging appropriately", {
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Missing Ps_col and outcome_col arguments
  testthat::expect_error(
    nonlinear_bins(data = data),
    regexp = "Both.*Ps_col.*outcome_col.*must be provided"
  )

  # Missing Ps_col only
  testthat::expect_error(
    nonlinear_bins(data = data, outcome_col = survival),
    regexp = "Ps_col.*must be provided"
  )

  # Missing outcome_col only
  testthat::expect_error(
    nonlinear_bins(data = data, Ps_col = Ps),
    regexp = "outcome_col.*must be provided"
  )

  # Set up the survival variable so that it is of class character
  bad_data <- data |> dplyr::mutate(survival = ifelse(survival == 1, "a", "b"))

  # Test a character survival column
  testthat::expect_error(
    nonlinear_bins(data = bad_data, Ps_col = Ps, outcome_col = survival),
    regexp = "must be of class.*logical, numeric, integer.*"
  )

  # Test the case when the data are not sufficiently dispersed and cause errors
  # usually when the data do not come from the same distribution as the
  # probability of survival metric
  bad_data <- data.frame(
    Ps = rbinom(10, 1, prob = 0.9),
    survival = rbinom(10, 1, 0.9)
  )

  testthat::expect_error(
    nonlinear_bins(
      data = bad_data,
      Ps_col = Ps,
      outcome_col = survival,
      divisor1 = 4,
      divisor2 = 4
    ),
    regexp = "Unable to calculate valid step sizes"
  )
})

testthat::test_that("nonlinear_bins handles basic functionality correctly", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Run the function
  result <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 5,
    divisor2 = 5,
    threshold_1 = 0.9,
    threshold_2 = 0.99
  )

  # Check intervals
  testthat::expect_type(result$intervals, "double")
  testthat::expect_true(length(result$intervals) > 1)

  # Check bin_stats structure
  testthat::expect_s3_class(result$bin_stats, "tbl_df")
  testthat::expect_named(
    result$bin_stats,
    c(
      "bin_number",
      "bin_start",
      "bin_end",
      "mean",
      "sd",
      "Pred_Survivors_b",
      "Pred_Deaths_b",
      "AntiS_b",
      "AntiM_b",
      "alive",
      "dead",
      "count",
      "percent"
    )
  )
})

testthat::test_that("nonlinear_bins produces reasonable bins for uniform data", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Run the function
  result <- nonlinear_bins(data, Ps_col = Ps, outcome_col = survival)

  # Check intervals cover correct range
  testthat::expect_true(min(result$intervals) > 0)
  testthat::expect_true(max(result$intervals) < 1)

  # Check bin stats are populated
  testthat::expect_true(all(result$bin_stats$count > 0))
})

testthat::test_that("nonlinear_bins respects divisor and threshold parameters", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Run with different divisors and thresholds
  result1 <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 10,
    divisor2 = 10
  )
  result2 <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 2,
    divisor2 = 2
  )

  # Check intervals differ based on divisors
  testthat::expect_true(length(result1$intervals) > length(result2$intervals))

  # Check thresholds affect intervals
  result3 <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    threshold_1 = 0.8,
    threshold_2 = 0.95
  )
  testthat::expect_true(length(result3$intervals) < length(result1$intervals))
})

testthat::test_that("nonlinear_bins handles invalid inputs gracefully", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Missing column
  testthat::expect_error(nonlinear_bins(
    data,
    Ps_col = NotAColumn,
    outcome_col = survival
  ))

  # Non-numeric column, warning will happen
  data <- data.frame(Ps = c("d", "e", "f"), survival = c("a", "b", "c"))
  testthat::expect_error(nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival
  ))

  # Check a logical outcome_col is binary
})

testthat::test_that("nonlinear_bins produces accurate bin statistics", {
  # Generate example data
  set.seed(123)

  # Parameters
  # Total number of patients
  n_patients <- 1000

  # Arbitrary group labels
  groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

  # Trauma types
  trauma_type_values <- sample(
    x = c("Blunt", "Penetrating"),
    size = n_patients,
    replace = TRUE
  )

  # RTS values
  rts_values <- sample(
    x = seq(from = 0, to = 7.8408, by = 0.005),
    size = n_patients,
    replace = TRUE
  )

  # patient ages
  ages <- sample(
    x = seq(from = 0, to = 100, by = 1),
    size = n_patients,
    replace = TRUE
  )

  # ISS scores
  iss_scores <- sample(
    x = seq(from = 0, to = 75, by = 1),
    size = n_patients,
    replace = TRUE
  )

  # Generate survival probabilities (Ps)
  Ps <- traumar::probability_of_survival(
    trauma_type = trauma_type_values,
    age = ages,
    rts = rts_values,
    iss = iss_scores
  )

  # Simulate survival outcomes based on Ps
  survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

  # Create data frame
  data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
    dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

  # Run the function
  result <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 3,
    divisor2 = 3
  )

  # Validate statistics by comparing the summarized values
  result_summary <- result$bin_stats

  # Select and sort the column
  survival_data <- data |> dplyr::pull(Ps) |> sort()
  total <- length(survival_data)

  # Step 1: Find indices for level thresholds
  loc_9A <- which(survival_data > 0.9) # Everything above 0.9 or other threshold
  loc_9B <- which(survival_data > 0.99) # Everything above 0.99 or other threshold
  loc_9C <- which(survival_data > 0.9 & survival_data <= 0.99) # Between 0.9 and 0.99 or other thresholds

  # Step 2: Define step sizes based on the data
  step1 <- round(min(loc_9A, na.rm = TRUE) / 3)
  step2 <- round(length(loc_9C) / 3)

  # Step 3: Define intervals
  # Check that loc_9A and loc_9B are finite before using them in seq()
  len <- unique(c(
    seq(1, min(loc_9A, na.rm = TRUE), by = step1), # From start to level_1
    seq(min(loc_9A, na.rm = TRUE), min(loc_9B, na.rm = TRUE), by = step2), # From level_1 to level_2
    max(loc_9B, na.rm = TRUE) # Up to max
  ))

  # Generate intervals based on these positions
  intervals <- unique(survival_data[len])

  # Create the summarized objects
  # Apply binning to each group separately
  data <- data |>
    dplyr::mutate(
      bin_number = .bincode(Ps, breaks = intervals, include.lowest = TRUE),
      bin_start = intervals[bin_number], # Start of the bin
      bin_end = intervals[bin_number + 1] # End of the bin
    )

  # Optionally group data by dynamic group_vars
  # Or run the bin statistics on the whole dataset
  grouped_stats <- data |>
    dplyr::group_by(bin_number, bin_start, bin_end) |>
    dplyr::summarize(
      mean = mean(Ps, na.rm = TRUE),
      sd = stats::sd(Ps, na.rm = TRUE),
      Pred_Survivors_b = sum(Ps, na.rm = TRUE),
      Pred_Deaths_b = sum(1 - Ps, na.rm = TRUE),
      AntiS_b = dplyr::if_else(
        dplyr::n() > 0,
        Pred_Survivors_b / dplyr::n(),
        NA_real_
      ),
      AntiM_b = dplyr::if_else(
        dplyr::n() > 0,
        Pred_Deaths_b / dplyr::n(),
        NA_real_
      ),
      alive = sum(survival == 1, na.rm = TRUE),
      dead = sum(survival == 0, na.rm = TRUE),
      count = dplyr::n(),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      percent = count / sum(count, na.rm = TRUE),
    ) |>
    dplyr::ungroup()

  # Validate the statistics
  testthat::expect_equal(grouped_stats, result_summary)
})

testthat::test_that("nonlinear_bins handles group_vars argument correctly", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  group_var <- sample(c("A", "B", "C"), 1000, replace = TRUE)
  data <- data.frame(Ps = Ps, survival = survival, group_var = group_var)

  # Run the function with group_vars
  result <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 5,
    divisor2 = 5,
    group_vars = "group_var"
  )

  # Ensure the result is grouped correctly by the specified variable
  testthat::expect_true(all(
    levels(factor(result$bin_stats$group_var)) %in% unique(group_var)
  ))
  testthat::expect_equal(
    length(unique(result$bin_stats$group_var)),
    length(unique(group_var))
  )
})

testthat::test_that("nonlinear_bins handles empty group_vars correctly", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival)

  # Run the function without group_vars
  result <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 5,
    divisor2 = 5
  )

  # Ensure no grouping is performed
  testthat::expect_false("group_var" %in% names(result$bin_stats))
})

testthat::test_that("nonlinear_bins handles multiple group_vars correctly", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  group_var1 <- sample(c("A", "B", "C"), 1000, replace = TRUE)
  group_var2 <- sample(1:3, 1000, replace = TRUE)
  data <- data.frame(
    Ps = Ps,
    survival = survival,
    group_var1 = group_var1,
    group_var2 = group_var2
  )

  # Run the function with multiple group_vars
  result <- rm_bin_summary(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    Divisor1 = 3,
    Divisor2 = 3,
    group_vars = c("group_var1", "group_var2")
  )

  # Ensure the result is grouped correctly by the specified variables
  testthat::expect_equal(
    length(unique(result$group_var1)),
    length(unique(group_var1))
  )
  testthat::expect_equal(
    length(unique(result$group_var2)),
    length(unique(group_var2))
  )
})

testthat::test_that("nonlinear_bins handles group_vars with no variability", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  group_var <- rep("A", 1000) # No variability in group_var
  data <- data.frame(Ps = Ps, survival = survival, group_var = group_var)

  # Run the function with group_vars
  result <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 5,
    divisor2 = 5,
    group_vars = "group_var"
  )

  # Check that result is not grouped by a constant variable
  testthat::expect_equal(length(unique(result$bin_stats$group_var)), 1)
})

testthat::test_that("nonlinear_bins correctly handles group_vars when there are few groups", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  group_var <- sample(c("A", "B"), 1000, replace = TRUE)
  data <- data.frame(Ps = Ps, survival = survival, group_var = group_var)

  # Run the function with a few group variables
  result <- nonlinear_bins(
    data,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 5,
    divisor2 = 5,
    group_vars = "group_var"
  )

  # Ensure that the bin statistics are separated by group_var
  testthat::expect_true(length(unique(result$bin_stats$group_var)) == 2)
})

testthat::test_that("nonlinear_bins performs all validation checks correctly", {
  # Generate example data
  set.seed(123)
  Ps <- plogis(rnorm(1000, mean = 2, sd = 1.5))
  survival <- rbinom(1000, 1, prob = 0.9)
  group_var1 <- sample(c("A", "B", "C"), 1000, replace = TRUE)
  group_var2 <- sample(1:3, 1000, replace = TRUE)
  data <- data.frame(
    Ps = Ps,
    survival = survival,
    group_var1 = group_var1,
    group_var2 = group_var2
  )

  # Test for non-data.frame input
  testthat::expect_error(
    nonlinear_bins(matrix(1:10, ncol = 2), Ps_col = Ps, outcome_col = survival),
    "data.*must be of class.*data\\.frame, tbl, tbl_df.*"
  )

  # Test for missing Ps_col and outcome_col
  testthat::expect_error(
    nonlinear_bins(data, Ps_col = NULL, outcome_col = NULL)
  )

  testthat::expect_error(
    nonlinear_bins(data, Ps_col = NULL, outcome_col = survival)
  )

  testthat::expect_error(
    nonlinear_bins(data, Ps_col = Ps, outcome_col = NULL)
  )

  # Test for non-numeric Ps_col
  testthat::expect_error(
    nonlinear_bins(data, Ps_col = "group_var1", outcome_col = survival)
  )

  # Test for Ps values outside the [0, 1] range
  data_invalid_ps <- data
  data_invalid_ps$Ps <- c(-0.1, 1.1)
  testthat::expect_error(
    nonlinear_bins(data_invalid_ps, Ps_col = Ps, outcome_col = survival)
  )

  # Test for non-binary outcome_col
  data_invalid_outcome <- data
  data_invalid_outcome$survival <- sample(1:3, 1000, replace = TRUE)
  testthat::expect_error(
    nonlinear_bins(data_invalid_outcome, Ps_col = Ps, outcome_col = survival),
    "outcome_col.*contains invalid values.*3, 2.*Valid values are.*0, 1.*"
  )

  # Test for non-character group_vars
  testthat::expect_error(
    nonlinear_bins(
      data,
      Ps_col = Ps,
      outcome_col = survival,
      group_vars = list(1, 2)
    ),
    "group_vars.*must be of class.*character.*or.*factor"
  )

  # Test for non-existent group_vars
  testthat::expect_error(
    nonlinear_bins(
      data,
      Ps_col = Ps,
      outcome_col = survival,
      group_vars = c("group_var1", "non_existent_var")
    ),
    "group_vars.*contains invalid column names.*non_existent_var.*Valid column names are.*Ps, survival, group_var1, group_var2.*"
  )
})

#' @title Assign Significance Codes Based on P-Values
#'
#' @description
#'
#' This function assigns significance codes to a p-value vector based on commonly
#' accepted significance thresholds. The significance codes are:
#' - `"***"` for p-values <= 0.001
#' - `"**"` for p-values <= 0.01 and > 0.001
#' - `"*"` for p-values <= 0.05 and > 0.01
#' - `"."` for p-values <= 0.1 and > 0.05
#' - `"<>"` for p-values > 0.1
#'
#' @param p_val_data A numeric vector representing the p-values to be
#'   categorized. The vector should contain p-values between 0 and 1.
#'
#' @returns A character vector with the assigned significance codes for each
#'   p-value.
#'
#' @export
#'
#' @examples
#' # Example usage of the stat_sig function
#' data <- data.frame(p_value = c(0.001, 0.03, 0.12, 0.05, 0.07))
#'
#' data |>
#'   dplyr::mutate(significance = stat_sig(p_val_data = p_value))
#'
#' @author Nicolas Foss, Ed.D., MS
#'
stat_sig <- function(p_val_data) {
  # Check if the input is numeric ----
  validate_numeric(
    input = p_val_data,
    min = 0,
    max = 1,
    type = "error",
    null_ok = FALSE
  )

  # Assign significance codes based on p-value thresholds ----
  significance_values <- ifelse(
    # Assign NA if p_val_data is NA or NULL
    is.na(p_val_data) | is.null(p_val_data),
    NA,
    # Check if p-value is less than or equal to 0.001
    ifelse(
      p_val_data <= 0.001,
      "***", # Assign "***" for p-values <= 0.001
      ifelse(
        p_val_data <= 0.01,
        "**", # Assign "**" for p-values <= 0.01
        ifelse(
          p_val_data <= 0.05,
          "*", # Assign "*" for p-values <= 0.05
          ifelse(
            p_val_data <= 0.1,
            ".", # Assign "." for p-values <= 0.1
            "<>" # Assign "<>" for p-values > 0.1
          )
        )
      )
    )
  )

  # End function ----
  return(significance_values)
}

test_that("stat_sig works within a mutate call", {
  # Create a data frame with a column of p-values
  data <- data.frame(p_value = c(0.0005, 0.02, 0.08, 0.15, 0.009))

  # Expected result based on the p-value thresholds
  expected_significance <- c("***", "*", ".", "<>", "**")

  # Apply stat_sig within mutate and check the result
  result <- data |>
    dplyr::mutate(significance = stat_sig(p_val_data = p_value))

  # Compare the 'significance' column with the expected result
  expect_equal(result$significance, expected_significance)
})

test_that("stat_sig works with p-values in the middle of thresholds in a mutate", {
  # Create a data frame with a column of p-values
  data <- data.frame(p_value = c(0.001, 0.01, 0.05, 0.1, 0.15))

  # Expected result for the given p-values
  expected_significance <- c("***", "**", "*", ".", "<>")

  # Apply stat_sig within mutate and check the result
  result <- data |>
    dplyr::mutate(significance = stat_sig(p_value))

  # Compare the 'significance' column with the expected result
  expect_equal(result$significance, expected_significance)
})

test_that("stat_sig works with an empty data frame in mutate", {
  # Create an empty data frame
  data <- data.frame(p_value = numeric(0))

  # Apply stat_sig within mutate (should return an empty 'significance' column)
  result <- data |>
    dplyr::mutate(significance = stat_sig(p_value))

  # Check that the resulting data frame has no rows and the correct column
  expect_equal(nrow(result), 0)
  expect_true("significance" %in% colnames(result))
})

test_that("stat_sig handles NA values in a mutate call", {
  # Create a data frame with NA values in p_value
  data <- data.frame(p_value = c(0.0005, NA, 0.05, 0.08, 0.15))

  # Expected result with significance codes and NA preserved
  expected_significance <- c("***", NA, "*", ".", "<>")

  # Apply stat_sig within mutate and check the result
  result <- data |>
    dplyr::mutate(significance = stat_sig(p_value))

  # Compare the 'significance' column with the expected result
  expect_equal(result$significance, expected_significance)
})

test_that("stat_sig correctly assigns significance codes", {
  # Test case with various p-values
  p_vals <- c(0.0005, 0.005, 0.02, 0.08, 0.15)
  expected <- c("***", "**", "*", ".", "<>")

  result <- stat_sig(p_vals)
  expect_equal(result, expected)
})

test_that("stat_sig handles p-values at the boundary correctly", {
  # Test case with boundary p-values
  p_vals <- c(0.001, 0.01, 0.05, 0.1, 0.15)
  expected <- c("***", "**", "*", ".", "<>")

  result <- stat_sig(p_vals)
  expect_equal(result, expected)
})

test_that("stat_sig throws an error for non-numeric input", {
  # Test non-numeric input
  expect_error(stat_sig("a string"), "p_val_data.*must be.*numeric")
})

test_that("stat_sig throws an error for p-values outside the [0, 1] range", {
  # Test p-values out of range
  expect_error(
    stat_sig(c(-0.01, 0.5, 1.2)),
    "p_val_data.*values must be contained within range.*0, 1.*Range of this input was.*-0.01, 1.2"
  )
})

test_that("stat_sig handles edge case of NULL vector", {
  # Test with a NULL
  expect_error(stat_sig(NULL), regexp = "p_val_data.*must not be NULL")
})

#' @title Convert Numbers into Readable Abbreviated Formats
#'
#' @description
#'
#' This function converts large numeric values into readable abbreviated formats
#' (e.g., 1,000 becomes "1k") with options for rounding, decimal precision, and
#' a custom prefix. It supports numbers up to the decillion range.
#'
#' @param x A numeric value or vector to be converted into a readable format.
#' @param digits Number of decimal places to display. Defaults to 2.
#' @param n_decimal `r lifecycle::badge("deprecated")` Use `digits` instead.
#' @param prefix An optional character string to prepend to the formatted number
#' (e.g., "$"). Defaults to `NULL`.
#' @param truncate A logical value indicating whether to truncate the numbers
#' before formatting. When `TRUE`, the function uses `base::signif()` to
#' truncate the numbers to the specified number of significant digits, making
#' the output more concise. When `FALSE`, the function uses `base::round()` to
#' round the numbers to the specified number of decimal places, preserving the
#' original scale of the number. Defaults to `FALSE`.
#'
#' @returns A character vector with the numbers formatted as abbreviated
#' strings. If `prefix` is provided, it prepends the formatted numbers.
#'
#' @export
#'
#' @examples
#' # Basic usage
#' pretty_number(1234)               # "1.23k"
#' pretty_number(1234567)            # "1.23m"
#' pretty_number(1234567890)         # "1.23b"
#'
#' # Adjusting decimal places
#' pretty_number(1234, digits = 1) # "1.2k"
#'
#' # Adding a prefix
#' pretty_number(1234, prefix = "$")  # "$1.23k"
#'
#' # Without rounding
#' pretty_number(1250, truncate = TRUE) # "1.2k"
#'
#' @author Nicolas Foss, Ed.D., MS
#'
pretty_number <- function(
  x,
  digits = 2,
  n_decimal = deprecated(),
  prefix = NULL,
  truncate = FALSE
) {
  # Handle deprecated n_decimal argument ----
  if (!missing(n_decimal)) {
    # Issue a warning
    lifecycle::deprecate_stop(
      when = "1.2.6",
      what = "pretty_number(n_decimal)",
      with = "pretty_number(digits)"
    )
  }

  # Enforce numeric or integer class on x ----
  validate_class(
    input = x,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error",
    calls = 2
  )

  # Error check: Ensure `digits` is an integer ----
  validate_class(
    input = digits,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error",
    calls = 2
  )

  # Error check: Ensure that `truncate` is logical ----
  validate_class(
    input = truncate,
    class_type = "logical",
    type = "error",
    calls = 2
  )

  # Enforce character class on prefix which can be null ----
  if (!is.null(prefix)) {
    validate_class(
      input = prefix,
      class_type = "character",
      null_ok = TRUE,
      calls = 2
    )
  }

  # Save the current options settings to restore them later ----
  old <- options()

  # Ensure that the original options are restored when the function exits, ----
  # even if an error occurs
  on.exit(options(old))

  # Set the 'scipen' option to a high value to prevent scientific notation ----
  # in the output. This ensures that large numbers are displayed in their
  # full numeric form rather than in scientific notation.
  options(scipen = 9999)

  # set values to different orders of magnitude ----
  thousand <- 1e3
  million <- 1e6
  billion <- 1e9
  trillion <- 1e12
  quadrillion <- 1e15
  quintillion <- 1e18
  sextillion <- 1e21
  septillion <- 1e24
  octillion <- 1e27
  nonillion <- 1e30
  decillion <- 1e33

  # Allow a toggle to truncate numbers so you can round something like ----
  # 555555 to "600k" with
  # > pretty_number(555555, truncate = TRUE, digits = 1)
  # "600k"
  if (truncate) {
    # When truncation is enabled, use signif() to truncate the numbers to the
    # specified number of significant digits. This ensures that the output is
    # concise and consistent.
    x <- signif(x, digits = digits)

    # Get the number of characters in the truncated value. This helps in ----
    # determining the appropriate suffix (e.g., "k", "m") based on the length of
    # the number.
    x_length <- nchar(trunc(x))
  } else {
    # When truncation is disabled, use round() to round the numbers to the ----
    # specified number of decimal places. This preserves the original scale of
    # the number.
    x <- round(x, digits = digits)

    # Get the number of characters in the rounded value. This helps in ----
    # determining the appropriate suffix (e.g., "k", "m") based on the length of
    # the number.
    x_length <- nchar(trunc(x))
  }

  # Classify the numeric value `x` into readable abbreviated formats ----

  # Define the suffixes for different orders of magnitude ----
  suffix <- c("k", "m", "b", "t", "qd", "qt", "sxt", "spt", "oct", "non", "dec")

  # Determine the appropriate suffix and format the value ----
  # based on the length of `x`
  x_val <- dplyr::case_when(
    is.na(x) ~ NA_character_,
    x_length %in% 4:6 ~
      # If the length of `x` is between 4 and 6 characters, divide by 1 thousand
      # and append the "k" suffix
      paste0(round(x / thousand, digits = digits), suffix[1]),
    x_length %in% 7:9 ~
      # If the length of `x` is between 7 and 9 characters, divide by 1 million
      # and append the "m" suffix
      paste0(round(x / million, digits = digits), suffix[2]),
    x_length %in% 10:12 ~
      # If the length of `x` is between 10 and 12 characters, divide by 1
      # billion and append the "b" suffix
      paste0(round(x / billion, digits = digits), suffix[3]),
    x_length %in% 13:15 ~
      # If the length of `x` is between 13 and 15 characters, divide by 1
      # trillion and append the "t" suffix
      paste0(round(x / trillion, digits = digits), suffix[4]),
    x_length %in% 16:18 ~
      # If the length of `x` is between 16 and 18 characters, divide by 1
      # quadrillion and append the "qd" suffix
      paste0(round(x / quadrillion, digits = digits), suffix[5]),
    x_length %in% 19:21 ~
      # If the length of `x` is between 19 and 21 characters, divide by
      # 1 quintillion and append the "qt" suffix
      paste0(round(x / quintillion, digits = digits), suffix[6]),
    x_length %in% 22:24 ~
      # If the length of `x` is between 22 and 24 characters, divide
      # by 1 sextillion and append the "sxt" suffix
      paste0(round(x / sextillion, digits = digits), suffix[7]),
    x_length %in% 25:27 ~
      # If the length of `x` is between 25 and 27 characters, divide
      # by 1 septillion and append the "spt" suffix
      paste0(round(x / septillion, digits = digits), suffix[8]),
    x_length %in% 28:30 ~
      # If the length of `x` is between 28 and 30 characters,
      # divide by 1 octillion and append the "oct" suffix
      paste0(round(x / octillion, digits = digits), suffix[9]),
    x_length %in% 31:33 ~
      # If the length of `x` is between 31 and 33 characters,
      # divide by 1 nonillion and append the "non" suffix
      paste0(
        round(x / nonillion, digits = digits),
        suffix[10]
      ),
    x_length %in% 34:36 ~
      # If the length of `x` is between 34 and 36 characters,
      # divide by 1 decillion and append the "dec" suffix
      paste0(
        round(x / decillion, digits = digits),
        suffix[11]
      ),
    # If the length of `x` does not match any of the above ----
    # ranges, round `x` to the specified number of decimal
    # places
    TRUE ~ paste0(round(x, digits = digits))
  )

  # If no prefix is provided, return the formatted value ----
  if (is.null(prefix)) {
    return(x_val)
  }

  # If a prefix is provided, prepend it to the formatted value ----
  x_val <- ifelse(is.na(x_val), NA_character_, paste0(prefix, x_val))

  # Return the final formatted value ----
  return(x_val)
}

test_that("pretty_number handles basic formatting", {
  expect_equal(pretty_number(1234), "1.23k")
  expect_equal(pretty_number(1234567), "1.23m")
  expect_equal(pretty_number(1234567890), "1.23b")
})

test_that("pretty_number respects the digits argument", {
  # set up options
  options(lifecycle_verbosity = "warning")

  # tests
  expect_equal(pretty_number(1234, digits = 1), "1.2k")
  expect_equal(pretty_number(1234, digits = 0), "1k")
  expect_error(
    pretty_number(1234, digits = "two")
  )
})

test_that("pretty_number adds prefix correctly", {
  expect_equal(pretty_number(1234, prefix = "$"), "$1.23k")
  expect_equal(pretty_number(1234, prefix = "€"), "€1.23k")
})

test_that("pretty_number handles rounding correctly", {
  expect_equal(pretty_number(1256, truncate = TRUE), "1.3k")
  expect_equal(pretty_number(1256, truncate = FALSE), "1.26k")
})

test_that("pretty_number supports large numbers up to decillions", {
  expect_equal(pretty_number(1e33), "1dec")
  expect_equal(pretty_number(1e30), "1non")
  expect_equal(pretty_number(1e27), "1oct")
})

test_that("pretty_number handles small numbers without formatting", {
  expect_equal(pretty_number(1), "1")
  expect_equal(pretty_number(999), "999")
})

test_that("pretty_number handles invalid x input", {
  expect_error(pretty_number("abc"), "x.*must be of class.*numeric, integer.*")
  expect_error(pretty_number(TRUE), "x.*must be of class.*numeric, integer.*")
})

test_that("pretty_number validates digits argument and warns about the deprecated digits argument", {
  expect_error(
    pretty_number(1234, digits = "two"),
    "digits.*must be of class.*numeric, integer.*."
  )
  expect_no_error(
    pretty_number(1234, digits = 2.5)
  )
})

test_that("pretty_number validates prefix argument", {
  expect_error(
    pretty_number(1234, prefix = 123),
    "prefix.*must be of class.*character"
  )
})

test_that("pretty_number handles NA and NULL values gracefully", {
  expect_error(pretty_number(NA))
  expect_error(pretty_number(NULL))
})

test_that("pretty_number handles vectorized input", {
  input <- c(1234, 5678, 9101112)
  result <- pretty_number(input)
  expected <- c("1.23k", "5.68k", "9.1m")
  expect_equal(result, expected)
})

test_that("pretty_number handles edge cases for large and small numbers", {
  expect_equal(pretty_number(1e12), "1t")
  expect_equal(pretty_number(1e6), "1m")
  expect_equal(pretty_number(1e-3), "0")
  expect_equal(pretty_number(0), "0")
})

test_that("n_decimal throws an error", {
  testthat::expect_error(
    pretty_number(1234567, n_decimal = 1)
  )
})

test_that("pretty_number() n_decimal is lifecycle-deprecated", {
  options(lifecycle_verbosity = "error")

  expect_error(
    pretty_number(1234, n_decimal = 1),
    class = "lifecycle_error_deprecated"
  )
})

#' @title Calculate Probability of Survival Using TRISS Method
#'
#' @description
#'
#' This function calculates the probability of survival (Ps) for trauma patients
#' based on the Trauma and Injury Severity Score (TRISS) methodology. TRISS
#' combines physiological and anatomical data to predict survival likelihood
#' using a logistic regression model. The function incorporates trauma type,
#' patient age, Revised Trauma Score (RTS), and Injury Severity Score (ISS) into
#' the calculation. Probability of survival is expressed as a percentage.
#'
#' @param trauma_type Character vector indicating the type of trauma ("Blunt" or
#'   "Penetrating"). Different methods exist for calculating probability of
#'   survival for burn patients, and so these records are excluded here.
#' @param age Numeric vector indicating the patient's age in years.
#' @param rts Numeric vector indicating the patient's Revised Trauma Score
#'   (RTS).
#' @param iss Numeric vector indicating the patient's Injury Severity Score
#'   (ISS).
#'
#' @returns Numeric vector of predicted probabilities of survival on a scale
#'   from 0 to 1.
#'
#' @details The methodology used in the calculation of survival probabilities
#' aligns with the coefficients published in Norouzi et al. (2013) and Merchant
#' et al. (2023). Consistent with Boyd et al. (1987),
#' `probability_of_survival()` does not treat patients under 15 years of age
#' differently and accounts for penetrating injuries similarly to other age
#' groups. Norouzi et al. (2013) and Merchant et al. (2023) use the updated
#' TRISS coefficients to calculate survival probabilities for penetrating
#' traumas with the same coefficients as for blunt traumas. If this approach is
#' preferred, please take note and adjust accordingly.
#'
#' @export
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @references
#'
#' Boyd CR, Tolson MA, Copes WS. (1987). Evaluating trauma care: the TRISS
#' method. Trauma Score and the Injury Severity Score. J Trauma. 1987
#' Apr;27(4):370-8. PMID: 3106646.
#'
#' Merchant AAH, Shaukat N, Ashraf N, Hassan S, Jarrar Z, Abbasi A, et al.
#' (2023). Which curve is better? A comparative analysis of trauma scoring
#' systems in a South Asian country. Trauma Surgery & Acute Care Open.
#' 2023;8:e001171. <doi:10.1136/tsaco-2023-001171>
#'
#' Norouzi V, Feizi I, Vatankhah S, Pourshaikhian M. (2013). Calculation of the
#' probability of survival for trauma patients based on trauma score and the
#' injury severity score model in fatemi hospital in ardabil. Arch Trauma Res.
#' 2013 Spring;2(1):30-5. <doi:10.5812/atr.9411>. Epub 2013 Jun 1. PMID:
#' 24396787; PMCID: PMC3876517.
#'
#' @examples
#' # Example usage:
#' trauma_data <- data.frame(
#'   Trauma_Type = c("Blunt", "Penetrating"),
#'   Patient_Age_Years = c(30, 60),
#'   RTS = c(7.84, 6.90),
#'   ISS = c(10, 25)
#' )
#'
#' # Run the function on example data
#' result <- trauma_data |>
#'   dplyr::mutate(Ps = probability_of_survival(
#'     trauma_type = Trauma_Type,
#'     age = Patient_Age_Years,
#'     rts = RTS,
#'     iss = ISS
#'   ))
#'
#' # Print the result
#' result
#'
probability_of_survival <- function(trauma_type, age, rts, iss) {
  # Ensure trauma_type is character or factor
  # Set up trauma_type validity checks ----
  validate_character_factor(input = trauma_type, type = "e")

  # Vector of valid trauma_type(s) ----
  valid_trauma_types <- c("Blunt", "Penetrating")

  # Check for valid values in trauma_type, ignoring NA ----
  validate_set(
    input = trauma_type,
    valid_set = valid_trauma_types,
    type = "w"
  )

  # Check age ----
  validate_numeric(
    input = age,
    min = 0,
    type = "warning",
    na_ok = TRUE,
    null_ok = TRUE
  )

  # Check rts ----
  validate_numeric(
    input = rts,
    min = 0,
    max = 7.8408,
    type = "warning",
    na_ok = TRUE,
    null_ok = TRUE
  )

  # Check iss ----
  validate_numeric(
    input = iss,
    min = 0,
    max = 75,
    type = "warning",
    na_ok = TRUE,
    null_ok = TRUE
  )

  # Assign age category ----
  # Age points are assigned as 0 if age is less than 55, otherwise 1
  age_points <- ifelse(age < 55, 0, 1)

  # Assign coefficients for b0 ----
  # Coefficient b0 is assigned based on trauma type
  b0 <- ifelse(
    trauma_type == "Blunt",
    -0.4499,
    ifelse(
      trauma_type == "Penetrating",
      -2.5355,
      NA_real_
    )
  )

  # Assign coefficients for b1 ----
  # Coefficient b1 is assigned based on trauma type
  b1 <- ifelse(
    trauma_type == "Blunt",
    0.8085,
    ifelse(
      trauma_type == "Penetrating",
      0.9934,
      NA_real_
    )
  )

  # Assign coefficients for b2 ----
  # Coefficient b2 is assigned based on trauma type
  b2 <- ifelse(
    trauma_type == "Blunt",
    -0.0835,
    ifelse(
      trauma_type == "Penetrating",
      -0.0651,
      NA_real_
    )
  )

  # Assign coefficients for b3 ----
  # Coefficient b3 is assigned based on trauma type
  b3 <- ifelse(
    trauma_type == "Blunt",
    -1.7430,
    ifelse(
      trauma_type == "Penetrating",
      -1.1360,
      NA_real_
    )
  )

  # Perform calculation ----
  # Calculate the logit value using the assigned coefficients and input
  # variables
  b <- b0 + (b1 * rts) + (b2 * iss) + (b3 * age_points)

  # Predicted probability of survival ----
  # Calculate the probability of survival using the logistic function
  survival_calc <- 1 / (1 + exp(-b))

  # End function ----
  # Return the calculated probability of survival
  return(survival_calc)
}

test_that("probability_of_survival handles valid inputs correctly", {
  trauma_type <- c("Blunt", "Penetrating")
  age <- c(30, 60)
  rts <- c(7.84, 6.90)
  iss <- c(10, 25)

  expected <- c(
    1 / (1 + exp(-(-0.4499 + 0.8085 * 7.84 - 0.0835 * 10 - (1.7430 * 0)))),
    1 / (1 + exp(-(-2.5355 + 0.9934 * 6.90 - 0.0651 * 25 - (1.1360 * 1))))
  )

  result <- probability_of_survival(
    trauma_type = trauma_type,
    age = age,
    rts = rts,
    iss = iss
  )
  expect_equal(result, expected)
})

# Test invalid trauma_type
test_that("probability_of_survival handles invalid trauma_type", {
  trauma_type <- c("Blunt", "Unknown")
  age <- c(30, 60)
  rts <- c(7.84, 6.90)
  iss <- c(10, 25)

  expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

# Test error in trauma_type
test_that("probability_of_survival handles invalid trauma_type", {
  trauma_type <- c(2, 1)
  age <- c(30, 60)
  rts <- c(7.84, 6.90)
  iss <- c(10, 25)

  expect_error(probability_of_survival(trauma_type, age, rts, iss))
})

# Test invalid age
test_that("probability_of_survival handles invalid age", {
  trauma_type <- c("Blunt", "Penetrating")
  age <- c(-5, 60) # Negative age
  rts <- c(7.84, 6.90)
  iss <- c(10, 25)

  expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

# Test invalid rts
test_that("probability_of_survival handles invalid rts", {
  trauma_type <- c("Blunt", "Penetrating")
  age <- c(30, 60)
  rts <- c(8, 6.90) # Out of range RTS
  iss <- c(10, 25)

  expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

# Test invalid iss
test_that("probability_of_survival handles invalid iss", {
  trauma_type <- c("Blunt", "Penetrating")
  age <- c(30, 60)
  rts <- c(7.84, 6.90)
  iss <- c(10, 80) # Out of range ISS

  expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

# Test handling of 'Burn' trauma type
test_that("probability_of_survival warns about 'Burn' trauma type", {
  trauma_type <- c("Blunt", "Burn")
  age <- c(30, 60)
  rts <- c(7.84, 6.90)
  iss <- c(10, 25)

  expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

# Test NA handling
test_that("probability_of_survival handles invalid values correctly", {
  trauma_type <- c("Blunt", "stuff")
  age <- c(30, 60)
  rts <- c(7.84, 6)
  iss <- c(10, 25)

  testthat::expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

# Test NA handling
test_that("probability_of_survival handles NA values correctly", {
  trauma_type <- c("Blunt", NA)
  age <- c(30, 60)
  rts <- c(7.84, NA)
  iss <- c(10, NA)

  expect_warning(probability_of_survival(trauma_type, age, rts, iss))
})

#' @title SEQIC Indicator 5 - Alcohol and Drug Screening
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Computes SEQIC Indicator 5a–5d for trauma system quality monitoring. These
#' indicators measure alcohol and drug screening rates among trauma patients at
#' trauma level I–IV facilities.
#'
#' @inheritParams seqic_indicator_1
#' @param blood_alcohol_content Unquoted column name for blood alcohol
#'   concentration. Numeric. A non-missing value indicates a test was performed.
#'   Values greater than zero are considered positive results.
#' @param drug_screen Unquoted column name for the drug screen result. Character
#'   or factor. May contain keywords (e.g., "opioid", "cocaine", "none"). The
#'   keywords used in this function correspond to the National Trauma Data Bank
#'   (NTDB) field values for the corresponding data element.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function:
#' \itemize{
#'   \item Filters to trauma records at trauma levels I–IV.
#'   \item Deduplicates by `unique_incident_id` to ensure one record per
#'     incident.
#'   \item Calculates four sub-measures:
#'     \itemize{
#'       \item {Indicator 5a:} Proportion of patients with a blood
#'         alcohol test performed.
#'       \item {Indicator 5b:} Among those tested, the proportion with
#'         BAC > 0.
#'       \item {Indicator 5c:} Proportion of patients with any recorded
#'         drug screen result.
#'       \item {Indicator 5d:} Among those with a drug result, the
#'         proportion that included a known positive drug (e.g., opioids,
#'         cocaine, THC).
#'     }
#'   \item Matches drug-related terms using regular expressions for a broad set
#'     of known substances. Matching is case-insensitive.
#' }
#'
#' @note
#'
#' Users must ensure input columns are correctly named and contain standardized
#' values where applicable. Drug screen values should ideally use consistent
#' naming or be mapped to recognizable substance terms prior to function use.
#'
#' @return A tibble summarizing SEQIC Indicator 5a–5d results. Includes
#'   numerator, denominator, and calculated proportion for each measure.
#'   Optionally includes 95% confidence intervals.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Create synthetic test data for Indicators 5a–5d
#' test_data <- tibble::tibble(
#'   id = as.character(1:10),
#'   trauma_level = rep(c("I", "II", "III", "IV", "V"), each = 2),
#'   bac = c(0.08, NA, 0, 0.02, NA, 0.15, NA, NA, 0, 0),
#'   drug = c(
#'     "opioid", "none", "cocaine", "none", NA,
#'     "benzodiazepine", "alcohol", "thc", "none", NA
#'   )
#' )
#'
#' # Run the indicator function
#' traumar::seqic_indicator_5(
#'   data = test_data,
#'   level = trauma_level,
#'   unique_incident_id = id,
#'   blood_alcohol_content = bac,
#'   drug_screen = drug
#' ) |>
#'   tidyr::pivot_longer(cols = -1, names_to = "Indicator", values_to =
#'   "Values")
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export

seqic_indicator_5 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  blood_alcohol_content,
  drug_screen,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error",
    logic = "or"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure `blood_alcohol_content` can be validated ----
  blood_alcohol_content_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ blood_alcohol_content }},
    var_name = "blood_alcohol_content"
  )

  # Validate `blood_alcohol_content` ----
  validate_numeric(
    input = blood_alcohol_content_check,
    min = 0,
    type = "error",
    var_name = "blood_alcohol_content"
  )

  # Ensure `drug_screen` can be validated
  drug_screen_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ drug_screen }},
    var_name = "drug_screen"
  )
  # Validate `drug_screen`
  validate_character_factor(
    input = drug_screen_check,
    type = "error",
    var_name = "drug_screen"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data`.
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Set up drug-related keyword matching via regular expressions ----
  ###___________________________________________________________________________

  # Options are consistent with the National Trauma Data Bank ----
  # responses as of the 2025 release
  # Define keyword vectors
  drug_keywords <- c(
    "alcohol",
    "bzo",
    "benzodiazepine",
    "amp",
    "amphetamine",
    "coc",
    "cocaine",
    "thc",
    "cannabinoid",
    "opi",
    "opioid",
    "pcp",
    "phencyclidine",
    "bar",
    "barbiturate",
    "mamp",
    "methamphetamine",
    "mdma",
    "ectasy",
    "mtd",
    "methadone",
    "tca",
    "tricyclic antidepressant",
    "oxy",
    "oxycodone",
    "none",
    "other"
  )

  # keywords for a positive test ----
  positive_drug_keywords <- setdiff(drug_keywords, "none")

  # Collapse into regular expression strings ----
  drug_pattern_terms <- stringr::str_c(drug_keywords, collapse = "|")
  positive_drug_pattern_terms <- stringr::str_c(
    positive_drug_keywords,
    collapse = "|"
  )

  # Final patterns (case-insensitive, non-capturing) ----
  drug_pattern <- sprintf("(?:%s)", drug_pattern_terms)
  positive_drug_pattern <- sprintf("(?:%s)", positive_drug_pattern_terms)

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  ###___________________________________________________________________________
  ### Compute numerator and denominator ----
  ### for each SEQIC Indicator 5 sub-measure
  ###___________________________________________________________________________
  seqic_5 <- data |>
    dplyr::filter({{ level }} %in% included_levels) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::summarize(
      # 5a: Proportion with BAC test performed
      numerator_5a = sum(!is.na({{ blood_alcohol_content }})),
      denominator_5a = dplyr::n(),
      seqic_5a = dplyr::if_else(
        denominator_5a > 0,
        numerator_5a / denominator_5a,
        NA_real_
      ),

      # 5b: Among those tested, proportion with BAC > 0 ----
      numerator_5b = sum({{ blood_alcohol_content }} > 0, na.rm = TRUE),
      denominator_5b = sum(!is.na({{ blood_alcohol_content }})),
      seqic_5b = dplyr::if_else(
        denominator_5b > 0,
        numerator_5b / denominator_5b,
        NA_real_
      ),

      # 5c: Proportion with any drug result (positive, none, or other) ----
      numerator_5c = sum(
        grepl(
          pattern = drug_pattern,
          x = {{ drug_screen }},
          ignore.case = TRUE
        ),
        na.rm = TRUE
      ),
      denominator_5c = dplyr::n(),
      seqic_5c = dplyr::if_else(
        denominator_5c > 0,
        numerator_5c / denominator_5c,
        NA_real_
      ),

      # 5d: Among those with a result, proportion with a positive  ----
      # drug result
      numerator_5d = sum(
        grepl(
          pattern = positive_drug_pattern,
          x = {{ drug_screen }},
          ignore.case = TRUE
        ),
        na.rm = TRUE
      ),
      denominator_5d = sum(
        grepl(
          pattern = drug_pattern,
          x = {{ drug_screen }},
          ignore.case = TRUE
        ),
        na.rm = TRUE
      ),
      seqic_5d = dplyr::if_else(
        denominator_5d > 0,
        numerator_5d / denominator_5d,
        NA_real_
      ),
      .by = {{ groups }}
    )

  if (!is.null(calculate_ci)) {
    seqic_5 <- seqic_5 |>
      dplyr::bind_cols(
        # Compute and bind all CI columns
        nemsqar::nemsqa_binomial_confint(
          data = seqic_5,
          x = numerator_5a,
          n = denominator_5a,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_5a = lower_ci, upper_ci_5a = upper_ci),

        nemsqar::nemsqa_binomial_confint(
          data = seqic_5,
          x = numerator_5b,
          n = denominator_5b,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_5b = lower_ci, upper_ci_5b = upper_ci),

        nemsqar::nemsqa_binomial_confint(
          data = seqic_5,
          x = numerator_5c,
          n = denominator_5c,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_5c = lower_ci, upper_ci_5c = upper_ci),

        nemsqar::nemsqa_binomial_confint(
          data = seqic_5,
          x = numerator_5d,
          n = denominator_5d,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_5d = lower_ci, upper_ci_5d = upper_ci)
      ) |>
      # Relocate CI columns immediately after their respective proportion ----
      # columns
      dplyr::relocate(lower_ci_5a, upper_ci_5a, .after = seqic_5a) |>
      dplyr::relocate(lower_ci_5b, upper_ci_5b, .after = seqic_5b) |>
      dplyr::relocate(lower_ci_5c, upper_ci_5c, .after = seqic_5c) |>
      dplyr::relocate(lower_ci_5d, upper_ci_5d, .after = seqic_5d)
  }

  # Assign label for ungrouped reporting, or sort grouped reporting ----
  if (is.null(groups)) {
    seqic_5 <- seqic_5 |>
      tibble::add_column(data = "population/sample", .before = "numerator_5a")
  } else {
    seqic_5 <- seqic_5 |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  # Return the final summarized data for Indicator 5. ----
  return(seqic_5)
}

# tests/testthat/test-seqic_indicator_5.R

testthat::test_that("seqic_indicator_5() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  valid_data <- tibble::tibble(
    level = factor(c("I", "II")),
    id = c("a", "b"),
    bac = c(0.08, 0),
    drug = c("opioid", "none")
  )

  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = TRUE,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = trauma_level,
      unique_incident_id = fake,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = trauma_level,
      unique_incident_id = id,
      blood_alcohol_content = something_else,
      drug_screen = drug
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = trauma_level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = FALSE
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_5() data validation errors", {
  # Minimal valid input for baseline
  valid_data <- tibble::tibble(
    level = factor(c("I", "II")),
    id = c("a", "b"),
    bac = c(0.08, 0),
    drug = c("opioid", "none")
  )

  # data must be a data.frame or tibble
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = list(),
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    "must be of class.*data.frame"
  )

  # level must be character or factor
  data_bad_level <- dplyr::mutate(valid_data, level = as.numeric(c(1, 2)))
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = data_bad_level,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    "must be of class.*character.*or.*factor"
  )

  # unique_incident_id must be character, factor, or numeric
  data_bad_id <- dplyr::mutate(valid_data, id = list(1:2))
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = data_bad_id,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    "unique_incident_id.*must be of class.*numeric, integer, character, factor"
  )

  # blood_alcohol_content must be numeric
  data_bad_bac <- dplyr::mutate(valid_data, bac = c("yes", "no"))
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = data_bad_bac,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    "blood_alcohol_content.*must be.*numeric"
  )

  # drug_screen must be character or factor
  data_bad_drug <- dplyr::mutate(
    valid_data,
    drug = as.Date(c("2022-01-01", "2022-01-02"))
  )
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = data_bad_drug,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug
    ),
    "must be of class.*character.*or.*factor"
  )

  # groups must be character vector
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug,
      groups = 123
    ),
    "groups.*must be of class.*character.*factor"
  )

  # groups must exist in data
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug,
      groups = "nonexistent_column"
    ),
    "groups.*contains invalid column names.*nonexistent_column).*Valid column names are.*level, id, bac, drug"
  )

  # calculate_ci must be "wilson", "clopper-pearson", or NULL
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug,
      calculate_ci = "bootstrap"
    ),
    "calculate_ci.*contains invalid values.*bootstrap.*Valid values are.*wilson, clopper-pearson"
  )

  # included_levels must be character, numeric, or factor
  testthat::expect_error(
    traumar::seqic_indicator_5(
      data = valid_data,
      level = level,
      unique_incident_id = id,
      blood_alcohol_content = bac,
      drug_screen = drug,
      included_levels = list("I", "II")
    ),
    "included_levels.*must be of class.*numeric, character, factor, integer"
  )
})

testthat::test_that("seqic_indicator_5() computes indicators correctly", {
  test_data <- tibble::tibble(
    id = as.character(1:10),
    trauma_level = rep(c("I", "II", "III", "IV", "V"), each = 2),
    bac = c(0.08, NA, 0, 0.02, NA, 0.15, NA, NA, 0, 0),
    drug = c(
      "opioid",
      "none",
      "cocaine",
      "none",
      NA,
      "benzodiazepine",
      "alcohol",
      "thc",
      "none",
      NA
    )
  )

  result <- traumar::seqic_indicator_5(
    data = test_data,
    level = trauma_level,
    unique_incident_id = id,
    blood_alcohol_content = bac,
    drug_screen = drug
  )

  testthat::expect_s3_class(result, "tbl_df")
  testthat::expect_named(
    object = result,
    expected = c(
      'data',
      'numerator_5a',
      'denominator_5a',
      'seqic_5a',
      'numerator_5b',
      'denominator_5b',
      'seqic_5b',
      'numerator_5c',
      'denominator_5c',
      'seqic_5c',
      'numerator_5d',
      'denominator_5d',
      'seqic_5d'
    )
  )
  testthat::expect_equal(nrow(result), 1)
  testthat::expect_equal(result$numerator_5a, 4)
  testthat::expect_equal(result$denominator_5a, 8)
  testthat::expect_equal(result$seqic_5a, 4 / 8)
  testthat::expect_equal(result$numerator_5b, 3)
  testthat::expect_equal(result$denominator_5b, 4)
  testthat::expect_equal(result$seqic_5b, 3 / 4)
  testthat::expect_equal(result$numerator_5c, 7)
  testthat::expect_equal(result$denominator_5c, 8)
  testthat::expect_equal(result$seqic_5c, 7 / 8)
  testthat::expect_equal(result$numerator_5d, 5)
  testthat::expect_equal(result$denominator_5d, 7)
  testthat::expect_equal(result$seqic_5d, 5 / 7)
})

testthat::test_that("seqic_indicator_5() handles empty input after filter", {
  empty_data <- tibble::tibble(
    id = c("1", "2"),
    trauma_level = c("V", "V"),
    bac = c(0.1, NA),
    drug = c("opioid", "none")
  )

  result <- traumar::seqic_indicator_5(
    data = empty_data,
    level = trauma_level,
    unique_incident_id = id,
    blood_alcohol_content = bac,
    drug_screen = drug
  )

  testthat::expect_equal(result$denominator_5a, 0)
  testthat::expect_true(all(is.na(result[c(
    "seqic_5a",
    "seqic_5b",
    "seqic_5c",
    "seqic_5d"
  )])))
})

testthat::test_that("seqic_indicator_5() returns confidence intervals if requested", {
  test_data <- tibble::tibble(
    id = as.character(1:6),
    trauma_level = rep("I", 6),
    bac = c(NA, 0.05, 0.00, 0.08, 0, 0),
    drug = c("thc", "none", "opioid", "thc", "none", "none")
  )

  result <- traumar::seqic_indicator_5(
    data = test_data,
    level = trauma_level,
    unique_incident_id = id,
    blood_alcohol_content = bac,
    drug_screen = drug,
    calculate_ci = "wilson"
  )

  testthat::expect_true(all(
    c("lower_ci_5a", "upper_ci_5a", "lower_ci_5d", "upper_ci_5d") %in%
      names(result)
  ))
  testthat::expect_type(result$lower_ci_5a, "double")
  testthat::expect_true(result$lower_ci_5a <= result$seqic_5a)
  testthat::expect_true(result$upper_ci_5a >= result$seqic_5a)
})

testthat::test_that("seqic_indicator_5() works with groupings", {
  grouped_data <- tibble::tibble(
    id = as.character(1:8),
    trauma_level = rep(c("I", "II"), each = 4),
    bac = c(0.1, NA, 0, 0, 0.15, NA, 0, NA),
    drug = c("thc", NA, "none", "opioid", "none", "opioid", "none", "none"),
    region = c("East", "East", "West", "West", "East", "East", "West", "West")
  )

  result <- traumar::seqic_indicator_5(
    data = grouped_data,
    level = trauma_level,
    unique_incident_id = id,
    blood_alcohol_content = bac,
    drug_screen = drug,
    groups = "region"
  )

  testthat::expect_s3_class(result, "tbl_df")
  testthat::expect_equal(nrow(result), 2)
  testthat::expect_setequal(result$region, c("East", "West"))
})

#' @title Calculate Trauma Hospital Performance Based on Robust and Validated
#'   Measures
#'
#' @description
#'
#' This function calculates trauma hospital performance based on the M, W, and Z
#' scores, which are derived from survival probability and mortality data, using
#' established methods. It computes the W-score, M-score, and Z-score based on
#' the provided dataset and calculates performance metrics for trauma programs.
#' For more information on the methods used in this function, please see
#' Champion et al. (1990) on the W score, and Flora (1978) and Boyd et al.
#' (1987) on the M and Z scores.
#'
#' @param df A data frame containing patient data.
#' @param Ps_col The name of the column containing the probability of survival
#'   (Ps). The values should be numeric and between 0 and 1. Values greater than
#'   1 will be automatically converted to decimal format by dividing by 100.
#' @param outcome_col The name of the column containing the binary outcome data.
#'
#'   The column should contain binary values indicating the patient outcome.
#'   Valid values include 1 (dead) and 0 (alive), or TRUE (dead) and FALSE
#'   (alive). The function will check values in this column and expects them to
#'   represent the outcome in a binary form.
#'
#' @param z_method A character vector indicating which method to use for
#'   calculating the Z-score. Must be one of "survival" or "mortality". The
#'   default is "survival".
#'
#' @return A tibble containing the following calculations:
#' \itemize{
#' \item `N_Patients`: The total number of patients included in the analysis.
#' \item `N_Survivors`: The total number of patients who survived, based on the
#' provided outcome data.
#' \item `N_Deaths`: The total number of patients who died, based on the
#' provided outcome data.
#' \item `Predicted_Survivors`: The total predicted number of survivors based on
#' the survival probability (`Ps`) for all patients.
#' \item `Predicted_Deaths`: The total predicted number of deaths, calculated as
#' `1 - Ps` for all patients.
#' \item `Patient_Estimate`: The estimated number of patients who survived that
#' were predicted to die, calculated based on the W-score. This value reflects
#' the difference between the actual and predicted number of deceased patients.
#' \item `W_Score`: The W-score, representing the difference between the
#' observed and expected number of survivors per 100 patients. A positive
#' W-score indicates that more patients survived than expected, while a negative
#' score indicates that fewer patients survived than expected.
#' \item `M_Score`: The M-score, which compares the observed patient case mix to
#' the Major Trauma Outcomes Study (MTOS) case mix. A higher score indicates
#' that the patient mix is more similar to MTOS, while a lower score indicates a
#' dissimilar mix. Based on the MTOS literature, an M_Score >= 0.88 indicates
#' that the Z_Score comes from distribution similar enough to the MTOS Ps
#' distribution.
#' \item `Z_Score`: The Z-score, which quantifies the difference between the
#' actual and predicted mortality (if `z_method = "mortality"`) or survival (if
#' `z_method = "survival"`). A Z-score > 1.96 is considered to point to the
#' statistical significance of the W-Score at alpha = 0.05 level for survival.
#' The positive Z_Score indicates that more patients survived than predicted,
#' while a negative Z-score indicates fewer survivors than predicted.
#'}
#'
#' @details
#'
#' The function checks whether the `outcome_col` contains values representing a
#' binary outcome. It also ensures that `Ps_col` contains numeric values within
#' the range 0 to 1. If any values exceed 1, a warning is issued. The patients
#' are then grouped into predefined Ps ranges, and the function compares the
#' fraction of patients in each range with the MTOS case mix distribution.
#'
#' Like other statistical computing functions, `trauma_performance()` is
#' happiest without missing data.  It is best to pass complete probability of
#' survival and outcome data to the function for optimal performance. With
#' smaller datasets, this is especially helpful.  However,
#' `trauma_performance()` will throw a warning about missing values, if any
#' exist in `Ps_col` and/or `outcome_col`.
#'
#' @note
#'
#' This function will produce the most reliable and interpretable results when
#' using a dataset that has one row per patient, with each column being a
#' feature.
#'
#' @examples
#' # Generate example data
#' set.seed(123)
#'
#' # Parameters
#' # Total number of patients
#' n_patients <- 5000
#'
#' # Arbitrary group labels
#' groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)
#'
#' # Trauma types
#' trauma_type_values <- sample(
#'   x = c("Blunt", "Penetrating"),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # RTS values
#' rts_values <- sample(
#'   x = seq(from = 0, to = 7.8408, by = 0.005),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # patient ages
#' ages <- sample(
#'   x = seq(from = 0, to = 100, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # ISS scores
#' iss_scores <- sample(
#'   x = seq(from = 0, to = 75, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # Generate survival probabilities (Ps)
#' Ps <- traumar::probability_of_survival(
#'   trauma_type = trauma_type_values,
#'   age = ages,
#'   rts = rts_values,
#'   iss = iss_scores
#' )
#'
#' # Simulate survival outcomes based on Ps
#' survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)
#'
#' # Create data frame
#' data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Calculate trauma performance (W, M, Z scores)
#' trauma_performance(data, Ps_col = Ps, outcome_col = death)
#'
#' @export
#'
#' @author Nicolas Foss, Ed.D., MS
#'
trauma_performance <- function(
  df,
  Ps_col,
  outcome_col,
  z_method = c("survival", "mortality")
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Check if the dataframe is valid ----
  validate_data_structure(
    input = df,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error",
    null_ok = FALSE
  )

  # Ensure Ps_col and outcome_col arguments are provided  ----
  # with tailored error messages
  if (missing(Ps_col) && missing(outcome_col)) {
    cli::cli_abort(
      "Both {.var Ps_col} and {.var outcome_col} arguments must be provided."
    )
  } else if (missing(Ps_col)) {
    cli::cli_abort("The {.var Ps_col} argument must be provided.")
  } else if (missing(outcome_col)) {
    cli::cli_abort("The {.var outcome_col} argument must be provided.")
  }

  # Pull and check the outcome column ----
  binary_data <- validate_data_pull(
    input = df,
    col = {{ outcome_col }},
    var_name = "outcome_col"
  )

  # Ensure the column is either logical or numeric ----
  validate_class(
    input = binary_data,
    class_type = c("numeric", "logical", "integer"),
    logic = "or",
    type = "error",
    var_name = "outcome_col"
  )

  # Get unique non-missing values to use in subsequent data validation ----
  non_missing <- stats::na.omit(binary_data)

  # Validate type and values ----
  if (is.logical(non_missing)) {
    # Logical vector: ensure only TRUE/FALSE (no coercion needed)
    validate_set(
      input = non_missing,
      valid_set = c(TRUE, FALSE),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.numeric(non_missing)) {
    # Numeric vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0, 1),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.integer(non_missing)) {
    # Integer vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0L, 1L),
      type = "error",
      var_name = "outcome_col"
    )
  }

  # Warn if missing ----
  validate_complete(
    input = binary_data,
    type = "warning",
    var_name = "outcome_col"
  )

  # Check if Ps column is numeric ----
  # dplyr::pull the Ps data
  Ps_check <- validate_data_pull(
    input = df,
    col = {{ Ps_col }},
    var_name = "Ps_col",
    calls = 5
  )

  # check the Ps_check remains continuous ----
  # Check if Ps column is continuous (values between 0 and 1)
  validate_numeric(
    input = Ps_check,
    min = 0,
    max = 1,
    type = "error",
    var_name = "Ps_col"
  )

  # Warn if any missings in Ps_col ----
  validate_complete(input = Ps_check, type = "warning", var_name = "Ps_col")

  # Validate the `z_method` argument
  z_method <- validate_choice(
    input = z_method,
    choices = c("survival", "mortality"),
    several.ok = FALSE,
    type = "error"
  )

  ### Initiate calculation of the W-Score ----

  # Total number of patients ----
  total_patients <- df |>
    nrow()

  # get n survivors ----
  total_survivors <- df |>
    dplyr::summarize(survivors = sum({{ outcome_col }} == 0, na.rm = TRUE)) |>
    dplyr::pull(survivors)

  # Number of patients who died ----
  total_deaths <- df |>
    dplyr::summarize(
      total_deaths = sum({{ outcome_col }} == 1, na.rm = TRUE)
    ) |>
    dplyr::pull(total_deaths)

  # Sum of Ps values for the patients
  sum_Ps <- df |>
    dplyr::summarize(sum_Ps = sum({{ Ps_col }}, na.rm = TRUE)) |>
    dplyr::pull(sum_Ps)

  # Calculate W-score
  W_score <- (total_patients - total_deaths - sum_Ps) / (total_patients / 100)

  ### Initiate process to calculate M-score ----
  # Bin patients into Ps ranges and calculate current fractions
  # Leverage the `trauma_case_mix` function from this package
  fractions_set <- trauma_case_mix(
    df = df,
    Ps_col = {{ Ps_col }},
    outcome_col = {{ outcome_col }}
  )

  # Take the M-Score ----
  M_score <- fractions_set |>
    dplyr::mutate(smallest_val = pmin(current_fraction, MTOS_distribution)) |>
    dplyr::summarize(M_score = sum(smallest_val, na.rm = TRUE)) |>
    dplyr::pull(M_score)

  ### Initiate process to calculate the Z-Score ----
  ### from Boyd et al. (1987)
  # get key statistics
  z_data <- df |>
    dplyr::mutate(
      prob_death = 1 - {{ Ps_col }},
      # scale factor to account for statistical variation ----
      scale_factor = {{ Ps_col }} * prob_death
    )

  # extract probability of death ----
  probability_death <- z_data$prob_death

  if (z_method == "mortality") {
    # get Z-Score, studying mortality (negative Z-Score is desired) ----
    Z_score <- z_data |>
      dplyr::summarize(
        # scale factor implemented ----
        z_score = (total_deaths - sum(prob_death, na.rm = TRUE)) /
          sqrt(sum(scale_factor))
      ) |>
      dplyr::pull(z_score)
  } else if (z_method == "survival") {
    # get Z-Score, studying survival (positive Z-Score is desired) ----
    Z_score <- z_data |>
      dplyr::summarize(
        z_score = (total_survivors - sum({{ Ps_col }}, na.rm = TRUE)) /
          sqrt(sum(scale_factor, na.rm = TRUE)) # scale factor implemented ----
      ) |>
      dplyr::pull(z_score)
  }

  # Return the scores as a dplyr::tibble ----
  result <- dplyr::tibble(
    N_Patients = total_patients,
    N_Survivors = total_survivors,
    N_Deaths = total_deaths,
    Predicted_Survivors = sum_Ps,
    Predicted_Deaths = sum(probability_death, na.rm = TRUE),
    # Positive patient estimate - Patients who survived, were predicted to die
    # Negative patient estimate - Patients who died, were predicted to survive
    Patient_Estimate = W_score * (total_patients / 100),
    # Number of patients that lived (positive) or died (negative) out of every
    # 100 patients treated in the sample
    W_Score = W_score,
    # M >= 0.88 is considered to indicate that the sample supplied by the user
    # is statistically similar to the MTOS probability of survival distribution
    M_Score = M_score,
    # z <= -1.96 and z >= 1.96 indicate statistically significant results
    Z_Score = Z_score
  )

  # Return the result ----
  return(result)
}

testthat::test_that("trauma_performance handles invalid dataframe input", {
  testthat::expect_error(
    trauma_performance(NULL, Ps_col = Ps, outcome_col = death),
    regexp = "df.*must not be NULL"
  )
})

testthat::test_that("trauma_performance checks for binary outcome column", {
  df <- tibble::tibble(Ps = c(0.8, 0.9), outcome_col = c(100, 1))
  testthat::expect_error(
    trauma_performance(df, Ps_col = Ps, outcome_col = outcome_col),
    regexp = "outcome_col.*contains invalid values such as.*100.*Valid values are.*0, 1"
  )
})

testthat::test_that("trauma_performance correctly processes probabilities above 1", {
  df <- tibble::tibble(Ps = c(12, 20), death = c(1, 0))
  expect_error(
    trauma_performance(df, Ps_col = Ps, outcome_col = death),
    regexp = "Ps_col.*values must be contained within range.*0, 1.*Range of this input was.*12, 20"
  )
})

testthat::test_that("trauma_performance validates Ps column is numeric", {
  df <- tibble::tibble(Ps = c("high", "medium"), death = c(1, 0))
  testthat::expect_error(
    trauma_performance(df, Ps_col = Ps, outcome_col = death),
    "Ps_col.*must be.*numeric"
  )
})

testthat::test_that("trauma_performance checks if Ps values are between 0 and 100", {
  df <- tibble::tibble(Ps = c(101, -1), death = c(1, 0))
  testthat::expect_error(
    trauma_performance(df, Ps_col = Ps, outcome_col = death),
    regexp = "Ps_col.*values must be contained within range.*0, 1.*Range of this input was.*-1, 101"
  )
})

testthat::test_that("trauma_performance calculates W-score", {
  df <- tibble::tibble(Ps = c(0.8, 0.9), death = c(1, 0))
  result <- trauma_performance(df, Ps_col = Ps, outcome_col = death)
  testthat::expect_true("W_Score" %in% names(result))
})

testthat::test_that("trauma_performance calculates M-score correctly", {
  df <- tibble::tibble(Ps = c(0.8, 0.9), death = c(1, 0))
  result <- trauma_performance(df, Ps_col = Ps, outcome_col = death)
  testthat::expect_true("M_Score" %in% names(result))
})

testthat::test_that("trauma_performance calculates Z-score with survival method", {
  df <- tibble::tibble(Ps = c(0.8, 0.9), death = c(1, 0))
  result <- trauma_performance(
    df,
    Ps_col = Ps,
    outcome_col = death,
    z_method = "survival"
  )
  testthat::expect_true("Z_Score" %in% names(result))
})

testthat::test_that("trauma_performance calculates Z-score with mortality method", {
  df <- tibble::tibble(Ps = c(0.8, 0.9), death = c(1, 0))
  result <- trauma_performance(
    df,
    Ps_col = Ps,
    outcome_col = death,
    z_method = "mortality"
  )
  testthat::expect_true("Z_Score" %in% names(result))
})

testthat::test_that("trauma_performance works when outcome_col is TRUE/FALSE", {
  df <- tibble::tibble(Ps = c(0.8, 0.9), death = c(TRUE, FALSE))
  result <- trauma_performance(df, Ps_col = Ps, outcome_col = death)
  testthat::expect_true("W_Score" %in% names(result))
})

testthat::test_that("trauma_performance handles edge case of empty dataframe", {
  df <- tibble::tibble(Ps = numeric(0), death = logical(0))

  result <- trauma_performance(df, Ps_col = Ps, outcome_col = death)

  # Additionally, check if the result is a tibble with the correct structure (optional)
  testthat::expect_s3_class(result, "tbl_df") # Check that the result is a tibble
  testthat::expect_equal(nrow(result), 1) # Verify that 9 rows are in the result
  testthat::expect_equal(ncol(result), 9) # Verify that there are 2 columns
})

# Test missing Ps_col and outcome_col columns and logical binary data
testthat::test_that("trauma_performance correctly validates missing Ps_col and outcome_col columns and logical binary data", {
  set.seed(123)
  n_patients <- 100
  Ps <- c(0.85, NA, 0.75, 0.6, 0.91)
  survival_outcomes <- as.character(c(1, 0, 0, 1, 0))
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_performance(data),
    regexp = "Both.*arguments must be provided"
  )

  testthat::expect_error(
    trauma_performance(data, Ps_col = Ps),
    regexp = "argument must be provided"
  )

  testthat::expect_error(
    trauma_performance(data, outcome_col = survival),
    regexp = "argument must be provided"
  )

  survival_outcomes <- c(1L, 2L, 2L, 1L, 2L)
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_performance(data, Ps_col = Ps, outcome_col = survival),
    regexp = "outcome_col.*contains invalid values such as.*Valid values are"
  )

  Ps <- c(0.85, 1, 0.75, 0.6, 0.91)
  survival_outcomes <- c(T, F, T, F, T)
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_s3_class(
    trauma_performance(data, Ps_col = Ps, outcome_col = survival),
    class = "tbl_df"
  )

  Ps <- c(0.85, NA, 0.75, 0.6, 0.91)
  survival_outcomes <- as.character(c(1, 0, 0, 1, 0))
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_performance(data, Ps_col = Ps, outcome_col = survival),
    regexp = "outcome_col.*must be of class.*numeric, logical, integer"
  )
})

#' @title SEQIC Indicator 3 - Presence of Probability of Survival Calculations
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' This function calculates Indicator 3, a measure of the proportion of trauma
#' incidents where the probability of survival is recorded. It filters the data
#' by trauma center level (I-IV), excluding burn cases, and computes the
#' proportion of incidents with a valid probability of survival value.
#'
#' @inheritParams seqic_indicator_1
#' @param trauma_type A column name indicating the type of trauma. The function
#'   filters out "Burn" cases.
#' @param probability_of_survival A column name for the probability of survival
#'   for each incident.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function:
#' \itemize{
#'   \item Filters trauma records to those with a trauma center level of I–IV.
#'   \item Excludes records with a trauma type of "Burn".
#'   \item Deduplicates by `unique_incident_id` to ensure one record per
#'   incident.
#'   \item Calculates the proportion of records with a non-missing
#'   `probability_of_survival`.
#' }
#'
#' @note
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @return A tibble summarizing SEQIC Indicator 3 results. Includes numerator,
#'   denominator, and performance rate for the indicator. 95% confidence
#'   intervals are provided optionally.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Create a synthetic test dataset
#' test_data <- tibble::tibble(
#'   unique_id = as.character(1:10),
#'   trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV", "I",
#'   "II"),
#'   trauma_category = c("Blunt", "Penetrating", "Burn", "Blunt", "Penetrating",
#'                       "Burn", "Blunt", "Penetrating", "Blunt", "Blunt"),
#'   survival_prob = c(0.95, 0.89, NA, 0.76, NA, 0.92, 0.88, NA, 0.97, 0.91)
#' )
#'
#' # Run the indicator function
#' traumar::seqic_indicator_3(
#'   data = test_data,
#'   level = trauma_level,
#'   trauma_type = trauma_category,
#'   unique_incident_id = unique_id,
#'   probability_of_survival = survival_prob,
#'   groups = "trauma_level"
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_3 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  trauma_type,
  unique_incident_id,
  probability_of_survival,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble.
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error",
    logic = "or"
  )

  # Make the `trauma_type` column accessible for validation. ----
  trauma_type_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ trauma_type }},
    var_name = "trauma_type"
  )

  # Validate `trauma_type` to ensure it's either character or factor. ----
  validate_character_factor(
    input = trauma_type_check,
    type = "error",
    var_name = "trauma_type"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Make the `probability_of_survival` column accessible for validation. ----
  probability_of_survival_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ probability_of_survival }},
    var_name = "probability_of_survival"
  )

  # Validate `probability_of_survival` ----
  validate_numeric(
    input = probability_of_survival_check,
    min = 0,
    max = 1,
    type = "error",
    var_name = "probability_of_survival"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data`.
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Filter the data for valid levels and exclude "Burn" trauma types. ----
  seqic_3 <- data |>
    dplyr::filter(
      {{ level }} %in% included_levels,
      {{ trauma_type }} != "Burn"
    ) |>
    dplyr::distinct(
      {{ unique_incident_id }},
      .keep_all = TRUE
    ) |>
    # Calculate the numerator and denominator for Indicator 3. ----
    dplyr::summarize(
      numerator_3 = sum(!is.na({{ probability_of_survival }})), # Count non-missing values in `probability_of_survival`.
      denominator_3 = dplyr::n(), # Count the total number of unique incidents.
      seqic_3 = dplyr::if_else(
        denominator_3 > 0,
        numerator_3 / denominator_3,
        NA_real_
      ), # Calculate the proportion of incidents with survival probability recorded.
      .by = {{ groups }} # Optionally group by specified columns.
    )

  # Optionally calculate confidence intervals for the proportions: ----
  if (!is.null(calculate_ci)) {
    seqic_3 <- seqic_3 |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_3,
          x = numerator_3, # Number of successes (non-missing probability of survival).
          n = denominator_3, # Total number of incidents.
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_3 = lower_ci, upper_ci_3 = upper_ci) # Rename the CI columns.
      )
  }

  # Add a label column ----
  # to indicate whether the data represents population or sample-level results.
  if (is.null(groups)) {
    seqic_3 <- seqic_3 |>
      tibble::add_column(data = "population/sample", .before = "numerator_3") # Add the label column.
  } else if (!is.null(groups)) {
    seqic_3 <- seqic_3 |>
      dplyr::arrange(!!!rlang::syms(groups)) # Arrange the results by the specified grouping variables.
  }

  # Return the final summarized data for Indicator 3. ----
  return(seqic_3)
}

# tests/testthat/test-seqic_indicator_3.R

testthat::test_that("seqic_indicator_3() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  test_data <- tibble::tibble(
    unique_id = c("1", "2", "3", "4", "5", "6"),
    trauma_level = c("I", "II", "III", "IV", "I", "III"),
    trauma_category = c(
      "Blunt",
      "Penetrating",
      "Burn",
      "Blunt",
      "Burn",
      "Penetrating"
    ),
    survival_prob = c(0.9, 0.8, NA, 0.95, NA, 0.88)
  )

  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = test_data,
      level = TRUE,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob,
      groups = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = test_data,
      level = trauma_level,
      trauma_type = "faked",
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob,
      groups = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = test_data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = FALSE,
      probability_of_survival = survival_prob,
      groups = NULL
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = test_data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = "some fake thing",
      groups = NULL
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_3: input validation", {
  data <- tibble::tibble(
    trauma_level = c("I", "II"),
    trauma_category = c("Blunt", "Penetrating"),
    unique_id = 1:2,
    survival_prob = c(0.9, 0.8)
  )

  # Invalid data
  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = "not_a_data",
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob
    ),
    "data.*must be of class.*data\\.frame, tbl, tbl_df"
  )

  # Set up error prone logical unique id
  logical_data <- tibble::tibble(
    trauma_level = c("I", "II"),
    trauma_category = c("Blunt", "Penetrating"),
    unique_id = logical(length = 2),
    survival_prob = c(0.9, 0.8)
  )

  # Invalid unique_id
  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = logical_data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob
    ),
    "unique_incident_id.*must be of class"
  )

  # Invalid level
  bad_data <- tibble::tibble(
    level = c(1L, 2L), # integer instead of character/factor
    trauma_type = c("Blunt", "Burn"),
    unique_id = c("1", "2"),
    survival_prob = c(0.95, 0.9)
  )

  expect_error(
    traumar::seqic_indicator_3(
      data = bad_data,
      level = level,
      trauma_type = trauma_type,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob,
      regex = "level.*must be of class.*character.*factor"
    )
  )

  # Invalid groups
  valid_data <- tibble::tibble(
    level = factor(c("I", "II")),
    trauma_type = c("Blunt", "Penetrating"),
    unique_id = c("1", "2"),
    survival_prob = c(0.9, 0.85)
  )

  expect_error(
    traumar::seqic_indicator_3(
      data = valid_data,
      level = level,
      trauma_type = trauma_type,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob,
      groups = list(1, 2) # not character strings
    ),
    regexp = "groups.*must be of class.*character.*factor"
  )

  # Invalid trauma_type
  bad_data <- dplyr::mutate(data, trauma_category = as.numeric(c(1, 2)))
  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = bad_data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob
    ),
    "must be of class"
  )

  # Invalid probability_of_survival
  bad_data <- dplyr::mutate(data, survival_prob = as.character(survival_prob))
  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = bad_data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob
    ),
    "probability_of_survival.*must be.*numeric"
  )

  # Invalid group var
  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob,
      groups = "bad_col"
    ),
    "groups.*contains invalid column names.*bad_col.*Valid column names are.*trauma_level, trauma_category, unique_id, survival_prob"
  )

  # Invalid calculate_ci
  testthat::expect_error(
    traumar::seqic_indicator_3(
      data = data,
      level = trauma_level,
      trauma_type = trauma_category,
      unique_incident_id = unique_id,
      probability_of_survival = survival_prob,
      calculate_ci = "not_a_method"
    ),
    "calculate_ci.*contains invalid values.*not_a_method.*Valid values are.*wilson, clopper-pearson"
  )
})

testthat::test_that("seqic_indicator_3: basic logic", {
  test_data <- tibble::tibble(
    unique_id = c("1", "2", "3", "4", "5", "6"),
    trauma_level = c("I", "II", "III", "IV", "I", "III"),
    trauma_category = c(
      "Blunt",
      "Penetrating",
      "Burn",
      "Blunt",
      "Burn",
      "Penetrating"
    ),
    survival_prob = c(0.9, 0.8, NA, 0.95, NA, 0.88)
  )

  res <- traumar::seqic_indicator_3(
    data = test_data,
    level = trauma_level,
    trauma_type = trauma_category,
    unique_incident_id = unique_id,
    probability_of_survival = survival_prob,
    groups = NULL
  )

  testthat::expect_s3_class(res, "tbl_df")
  testthat::expect_named(
    res,
    c("data", "numerator_3", "denominator_3", "seqic_3")
  )
  testthat::expect_equal(res$numerator_3, 4)
  testthat::expect_equal(res$denominator_3, 4) # burn cases excluded
  testthat::expect_equal(res$seqic_3, 4 / 4)
})

testthat::test_that("seqic_indicator_3: grouped results and confidence intervals", {
  test_data <- tibble::tibble(
    unique_id = as.character(1:6),
    trauma_level = c("I", "I", "II", "II", "III", "IV"),
    trauma_category = c(
      "Blunt",
      "Burn",
      "Penetrating",
      "Blunt",
      "Burn",
      "Blunt"
    ),
    survival_prob = c(0.9, NA, 0.85, NA, NA, 0.95)
  )

  res <- traumar::seqic_indicator_3(
    data = test_data,
    level = trauma_level,
    trauma_type = trauma_category,
    unique_incident_id = unique_id,
    probability_of_survival = survival_prob,
    groups = "trauma_level",
    calculate_ci = "wilson"
  )

  testthat::expect_s3_class(res, "tbl_df")
  testthat::expect_true("trauma_level" %in% colnames(res))
  testthat::expect_true(all(
    c(
      "numerator_3",
      "denominator_3",
      "seqic_3",
      "lower_ci_3",
      "upper_ci_3"
    ) %in%
      names(res)
  ))
})

#' @title SEQIC Indicator 4 - Autopsy and Long LOS Without Autopsy
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Computes SEQIC Indicator 4a and 4b for trauma center performance. Indicator
#' 4a captures the proportion of deceased trauma patients at trauma level I–IV
#' facilities who had an autopsy performed. Indicator 4b identifies deceased
#' trauma patients with a prolonged length of stay (LOS > 3 days) but without an
#' autopsy.
#'
#' @inheritParams seqic_indicator_1
#' @param ed_disposition Column representing the emergency department
#'   disposition. For a record to be picked up in this function, the ED
#'   dispostion must be documented as "Deceased/Expired".
#' @param ed_LOS Column for the calculated ED length of stay, measured in
#'   minutes.
#' @param hospital_disposition Column representing the hospital disposition. For
#'   a record to be picked up in this function, the hospital dispostion must be
#'   documented as "Deceased/Expired".
#' @param hospital_LOS Column for the calculated hospital length of stay,
#'   measured in minutes.
#' @param autopsy Unquoted column name indicating whether an autopsy was
#'   performed. Expected values: `"Yes"` or `NA`.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function:
#' \itemize{
#'   \item Filters trauma records to those with a trauma center level of I–IV.
#'   \item Identifies records where the patient died, based on ED or hospital
#'   disposition.
#'   \item Deduplicates by `unique_incident_id` to ensure one record per
#'   incident.
#'   \item For Indicator 4a, calculates the proportion of deceased patients who
#'   received an autopsy.
#'   \item For Indicator 4b, calculates the proportion of deceased patients with
#'   a hospital or ED length of stay greater than 72 hours (4320 minutes) and no
#'   autopsy performed.
#' }
#'
#' @note
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @return A tibble summarizing SEQIC Indicator 4a and 4b results. Includes
#'   numerator, denominator, and performance rate for the indicator. 95%
#'   confidence intervals are provided optionally.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Create a synthetic test dataset
#' test_data <- tibble::tibble(
#'   id = as.character(1:8),
#'   trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV"),
#'   ed_disp = c(
#'     "Operating Room",
#'     "Admitted",
#'     "Deceased/Expired",
#'     "Transferred",
#'     "Deceased/Expired",
#'     "Deceased/Expired",
#'     "Admitted",
#'     "Deceased/Expired"
#'   ),
#'   ed_los = c(120, 200, 5000, 180, 3000, 4321, 60, 4000),
#'   hosp_disp = c(
#'     "Deceased/Expired",
#'     "Deceased/Expired",
#'     "Deceased/Expired",
#'     "Discharged",
#'     "Deceased/Expired",
#'     "Deceased/Expired",
#'     "Discharged",
#'     "Deceased/Expired"
#'   ),
#'   hosp_los = c(3000, 4500, 1000, 200, 5000, 4400, 150, 3000),
#'   autopsy_done = c("Yes", "No", "No", NA, "Yes", "No", NA, "Yes")
#' )
#'
#' # Run the indicator function
#' traumar::seqic_indicator_4(
#'   data = test_data,
#'   level = trauma_level,
#'   ed_disposition = ed_disp,
#'   ed_LOS = ed_los,
#'   hospital_disposition = hosp_disp,
#'   hospital_LOS = hosp_los,
#'   unique_incident_id = id,
#'   autopsy = autopsy_done
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
seqic_indicator_4 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  ed_disposition,
  ed_LOS,
  hospital_disposition,
  hospital_LOS,
  unique_incident_id,
  autopsy,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # Ensure that `ed_disposition` can be validated ----
  ed_disp_check <- validate_data_pull(
    input = data,
    col = {{ ed_disposition }},
    type = "error",
    var_name = "ed_disposition"
  )

  # Validate `ed_disposition` ----
  validate_character_factor(
    input = ed_disp_check,
    type = "error",
    var_name = "ed_disposition"
  )

  # Ensure that `hospital_disposition` can be validated ----
  hospital_disp_check <- validate_data_pull(
    input = data,
    col = {{ hospital_disposition }},
    type = "error",
    var_name = "hospital_disposition"
  )

  # Validate `hospital_disposition` ----
  validate_character_factor(
    input = hospital_disp_check,
    type = "error",
    var_name = "hospital_disposition"
  )

  # Ensure that ed_LOS can be validated ----
  ed_los_check <- validate_data_pull(
    input = data,
    col = {{ ed_LOS }},
    type = "error",
    var_name = "ed_LOS"
  )

  # Validate `ed_LOS` ----
  validate_numeric(
    input = ed_los_check,
    min = NULL,
    max = NULL,
    type = "error",
    var_name = "ed_LOS"
  )

  # Ensure `hospital_LOS` can be validated ----
  hospital_los_check <- validate_data_pull(
    input = data,
    col = {{ hospital_LOS }},
    type = "error",
    var_name = "hospital_LOS"
  )

  # Validate `hospital_LOS` ----
  validate_numeric(
    input = hospital_los_check,
    min = NULL,
    max = NULL,
    type = "error",
    var_name = "hospital_LOS"
  )

  # Ensure that `autopsy` can be validated ----
  autopsy_check <- validate_data_pull(
    input = data,
    col = {{ autopsy }},
    type = "error",
    var_name = "autopsy"
  )

  # Validate `autopsy` ----
  validate_character_factor(
    input = autopsy_check,
    type = "error",
    var_name = "autopsy"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data` ----
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  ###___________________________________________________________________________
  # Indicator 4A - Presence of Autopsy in Deceased Patients at Trauma Centers
  ###___________________________________________________________________________
  seqic_4a <- data |>
    dplyr::filter(
      # Limit to valid trauma levels
      {{ level }} %in% included_levels,
      # Include cases where ED or hospital disposition is "Deceased/Expired"
      dplyr::if_any(
        c({{ ed_disposition }}, {{ hospital_disposition }}),
        ~ . == "Deceased/Expired"
      )
    ) |>
    # Ensure unique incidents (in case of duplicated records)
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    # Compute numerator (autopsies = "Yes") and denominator
    dplyr::summarize(
      numerator_4a = sum({{ autopsy }} == "Yes", na.rm = TRUE),
      denominator_4a = dplyr::n(),
      seqic_4a = dplyr::if_else(
        denominator_4a > 0,
        numerator_4a / denominator_4a,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # Optional confidence intervals for 4a ----
  if (!is.null(calculate_ci)) {
    seqic_4a <- seqic_4a |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_4a,
          x = numerator_4a,
          n = denominator_4a,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_4a = lower_ci, upper_ci_4a = upper_ci)
      ) |>
      dplyr::relocate(lower_ci_4a, .after = seqic_4a) |>
      dplyr::relocate(upper_ci_4a, .after = lower_ci_4a)
  }

  ###___________________________________________________________________________
  # Indicator 4B - No Autopsy + Long LOS in Deceased Patients ----
  ###___________________________________________________________________________
  seqic_4b <- data |>
    dplyr::filter(
      {{ level }} %in% included_levels,
      dplyr::if_any(
        c({{ ed_disposition }}, {{ hospital_disposition }}),
        ~ . == "Deceased/Expired"
      ),
      # Long LOS (> 72 hours = 4320 minutes)
      dplyr::if_any(c({{ ed_LOS }}, {{ hospital_LOS }}), ~ . > 4320)
    ) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    # Numerator = no autopsy or autopsy not "Yes"
    dplyr::summarize(
      numerator_4b = sum(is.na({{ autopsy }}) | {{ autopsy }} != "Yes"),
      denominator_4b = dplyr::n(),
      seqic_4b = dplyr::if_else(
        denominator_4b > 0,
        numerator_4b / denominator_4b,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # Optional confidence intervals for 4b ----
  if (!is.null(calculate_ci)) {
    seqic_4b <- seqic_4b |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_4b,
          x = numerator_4b,
          n = denominator_4b,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_4b = lower_ci, upper_ci_4b = upper_ci)
      ) |>
      dplyr::relocate(lower_ci_4b, .after = seqic_4b) |>
      dplyr::relocate(upper_ci_4b, .after = lower_ci_4b)
  }

  # Combine 4a and 4b results; assign label for ungrouped reporting. ----
  if (is.null(groups)) {
    seqic_4 <- dplyr::bind_cols(
      seqic_4a,
      seqic_4b
    ) |>
      tibble::add_column(data = "population/sample", .before = "numerator_4a")
  } else {
    seqic_4 <- seqic_4a |>
      dplyr::full_join(seqic_4b, by = dplyr::join_by(!!!rlang::syms(groups))) |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  # Return the final summarized data for Indicator 4. ----
  return(seqic_4)
}

# tests/testthat/test-seqic_indicator_4.R

testthat::test_that("seqic_indicator_4() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  data <- tibble::tibble(
    id = as.character(1:8),
    trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV"),
    ed_disp = c(
      "Operating Room",
      "Admitted",
      "Deceased/Expired",
      "Transferred",
      "Deceased/Expired",
      "Deceased/Expired",
      "Admitted",
      "Deceased/Expired"
    ),
    ed_los = c(120, 200, 5000, 180, 3000, 4321, 60, 4000),
    hosp_disp = c(
      "Deceased/Expired",
      "Deceased/Expired",
      "Deceased/Expired",
      "Discharged",
      "Deceased/Expired",
      "Deceased/Expired",
      "Discharged",
      "Deceased/Expired"
    ),
    hosp_los = c(3000, 4500, 1000, 200, 5000, 4400, 150, 3000),
    autopsy_done = c("Yes", "No", "No", NA, "Yes", "No", NA, "Yes")
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = FAKE,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = trauma_level,
      ed_disposition = NOT_TRUE,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = "something else",
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = other_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = a_fake_column,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = error,
      autopsy = autopsy_done
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_4(
      data = data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = FALSE
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_4() input validation works as expected", {
  valid_data <- tibble::tibble(
    trauma_level = factor(rep("I", 2)),
    ed_disp = c("Deceased/Expired", "Deceased/Expired"),
    ed_los = c(3000, 4000),
    hosp_disp = c("Deceased/Expired", "Deceased/Expired"),
    hosp_los = c(4000, 5000),
    autopsy_done = c("Yes", "No"),
    id = c("1", "2"),
    group_var = c("A", "B")
  )

  # data must be data.frame or tibble
  expect_error(
    traumar::seqic_indicator_4(
      data = list(),
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "data.*must be of class.*data\\.frame, tbl, tbl_df"
  )

  # level must be character or factor
  bad_data <- valid_data |> dplyr::mutate(trauma_level = as.numeric(1:2))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "must be of class.*character.*factor"
  )

  # ed_disposition must be character or factor
  bad_data <- valid_data |> dplyr::mutate(ed_disp = as.numeric(1:2))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "ed_disposition.*character.*factor"
  )

  # hospital_disposition must be character or factor
  bad_data <- valid_data |> dplyr::mutate(hosp_disp = as.numeric(1:2))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "hospital_disposition.*character.*factor"
  )

  # ed_LOS must be numeric
  bad_data <- valid_data |> dplyr::mutate(ed_los = as.character(ed_los))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "ed_LOS.*numeric"
  )

  # hospital_LOS must be numeric
  bad_data <- valid_data |> dplyr::mutate(hosp_los = as.character(hosp_los))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "hospital_LOS.*numeric"
  )

  # autopsy must be character or factor
  bad_data <- valid_data |> dplyr::mutate(autopsy_done = as.numeric(1:2))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "autopsy.*character.*factor"
  )

  # unique_incident_id must be character, factor, or numeric
  bad_data <- valid_data |> dplyr::mutate(id = list(1, 2))
  expect_error(
    traumar::seqic_indicator_4(
      data = bad_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done
    ),
    "unique_incident_id.*must be of class.*numeric, integer, character, factor"
  )

  # groups must be character vector and present in data
  expect_error(
    traumar::seqic_indicator_4(
      data = valid_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done,
      groups = list()
    ),
    "groups.*must be of class.*character.*factor"
  )

  expect_error(
    traumar::seqic_indicator_4(
      data = valid_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done,
      groups = c("bad_col")
    ),
    "groups.*contains invalid column names.*bad_col.*Valid column names are.*trauma_level, ed_disp, ed_los, hosp_disp, hosp_los, autopsy_done, id, group_var"
  )

  # calculate_ci must be NULL, "wilson", or "clopper-pearson"
  expect_error(
    traumar::seqic_indicator_4(
      data = valid_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done,
      calculate_ci = "jackknife"
    ),
    "calculate_ci.*contains invalid values.*jackknife.*Valid values are.*wilson, clopper-pearson"
  )

  # included_levels must be character, factor, or numeric
  expect_error(
    traumar::seqic_indicator_4(
      data = valid_data,
      level = trauma_level,
      ed_disposition = ed_disp,
      ed_LOS = ed_los,
      hospital_disposition = hosp_disp,
      hospital_LOS = hosp_los,
      unique_incident_id = id,
      autopsy = autopsy_done,
      included_levels = list("I", "II")
    ),
    "included_levels.*must be of class.*numeric, character, factor, integer"
  )
})

testthat::test_that("seqic_indicator_4 computes 4a and 4b correctly", {
  data <- tibble::tibble(
    id = as.character(1:8),
    trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV"),
    ed_disp = c(
      "Operating Room",
      "Admitted",
      "Deceased/Expired",
      "Transferred",
      "Deceased/Expired",
      "Deceased/Expired",
      "Admitted",
      "Deceased/Expired"
    ),
    ed_los = c(120, 200, 5000, 180, 3000, 4321, 60, 4000),
    hosp_disp = c(
      "Deceased/Expired",
      "Deceased/Expired",
      "Deceased/Expired",
      "Discharged",
      "Deceased/Expired",
      "Deceased/Expired",
      "Discharged",
      "Deceased/Expired"
    ),
    hosp_los = c(3000, 4500, 1000, 200, 5000, 4400, 150, 3000),
    autopsy_done = c("Yes", "No", "No", NA, "Yes", "No", NA, "Yes")
  )

  result <- traumar::seqic_indicator_4(
    data = data,
    level = trauma_level,
    ed_disposition = ed_disp,
    ed_LOS = ed_los,
    hospital_disposition = hosp_disp,
    hospital_LOS = hosp_los,
    unique_incident_id = id,
    autopsy = autopsy_done
  )

  testthat::expect_s3_class(result, "tbl_df")
  testthat::expect_true(all(
    c(
      "numerator_4a",
      "denominator_4a",
      "seqic_4a",
      "numerator_4b",
      "denominator_4b",
      "seqic_4b"
    ) %in%
      names(result)
  ))
  testthat::expect_equal(result$numerator_4a, 3) # 3 had autopsy = "Yes"
  testthat::expect_equal(result$denominator_4a, 6) # 6 deaths
  testthat::expect_equal(result$numerator_4b, 3) # 3 long LOS and no autopsy
  testthat::expect_equal(result$denominator_4b, 4)
})

testthat::test_that("seqic_indicator_4 computes correctly with grouping", {
  data <- tibble::tibble(
    id = as.character(1:4),
    trauma_level = c("I", "II", "I", "II"),
    ed_disp = rep("Deceased/Expired", 4),
    ed_los = c(100, 100, 5000, 5000),
    hosp_disp = rep("Deceased/Expired", 4),
    hosp_los = c(100, 100, 5000, 5000),
    autopsy_done = c("Yes", "No", "No", "No"),
    hospital = c("A", "A", "B", "B")
  )

  result <- traumar::seqic_indicator_4(
    data = data,
    level = trauma_level,
    ed_disposition = ed_disp,
    ed_LOS = ed_los,
    hospital_disposition = hosp_disp,
    hospital_LOS = hosp_los,
    unique_incident_id = id,
    autopsy = autopsy_done,
    groups = "hospital"
  )

  testthat::expect_equal(nrow(result), 2)
  testthat::expect_equal(result$numerator_4a[result$hospital == "A"], 1)
  testthat::expect_equal(result$numerator_4b[result$hospital == "B"], 2)
})

testthat::test_that("seqic_indicator_4 returns confidence intervals", {
  data <- tibble::tibble(
    id = as.character(1:5),
    trauma_level = rep("I", 5),
    ed_disp = rep("Deceased/Expired", 5),
    ed_los = c(100, 100, 100, 5000, 5000),
    hosp_disp = rep("Deceased/Expired", 5),
    hosp_los = c(100, 100, 100, 5000, 5000),
    autopsy_done = c("Yes", "Yes", "No", "No", "No")
  )

  result <- traumar::seqic_indicator_4(
    data = data,
    level = trauma_level,
    ed_disposition = ed_disp,
    ed_LOS = ed_los,
    hospital_disposition = hosp_disp,
    hospital_LOS = hosp_los,
    unique_incident_id = id,
    autopsy = autopsy_done,
    calculate_ci = "wilson"
  )

  testthat::expect_true("lower_ci_4a" %in% names(result))
  testthat::expect_true("upper_ci_4b" %in% names(result))
  testthat::expect_type(result$lower_ci_4a, "double")
})

#' @title Validate Set Equality
#'
#' @description
#' This function checks if all elements of an input are within a specified set
#' of valid values. Depending on the specified type, it will either throw an
#' error, issue a warning, or send a message.
#'
#' @inheritParams validate_numeric
#' @param valid_set A vector of valid values.
#' @inheritParams validate_data_pull
#'
#' @return NULL. The function is used for its side effects.
#'
#' @author
#' Nicolas Foss, Ed.D., MS
#'
#' @keywords internal
#' @noRd
#'
validate_set <- function(
  input,
  valid_set,
  type = c("error", "warning", "message"),
  na_ok = TRUE,
  null_ok = TRUE,
  var_name = NULL,
  calls = NULL
) {
  # Validate the type argument
  type <- match.arg(arg = type, choices = c("error", "warning", "message"))

  # Define number of callers to go back
  calls <- ifelse(is.null(calls), 2, calls)

  # Get the input name, optionally using var_name
  if (is.null(var_name)) {
    input_name <- deparse(substitute(input))
  } else {
    # Validate var_name
    validate_character_factor(input = var_name, type = "error", calls = 1)

    # Initialize input_name using var_name
    input_name <- var_name
  }

  # Check if the input is NULL
  if (is.null(input)) {
    if (!null_ok) {
      validate_error_type(
        input = input_name,
        message = "must not be NULL.",
        type = "error",
        calls = calls
      )
    }
    return(NULL)
  }

  # Check for NA values if na_ok is FALSE
  if (!na_ok && any(is.na(input))) {
    validate_error_type(
      input = input_name,
      message = "must not contain NA values.",
      type = "error",
      calls = calls
    )
  }

  # Check if all elements of the input are within the valid set
  invalid_values <- setdiff(x = unique(input), y = valid_set)

  # Check to ensure the invalid_values are not empty
  if (length(invalid_values) > 0) {
    if (length(valid_set) <= 10) {
      # Clip invalid_values down to a length of <= 10
      invalid_values <- head(invalid_values, n = 10)

      # Call the validate_error_type function to handle the message display
      # For small valid_set
      validate_error_type(
        input = input_name,
        message = glue::glue(
          " contains invalid values such as {cli::col_grey(paste0('(', paste0(invalid_values, collapse = ', '), ')'))}. Valid values are: {cli::col_blue(paste0('(', paste0(valid_set, collapse = ', '), ')'))}."
        ),
        type = type,
        calls = calls
      )
    } else {
      # Call the validate_error_type function to handle the message display
      # For large valid_set

      # Clip valid_set down to a length of <= 10
      valid_set <- head(valid_set, n = 10)

      # Clip invalid_values down to a length of <= 10
      invalid_values <- head(invalid_values, n = 10)

      # Modified messaging
      validate_error_type(
        input = input_name,
        message = glue::glue(
          " contains invalid values such as {cli::col_grey(paste0('(', paste0(invalid_values, collapse = ', '), ')'))}. Some examples of valid values are: {cli::col_blue(paste0('(', paste0(valid_set, collapse = ', '), '...', ')'))}."
        ),
        type = type,
        calls = calls
      )
    }
  }
}

#' @title Normalize a Numeric Vector
#'
#' @description
#'
#' This function normalizes a numeric or integer vector using one of two
#' methods: min-max normalization (scales data to the range (0, 1)) or z-score
#' normalization (centers data around 0 with a standard deviation of 1).
#'
#' @param x A numeric or integer vector to be normalized.
#' @param method A character string specifying the normalization method. Options
#'   are `"min_max"` for min-max normalization or `"z_score"` for z-score
#'   normalization. If no method is provided, the default is `"min_max"`.
#'
#' @returns A numeric vector of the same length as `x`, containing the
#'   normalized values.
#'
#' @export
#'
#' @examples
#' # Example data
#' data <- c(10, 20, 30, 40, 50, NA)
#'
#' # Min-max normalization
#' normalize(data, method = "min_max")
#'
#' # Z-score normalization
#' normalize(data, method = "z_score")
#'
#' # Default behavior (min-max normalization)
#' normalize(data)
#'
#' @author Nicolas Foss, Ed.D., MS
#'
normalize <- function(x, method = c("min_max", "z_score")) {
  # Ensure that x is numeric or integer
  validate_class(input = x, class_type = c("numeric", "integer"), logic = "or")

  # Check for empty input ----
  if (length(x) < 1) {
    validate_length(input = x, min_length = 1, type = "warning")
    return(numeric(0))
  }
  # Validate method ----
  method <- match.arg(
    arg = method,
    choices = c("min_max", "z_score"),
    several.ok = FALSE
  )

  # Check if the method is "min_max" ----
  if (method == "min_max") {
    # Perform min-max normalization
    normalized_data <-
      (x - base::min(x, na.rm = TRUE)) /
      (base::max(x, na.rm = TRUE) - base::min(x, na.rm = TRUE))

    # Check if the method is "z_score" ----
  } else if (method == "z_score") {
    # Calculate the mean of x, ignoring NA values
    mean_x <- base::mean(x, na.rm = TRUE)

    # Calculate the standard deviation of x, ignoring NA values ----
    std_dev_x <- stats::sd(x, na.rm = TRUE)

    # Subtract the mean from each value in x ----
    x_minus_mean <- x - mean_x

    # Perform z-score normalization ----
    normalized_data <- x_minus_mean / std_dev_x
  }

  # Return the normalized data ----
  return(normalized_data)
}

test_that("normalize handles min-max normalization correctly", {
  x <- c(10, 20, 30, 40, 50)
  expected <- (x - min(x)) / (max(x) - min(x))
  result <- normalize(x, method = "min_max")
  expect_equal(result, expected)
})

test_that("normalize handles z-score normalization correctly", {
  x <- c(10, 20, 30, 40, 50)
  expected <- (x - mean(x)) / sd(x)
  result <- normalize(x, method = "z_score")
  expect_equal(result, expected)
})

test_that("normalize handles NA values correctly", {
  x <- c(10, 20, NA, 40, 50)

  # Min-max normalization
  expected_min_max <- (x - min(x, na.rm = TRUE)) /
    (max(x, na.rm = TRUE) - min(x, na.rm = TRUE))
  result_min_max <- normalize(x, method = "min_max")
  expect_equal(result_min_max, expected_min_max)

  # Z-score normalization
  mean_x <- mean(x, na.rm = TRUE)
  sd_x <- sd(x, na.rm = TRUE)
  expected_z_score <- (x - mean_x) / sd_x
  result_z_score <- normalize(x, method = "z_score")
  expect_equal(result_z_score, expected_z_score)
})

test_that("normalize defaults to min-max normalization when method is not supplied", {
  x <- c(10, 20, 30, 40, 50)
  expected <- (x - min(x)) / (max(x) - min(x))
  result <- normalize(x)
  expect_equal(result, expected)
})

test_that("normalize handles invalid method input by defaulting to min-max normalization", {
  x <- c(10, 20, 30, 40, 50)
  expected <- (x - min(x)) / (max(x) - min(x))
  expect_error(normalize(x, method = "invalid"))
})

test_that("normalize raises an error for non-numeric inputs", {
  expect_error(normalize(c("a", "b", "c")))
})

test_that("normalize handles integer inputs correctly", {
  x <- c(10L, 20L, 30L, 40L, 50L)
  expected_min_max <- (x - min(x)) / (max(x) - min(x))
  expected_z_score <- (x - mean(x)) / sd(x)

  result_min_max <- normalize(x, method = "min_max")
  result_z_score <- normalize(x, method = "z_score")

  expect_equal(result_min_max, expected_min_max)
  expect_equal(result_z_score, expected_z_score)
})

test_that("normalize handles empty input gracefully", {
  x <- numeric(0)

  expect_warning(
    result <- normalize(x, method = "min_max")
  )
  expect_equal(result, numeric(0))

  expect_warning(
    result <- normalize(x, method = "z_score")
  )
  expect_equal(result, numeric(0))
})

#' @title Relative Mortality Metric (RMM) Calculation
#'
#' @description
#' Calculates the Relative Mortality Metric (RMM) from Napoli et al. (2017)
#' based on patient survival probabilities (Ps) and actual outcomes. The
#' function groups patients into bins based on their survival probability scores
#' (Ps) and computes a weighted mortality metric along with confidence
#' intervals. For more information on the methods used in this function, see as
#' well Schroeder et al. (2019), and Kassar et al. (2016).
#'
#' The Relative Mortality Metric (RMM) quantifies the performance of a center in
#' comparison to the anticipated mortality based on the TRISS national
#' benchmark. The RMM measures the difference between observed and expected
#' mortality, with a range from -1 to 1.
#'
#' - An RMM of 0 indicates that the observed mortality aligns with the expected
#' national benchmark across all acuity levels.
#' - An RMM greater than 0 indicates better-than-expected performance, where
#' the center is outperforming the national benchmark.
#' - An RMM less than 0 indicates under-performance, where the center’s observed
#' mortality is higher than the expected benchmark.
#'
#' This metric helps assess how a center's mortality compares to the national
#' standards, guiding quality improvement efforts. `rmm()` utilizes bootstrap
#' sampling to calculate the confidence intervals via the standard error method.
#'
#' @param data A data frame or tibble containing the data.
#' @param Ps_col The name of the column containing the survival probabilities
#'   (Ps). Should be numeric on a scale from 0 to 1.
#' @param outcome_col The name of the column containing the outcome data. It
#'   should be binary, with values indicating patient survival. A value of `1`
#'   should represent "alive" (survived), while `0` should represent "dead" (did
#'   not survive).  `TRUE/FALSE` are accepted as well. Ensure the column
#'   contains only these possible values.
#' @param group_vars Optional character vector specifying grouping variables for
#'   stratified analysis. If `NULL`, the calculation is performed on the entire
#'   dataset.
#' @param n_samples A numeric value indicating the number of bootstrap samples
#'   to take from the data source.
#' @param Divisor1 A positive numeric value controlling the coarseness of bins
#'   for Ps values below `Threshold_1`. It scales the number of steps from the
#'   start of the dataset up to the `Threshold_1` cut point. Larger values
#'   produce fewer, broader bins; smaller values produce more, narrower bins.
#'   Defaults to `5`.
#' @param Divisor2 A positive numeric value controlling the coarseness of bins
#'   for Ps values between `Threshold_1` and `Threshold_2`. Larger values yield
#'   wider bins, and smaller values yield narrower bins in this range. Defaults
#'   to `5`.
#' @param Threshold_1 A numeric value that defines the lower bound of the
#'   high-survival probability range in `Ps_col`. The function identifies the
#'   first index where `Ps_col` exceeds this value and begins applying smaller
#'   bin widths from that point onward. Defaults to `0.9`, meaning binning
#'   changes once Ps > 0.90.
#' @param Threshold_2 A numeric value that defines the upper bound of the
#'   high-survival probability range in `Ps_col`. The function identifies the
#'   first index where `Ps_col` exceeds this value. Between `Threshold_1` and
#'   `Threshold_2`, finer binning is applied; above `Threshold_2`, binning may
#'   again change. Defaults to `0.99`, meaning the special binning range is
#'   between Ps values of 0.90 and 0.99.
#' @param bootstrap_ci A logical indicating whether to return the relative
#'   mortality metric estimate and 95% confidence intervals using bootstrap
#'   sampling. Default is `TRUE`.
#' @param pivot A logical indicating whether to return the results in a long
#'   format (`pivot = TRUE`) or wide format (`pivot = FALSE`, default). Use with
#'   caution in tandem with `group_vars` if the grouping variable is of a
#'   different class than `rmm()`'s outputs, such as `factor` or `character`
#'   grouping variables.
#' @param seed Optional numeric value to set a random seed for reproducibility.
#'   If `NULL` (default), no seed is set.
#'
#' @details
#' Like other statistical computing functions, `rmm()` is happiest without
#' missing data.  It is best to pass complete probability of survival and
#' mortality outcome data to the function for optimal performance. With smaller
#' datasets, this is especially helpful.  However, `rmm()` will throw a warning
#' about missing values, if any exist in `Ps_col` and/or `outcome_col`.
#'
#' `rmm()` assumes `Ps_col` contains probabilities derived from
#' real-world inputs for the Trauma Injury Severity Score (TRISS) model.
#' Synthetic or low-variability data (especially with small sample sizes) may
#' not reflect the distribution of TRISS-derived survival probabilities. This
#' can result in unstable estimates or function failure due to insufficient
#' dispersion. With small sample sizes, it may be important to use smaller
#' values with the divisor arguments and adjust the thresholds (based on the
#' distribution of the `Ps_col` values) to create bins that better accommodate
#' the data.
#'
#' Due to the use of bootstrap sampling within the function, users should
#' consider setting the random number `seed` within `rmm()` for reproducibility.
#'
#' @returns A tibble containing the Relative Mortality Metric (RMM) and related
#'   statistics:
#'   - `population_RMM_LL`: The lower bound of the 95% confidence interval for
#'     the population RMM.
#'   - `population_RMM`: The final calculated Relative Mortality Metric for the
#'     population existing in `data`.
#'   - `population_RMM_UL`: The upper bound of the 95% confidence interval for
#'     the population RMM.
#'   - `population_CI`: The confidence interval width for the population RMM.
#'   - `bootstrap_RMM_LL`: The lower bound of the 95% confidence interval for
#'     the bootstrap RMM. (optional, if `bootstrap_ci = TRUE`)
#'   - `bootstrap_RMM`: The average RMM value calculated for the bootstrap
#'     sample. (optional, if `bootstrap_ci = TRUE`)
#'   - `bootstrap_RMM_UL`: The upper bound of the 95% confidence interval for
#'     the bootstrap RMM. (optional, if `bootstrap_ci = TRUE`)
#'   - `bootstrap_CI`: The width of the 95% confidence interval for the
#'     bootstrap RMM. (optional, if `bootstrap_ci = TRUE`)
#'   - If `pivot = TRUE`, the results will be in long format with two columns:
#'     `stat` and `value`, where each row corresponds to one of the calculated
#'     statistics.
#'   - If `pivot = FALSE` (default), the results will be returned in wide
#'     format, with each statistic as a separate column.
#'
#' @note
#'
#' This function will produce the most reliable and interpretable results when
#' using a dataset that has one row per patient, with each column being a
#' feature.
#'
#' By default, `rmm()` derives bin cut points from the full dataset’s
#' distribution. This ensures comparability across groups when `group_vars` is
#' used. To tailor results to a specific group (e.g., a single hospital), filter
#' the dataset to that subgroup before calling `rmm()`. The function will then
#' compute bins and related statistics using only that subset’s `Ps_col`
#' distribution. When `group_vars` is used, and ff a group lacks
#' observations within one or more bins, `rm_bin_summary()` will compute
#' statistics only for the bins that contain data. Bins with no observations are
#' excluded from the summary for that group.
#'
#' @export
#'
#' @references
#'
#' Kassar, O.M., Eklund, E.A., Barnhardt, W.F., Napoli, N.J., Barnes, L.E.,
#' Young, J.S. (2016). Trauma survival margin analysis: A dissection of trauma
#' center performance through initial lactate. The American Surgeon, 82(7),
#' 649-653. <doi:10.1177/000313481608200733>
#'
#' Napoli, N. J., Barnhardt, W., Kotoriy, M. E., Young, J. S., & Barnes, L. E.
#' (2017). Relative mortality analysis: A new tool to evaluate clinical
#' performance in trauma centers. IISE Transactions on Healthcare Systems
#' Engineering, 7(3), 181–191. <doi:10.1080/24725579.2017.1325948>
#'
#' Schroeder, P. H., Napoli, N. J., Barnhardt, W. F., Barnes, L. E., & Young, J.
#' S. (2018). Relative mortality analysis of the “golden hour”: A comprehensive
#' acuity stratification approach to address disagreement in current literature.
#' Prehospital Emergency Care, 23(2), 254–262.
#' <doi:10.1080/10903127.2018.1489021>
#'
#' @seealso [probability_of_survival()], [rm_bin_summary()], and
#'   [nonlinear_bins()]
#'
#' @examples
#' # Generate example data
#' set.seed(123)
#'
#' # Parameters
#' # Total number of patients
#' n_patients <- 5000
#'
#' # Arbitrary group labels
#' groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)
#'
#' # Trauma types
#' trauma_type_values <- sample(
#'   x = c("Blunt", "Penetrating"),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # RTS values
#' rts_values <- sample(
#'   x = seq(from = 0, to = 7.8408, by = 0.005),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # patient ages
#' ages <- sample(
#'   x = seq(from = 0, to = 100, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # ISS scores
#' iss_scores <- sample(
#'   x = seq(from = 0, to = 75, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # Generate survival probabilities (Ps)
#' Ps <- traumar::probability_of_survival(
#'   trauma_type = trauma_type_values,
#'   age = ages,
#'   rts = rts_values,
#'   iss = iss_scores
#' )
#'
#' # Simulate survival outcomes based on Ps
#' survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)
#'
#' # Create data frame
#' data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Example usage of the `rmm` function
#' rmm(data = data, Ps_col = Ps,
#'     outcome_col = survival,
#'     Divisor1 = 4,
#'     Divisor2 = 4,
#'     n_samples = 10
#'     )
#'
#' # pivot!
#' rmm(data = data, Ps_col = Ps,
#'     outcome_col = survival,
#'     Divisor1 = 4,
#'     Divisor2 = 4,
#'     n_samples = 10,
#'     pivot = TRUE
#'     )
#'
#' # Create example grouping variable (e.g., hospital)
#' hospital <- sample(c("Hospital A", "Hospital B"), n_patients, replace = TRUE)
#'
#' # Create data frame
#' data <- data.frame(
#'   Ps = Ps,
#'   survival = survival_outcomes,
#'   hospital = hospital
#' ) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Example usage of the `rmm` function with grouping by hospital
#' rmm(
#'   data = data,
#'   Ps_col = Ps,
#'   outcome_col = survival,
#'   group_vars = "hospital",
#'   Divisor1 = 4,
#'   Divisor2 = 4,
#'   n_samples = 10
#' )
#'
#' # Pivoted output for easier visualization
#' rmm(
#'   data = data,
#'   Ps_col = Ps,
#'   outcome_col = survival,
#'   group_vars = "hospital",
#'   Divisor1 = 4,
#'   Divisor2 = 4,
#'   n_samples = 10,
#'   pivot = TRUE
#' )
#'
#' @author Nicolas Foss, Ed.D, MS, original implementation in MATLAB by Nicholas
#'   J. Napoli, Ph.D., MS
#'
rmm <- function(
  data,
  Ps_col,
  outcome_col,
  group_vars = NULL,
  n_samples = 100,
  Divisor1 = 5,
  Divisor2 = 5,
  Threshold_1 = 0.9,
  Threshold_2 = 0.99,
  bootstrap_ci = TRUE,
  pivot = FALSE,
  seed = NULL
) {
  # Validation checks using `cli` for robust error messaging: ----
  # Ensures the input data is a data frame or tibble.
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error"
  )

  # check the n_samples value ----
  validate_class(
    input = n_samples,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error"
  )

  # validate Divisor1 ----
  validate_class(
    input = Divisor1,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error"
  )

  # validate Divisor2 ----
  validate_class(
    input = Divisor2,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error"
  )

  # validate Threshold_1 ----
  validate_numeric(input = Threshold_1, min = 0, max = 1, type = "error")

  # validate Threshold_2 ----
  validate_numeric(input = Threshold_2, min = 0, max = 1, type = "error")

  # Validate the CI argument ----
  validate_class(input = bootstrap_ci, class_type = "logical", type = "error")

  # Validate the pivot argument ----
  validate_class(input = pivot, class_type = "logical", type = "error")

  # Ensure Ps_col and outcome_col arguments are provided  ----
  # with tailored error messages
  if (missing(Ps_col) && missing(outcome_col)) {
    cli::cli_abort(
      "Both {.var Ps_col} and {.var outcome_col} arguments must be provided."
    )
  } else if (missing(Ps_col)) {
    cli::cli_abort("The {.var Ps_col} argument must be provided.")
  } else if (missing(outcome_col)) {
    cli::cli_abort("The {.var outcome_col} argument must be provided.")
  }

  # Pull and check the outcome column ----
  binary_data <- validate_data_pull(
    input = data,
    col = {{ outcome_col }},
    calls = 5,
    var_name = "outcome_col"
  )

  # Ensure the column is either logical or numeric ----
  validate_class(
    input = binary_data,
    class_type = c("numeric", "logical", "integer"),
    logic = "or",
    type = "error",
    var_name = "outcome_col"
  )

  # Get unique non-missing values to use in subsequent data validation ----
  non_missing <- stats::na.omit(binary_data)

  # Validate type and values ----
  if (is.logical(non_missing)) {
    # Logical vector: ensure only TRUE/FALSE (no coercion needed)
    validate_set(
      input = non_missing,
      valid_set = c(TRUE, FALSE),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.numeric(non_missing)) {
    # Numeric vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0, 1),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.integer(non_missing)) {
    # Integer vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0L, 1L),
      type = "error",
      var_name = "outcome_col"
    )
  } else {
    # Not logical or numeric ----
    cli::cli_abort(
      "The {.var outcome_col} must be either logical (TRUE/FALSE) or numeric/integer (1/0)."
    )
  }

  # Warn if missing ----
  validate_complete(
    input = binary_data,
    type = "warning",
    var_name = "outcome_col"
  )

  # Check if Ps column is numeric ----
  # dplyr::pull the Ps data
  Ps_check <- validate_data_pull(
    input = data,
    col = {{ Ps_col }},
    var_name = "Ps_col",
    calls = 5
  )

  # check the Ps_check remains continuous ----
  # Check if Ps column is continuous (values between 0 and 1)
  validate_numeric(
    input = Ps_check,
    min = 0,
    max = 1,
    type = "error",
    var_name = "Ps_col"
  )

  # Warn if any missings in Ps_col ----
  validate_complete(input = Ps_check, type = "warning", var_name = "Ps_col")

  # Validate the seed argument ----
  validate_class(
    input = seed,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error",
    null_ok = TRUE
  )

  # Set the random seed if a value is given ----
  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Check if all elements in group_vars are strings ----
  # (i.e., character vectors)
  validate_character_factor(input = group_vars, type = "error", null_ok = TRUE)

  # Check if all group_vars exist in the data ----
  validate_names(
    input = data,
    check_names = group_vars,
    type = "error",
    var_name = "group_vars",
    null_ok = TRUE
  )

  # Treat the column-names-as-strings as symbols ----
  if (!is.null(group_vars)) {
    group_vars_syms <- rlang::syms(group_vars)
  } else if (is.null(group_vars)) {
    group_vars_syms <- NULL
  }

  # Assume same distribution of POS scores over years ----
  # Dynamically assign bins for POS scores using non-linear process
  # specified by Napoli et al. 2017
  # those methods are adapted using this function

  # get the population level bins ----
  bin_data <- nonlinear_bins(
    data = data,
    Ps_col = {{ Ps_col }},
    outcome_col = {{ outcome_col }},
    group_vars = group_vars,
    divisor1 = Divisor1,
    divisor2 = Divisor2,
    threshold_1 = Threshold_1,
    threshold_2 = Threshold_2
  )

  if (bootstrap_ci) {
    # Bootstrap process ----
    bootstrap_data <- data |>
      dplyr::select({{ Ps_col }}, {{ outcome_col }}, !!!group_vars_syms) |> # Select only relevant columns
      dplyr::group_by(!!!group_vars_syms) |>
      infer::generate(reps = n_samples, type = "bootstrap") |> # Generate bootstrap samples
      dplyr::ungroup()

    # bootstrapping to get bins for the population to then create ----
    # the confidence intervals
    # Nest data by replicate and optional group_vars and apply nonlinear_bins
    bin_data_boot <- bootstrap_data |>
      tidyr::nest(data = -replicate) |> # Nest data
      dplyr::mutate(
        bins = purrr::map(
          data,
          ~ nonlinear_bins(
            .x,
            Ps_col = {{ Ps_col }},
            outcome_col = {{ outcome_col }},
            group_vars = group_vars,
            divisor1 = Divisor1,
            divisor2 = Divisor2,
            threshold_1 = Threshold_1,
            threshold_2 = Threshold_2
          )
        )
      ) |>
      dplyr::mutate(
        bins_temp = purrr::map(bins, ~ .x$bin_stats)
      ) |>
      tidyr::unnest(bins_temp) |>
      dplyr::select(-bins)
  }

  # Initialize the bin_df to hold bin statistics ----
  bin_df <- bin_data$bin_stats |>
    dplyr::select(!!!group_vars_syms, bin_number, bin_start, bin_end) |>
    # Calculate the midpoint of each bin using the start and end points
    dplyr::mutate(midpoint = (bin_end + bin_start) / 2) |>
    dplyr::arrange(!!!group_vars_syms, bin_number) # Sort the bins by bin_number

  if (bootstrap_ci) {
    # Initialize the bind_df_boot to hold bin statistics for each replicate ----
    # Initialize the bin_df with bootstrap samples
    bin_df_boot <- bin_data_boot |>
      dplyr::select(
        !!!group_vars_syms,
        replicate,
        bin_number,
        bin_start,
        bin_end
      ) |>
      # Calculate the midpoint of each bin for each bootstrap replicate ----
      dplyr::mutate(midpoint = (bin_end + bin_start) / 2) |>
      dplyr::arrange(!!!group_vars_syms, replicate, bin_number) # Sort by replicate and bin_number
  }
  # Summarize bin-level statistics: ----
  # - TA_b: Total alive (patients in the bin that survived)
  # - TD_b: Total dead (patients in the bin that did not survive)
  # - N_b: Total number of observations (patients in the bin)
  # - EM_b: Estimated mortality for the bin (TD_b / (TA_b + TD_b))
  bin_summary <- bin_data$bin_stats |>
    dplyr::group_by(!!!group_vars_syms, bin_number) |> # Perform this calculation for each bin
    dplyr::summarize(
      TA_b = sum(alive, na.rm = TRUE), # Total number of survivors in the bin
      TD_b = sum(dead, na.rm = TRUE), # Total number of deaths in the bin
      N_b = sum(count, na.rm = TRUE), # Total number of patients in the bin
      EM_b = TD_b / N_b, # Estimated mortality (TD_b / total patients)
      AntiS_b = AntiS_b, # keep the predicted survival data
      AntiM_b = AntiM_b, # keep the predicted mortality data
      .groups = "drop"
    ) |>
    dplyr::arrange(!!!group_vars_syms, bin_number) # Arrange the bins by bin_number

  if (bootstrap_ci) {
    # Summarize bin-level statistics for the boostrapped data: ----
    # - TA_b: Total alive (patients in the bin that survived)
    # - TD_b: Total dead (patients in the bin that did not survive)
    # - N_b: Total number of observations (patients in the bin)
    # - EM_b: Estimated mortality for the bin (TD_b / (TA_b + TD_b))
    bin_summary_boot <- bin_data_boot |>
      # Perform this calculation for each replicate and bin
      dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
      dplyr::summarize(
        TA_b = sum(alive, na.rm = TRUE), # Total number of survivors in the bin
        TD_b = sum(dead, na.rm = TRUE), # Total number of deaths in the bin
        N_b = sum(count, na.rm = TRUE), # Total number of patients in the bin
        EM_b = TD_b / N_b, # Estimated mortality (TD_b / total patients)
        AntiS_b = AntiS_b, # keep the predicted survival data
        AntiM_b = AntiM_b, # keep the predicted mortality data
        .groups = "drop"
      ) |>
      dplyr::arrange(!!!group_vars_syms, replicate, bin_number) # Arrange the bins by bin_number
  }

  if (is.null(group_vars)) {
    # Join the bin statistics (bin_summary) with the bin_df for further ----
    # calculations. The merged data will contain the bin information and
    # corresponding statistics
    # Not using AntiM_b = -1 * midpoint + 1
    # i.e., Anticipated mortality (1 - midpoint, reversed scale) ----
    bin_stats <- bin_summary |>
      dplyr::left_join(bin_df, by = dplyr::join_by(bin_number)) |>
      dplyr::group_by(!!!group_vars_syms, bin_number) |>
      dplyr::mutate(
        R_b = bin_end - bin_start # Calculate the bin width (R_b = end - start)
      ) |>
      dplyr::ungroup()
  } else {
    bin_stats <- bin_summary |>
      dplyr::left_join(
        bin_df,
        by = dplyr::join_by(!!!rlang::syms(group_vars), bin_number)
      ) |>
      dplyr::group_by(!!!group_vars_syms, bin_number) |>
      dplyr::mutate(
        R_b = bin_end - bin_start # Calculate the bin width (R_b = end - start)
      ) |>
      dplyr::ungroup()
  }

  if (bootstrap_ci) {
    if (is.null(group_vars)) {
      # For the bootstrapped data ----
      # Join the bin statistics (bin_summary) with the bin_df_boot for further calculations
      # The merged data will contain the bin information and corresponding statistics
      # Not using AntiM_b = -1 * midpoint + 1
      # i.e., Anticipated mortality (1 - midpoint, reversed scale)
      bin_stats_boot <- bin_summary_boot |>
        dplyr::left_join(
          bin_df_boot,
          by = dplyr::join_by(replicate, bin_number)
        ) |>
        dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
        dplyr::mutate(
          R_b = bin_end - bin_start, # Calculate the bin width (R_b = end - start)
        ) |>
        dplyr::ungroup()
    } else {
      bin_stats_boot <- bin_summary_boot |>
        dplyr::left_join(
          bin_df_boot,
          by = dplyr::join_by(!!!rlang::syms(group_vars), replicate, bin_number)
        ) |>
        dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
        dplyr::mutate(
          R_b = bin_end - bin_start # Calculate the bin width (R_b = end - start)
        ) |>
        dplyr::ungroup()
    }
  }

  # Calculate the Relative Mortality Metric (RMM): ----
  # RMM is calculated by:
  # - Computing the weighted difference between anticipated and observed mortality.
  # - Normalizing by the weighted anticipated mortality.
  rmm_result <- bin_stats |>
    dplyr::group_by(!!!group_vars_syms) |>
    dplyr::summarize(
      numerator = sum(R_b * (AntiM_b - EM_b), na.rm = TRUE), # Weighted numerator (difference between anticipated and observed mortality)
      denominator = sum(R_b * AntiM_b, na.rm = TRUE), # Weighted denominator (anticipated mortality)
      population_RMM = numerator / denominator, # Final RMM calculation
      population_RMM = pmin(
        1,
        pmax(-1, population_RMM, na.rm = TRUE),
        na.rm = TRUE
      ), # Ensure RMM is within [-1, 1]
      population_CI = 1.96 *
        sqrt(
          (sum(AntiM_b, na.rm = TRUE) * sum(AntiS_b, na.rm = TRUE)) /
            sum(N_b, na.rm = TRUE)
        ),
      # Lower bound of 95% CI
      population_RMM_LL = pmax(
        -1,
        population_RMM - population_CI,
        na.rm = TRUE
      ), # Clip LL
      # Upper bound of 95% CI
      population_RMM_UL = pmin(1, population_RMM + population_CI, na.rm = TRUE), # Clip UL,
      .groups = "drop"
    ) |>
    dplyr::relocate(population_RMM_LL, .before = population_RMM) |>
    dplyr::relocate(population_CI, .after = population_RMM_UL)

  if (bootstrap_ci) {
    # For the bootstrapped data ----
    # Calculate the Relative Mortality Metric (RMM) and its upper and lower
    # confidence intervals:
    # RMM is calculated by: ----
    # - Computing the weighted difference between anticipated and observed mortality.
    # - Normalizing by the weighted anticipated mortality.
    # The confidence intervals are adjusted based on the weighted error bound.
    rmm_result_boot <- bin_stats_boot |>
      dplyr::group_by(!!!group_vars_syms, replicate) |>
      dplyr::summarize(
        numerator = sum(R_b * (AntiM_b - EM_b), na.rm = TRUE), # Weighted numerator (difference between anticipated and observed mortality)
        denominator = sum(R_b * AntiM_b, na.rm = TRUE), # Weighted denominator (anticipated mortality)
        RMM = numerator / denominator, # Final RMM calculation
        RMM = pmin(1, pmax(-1, RMM, na.rm = TRUE), na.rm = TRUE), # Ensure RMM is within [-1, 1]
        .groups = "drop"
      ) |>
      dplyr::ungroup()

    # Calculate mean, standard deviation, and 95% confidence intervals ----
    rmm_result_ci <- rmm_result_boot |>
      dplyr::group_by(!!!group_vars_syms) |>
      dplyr::summarize(
        bootstrap_RMM = mean(RMM, na.rm = TRUE), # Mean RMM
        sd_bootstrap_RMM = sd(RMM, na.rm = TRUE), # Standard deviation of RMM
        se_bootstrap_RMM = sd_bootstrap_RMM / sqrt(n_samples),
        bootstrap_CI = 1.96 * se_bootstrap_RMM, # Standard error
        # Lower bound of 95% CI
        bootstrap_RMM_LL = pmax(-1, bootstrap_RMM - bootstrap_CI, na.rm = TRUE), # Clip LL
        # Upper bound of 95% CI
        bootstrap_RMM_UL = pmin(1, bootstrap_RMM + bootstrap_CI, na.rm = TRUE), # Clip UL
        .groups = "drop"
      ) |>
      dplyr::ungroup()
  }

  if (bootstrap_ci) {
    if (is.null(group_vars)) {
      # add the confidence intervals from the bootstrap distribution
      # to the final result
      rmm_result_final <- rmm_result |>
        dplyr::bind_cols(rmm_result_ci) |>
        dplyr::relocate(bootstrap_RMM_LL, .before = bootstrap_RMM) |>
        dplyr::relocate(bootstrap_RMM_UL, .after = bootstrap_RMM) |>
        dplyr::relocate(bootstrap_CI, .after = bootstrap_RMM_UL) |>
        dplyr::select(
          -numerator,
          -denominator,
          -sd_bootstrap_RMM,
          -se_bootstrap_RMM
        )
    } else {
      rmm_result_final <- rmm_result |>
        dplyr::left_join(
          rmm_result_ci,
          by = dplyr::join_by(!!!rlang::syms(group_vars))
        ) |>
        dplyr::relocate(bootstrap_RMM_LL, .before = bootstrap_RMM) |>
        dplyr::relocate(bootstrap_RMM_UL, .after = bootstrap_RMM) |>
        dplyr::relocate(bootstrap_CI, .after = bootstrap_RMM_UL) |>
        dplyr::select(
          -numerator,
          -denominator,
          -sd_bootstrap_RMM,
          -se_bootstrap_RMM
        )
    }
  } else {
    rmm_result_final <- rmm_result |>
      dplyr::select(
        -numerator,
        -denominator
      )
  }

  # Return the final result containing the RMM and its confidence intervals ----
  # optionally, pivot
  if (pivot && is.null(group_vars)) {
    rmm_result_final <- rmm_result_final |>
      tidyr::pivot_longer(
        tidyselect::everything(),
        names_to = "stat",
        values_to = "value"
      )

    return(rmm_result_final)
  } else if (pivot && !is.null(group_vars)) {
    rmm_result_final <- rmm_result_final |>
      tidyr::pivot_longer(
        -dplyr::all_of(group_vars),
        names_to = "stat",
        values_to = "value"
      )

    return(rmm_result_final)
  } else if (!pivot) {
    # wide result
    return(rmm_result_final)
  }
}

#' @title Bin-Level Summary for Relative Mortality Metric (RMM)
#'
#' @description
#'
#' Calculates a bin-level summary for the Relative Mortality Metric (RMM) from
#' Napoli et al. (2017) by grouping data into bins based on survival
#' probabilities (Ps) and summarizing outcomes within each bin. This function
#' returns statistics such as total alive, total dead, estimated mortality,
#' anticipated mortality, and confidence intervals for each bin. For more
#' information on the methods used in this function, see as well Schroeder et
#' al. (2019), and Kassar et al. (2016).
#'
#' The Relative Mortality Metric (RMM) quantifies the performance of a center in
#' comparison to the anticipated mortality based on the TRISS national
#' benchmark. The RMM measures the difference between observed and expected
#' mortality, with a range from -1 to 1.
#'
#' - An RMM of 0 indicates that the observed mortality aligns with the expected
#' national benchmark across all acuity levels.
#' - An RMM greater than 0 indicates better-than-expected performance, where
#' the center is outperforming the national benchmark.
#' - An RMM less than 0 indicates under-performance, where the center’s observed
#' mortality is higher than the expected benchmark.
#'
#' This metric helps assess how a center's mortality compares to the national
#' standards, guiding quality improvement efforts.`rm_bin_summary()` utilizes
#' bootstrap sampling to calculate the confidence intervals via the standard
#' error method.
#'
#' @inheritParams rmm
#'
#' @details
#' Like other statistical computing functions, `rm_bin_summary()` is happiest
#' without missing data.  It is best to pass complete probability of survival
#' and mortality outcome data to the function for optimal performance. With
#' smaller datasets, this is especially helpful.  However, `rm_bin_summary()`
#' will throw a warning about missing values, if any exist in `Ps_col` and/or
#' `outcome_col`.
#'
#' `rm_bin_summary()` assumes `Ps_col` contains probabilities derived from
#' real-world inputs for the Trauma Injury Severity Score (TRISS) model.
#' Synthetic or low-variability data (especially with small sample sizes) may
#' not reflect the distribution of TRISS-derived survival probabilities. This
#' can result in unstable estimates or function failure due to insufficient
#' dispersion. With small sample sizes, it may be important to use smaller
#' values with the divisor arguments and adjust the thresholds (based on the
#' distribution of the `Ps_col` values) to create bins that better accommodate
#' the data.
#'
#' Due to the use of bootstrap sampling within the function, users should
#' consider setting the random number seed within `rm_bin_summary()` using the
#' `seed` argument for reproducibility.
#'
#' @returns A tibble containing bin-level statistics including:
#'   - `bin_number`: The bin to which each record was assigned.
#'   - `TA_b`: Total alive in each bin (number of patients who survived).
#'   - `TD_b`: Total dead in each bin (number of patients who did not survive).
#'   - `N_b`: Total number of patients in each bin.
#'   - `EM_b`: Estimated mortality rate for each bin (TD_b / (TA_b + TD_b)).
#'   - `AntiS_b`: The anticipated survival rate for each bin.
#'   - `AntiM_b`: The anticipated mortality rate for each bin.
#'   - `bin_start`: The lower bound of the survival probability range for each
#'      bin.
#'   - `bin_end`: The upper bound of the survival probability range for each
#'      bin.
#'   - `midpoint`: The midpoint of the bin range (calculated as
#'      (bin_start + bin_end) / 2).
#'   - `R_b`: The width of each bin (bin_end - bin_start).
#'   - `population_RMM_LL`: The lower bound of the 95% confidence interval for
#'      the population RMM.
#'   - `population_RMM`: The final calculated Relative Mortality Metric for the
#'      population existing in `data`.
#'   - `population_RMM_UL`: The upper bound of the 95% confidence interval for
#'      the population RMM.
#'   - `population_CI`: The confidence interval width for the population RMM.
#'   - `bootstrap_RMM_LL`: The lower bound of the 95% confidence interval for
#'      the bootstrap RMM. (optional, if `bootstrap_ci = TRUE`)
#'   - `bootstrap_RMM`: The average RMM value calculated for the bootstrap
#'      sample. (optional, if `bootstrap_ci = TRUE`)
#'   - `bootstrap_RMM_UL`: The upper bound of the 95% confidence interval for
#'      the bootstrap RMM. (optional, if `bootstrap_ci = TRUE`)
#'   - `bootstrap_CI`: The width of the 95% confidence interval for the
#'      bootstrap RMM. (optional, if `bootstrap_ci = TRUE`)
#'
#' @note
#'
#' This function will produce the most reliable and interpretable results when
#' using a dataset that has one row per patient, with each column being a
#' feature.
#'
#' By default, `rm_bin_summary()` derives bin cut points from the full dataset’s
#' distribution. This ensures comparability across groups when `group_vars` is
#' used. To tailor results to a specific group (e.g., a single hospital), filter
#' the dataset to that subgroup before calling `rm_bin_summary()`. The function
#' will then compute bins and related statistics using only that subset’s
#' `Ps_col` distribution. When `group_vars` is used, and ff a group lacks
#' observations within one or more bins, `rm_bin_summary()` will compute
#' statistics only for the bins that contain data. Bins with no observations are
#' excluded from the summary for that group.

#'
#' @export
#'
#' @references
#'
#' Kassar, O.M., Eklund, E.A., Barnhardt, W.F., Napoli, N.J., Barnes, L.E.,
#' Young, J.S. (2016). Trauma survival margin analysis: A dissection of trauma
#' center performance through initial lactate. The American Surgeon, 82(7),
#' 649-653. <doi:10.1177/000313481608200733>
#'
#' Napoli, N. J., Barnhardt, W., Kotoriy, M. E., Young, J. S., & Barnes, L. E.
#' (2017). Relative mortality analysis: A new tool to evaluate clinical
#' performance in trauma centers. IISE Transactions on Healthcare Systems
#' Engineering, 7(3), 181–191. <doi:10.1080/24725579.2017.1325948>
#'
#' Schroeder, P. H., Napoli, N. J., Barnhardt, W. F., Barnes, L. E., & Young, J.
#' S. (2018). Relative mortality analysis of the “golden hour”: A comprehensive
#' acuity stratification approach to address disagreement in current literature.
#' Prehospital Emergency Care, 23(2), 254–262.
#' <doi:10.1080/10903127.2018.1489021>
#'
#' @seealso [probability_of_survival()], [rmm()], and [nonlinear_bins()]
#'
#' @examples
#' # Generate example data
#' set.seed(123)
#'
#' # Parameters
#' # Total number of patients
#' n_patients <- 5000
#'
#' # Arbitrary group labels
#' groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)
#'
#' # Trauma types
#' trauma_type_values <- sample(
#'   x = c("Blunt", "Penetrating"),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # RTS values
#' rts_values <- sample(
#'   x = seq(from = 0, to = 7.8408, by = 0.005),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # patient ages
#' ages <- sample(
#'   x = seq(from = 0, to = 100, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # ISS scores
#' iss_scores <- sample(
#'   x = seq(from = 0, to = 75, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # Generate survival probabilities (Ps)
#' Ps <- traumar::probability_of_survival(
#'   trauma_type = trauma_type_values,
#'   age = ages,
#'   rts = rts_values,
#'   iss = iss_scores
#' )
#'
#' # Simulate survival outcomes based on Ps
#' survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)
#'
#' # Create data frame
#' data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Example usage of the `rm_bin_summary()` function
#' rm_bin_summary(
#'   data = data,
#'   Ps_col = Ps,
#'   outcome_col = survival,
#'   n_samples = 10,
#'   Divisor1 = 4,
#'   Divisor2 = 4
#' )
#'
#' # Create example grouping variable (e.g., hospital)
#' hospital <- sample(c("Hospital A", "Hospital B"), n_patients, replace = TRUE)
#'
#' # Create data frame
#' data <- data.frame(
#'   Ps = Ps,
#'   survival = survival_outcomes,
#'   hospital = hospital
#' ) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Example usage of the `rm_bin_summary()` function with grouping
#' rm_bin_summary(
#'   data = data,
#'   Ps_col = Ps,
#'   outcome_col = survival,
#'   group_vars = "hospital", # Stratifies by hospital
#'   n_samples = 10,
#'   Divisor1 = 4,
#'   Divisor2 = 4
#' )
#'
#' @author Nicolas Foss, Ed.D, MS, original implementation in MATLAB by Nicholas
#'   J. Napoli, Ph.D., MS
#'
rm_bin_summary <- function(
  data,
  Ps_col,
  outcome_col,
  group_vars = NULL,
  n_samples = 100,
  Divisor1 = 5,
  Divisor2 = 5,
  Threshold_1 = 0.9,
  Threshold_2 = 0.99,
  bootstrap_ci = TRUE,
  seed = NULL
) {
  # Validation checks using `cli` for robust error messaging: ----
  # Ensures the input data is a data frame or tibble.
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error"
  )

  # check the n_samples value ----
  validate_class(
    input = n_samples,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error"
  )

  # validate Divisor1 ----
  validate_class(
    input = Divisor1,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error"
  )

  # validate Divisor2 ----
  validate_class(
    input = Divisor2,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error"
  )

  # validate Threshold_1 ----
  validate_numeric(input = Threshold_1, min = 0, max = 1, type = "error")

  # validate Threshold_2 ----
  validate_numeric(input = Threshold_2, min = 0, max = 1, type = "error")

  # Validate the CI argument ----
  validate_class(input = bootstrap_ci, class_type = "logical", type = "error")

  # Ensure Ps_col and outcome_col arguments are provided ----
  # with tailored error messages
  if (missing(Ps_col) && missing(outcome_col)) {
    cli::cli_abort(
      "Both {.var Ps_col} and {.var outcome_col} arguments must be provided."
    )
  } else if (missing(Ps_col)) {
    cli::cli_abort("The {.var Ps_col} argument must be provided.")
  } else if (missing(outcome_col)) {
    cli::cli_abort("The {.var outcome_col} argument must be provided.")
  }

  # Pull and check the outcome column ----
  binary_data <- validate_data_pull(
    input = data,
    col = {{ outcome_col }},
    calls = 5,
    var_name = "outcome_col"
  )

  # Ensure the column is either logical or numeric ----
  validate_class(
    input = binary_data,
    class_type = c("numeric", "logical", "integer"),
    logic = "or",
    type = "error",
    var_name = "outcome_col"
  )

  # Get unique non-missing values ----
  non_missing <- stats::na.omit(binary_data)

  # Validate type and values ----
  if (is.logical(non_missing)) {
    # Logical vector: ensure only TRUE/FALSE (no coercion needed) ----
    validate_set(
      input = non_missing,
      valid_set = c(TRUE, FALSE),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.numeric(non_missing)) {
    # Numeric vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0, 1),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.integer(non_missing)) {
    # Integer vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0L, 1L),
      type = "error",
      var_name = "outcome_col"
    )
  } else {
    # Not logical or numeric ----
    cli::cli_abort(
      "The {.var outcome_col} must be either logical (TRUE/FALSE) or numeric (1/0)."
    )
  }

  # Warn if missing ----
  validate_complete(
    input = binary_data,
    type = "warning",
    var_name = "outcome_col"
  )

  # Check if Ps column is numeric ----
  # dplyr::pull the Ps data
  Ps_check <- validate_data_pull(
    input = data,
    col = {{ Ps_col }},
    var_name = "Ps_col",
    calls = 5
  )

  # check the Ps_check remains continuous ----
  # Check if Ps column is continuous (values between 0 and 1)
  validate_numeric(
    input = Ps_check,
    min = 0,
    max = 1,
    type = "error",
    var_name = "Ps_col"
  )

  # Warn if any missings in Ps_col ----
  validate_complete(input = Ps_check, type = "warning", var_name = "Ps_col")

  # Validate the seed argument ----
  validate_class(
    input = seed,
    class_type = c("numeric", "integer"),
    logic = "or",
    type = "error",
    null_ok = TRUE
  )

  # Set the random seed if a value is given ----
  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Check if all elements in group_vars are strings ----
  # (i.e., character vectors)
  validate_character_factor(input = group_vars, type = "error", null_ok = TRUE)

  # Check if all group_vars exist in the data ----
  validate_names(
    input = data,
    check_names = group_vars,
    type = "error",
    var_name = "group_vars",
    null_ok = TRUE
  )

  # Treat the column-names-as-strings as symbols ----
  if (!is.null(group_vars)) {
    group_vars_syms <- rlang::syms(group_vars)
  } else if (is.null(group_vars)) {
    group_vars_syms <- NULL
  }

  # Assume same distribution of POS scores over years ----
  # Dynamically assign bins for POS scores using non-linear process
  # specified by Napoli et al. 2017
  # those methods are adapted using this function

  # get the population level bins ----
  bin_data <- nonlinear_bins(
    data,
    Ps_col = {{ Ps_col }},
    outcome_col = {{ outcome_col }},
    group_vars = group_vars,
    divisor1 = Divisor1,
    divisor2 = Divisor2,
    threshold_1 = Threshold_1,
    threshold_2 = Threshold_2
  )

  if (bootstrap_ci) {
    # Bootstrap process ----
    bootstrap_data <- data |>
      dplyr::select({{ Ps_col }}, {{ outcome_col }}, !!!group_vars_syms) |> # Select only relevant columns
      dplyr::group_by(!!!group_vars_syms) |>
      infer::generate(reps = n_samples, type = "bootstrap") |> # Generate bootstrap samples
      dplyr::ungroup()

    # bootstrapping to get bins for the population to then create ----
    # the confidence intervals
    # Nest data by replicate and apply nonlinear_bins
    bin_data_boot <- bootstrap_data |>
      tidyr::nest(data = -replicate) |> # Nest data by replicate
      dplyr::mutate(
        bins = purrr::map(
          data,
          ~ nonlinear_bins(
            .x,
            Ps_col = {{ Ps_col }},
            outcome_col = {{ outcome_col }},
            group_vars = group_vars,
            divisor1 = Divisor1,
            divisor2 = Divisor2,
            threshold_1 = Threshold_1,
            threshold_2 = Threshold_2
          )
        )
      ) |>
      dplyr::mutate(
        bins_temp = purrr::map(bins, ~ .x$bin_stats)
      ) |>
      tidyr::unnest(bins_temp) |>
      dplyr::select(-bins)
  }

  # Initialize the bin_df to hold bin statistics ----
  bin_df <- bin_data$bin_stats |>
    dplyr::select(!!!group_vars_syms, bin_number, bin_start, bin_end) |>
    # Calculate the midpoint of each bin using the start and end points
    dplyr::mutate(midpoint = (bin_end + bin_start) / 2) |>
    dplyr::arrange(!!!group_vars_syms, bin_number) # Sort the bins by bin_number

  if (bootstrap_ci) {
    # Initialize the bind_df_boot to hold bin statistics for each replicate ----
    # Initialize the bin_df with bootstrap samples
    bin_df_boot <- bin_data_boot |>
      dplyr::select(
        !!!group_vars_syms,
        replicate,
        bin_number,
        bin_start,
        bin_end
      ) |>
      # Calculate the midpoint of each bin for each bootstrap replicate ----
      dplyr::mutate(midpoint = (bin_end + bin_start) / 2) |>
      dplyr::arrange(!!!group_vars_syms, replicate, bin_number) # Sort by replicate and bin_number
  }

  # Summarize bin-level statistics: ----
  # - TA_b: Total alive (patients in the bin that survived)
  # - TD_b: Total dead (patients in the bin that did not survive)
  # - N_b: Total number of observations (patients in the bin)
  # - EM_b: Estimated mortality for the bin (TD_b / (TA_b + TD_b))
  bin_summary <- bin_data$bin_stats |>
    dplyr::group_by(!!!group_vars_syms, bin_number) |> # Perform this calculation for each bin
    dplyr::summarize(
      TA_b = sum(alive, na.rm = TRUE), # Total number of survivors in the bin
      TD_b = sum(dead, na.rm = TRUE), # Total number of deaths in the bin
      N_b = sum(count, na.rm = TRUE), # Total number of patients in the bin
      EM_b = TD_b / N_b, # Estimated mortality (TD_b / total patients)
      AntiS_b = AntiS_b, # keep the predicted survival data
      AntiM_b = AntiM_b, # keep the predicted mortality data
      .groups = "drop"
    ) |>
    dplyr::arrange(!!!group_vars_syms, bin_number) # Arrange the bins by bin_number

  if (bootstrap_ci) {
    # Summarize bin-level statistics for the boostrapped data: ----
    # - TA_b: Total alive (patients in the bin that survived)
    # - TD_b: Total dead (patients in the bin that did not survive)
    # - N_b: Total number of observations (patients in the bin)
    # - EM_b: Estimated mortality for the bin (TD_b / (TA_b + TD_b))
    bin_summary_boot <- bin_data_boot |>
      # Perform this calculation for each replicate and bin ----
      dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
      dplyr::summarize(
        TA_b = sum(alive, na.rm = TRUE), # Total number of survivors in the bin
        TD_b = sum(dead, na.rm = TRUE), # Total number of deaths in the bin
        N_b = sum(count, na.rm = TRUE), # Total number of patients in the bin
        EM_b = TD_b / N_b, # Estimated mortality (TD_b / total patients)
        AntiS_b = AntiS_b, # keep the predicted survival data
        AntiM_b = AntiM_b, # keep the predicted mortality data
        .groups = "drop"
      ) |>
      dplyr::arrange(!!!group_vars_syms, replicate, bin_number) # Arrange the bins by bin_number
  }

  if (is.null(group_vars)) {
    # Join the bin statistics (bin_summary) with the bin_df for further ----
    # calculations. The merged data will contain the bin information and
    # corresponding statistics
    # Not using AntiM_b = -1 * midpoint + 1
    # i.e., Anticipated mortality (1 - midpoint, reversed scale)
    bin_stats <- bin_summary |>
      dplyr::left_join(bin_df, by = dplyr::join_by(bin_number)) |>
      dplyr::group_by(!!!group_vars_syms, bin_number) |>
      dplyr::mutate(
        R_b = bin_end - bin_start # Calculate the bin width (R_b = end - start)
      ) |>
      dplyr::ungroup()
  } else {
    bin_stats <- bin_summary |>
      dplyr::left_join(
        bin_df,
        by = dplyr::join_by(!!!rlang::syms(group_vars), bin_number)
      ) |>
      dplyr::group_by(!!!group_vars_syms, bin_number) |>
      dplyr::mutate(
        R_b = bin_end - bin_start # Calculate the bin width (R_b = end - start)
      ) |>
      dplyr::ungroup()
  }

  if (bootstrap_ci) {
    if (is.null(group_vars)) {
      # For the bootstrapped data ----
      # Join the bin statistics (bin_summary) with the bin_df_boot for further calculations
      # The merged data will contain the bin information and corresponding statistics
      # Not using AntiM_b = -1 * midpoint + 1
      # i.e., Anticipated mortality (1 - midpoint, reversed scale)
      bin_stats_boot <- bin_summary_boot |>
        dplyr::left_join(
          bin_df_boot,
          by = dplyr::join_by(replicate, bin_number)
        ) |>
        dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
        dplyr::mutate(
          R_b = bin_end - bin_start, # Calculate the bin width (R_b = end - start)
        ) |>
        dplyr::ungroup()
    } else {
      bin_stats_boot <- bin_summary_boot |>
        dplyr::left_join(
          bin_df_boot,
          by = dplyr::join_by(!!!rlang::syms(group_vars), replicate, bin_number)
        ) |>
        dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
        dplyr::mutate(
          R_b = bin_end - bin_start # Calculate the bin width (R_b = end - start)
        ) |>
        dplyr::ungroup()
    }
  }

  # Calculate the Relative Mortality Metric (RMM): ----
  # RMM is calculated by:
  # - Computing the weighted difference between anticipated and observed mortality.
  # - Normalizing by the weighted anticipated mortality.
  rmm_result <- bin_stats |>
    dplyr::group_by(!!!group_vars_syms, bin_number) |>
    dplyr::mutate(
      numerator = R_b * (AntiM_b - EM_b), # Weighted numerator (difference between anticipated and observed mortality)
      denominator = R_b * AntiM_b, # Weighted denominator (anticipated mortality)
      population_RMM = numerator / denominator, # Final RMM calculation
      population_RMM = pmin(
        1,
        pmax(-1, population_RMM, na.rm = TRUE),
        na.rm = TRUE
      ), # Ensure RMM is within [-1, 1]
      population_CI = 1.96 *
        sqrt(
          (sum(AntiM_b, na.rm = TRUE) * sum(AntiS_b, na.rm = TRUE)) /
            sum(N_b, na.rm = TRUE)
        ),
      # Lower bound of 95% CI
      population_RMM_LL = pmax(
        -1,
        population_RMM - population_CI,
        na.rm = TRUE
      ), # Clip LL
      # Upper bound of 95% CI
      population_RMM_UL = pmin(1, population_RMM + population_CI, na.rm = TRUE), # Clip UL,
    ) |>
    dplyr::ungroup() |>
    dplyr::relocate(population_RMM_LL, .before = population_RMM) |>
    dplyr::relocate(population_CI, .after = population_RMM_UL)

  if (bootstrap_ci) {
    # For the bootstrapped data ----
    # Calculate the Relative Mortality Metric (RMM) and its upper and lower confidence intervals:
    # RMM is calculated by:
    # - Computing the weighted difference between anticipated and observed mortality.
    # - Normalizing by the weighted anticipated mortality.
    # The confidence intervals are adjusted based on the weighted error bound.
    rmm_result_boot <- bin_stats_boot |>
      dplyr::group_by(!!!group_vars_syms, replicate, bin_number) |>
      dplyr::mutate(
        numerator = sum(R_b * (AntiM_b - EM_b), na.rm = TRUE), # Weighted numerator (difference between anticipated and observed mortality)
        denominator = sum(R_b * AntiM_b, na.rm = TRUE), # Weighted denominator (anticipated mortality)
        RMM = numerator / denominator, # Final RMM calculation
        RMM = pmin(1, pmax(-1, RMM, na.rm = TRUE), na.rm = TRUE), # Ensure RMM is within [-1, 1]
      ) |>
      dplyr::ungroup()

    # Calculate mean, standard deviation, and 95% confidence intervals ----
    rmm_result_ci <- rmm_result_boot |>
      dplyr::group_by(!!!group_vars_syms, bin_number) |>
      dplyr::summarize(
        bootstrap_RMM = mean(RMM, na.rm = TRUE), # Mean RMM
        sd_bootstrap_RMM = sd(RMM, na.rm = TRUE), # Standard deviation of RMM
        se_bootstrap_RMM = sd_bootstrap_RMM / sqrt(n_samples),
        bootstrap_CI = 1.96 * se_bootstrap_RMM, # Standard error
        # Lower bound of 95% CI
        bootstrap_RMM_LL = pmax(-1, bootstrap_RMM - bootstrap_CI, na.rm = TRUE), # Clip LL
        # Upper bound of 95% CI
        bootstrap_RMM_UL = pmin(1, bootstrap_RMM + bootstrap_CI, na.rm = TRUE), # Clip UL
        .groups = "drop"
      )

    # add the confidence intervals from the bootstrap distribution ----
    # to the final result
    rmm_result_final <- rmm_result |>
      dplyr::left_join(
        rmm_result_ci,
        by = dplyr::join_by(!!!rlang::syms(group_vars), bin_number)
      ) |>
      dplyr::relocate(bootstrap_RMM_LL, .before = bootstrap_RMM) |>
      dplyr::relocate(bootstrap_RMM_UL, .after = bootstrap_RMM) |>
      dplyr::relocate(bootstrap_CI, .after = bootstrap_RMM_UL) |>
      dplyr::select(
        -numerator,
        -denominator,
        -sd_bootstrap_RMM,
        -se_bootstrap_RMM
      )
  } else {
    rmm_result_final <- rmm_result |>
      dplyr::select(-numerator, -denominator)
  }

  # complete ----
  return(rmm_result_final)
}

###_____________________________________________________________________________
# test the rmm function
###_____________________________________________________________________________

testthat::test_that("rmm() input validation for Ps_col and outcome_col", {
  testthat::expect_error(
    traumar::rmm(data = dplyr::tibble(x = 1:5)),
    "Both `Ps_col` and `outcome_col` arguments must be provided."
  )
})

testthat::test_that("rmm() rejects outcome_col with wrong data type", {
  df <- dplyr::tibble(Ps = runif(10), outcome = letters[1:10])
  testthat::expect_error(
    traumar::rmm(data = df, Ps_col = Ps, outcome_col = outcome),
    "outcome_col.*must be of class.*numeric, logical, integer"
  )
})

testthat::test_that("rmm() validates bootstrap_ci is logical", {
  df <- dplyr::tibble(Ps = runif(10), outcome = rbinom(10, 1, 0.5))
  testthat::expect_error(
    traumar::rmm(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      bootstrap_ci = "yes"
    ),
    "bootstrap_ci.*logical.*"
  )
})

testthat::test_that("rmm() fails if seed is not numeric", {
  df <- dplyr::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    outcome = rbinom(1000, 1, 0.5)
  )
  testthat::expect_error(
    traumar::rmm(data = df, Ps_col = Ps, outcome_col = outcome, seed = "foo")
  )
})

testthat::test_that("rmm() validates group_vars are character vectors", {
  df <- dplyr::tibble(Ps = runif(10), outcome = rbinom(10, 1, 0.5))
  testthat::expect_error(
    traumar::rmm(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      group_vars = list(1)
    ),
    "group_vars.*must be of class.*character.*factor"
  )
})

testthat::test_that("rmm() validates group_vars exist in data", {
  df <- dplyr::tibble(Ps = runif(10), outcome = rbinom(10, 1, 0.5))
  testthat::expect_error(
    traumar::rmm(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      group_vars = c("fake_col")
    ),
    "group_vars.*contains invalid column names.*fake_col.*Valid column names are.*Ps, outcome"
  )
})

testthat::test_that("rmm function validates inputs correctly", {
  # Test if data is a data frame
  testthat::expect_error(rmm(data = NULL, Ps_col = Ps, outcome_col = survival))

  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )
  testthat::expect_error(rmm(data = df, outcome_col = survival, n_samples = 5))

  # Test missing outcome_col
  testthat::expect_error(rmm(data = df, Ps_col = Ps))

  # Test non-binary outcome_col
  df_non_binary <- tibble::tibble(
    Ps = plogis(rnorm(5, mean = 2, sd = 1.5)),
    survival = c(1, 2, 3, 4, 5)
  )
  testthat::expect_error(rmm(
    data = df_non_binary,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5
  ))

  # Test non-numeric Ps_col
  df_non_numeric <- tibble::tibble(Ps = c("a", "b", "c"), survival = c(1, 0, 1))
  testthat::expect_error(rmm(
    data = df_non_numeric,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5
  ))

  # Test Ps values > 1
  df_ps_above_1 <- tibble::tibble(
    Ps = sample(30:150, size = 1000, replace = TRUE),
    survival = sample(c(1, 0), size = 1000, replace = TRUE)
  )
  testthat::expect_error(rmm(
    data = df_ps_above_1,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5
  ))

  #Test incorrect input to n_samples
  testthat::expect_error(rmm(
    data = data.frame(Ps = c(0.1, 0.5, 0.9, .005), survival = c(1, 0, 1, 0)),
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = "1000"
  ))
})

testthat::test_that("rmm function computes binning correctly", {
  set.seed(01232025)

  # Test the binning process with some mock data
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  # Check if binning works as expected
  result <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    Divisor1 = 5,
    Divisor2 = 5,
    n_samples = 5
  )
  testthat::expect_true(all(
    c(
      "population_RMM_LL",
      "population_RMM",
      "population_RMM_UL",
      "population_CI",
      "bootstrap_RMM_LL",
      "bootstrap_RMM",
      "bootstrap_RMM_UL",
      "bootstrap_CI"
    ) %in%
      colnames(result)
  ))

  # Test that RMM is calculated
  testthat::expect_true(all(!is.na(result$population_RMM)))
  testthat::expect_true(all(!is.na(result$bootstrap_RMM)))
})

testthat::test_that("rmm function calculates RMM and its confidence intervals", {
  set.seed(01232025)

  # Test with mock data for RMM calculation
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  result <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    Divisor1 = 5,
    Divisor2 = 5,
    n_samples = 5
  )

  # Test for upper and lower bounds of RMM
  testthat::expect_true(all(result$population_RMM_LL <= result$population_RMM))
  testthat::expect_true(all(result$population_RMM_UL >= result$population_RMM))
  testthat::expect_true(all(result$bootstrap_RMM_LL <= result$bootstrap_RMM))
  testthat::expect_true(all(result$bootstrap_RMM_UL >= result$bootstrap_RMM))
})

testthat::test_that("rmm function handles the pivot argument correctly", {
  # Test with pivot = TRUE
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  result_pivot <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    pivot = TRUE,
    n_samples = 5
  )
  testthat::expect_true("stat" %in% colnames(result_pivot))
  testthat::expect_true("value" %in% colnames(result_pivot))

  # Test without pivot (default is FALSE)
  result_no_pivot <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    pivot = FALSE,
    n_samples = 5
  )
  testthat::expect_false("stat" %in% colnames(result_no_pivot))
  testthat::expect_false("value" %in% colnames(result_no_pivot))
})

testthat::test_that("rmm function handles edge cases correctly", {
  # Custom expectation function to test for the presence of a specific warning
  # pattern
  expect_warning_present <- function(expr, pattern) {
    # Initialize an empty character vector to store warning messages
    warnings <- character()

    # Evaluate the expression while intercepting all warning conditions
    withCallingHandlers(
      # Force evaluation of the expression to ensure any lazy evaluation is
      # triggered
      force(expr),

      # Define a handler specifically for warning conditions
      warning = function(w) {
        # Append the current warning message to the 'warnings' vector
        warnings <<- c(warnings, conditionMessage(w))

        # Prevent the warning from being printed to the console during tests
        invokeRestart("muffleWarning")
      }
    )

    # Assert that at least one warning matches the specified regular expression
    # pattern
    testthat::expect_true(
      # Logical test: does any warning message match the provided pattern?
      any(stringr::str_detect(warnings, pattern)),

      # Message to display if the expectation fails
      info = paste("Expected warning matching:", pattern)
    )
  }

  # Generate example data
  set.seed(123)

  # Parameters
  # Total number of patients
  n_patients <- 5000

  # Arbitrary group labels
  groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

  # Trauma types
  trauma_type_values <- sample(
    x = c("Blunt", "Penetrating"),
    size = n_patients,
    replace = TRUE
  )

  # RTS values
  rts_values <- sample(
    x = seq(from = 0, to = 7.8408, by = 0.005),
    size = n_patients,
    replace = TRUE
  )

  # patient ages
  ages <- sample(
    x = seq(from = 0, to = 100, by = 1),
    size = n_patients,
    replace = TRUE
  )

  # ISS scores
  iss_scores <- sample(
    x = seq(from = 0, to = 75, by = 1),
    size = n_patients,
    replace = TRUE
  )

  # Generate survival probabilities (Ps)
  Ps <- traumar::probability_of_survival(
    trauma_type = trauma_type_values,
    age = ages,
    rts = rts_values,
    iss = iss_scores
  )

  # Simulate survival outcomes based on Ps
  survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

  # Create data frame
  data <- data.frame(
    Ps = Ps,
    survival = survival_outcomes,
    groups = groups
  ) |>
    dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

  # Test with edge case: only one row
  df_one_row <- data |> dplyr::slice_sample(n = 1)
  testthat::expect_error(
    rmm(
      data = df_one_row,
      Ps_col = Ps,
      outcome_col = survival
    ),
    regexp = "must have a length within range.*2, Inf"
  )

  # Test with all NA values in Ps
  df_na_ps <- data |>
    dplyr::mutate(Ps = dplyr::if_else(Ps < 0.01, NA_real_, Ps))

  # Test that the rmm() function emits a warning about missing values in Ps_col
  expect_warning_present(
    # Expression to evaluate: calling rmm() with deliberately missing Ps values
    rmm(
      data = df_na_ps, # Dataset with intentionally introduced NA values in Ps
      Ps_col = Ps, # Name of the probability score column being tested
      outcome_col = survival # Outcome variable for the model
    ),

    # Regular expression to match the expected warning message about missing Ps
    # values
    "Ps_col.*missing values detected"
  )

  # Create a dataset with deliberately missing values in the outcome column
  # (survival)
  df_na_outcome <- data |>
    dplyr::mutate(
      survival = dplyr::if_else(Ps < 0.01, NA_real_, survival)
    )

  # Use the custom expect_warning_present() function to test for the specific warning
  expect_warning_present(
    # Expression to evaluate: run rmm() on the dataset with missing outcome values
    rmm(
      data = df_na_outcome,
      Ps_col = Ps,
      outcome_col = survival
    ),

    # Regular expression to match the expected warning message about missing outcome values
    "outcome_col.*missing values detected"
  )
})

testthat::test_that("rmm function handles the seed argument correctly", {
  # Test with a fixed seed value to ensure reproducibility
  set.seed(123)
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  result_1 <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5,
    seed = 123
  )
  result_2 <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5,
    seed = 123
  )

  # Ensure the results are identical with the same seed
  testthat::expect_identical(result_1, result_2)

  # Test with a different seed to check if results change
  result_3 <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5,
    seed = 456
  )

  testthat::expect_false(identical(result_1, result_3))
})

testthat::test_that("rmm function handles the group_vars argument correctly", {
  set.seed(01232025)

  # Test with grouping by a variable (e.g., a factor)
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9),
    group = rep(letters[1:5], each = 200)
  )

  result_grouped <- rmm(
    data = df,
    Ps_col = Ps,
    outcome_col = survival,
    group_vars = "group",
    n_samples = 100
  )

  # Test if the output contains the group variable
  testthat::expect_true("group" %in% colnames(result_grouped))

  # Ensure that results are calculated per group
  group_means <- result_grouped %>%
    dplyr::group_by(group) %>%
    dplyr::summarize(mean_rmm = mean(population_RMM, na.rm = TRUE))
  testthat::expect_true(all(!is.na(group_means$mean_rmm)))
})

testthat::test_that("rmm function handles edge cases with seed and group_vars", {
  # Test with missing group_vars
  df_missing_group <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )
  result_no_group <- rmm(
    data = df_missing_group,
    Ps_col = Ps,
    outcome_col = survival,
    n_samples = 5,
    seed = 123
  )

  # Test that no error occurs and RMM is calculated
  testthat::expect_true(all(!is.na(result_no_group$population_RMM)))

  # Test with group_vars but no valid grouping
  df_invalid_group <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9),
    group = rep(NA, 1000)
  )

  testthat::expect_error(rmm(
    data = df_invalid_group,
    Ps_col = Ps,
    outcome_col = survival,
    group_vars = group,
    n_samples = 5,
    seed = 123
  ))
})

testthat::test_that("rmm function handles missing values with seed and group_vars correctly", {
  # Custom expectation function to test for the presence of a specific warning pattern
  expect_warning_present <- function(expr, pattern) {
    # Initialize an empty character vector to store warning messages
    warnings <- character()

    # Evaluate the expression while intercepting all warning conditions
    withCallingHandlers(
      # Force evaluation of the expression to ensure any lazy evaluation is triggered
      force(expr),

      # Define a handler specifically for warning conditions
      warning = function(w) {
        # Append the current warning message to the 'warnings' vector
        warnings <<- c(warnings, conditionMessage(w))

        # Prevent the warning from being printed to the console during tests
        invokeRestart("muffleWarning")
      }
    )

    # Assert that at least one warning matches the specified regular expression pattern
    testthat::expect_true(
      # Logical test: does any warning message match the provided pattern?
      any(stringr::str_detect(warnings, pattern)),

      # Message to display if the expectation fails
      info = paste("Expected warning matching:", pattern)
    )
  }

  # Create a larger dataset with missing values in Ps, survival, and group variables
  set.seed(123)
  df_missing_values <- tibble::tibble(
    Ps = c(rep(NA, 200), plogis(rnorm(800, mean = 2, sd = 1.5))), # 200 NAs in Ps
    survival = rbinom(1000, 1, prob = 0.8), # Binary outcome, no NAs here
    group = rep(c("A", "B", "C", "D", "E"), each = 200) # Group variable with 5 groups
  )

  # Expect a warning when running rmm() with missing values in the dataset (Ps
  # column missing)
  expect_warning_present(
    rmm(
      data = df_missing_values,
      Ps_col = Ps,
      outcome_col = survival,
      group_vars = "group",
      n_samples = 5,
      seed = 123
    ),
    "Ps_col.*missing values detected"
  )

  # Create a dataset with missing values in the *outcome* column
  df_missing_values <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)), # Ps column fully populated
    survival = c(rep(NA, 200), rbinom(800, 1, prob = 0.8)), # 200 NAs in survival
    group = rep(c("A", "B", "C", "D", "E"), each = 200) # Same group structure
  )

  # Expect a warning when running rmm() with missing values in the *outcome* column
  expect_warning_present(
    rmm(
      data = df_missing_values,
      Ps_col = Ps,
      outcome_col = survival,
      group_vars = "group",
      n_samples = 5,
      seed = 123
    ),
    "outcome_col.*missing values detected"
  )
})

testthat::test_that("rm_bin_summary has correct validation of outcome_col when it is logical", {
  set.seed(01232025)

  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = sample(
      x = c(TRUE, FALSE),
      size = 1000,
      replace = TRUE,
      prob = c(0.9, 0.1)
    )
  )

  testthat::expect_no_error(
    rmm(data = df, Ps_col = Ps, outcome_col = survival)
  )
})

###_____________________________________________________________________________
# test the rm_bin_summary function
###_____________________________________________________________________________

testthat::test_that("rm_bin_summary() input validation for Ps_col and outcome_col", {
  testthat::expect_error(
    traumar::rm_bin_summary(data = dplyr::tibble(x = 1:5)),
    "Both `Ps_col` and `outcome_col` arguments must be provided."
  )
})

testthat::test_that("rm_bin_summary() rejects outcome_col with wrong data type", {
  df <- dplyr::tibble(Ps = runif(10), outcome = letters[1:10])
  testthat::expect_error(
    traumar::rm_bin_summary(data = df, Ps_col = Ps, outcome_col = outcome),
    "outcome_col.*must be of class.*numeric, logical, integer"
  )
})

testthat::test_that("rm_bin_summary() validates bootstrap_ci is logical", {
  df <- dplyr::tibble(Ps = runif(10), outcome = rbinom(10, 1, 0.5))
  testthat::expect_error(
    traumar::rm_bin_summary(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      bootstrap_ci = "yes"
    ),
    "bootstrap_ci.*logical.*"
  )
})

testthat::test_that("rm_bin_summary() warns if seed is not numeric", {
  df <- dplyr::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    outcome = rbinom(1000, 1, 0.5)
  )
  testthat::expect_error(
    traumar::rm_bin_summary(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      seed = "foo"
    ),
    "seed.*must be of class.*numeric, integer"
  )
})

testthat::test_that("rm_bin_summary() validates group_vars are character vectors", {
  df <- dplyr::tibble(Ps = runif(10), outcome = rbinom(10, 1, 0.5))
  testthat::expect_error(
    traumar::rm_bin_summary(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      group_vars = list(1)
    ),
    "group_vars.*must be of class.*character.*factor"
  )
})

testthat::test_that("rm_bin_summary() validates group_vars exist in data", {
  df <- dplyr::tibble(Ps = runif(10), outcome = rbinom(10, 1, 0.5))
  testthat::expect_error(
    traumar::rm_bin_summary(
      data = df,
      Ps_col = Ps,
      outcome_col = outcome,
      group_vars = c("fake_col")
    ),
    "group_vars.*contains invalid column names.*fake_col.*Valid column names are.*Ps, outcome"
  )
})

testthat::test_that("rm_bin_summary validates inputs correctly", {
  # Test if data is a data frame
  testthat::expect_error(rm_bin_summary(
    data = NULL,
    Ps_col = Ps,
    outcome_col = survival
  ))

  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )
  testthat::expect_error(rm_bin_summary(data = df, outcome_col = survival))

  # Test missing outcome_col
  testthat::expect_error(rm_bin_summary(data = df, Ps_col = Ps))

  # Test non-binary outcome_col
  df_non_binary <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = sample(1:3, 1000, replace = TRUE)
  )
  testthat::expect_error(rm_bin_summary(
    data = df_non_binary,
    Ps_col = Ps,
    outcome_col = survival
  ))

  # Test non-numeric Ps_col
  df_non_numeric <- tibble::tibble(Ps = c("a", "b", "c"), survival = c(1, 0, 1))
  testthat::expect_error(rm_bin_summary(
    data = df_non_numeric,
    Ps_col = Ps,
    outcome_col = survival
  ))

  # Test Ps values > 1
  df_ps_above_1 <- tibble::tibble(Ps = c(150, 80, 30), survival = c(1, 0, 1))
  testthat::expect_error(rm_bin_summary(
    data = df_ps_above_1,
    Ps_col = Ps,
    outcome_col = survival
  ))
})

testthat::test_that("nonlinear_bins produces correct bin data", {
  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  bin_data <- nonlinear_bins(
    df,
    Ps_col = Ps,
    outcome_col = survival,
    divisor1 = 5,
    divisor2 = 5,
    threshold_1 = 0.9,
    threshold_2 = 0.99
  )

  testthat::expect_true("bin_stats" %in% names(bin_data))
  testthat::expect_true("intervals" %in% names(bin_data))
  testthat::expect_equal(nrow(bin_data$bin_stats), 10) # assuming 5 bins
})

testthat::test_that("bootstrap data has the correct number of samples", {
  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  bootstrap_data <- df |>
    infer::generate(reps = 100, type = "bootstrap")

  testthat::expect_equal(nrow(bootstrap_data), 100 * nrow(df))
})

testthat::test_that("bin statistics are calculated correctly", {
  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  try_function <- rm_bin_summary(df, Ps, survival, n_samples = 5)

  testthat::expect_true("TA_b" %in% names(try_function))
  testthat::expect_true("TD_b" %in% names(try_function))
  testthat::expect_true("EM_b" %in% names(try_function))
})

testthat::test_that("RMM is calculated correctly", {
  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  rmm_result <- df |>
    rm_bin_summary(Ps, survival, n_samples = 5)

  testthat::expect_true("population_RMM" %in% names(rmm_result))
  testthat::expect_true(
    all(rmm_result$population_RMM >= -1) || all(rmm_result$population_RMM <= 1)
  )
})

testthat::test_that("confidence intervals are correctly computed in final RMM", {
  set.seed(10232015)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(10000, mean = 2, sd = 1.5)),
    survival = rbinom(10000, 1, prob = 0.9)
  )

  rmm_result <- rm_bin_summary(df, Ps, survival, n_samples = 100)

  testthat::expect_true("bootstrap_RMM_LL" %in% names(rmm_result))
  testthat::expect_true("bootstrap_RMM_UL" %in% names(rmm_result))
  testthat::expect_true("bootstrap_CI" %in% names(rmm_result))
  testthat::expect_true(all(
    rmm_result$population_RMM_UL > rmm_result$population_RMM_LL
  )) # CI upper should be greater than lower
})

testthat::test_that("RMM final data is correctly sorted by bin_number", {
  set.seed(01232025)

  # Test missing Ps_col
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  rmm_result_final <- rm_bin_summary(df, Ps, survival, n_samples = 5)

  testthat::expect_equal(min(rmm_result_final$bin_number), 1)
  testthat::expect_equal(max(rmm_result_final$bin_number), 10) # Assuming 10 bins
})

testthat::test_that("rm_bin_summary validates group_vars correctly", {
  set.seed(01232025)

  # Test valid grouping
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9),
    group = sample(letters[1:5], 1000, replace = TRUE)
  )

  grouped_result <- rm_bin_summary(
    df,
    Ps,
    survival,
    group_vars = "group",
    n_samples = 5
  )
  testthat::expect_true("group" %in% names(grouped_result))
  testthat::expect_true(all(grouped_result$group %in% unique(df$group)))

  # Test invalid group_vars (non-existent column)
  testthat::expect_error(rm_bin_summary(
    df,
    Ps,
    survival,
    group_vars = non_existent_col
  ))
})

testthat::test_that("rm_bin_summary validates Ps_col and outcome_col correctly", {
  # Custom expectation function to test for the presence of a specific warning pattern
  expect_warning_present <- function(expr, pattern) {
    # Initialize an empty character vector to store warning messages
    warnings <- character()

    # Evaluate the expression while intercepting all warning conditions
    withCallingHandlers(
      # Force evaluation of the expression to ensure any lazy evaluation is triggered
      force(expr),

      # Define a handler specifically for warning conditions
      warning = function(w) {
        # Append the current warning message to the 'warnings' vector
        warnings <<- c(warnings, conditionMessage(w))

        # Prevent the warning from being printed to the console during tests
        invokeRestart("muffleWarning")
      }
    )

    # Assert that at least one warning matches the specified regular expression pattern
    testthat::expect_true(
      # Logical test: does any warning message match the provided pattern?
      any(stringr::str_detect(warnings, pattern)),

      # Message to display if the expectation fails
      info = paste("Expected warning matching:", pattern)
    )
  }

  # Set seed for reproducibility
  set.seed(01232025)

  # Create a dataset with valid grouping and complete data
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)), # Probability scores (Ps)
    survival = rbinom(1000, 1, prob = 0.9), # Binary outcome variable
    group = sample(letters[1:5], 1000, replace = TRUE) # Random group assignment (5 groups)
  )

  # Introduce missing values into the Ps column to create an invalid input scenario
  bad_ps <- df |>
    dplyr::mutate(
      Ps = c(rep(NA, 50), plogis(rnorm(950, mean = 2, sd = 1.5)))
    )

  # Expect a warning for missing values in Ps_col when running rm_bin_summary()
  expect_warning_present(
    rm_bin_summary(
      bad_ps,
      Ps,
      survival,
      n_samples = 5
    ),
    "Ps_col.*missing values detected"
  )

  # Introduce missing values into the outcome (survival) column
  bad_outcome <- df |>
    dplyr::mutate(
      survival = c(rep(NA, 50), rbinom(950, size = 1, prob = 0.9))
    )

  # Expect a warning for missing values in outcome_col when running rm_bin_summary()
  expect_warning_present(
    rm_bin_summary(
      bad_outcome,
      Ps,
      survival,
      n_samples = 5
    ),
    "outcome_col.*missing values detected"
  )
})

testthat::test_that("rm_bin_summary handles the seed argument correctly", {
  # Test consistency with seed
  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  result1 <- rm_bin_summary(df, Ps, survival, n_samples = 5, seed = 12345)
  result2 <- rm_bin_summary(df, Ps, survival, n_samples = 5, seed = 12345)

  # Ensure the results are the same for the same seed
  testthat::expect_equal(result1, result2)

  # Test different seed values result in different outputs
  result3 <- rm_bin_summary(df, Ps, survival, n_samples = 5, seed = 67890)
  testthat::expect_false(identical(result1, result3))
})

testthat::test_that("rm_bin_summary handles extreme edge cases for Ps values", {
  # Test edge cases for Ps column (extremely low/high values)
  df_extreme <- tibble::tibble(Ps = c(1e-10, 1, 1e+10), survival = c(1, 0, 1))

  # Ensure that error is thrown with values above 1 or less than 0
  testthat::expect_error(rm_bin_summary(
    df_extreme,
    Ps,
    survival,
    n_samples = 5
  ))
})

testthat::test_that("rm_bin_summary produces error with invalid n_samples", {
  set.seed(01232025)

  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  # Invalid n_samples (non-integer, negative)
  testthat::expect_error(rm_bin_summary(df, Ps, survival, n_samples = -1))
  testthat::expect_error(rm_bin_summary(df, Ps, survival, n_samples = 3.5))
})

testthat::test_that("rm_bin_summary with missing group_vars argument", {
  set.seed(01232025)

  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = rbinom(1000, 1, prob = 0.9)
  )

  # Ensure no error when group_vars is not supplied
  no_group_result <- rm_bin_summary(df, Ps, survival, n_samples = 5)
  testthat::expect_true("bin_number" %in% names(no_group_result))
  testthat::expect_true(nrow(no_group_result) > 0)
})

testthat::test_that("rm_bin_summary has correct validation of outcome_col when it is logical", {
  set.seed(01232025)

  df <- tibble::tibble(
    Ps = plogis(rnorm(1000, mean = 2, sd = 1.5)),
    survival = sample(
      x = c(TRUE, FALSE),
      size = 1000,
      replace = TRUE,
      prob = c(0.9, 0.1)
    )
  )

  testthat::expect_no_error(
    rm_bin_summary(data = df, Ps_col = Ps, outcome_col = survival)
  )
})

#' @title View the Current Patient Population Case Mix Compared to the Major
#'   Trauma Study Case Mix
#'
#' @description
#'
#' This function compares the current patient population's case mix (based on
#' probability of survival, Ps) to the MTOS case mix by binning patients into
#' specific Ps ranges. It returns the fraction of patients in each range and
#' compares it to the MTOS distribution. For more information on the methods
#' used in these calculations, please see Flora (1978) and Boyd et al. (1987).
#'
#' @param df A data frame containing patient data.
#' @param Ps_col The name of the column containing the probability of survival
#'   (Ps) values.
#' @param outcome_col The name of the column containing the binary outcome data
#'   (valid values are 1 or TRUE for alive, 0 or FALSE for dead).
#'
#' @details
#'
#' The function checks whether the `outcome_col` contains values representing a
#' binary outcome. It also ensures that `Ps_col` contains numeric values within
#' the range 0 to 1. If any values exceed 1, a warning is issued. The patients
#' are then grouped into predefined Ps ranges, and the function compares the
#' fraction of patients in each range with the MTOS case mix distribution.
#'
#' Like other statistical computing functions, `trauma_case_mix()` is happiest
#' without missing data.  It is best to pass complete probability of survival
#' and outcome data to the function for optimal performance. With smaller
#' datasets, this is especially helpful.  However, `trauma_case_mix()` will
#' throw a warning about missing values, if any exist in `Ps_col` and/or
#' `outcome_col`.
#'
#' @return A data frame containing:
#'   \itemize{
#'     \item \code{Ps_range}: The probability of survival range category.
#'     \item \code{current_fraction}: The fraction of patients in the current
#'     dataset within each Ps range.
#'     \item \code{MTOS_distribution}: The reference distribution of patients in
#'     each Ps range based on the MTOS study.
#'     \item \code{survivals}: The number of observed survivors (outcome = 1) in
#'     each Ps range.
#'     \item \code{predicted_survivals}: The sum of predicted survivals (sum of
#'     Ps values) in each Ps range.
#'     \item \code{deaths}: The number of observed deaths (outcome = 0) in each
#'     Ps range.
#'     \item \code{predicted_deaths}: The sum of predicted deaths (sum of 1 - Ps
#'     values) in each Ps range.
#'     \item \code{count}: The total number of patients in each Ps range.
#'   }
#'
#' @note
#'
#' This function will produce the most reliable and interpretable results when
#' using a dataset that has one row per patient, with each column being a
#' feature.
#'
#' @examples
#' # Generate example data
#' set.seed(123)
#'
#' # Parameters
#' # Total number of patients
#' n_patients <- 5000
#'
#' # Arbitrary group labels
#' groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)
#'
#' # Trauma types
#' trauma_type_values <- sample(
#'   x = c("Blunt", "Penetrating"),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # RTS values
#' rts_values <- sample(
#'   x = seq(from = 0, to = 7.8408, by = 0.005),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # patient ages
#' ages <- sample(
#'   x = seq(from = 0, to = 100, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # ISS scores
#' iss_scores <- sample(
#'   x = seq(from = 0, to = 75, by = 1),
#'   size = n_patients,
#'   replace = TRUE
#' )
#'
#' # Generate survival probabilities (Ps)
#' Ps <- traumar::probability_of_survival(
#'   trauma_type = trauma_type_values,
#'   age = ages,
#'   rts = rts_values,
#'   iss = iss_scores
#' )
#'
#' # Simulate survival outcomes based on Ps
#' survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)
#'
#' # Create data frame
#' data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
#'   dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
#'
#' # Compare the current case mix with the MTOS case mix
#' trauma_case_mix(data, Ps_col = Ps, outcome_col = death)
#'
#' @export
#'
#' @author Nicolas Foss, Ed.D., MS
#'
trauma_case_mix <- function(df, Ps_col, outcome_col) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Check if the dataframe is valid ----
  validate_data_structure(
    input = df,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error",
    null_ok = FALSE
  )

  # Ensure Ps_col and outcome_col arguments are provided  ----
  # with tailored error messages
  if (missing(Ps_col) && missing(outcome_col)) {
    cli::cli_abort(
      "Both {.var Ps_col} and {.var outcome_col} arguments must be provided."
    )
  } else if (missing(Ps_col)) {
    cli::cli_abort("The {.var Ps_col} argument must be provided.")
  } else if (missing(outcome_col)) {
    cli::cli_abort("The {.var outcome_col} argument must be provided.")
  }

  # Pull and check the outcome column ----
  binary_data <- validate_data_pull(
    input = df,
    col = {{ outcome_col }},
    var_name = "outcome_col"
  )

  # Ensure the column is either logical or numeric ----
  validate_class(
    input = binary_data,
    class_type = c("numeric", "logical", "integer"),
    logic = "or",
    type = "error",
    var_name = "outcome_col"
  )

  # Get unique non-missing values to use in subsequent data validation ----
  non_missing <- stats::na.omit(binary_data)

  # Validate type and values ----
  if (is.logical(non_missing)) {
    # Logical vector: ensure only TRUE/FALSE (no coercion needed)
    validate_set(
      input = non_missing,
      valid_set = c(TRUE, FALSE),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.numeric(non_missing)) {
    # Numeric vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0, 1),
      type = "error",
      var_name = "outcome_col"
    )
  } else if (is.integer(non_missing)) {
    # Integer vector: ensure strictly 0 or 1 ----
    validate_set(
      input = non_missing,
      valid_set = c(0L, 1L),
      type = "error",
      var_name = "outcome_col"
    )
  }

  # Warn if missing ----
  validate_complete(
    input = binary_data,
    type = "warning",
    var_name = "outcome_col"
  )

  # Check if Ps column is numeric ----
  # dplyr::pull the Ps data
  Ps_check <- validate_data_pull(
    input = df,
    col = {{ Ps_col }},
    var_name = "Ps_col",
    calls = 5
  )

  # check the Ps_check remains continuous ----
  # Check if Ps column is continuous (values between 0 and 1)
  validate_numeric(
    input = Ps_check,
    min = 0,
    max = 1,
    type = "error",
    var_name = "Ps_col"
  )

  # Warn if any missings in Ps_col ----
  validate_complete(input = Ps_check, type = "warning", var_name = "Ps_col")

  # Set the Ps range order for the function ----
  Ps_range_order <- c(
    "0.96 - 1.00",
    "0.91 - 0.95",
    "0.76 - 0.90",
    "0.51 - 0.75",
    "0.26 - 0.50",
    "0.00 - 0.25"
  )

  # Define the MTOS Ps distribution ----
  MTOS_distribution <- tibble::tibble(
    Ps_range = factor(
      c(
        "0.96 - 1.00",
        "0.91 - 0.95",
        "0.76 - 0.90",
        "0.51 - 0.75",
        "0.26 - 0.50",
        "0.00 - 0.25"
      ),
      levels = Ps_range_order
    ),
    MTOS_distribution = c(0.842, 0.053, 0.052, 0, 0.043, 0.01)
  )

  # Bin patients into Ps ranges and calculate current fractions ----
  fractions_set <- df |>
    # Mutate the dataframe to add new columns
    dplyr::mutate(
      # Create a new column Ps_range based on the value of Ps_col
      Ps_range = dplyr::case_when(
        {{ Ps_col }} >= 0.96 ~ "0.96 - 1.00", # Ps_col >= 0.96
        {{ Ps_col }} >= 0.91 ~ "0.91 - 0.95", # Ps_col >= 0.91
        {{ Ps_col }} >= 0.76 ~ "0.76 - 0.90", # Ps_col >= 0.76
        {{ Ps_col }} >= 0.51 ~ "0.51 - 0.75", # Ps_col >= 0.51
        {{ Ps_col }} >= 0.26 ~ "0.26 - 0.50", # Ps_col >= 0.26
        TRUE ~ "0.00 - 0.25" # Default case for Ps_col < 0.26
      )
    ) |>
    # Summarize the dataframe to calculate various statistics ----
    dplyr::summarize(
      # Calculate the number of survivals
      survivals = sum({{ outcome_col }} == 1, na.rm = TRUE),
      # Calculate the predicted number of survivals
      predicted_survivals = sum({{ Ps_col }}, na.rm = TRUE),
      # Calculate the number of deaths
      deaths = sum({{ outcome_col }} == 0, na.rm = TRUE),
      # Calculate the predicted number of deaths
      predicted_deaths = sum(1 - {{ Ps_col }}, na.rm = TRUE),
      # Count the number of rows in each Ps_range
      count = dplyr::n(),
      # Calculate the current fraction of each Ps_range
      current_fraction = dplyr::n() / nrow(df),
      # Group by Ps_range
      .by = Ps_range
    ) |>
    # Join the summarized dataframe with MTOS_distribution by Ps_range ----
    dplyr::left_join(MTOS_distribution, by = dplyr::join_by(Ps_range)) |>
    # Arrange the dataframe by Ps_range
    dplyr::arrange(Ps_range) |>
    # Relocate the current_fraction column to be after Ps_range
    dplyr::relocate(current_fraction, .after = Ps_range) |>
    # Relocate the MTOS_distribution column to be after current_fraction
    dplyr::relocate(MTOS_distribution, .after = current_fraction) |>
    tibble::as_tibble()

  # Return the result as a tibble ----
  return(fractions_set)
}

# Test 1: Basic functionality with valid input data
testthat::test_that("Valid input returns correct output", {
  set.seed(123)
  n_patients <- 100
  Ps <- plogis(rnorm(n_patients, mean = 2, sd = 1.5))
  survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)
  data <- data.frame(Ps = Ps, survival = survival_outcomes) |>
    dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

  result <- trauma_case_mix(data, Ps_col = Ps, outcome_col = death)
  testthat::expect_equal(ncol(result), 8)
  testthat::expect_true("Ps_range" %in% colnames(result))
  testthat::expect_true("current_fraction" %in% colnames(result))
  testthat::expect_true("MTOS_distribution" %in% colnames(result))
})

# Test 2: Handles Ps values greater than 1, converting to decimal
testthat::test_that("Ps values > 1 cause an error", {
  set.seed(123)
  surv_probs <- sample(1:100, 100, replace = T)
  survival_outcomes <- rbinom(100, size = 1, prob = 0.9)
  data <- data.frame(Ps = surv_probs, death = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = death),
    regexp = "Ps_col.*values must be contained within range.*0, 1.*Range of this input was.*4, 99"
  )
})

# Test 3: Invalid outcome column (non-binary)
testthat::test_that("Invalid binary outcome column throws error", {
  set.seed(123)
  n_patients <- 100
  Ps <- plogis(rnorm(n_patients, mean = 2, sd = 1.5))
  survival_outcomes <- rnorm(n_patients, mean = 0.5, sd = 0.2) # Non-binary outcome
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = survival),
    regexp = "outcome_col: contains invalid values such as.*Valid values are.*0, 1"
  )
})

# Test 4: Invalid Ps column (non-numeric)
testthat::test_that("Non-numeric Ps column throws error", {
  set.seed(123)
  n_patients <- 100
  Ps <- letters[1:100] # Non-numeric Ps values
  survival_outcomes <- rbinom(n_patients, size = 1, prob = 0.7)
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = survival),
    regexp = "Ps_col.*must be.*numeric"
  )
})

# Test 5: Ps column contains values outside the expected range (0 to 100)
testthat::test_that("Ps values outside 0-1 range throw error", {
  set.seed(123)
  n_patients <- 10
  Ps <- c(5, 10, 20, 105, -10, 0.85, -3, 200, 1.5, 80) # Invalid Ps values
  survival_outcomes <- rbinom(n_patients, size = 1, prob = 0.9)
  data <- data.frame(Ps = Ps, survival = survival_outcomes) |>
    dplyr::mutate(death = dplyr::if_else(survival >= 0.5, 0, 1))

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = death),
    regexp = "Ps_col.*values must be contained within range.*0, 1.*Range of this input was.*-10, 200"
  )
})

# Test 6: Non-data frame input should throw an error
testthat::test_that("Invalid df input throws error", {
  testthat::expect_error(
    trauma_case_mix(df = NULL, Ps_col = Ps, outcome_col = death),
    "df.*must not be NULL"
  )
})

# Test 7: Valid input data with missing values (NA) in Ps_col or outcome_col
testthat::test_that("NA values are correctly handled", {
  set.seed(123)
  n_patients <- 100
  Ps <- c(0.85, NA, 0.75, 0.6, 0.91)
  survival_outcomes <- c(1, 0, 0, 1, 0)
  data <- data.frame(Ps = Ps, survival = survival_outcomes) |>
    dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

  testthat::expect_warning(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = death),
    regexp = "Ps_col.*missing values detected.*Found 1 missing value\\(s\\) out of 5 total values for 20% global missingness"
  )

  Ps <- c(0.85, 0.2, 0.75, 0.6, 0.91)
  survival_outcomes <- c(1, NA, 0, 1, 0)
  data <- data.frame(Ps = Ps, survival = survival_outcomes) |>
    dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))
  testthat::expect_warning(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = death),
    regexp = "outcome_col.*missing values detected.*Found 1 missing value\\(s\\) out of 5 total values for 20% global missingness"
  )
})

# Test 8: non-logical and non-numeric argument to outcome_col
testthat::test_that("non-logical and non-numeric argument to outcome_col do not work", {
  set.seed(123)
  n_patients <- 100
  Ps <- c(0.85, NA, 0.75, 0.6, 0.91)
  survival_outcomes <- as.character(c(1, 0, 0, 1, 0))
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = survival),
    regexp = "outcome_col.*must be of class.*numeric, logical, integer"
  )
})

# Test 9: missing Ps_col and outcome_col columns and logical binary data
testthat::test_that("trauma_case_mix correctly validates missing Ps_col and outcome_col columns and logical binary data", {
  set.seed(123)
  n_patients <- 100
  Ps <- c(0.85, NA, 0.75, 0.6, 0.91)
  survival_outcomes <- as.character(c(1, 0, 0, 1, 0))
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data),
    regexp = "Both.*arguments must be provided"
  )

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps),
    regexp = "argument must be provided"
  )

  testthat::expect_error(
    trauma_case_mix(data, outcome_col = survival),
    regexp = "argument must be provided"
  )

  survival_outcomes <- c(1L, 2L, 2L, 1L, 2L)
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = survival),
    regexp = "outcome_col.*contains invalid values such as.*Valid values are"
  )

  survival_outcomes <- c(T, F, T, F, T)
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_warning(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = survival)
  )

  survival_outcomes <- as.character(c(1, 0, 0, 1, 0))
  data <- data.frame(Ps = Ps, survival = survival_outcomes)

  testthat::expect_error(
    trauma_case_mix(data, Ps_col = Ps, outcome_col = survival),
    regexp = "outcome_col.*must be of class.*numeric, logical, integer"
  )
})

#' @title SEQIC Indicator 13 – Validation of  Trauma Registry Records
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Calculates the proportion of trauma records that meet or exceed
#' a threshold for data validity among facilities at the specified trauma center
#' levels. Optionally computes confidence intervals.
#'
#' @inheritParams seqic_indicator_1
#' @param validity_score Numeric. The proportion of each trauma registry record
#'   that is valid, expressed as a percentage (0–100). Typically calculated by
#'   the registry system.
#' @param validity_threshold Numeric. The minimum acceptable validity percentage
#'   threshold for records to be counted in the numerator. Defaults to `85`.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details
#'
#' This function:
#' \itemize{
#'   \item Filters to include only patients treated at trauma centers with
#'   levels specified in `included_levels` (default: Levels I–IV).
#'   \item Deduplicates the dataset using `unique_incident_id` to ensure each
#'   incident is counted only once.
#'   \item Flags records with a `validity_score` greater than or equal to the
#'   specified `validity_threshold` threshold (default: 85).
#'   \item Calculates the proportion of valid records among all included
#'   records.
#'   \item Optionally calculates binomial confidence intervals using the method
#'   specified in `calculate_ci` via `nemsqa_binomial_confint()`.
#'   \item Adds a "Population/Sample" label unless grouping is applied via
#'   `groups`.
#' }
#'
#' Users must ensure that appropriate column names are passed using tidy
#' evaluation (bare column names) and that the input data has been cleaned and
#' includes no missing or malformed identifiers, trauma level classifications,
#' or validity scores.
#'
#' @return A tibble summarizing SEQIC Indicator 13 results. Includes numerator,
#'   denominator, and performance rate 95% confidence intervals are included if
#'   requested.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Simulated data for SEQIC Indicator 13
#' test_data <- tibble::tibble(
#'   id = as.character(1:12),
#'   trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV", "I", "II",
#'   "III", "IV"),
#'   validity = c(90, 80, 88, 92, 86, 75, 89, 70, 95, 85, 83, 87)
#' )
#'
#' # Run the function
#' traumar::seqic_indicator_13(
#'   data = test_data,
#'   level = trauma_level,
#'   included_levels = c("I", "II", "III", "IV"),
#'   unique_incident_id = id,
#'   validity_score = validity,
#'   validity_threshold = 85,
#'   calculate_ci = "wilson"
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_13 <-
  function(
    data,
    level,
    included_levels = c("I", "II", "III", "IV"),
    unique_incident_id,
    validity_score,
    validity_threshold = 85,
    groups = NULL,
    calculate_ci = NULL,
    ...
  ) {
    ###___________________________________________________________________________
    ### Data validation
    ###___________________________________________________________________________

    # Validate if `data` is a data frame or tibble. ----
    validate_data_structure(
      input = data,
      structure_type = c("data.frame", "tbl", "tbl_df"),
      type = "error"
    )

    # make the `level` column accessible for validation ----
    level_check <- validate_data_pull(
      input = data,
      type = "error",
      col = {{ level }},
      var_name = "level"
    )

    # validate `level` ----
    validate_character_factor(
      input = level_check,
      type = "error",
      var_name = "level"
    )

    # make the `unique_incident_id` column accessible for validation ----
    unique_incident_id_check <- validate_data_pull(
      input = data,
      type = "error",
      col = {{ unique_incident_id }},
      var_name = "unique_incident_id"
    )

    # Validate `unique_incident_id` ----
    validate_class(
      input = unique_incident_id_check,
      class_type = c("numeric", "integer", "character", "factor"),
      logic = "or",
      type = "error",
      var_name = "unique_incident_id"
    )

    # Ensure that `validity_score` can be validated
    validity_score_check <- validate_data_pull(
      input = data,
      col = {{ validity_score }},
      type = "error",
      var_name = "validity_score"
    )

    # Validate that validity_score is numeric
    validate_numeric(
      input = validity_score_check,
      min = 0,
      max = 100,
      type = "error",
      var_name = "validity_score"
    )

    # Validate that validity_threshold is numeric
    validate_numeric(
      input = validity_threshold,
      min = 0,
      max = 100,
      type = "error"
    )

    # Check if all elements in groups are strings (i.e., character vectors)
    if (!is.null(groups)) {
      if (!is.character(groups)) {
        cli::cli_abort(c(
          "All elements in {.var groups} must be strings.",
          "i" = "You passed an object of class {.cls {class(groups)}} to {.var groups}."
        ))
      }
    }

    # Check if all elements in groups are strings (i.e., character vectors) ----
    validate_character_factor(input = groups, type = "error", null_ok = TRUE)

    # Check if all `groups` exist in the `data` ----
    validate_names(
      input = data,
      check_names = groups,
      type = "error",
      var_name = "groups",
      null_ok = TRUE
    )

    # Validate the `calculate_ci` argument ----
    calculate_ci <- validate_choice(
      input = calculate_ci,
      choices = c("wilson", "clopper-pearson"),
      several.ok = FALSE,
      type = "error",
      null_ok = TRUE,
      var_name = "calculate_ci"
    )

    # Validate the `included_levels` argument ----
    validate_class(
      input = included_levels,
      class_type = c("numeric", "character", "factor", "integer"),
      type = "error",
      logic = "or"
    )

    ###_________________________________________________________________________
    ### Calculations ----
    ###_________________________________________________________________________

    seqic_13 <- data |>
      # Filter the dataset to include only records from facilities matching ----
      # the trauma levels specified in `included_levels` (default: I–IV).
      dplyr::filter({{ level }} %in% included_levels) |>

      # Deduplicate the dataset to ensure each trauma incident ----
      # is represented only once. This is essential to avoid overcounting when
      # calculating proportions. The `unique_incident_id` column is assumed to
      # uniquely identify each incident.
      dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>

      # Summarize the filtered and deduplicated dataset: ----
      dplyr::summarize(
        # Count the number of valid records ----
        # that meet or exceed the threshold set by the `validity` argument
        # (default: 85). Missing values are ignored.
        numerator_13 = sum(
          {{ validity_score }} >= {{ validity_threshold }},
          na.rm = TRUE
        ),

        # Count the total number of records in the denominator ----
        # after filtering and deduplication. This represents all valid candidate
        # records.
        denominator_13 = dplyr::n(),

        # Calculate the proportion of valid records ----
        # (numerator / denominator), rounded to 3 decimal places for reporting
        # clarity.
        seqic_13 = dplyr::if_else(
          denominator_13 > 0,
          numerator_13 / denominator_13,
          NA_real_
        ),
        .by = {{ groups }}
      )

    # Optionally calculate confidence intervals for the proportions: ----
    # - If `calculate_ci` is provided, apply a binomial confidence interval
    #   method (Wilson or Clopper-Pearson).
    # - `nemsqa_binomial_confint()` calculates the confidence intervals for the
    # proportion.
    # - Select only the relevant columns and rename the CI columns for clarity.
    if (!is.null(calculate_ci)) {
      seqic_13 <- seqic_13 |>
        dplyr::bind_cols(
          nemsqar::nemsqa_binomial_confint(
            data = seqic_13,
            x = numerator_13, # Number of successes (numerator).
            n = denominator_13, # Number of trials (denominator).
            method = calculate_ci, # Confidence interval calculation method (e.g., "wilson").
            ...
          ) |>
            dplyr::select(lower_ci, upper_ci) |>
            dplyr::rename(lower_ci_13 = lower_ci, upper_ci_13 = upper_ci) # Rename CI columns.
        )
    }

    # Assign a label to indicate whether the data represents population or ----
    # sample-level results:
    # - If no grouping is applied, label the data as "Population/Sample". ----
    if (is.null(groups)) {
      seqic_13 <- seqic_13 |>
        tibble::add_column(data = "population/sample", .before = "numerator_13") # Add the label column.
    } else if (!is.null(groups)) {
      seqic_13 <- seqic_13 |>
        dplyr::arrange(!!!rlang::syms(groups))
    }

    # Return SEQIC Indicator 13 results as a tibble ----
    return(seqic_13)
  }

# tests/testthat/test-seqic_indicator_13.R

testthat::test_that("seqic_indicator_13() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "III", "IV", "I", "II"),
    validity = c(90, 85, 75, 92, 86, 70)
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = TRUE,
      unique_incident_id = id,
      validity_score = validity,
      validity_threshold = 85
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      included_levels = logical(length = 6),
      unique_incident_id = id,
      validity_score = validity,
      validity_threshold = 85
    ),
    regexp = "included_levels.*must be of class.*numeric, character, factor, integer"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = fake,
      unique_incident_id = id,
      validity_score = validity,
      validity_threshold = 85
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      unique_incident_id = "faked",
      validity_score = validity,
      validity_threshold = 85
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      validity_score = a_fake,
      validity_threshold = 85
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_13: input validation", {
  data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("I", "II", "III", "IV"),
    validity = c(90, 85, 75, 92)
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = "not a data",
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity
    ),
    "data.*must be of class.*data\\.frame, tbl, tbl_df"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data |> dplyr::mutate(trauma_level = 1:4),
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity
    ),
    "level.*must be of class.*character.*factor"
  )

  bad_id <- data |> dplyr::mutate(id = c(T, T, F, F))
  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = bad_id,
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity
    ),
    "must be of class"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data |> dplyr::mutate(validity = as.character(validity)),
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity
    ),
    "validity_score.*must be.*numeric"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data |> dplyr::mutate(validity = c(90, -1, 105, 92)),
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity
    ),
    "validity_score.*values must be contained within range.*0, 100.*Range of this input was.*-1, 105"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity,
      validity_threshold = "high"
    ),
    "validity_threshold.*must be.*numeric"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity,
      groups = 123
    ),
    "must be strings"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity,
      groups = c("nope")
    ),
    "groups.*contains invalid column names.*nope.*Valid column names are.*id, trauma_level, validity"
  )

  testthat::expect_error(
    traumar::seqic_indicator_13(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      validity_score = validity,
      calculate_ci = "bogus"
    ),
    "calculate_ci.*contains invalid values.*bogus.*Valid values are.*wilson, clopper-pearson"
  )
})

testthat::test_that("seqic_indicator_13: correct computation", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "III", "IV", "I", "II"),
    validity = c(90, 85, 75, 92, 86, 70)
  )

  out <- traumar::seqic_indicator_13(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    validity_score = validity,
    validity_threshold = 85
  )

  testthat::expect_equal(nrow(out), 1)
  testthat::expect_equal(out$numerator_13, 4)
  testthat::expect_equal(out$denominator_13, 6)
  testthat::expect_equal(out$seqic_13, 4 / 6)
  testthat::expect_equal(out$data, "population/sample")
})

testthat::test_that("seqic_indicator_13: grouping works", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "I", "II", "II", "III", "III"),
    region = c("A", "A", "A", "B", "B", "B"),
    validity = c(90, 84, 88, 70, 95, 60)
  )

  out <- traumar::seqic_indicator_13(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    validity_score = validity,
    groups = "region"
  )

  testthat::expect_equal(nrow(out), 2)
  testthat::expect_equal(
    out |> dplyr::filter(region == "A") |> dplyr::pull(numerator_13),
    2
  )
  testthat::expect_equal(
    out |> dplyr::filter(region == "A") |> dplyr::pull(denominator_13),
    3
  )
})

testthat::test_that("seqic_indicator_13: CI calculation works", {
  data <- tibble::tibble(
    id = 1:10,
    trauma_level = rep("I", 10),
    validity = c(rep(90, 7), rep(70, 3))
  )

  out <- traumar::seqic_indicator_13(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    validity_score = validity,
    calculate_ci = "wilson"
  )

  testthat::expect_true("lower_ci_13" %in% names(out))
  testthat::expect_true("upper_ci_13" %in% names(out))
  testthat::expect_true(out$lower_ci_13 < out$seqic_13)
  testthat::expect_true(out$upper_ci_13 > out$seqic_13)
})

testthat::test_that("seqic_indicator_13: zero denominator handled", {
  data <- tibble::tibble(
    id = 1:3,
    trauma_level = c("V", "V", "V"),
    validity = c(90, 95, 88)
  )

  out <- traumar::seqic_indicator_13(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    validity_score = validity
  )

  testthat::expect_equal(out$denominator_13, 0)
  testthat::expect_true(is.na(out$seqic_13))
})

testthat::test_that("seqic_indicator_13: deduplication works", {
  data <- tibble::tibble(
    id = c(1, 1, 2, 3, 4),
    trauma_level = c("I", "I", "I", "I", "I"),
    validity = c(90, 90, 80, 85, 95)
  )

  out <- traumar::seqic_indicator_13(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    validity_score = validity
  )

  testthat::expect_equal(out$denominator_13, 4) # unique ids only
})

#' @title SEQIC Indicator 8 - Survival by Risk Group
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Calculates the proportion of patients who survived based on risk groups
#' existing in the data among trauma patients transported to Level I–IV trauma
#' centers.
#'
#' @inheritParams seqic_indicator_1
#' @param mortality_indicator A logical, character, or factor variable
#'   indicating whether the patient died at the trauma center. Accepts values
#'   like `TRUE`/`FALSE` or `"Yes"`/`"No"`.
#' @param risk_group A character or factor column indicating the patient's risk
#'   group (e.g., "High", "Moderate", "Low"). See risk definitions below.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details
#'
#' \itemize{
#'   \item Filters the dataset to include only trauma center levels I through
#'   IV.
#'   \item Deduplicates the dataset using `unique_incident_id` to ensure one
#'   record per incident.
#'   \item Accepts a mortality indicator that may be logical, character, or
#'   factor, and identifies survivors as those with values of `FALSE` or `"No"`.
#'   \item Requires a predefined `risk_group` variable representing categories
#'   such as "Low", "Moderate", or "High" risk.
#'   \item Calculates overall survival proportions and survival proportions
#'   stratified by risk group.
#'   \item Optionally includes 95% confidence intervals using binomial methods
#'   if `calculate_ci` is specified.
#' }
#'
#' @note
#'
#' This function calculates survival outcomes for patients transported
#' to trauma centers, stratified by risk of mortality. Risk groups—low,
#' moderate, and high— are defined by the Iowa System Evaluation and Quality
#' Improvement Committee (SEQIC) as described below. Users may also apply
#' alternative risk stratification methods if preferred.
#'
#' \itemize{
#'   \item Abnormal Physiology Criteria: GCS 3–5; Respirations <5 or >30 per
#'   minute; Systolic BP <60 mm Hg
#'   \item High Risk: Probability of Survival < 0.2; ISS > 41; ISS > 24 with
#'   abnormal physiology
#'   \item Moderate Risk: Probability of Survival 0.2–0.5; ISS 16–41
#'   \item Low Risk: Probability of Survival > 0.5; ISS < 16; Normal physiology
#' }
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @returns
#' A named list with two tibbles:
#'
#' `overall`: A tibble summarizing overall mortality among trauma patients,
#' grouped by the variables specified in `groups`. Columns include:
#'
#' \itemize{
#'   \item `numerator_8_all` (number of survivors),
#'   \item `denominator_8_all` (total number
#'   of unique trauma incidents),
#'   \item `seqic_8_all` (survival proportion), and optionally
#'   \item `lower_ci_8`,
#'   \item `upper_ci_8` (confidence interval bounds if `calculate_ci` is
#'   specified).
#'   }
#'
#' `risk_group`: A tibble summarizing mortality stratified by risk group and any
#' additional grouping variables. Columns include:
#'
#' \itemize{
#'   \item `risk_group` (used for stratification),
#'   \item `numerator_8_risk` (survivors per group),
#'   \item `denominator_8_risk` (total incidents per group),
#'   \item `seqic_8_risk` (survival proportion per group), and optionally
#'   \item `lower_ci_8_risk`,
#'   \item `upper_ci_8_risk` (confidence interval bounds if `calculate_ci` is
#'   specified).
#' }
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Simulated dataset for SEQIC Indicator 8
#' test_data <- tibble::tibble(
#'   id = as.character(1:12),
#'   trauma_level = c("I", "II", "III", "IV", "V", "II", "I", "III", "IV", "II",
#'   "I", "III"),
#'   mortality = c(FALSE, "No", TRUE, "Yes", FALSE, TRUE, "No", FALSE, "Yes",
#'   FALSE, TRUE, "No"),
#'   risk = c("High", "High", "Moderate", "Moderate", "Low", "Low", "High",
#'   "Moderate", "Low", "Moderate", "High", "Low")
#' )
#'
#' # Run indicator 8 function
#' traumar::seqic_indicator_8(
#'   data = test_data,
#'   level = trauma_level,
#'   unique_incident_id = id,
#'   mortality_indicator = mortality,
#'   risk_group = risk
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_8 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  mortality_indicator,
  risk_group,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure that the `mortality_indicator`
  mortality_indicator_check <- validate_data_pull(
    input = data,
    col = {{ mortality_indicator }},
    type = "error",
    var_name = "mortality_indicator"
  )

  # Validate the `mortality_indicator` column
  validate_class(
    input = mortality_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "mortality_indicator"
  )

  # Ensure that `risk_group` can be validated
  risk_group_check <- validate_data_pull(
    input = data,
    col = {{ risk_group }},
    type = "error",
    var_name = "risk_group"
  )

  # Validate the `risk_group` column
  validate_character_factor(
    input = risk_group_check,
    type = "error",
    var_name = "risk_group"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data`.
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Initiate the output as a list ----
  seqic_8 <- list()

  # Overall mortality, one row per unique incident ----
  seqic_8_all <- data |>
    dplyr::filter({{ level }} %in% included_levels) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::summarize(
      numerator_8_all = sum(
        !({{ mortality_indicator }} %in% c(TRUE, "Yes")),
        na.rm = TRUE
      ),
      denominator_8_all = dplyr::n(),
      seqic_8_all = dplyr::if_else(
        denominator_8_all > 0,
        numerator_8_all / denominator_8_all,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # Mortality stratified by risk group ----
  seqic_8_risk <- data |>
    dplyr::filter({{ level }} %in% included_levels) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::summarize(
      numerator_8_risk = sum(
        !({{ mortality_indicator }} %in% c(TRUE, "Yes")),
        na.rm = TRUE
      ),
      denominator_8_risk = dplyr::n(),
      seqic_8_risk = dplyr::if_else(
        denominator_8_risk > 0,
        numerator_8_risk / denominator_8_risk,
        NA_real_
      ),
      .by = c({{ groups }}, {{ risk_group }})
    )

  # Compute confidence intervals if requested ----
  if (!is.null(calculate_ci)) {
    seqic_8_all <- seqic_8_all |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_8_all,
          x = numerator_8_all,
          n = denominator_8_all,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_8 = lower_ci, upper_ci_8 = upper_ci)
      )

    seqic_8_risk <- seqic_8_risk |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_8_risk,
          x = numerator_8_risk,
          n = denominator_8_risk,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_8_risk = lower_ci, upper_ci_8_risk = upper_ci)
      )
  }

  # Label output or arrange by grouping vars ----
  if (is.null(groups)) {
    seqic_8$overall <- seqic_8_all |>
      tibble::add_column(data = "population/sample", .before = 1)
    seqic_8$risk_group <- seqic_8_risk |>
      tibble::add_column(data = "population/sample risk groups", .before = 1) |>
      dplyr::arrange({{ risk_group }})
  } else {
    seqic_8$overall <- seqic_8_all |>
      dplyr::arrange(!!!rlang::syms(groups))
    seqic_8$risk_group <- seqic_8_risk |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  # Return the final summarized dataset. ----
  return(seqic_8)
}

# tests/testthat/test-seqic_indicator_8.R

testthat::test_that("seqic_indicator_8() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  test_data <- dplyr::tibble(
    id = as.character(1:10),
    trauma_level = c("I", "II", "III", "IV", "V", "II", "I", "III", "IV", "II"),
    mortality = c(
      FALSE,
      "No",
      TRUE,
      "Yes",
      FALSE,
      TRUE,
      "No",
      FALSE,
      "Yes",
      TRUE
    ),
    risk = c(
      "High",
      "High",
      "Moderate",
      "Moderate",
      "Low",
      "Low",
      "High",
      "Moderate",
      "Low",
      "Moderate"
    )
  )

  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = test_data,
      level = "fake",
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = test_data,
      level = trauma_level,
      unique_incident_id = not_a_column,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = still_alive,
      risk_group = risk
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = test_data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = low_risk_group
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_8() data validation - data must be a data.frame or tibble", {
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = "not_a_data",
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    "data.*must be of class.*data.frame, tbl, tbl_df"
  )
})

testthat::test_that("seqic_indicator_8() data validation - level must be character or factor", {
  data <- tibble::tibble(
    trauma_level = as.numeric(1:3),
    id = as.character(1:3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    "level.*must be of class.*character.*factor"
  )
})

testthat::test_that("seqic_indicator_8() data validation - unique_incident_id must be character, numeric, or factor", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = list(1, 2, 3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    "unique_incident_id.*must be of class"
  )
})

testthat::test_that("seqic_indicator_8() data validation - mortality_indicator must be character, factor, or logical", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = as.character(1:3),
    mortality = as.Date(c("2023-01-01", "2023-01-02", "2023-01-03")),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    "mortality_indicator.*must be of class.*logical, character, factor"
  )
})

testthat::test_that("seqic_indicator_8() data validation - risk_group must be character or factor", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = as.character(1:3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = as.Date(c("2023-01-01", "2023-01-02", "2023-01-03"))
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk
    ),
    "risk_group.*must be of class.*character.*factor"
  )
})

testthat::test_that("seqic_indicator_8() data validation - groups must be character vector", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = as.character(1:3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk,
      groups = 1
    ),
    "groups.*must be of class.*character.*factor"
  )
})

testthat::test_that("seqic_indicator_8() data validation - all groups must exist in data", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = as.character(1:3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk,
      groups = c("nonexistent_var")
    ),
    "groups.*contains invalid column names.*nonexistent_var.*Valid column names are.*trauma_level, id, mortality, risk"
  )
})

testthat::test_that("seqic_indicator_8() data validation - calculate_ci must be wilson or clopper-pearson", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = as.character(1:3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk,
      calculate_ci = "invalid"
    ),
    "calculate_ci.*contains invalid values.*invalid.*Valid values are.*wilson, clopper-pearson"
  )
})

testthat::test_that("seqic_indicator_8() data validation - included_levels must be character, numeric, or factor", {
  data <- tibble::tibble(
    trauma_level = as.character(c("I", "II", "III")),
    id = as.character(1:3),
    mortality = c(TRUE, FALSE, TRUE),
    risk = c("Low", "Moderate", "High")
  )
  testthat::expect_error(
    traumar::seqic_indicator_8(
      data = data,
      level = trauma_level,
      unique_incident_id = id,
      mortality_indicator = mortality,
      risk_group = risk,
      included_levels = list("I", "II")
    ),
    "included_levels.*must be of class"
  )
})

testthat::test_that("seqic_indicator_8 correctly calculates overall and risk group survival", {
  test_data <- dplyr::tibble(
    id = as.character(1:10),
    trauma_level = c("I", "II", "III", "IV", "V", "II", "I", "III", "IV", "II"),
    mortality = c(
      FALSE,
      "No",
      TRUE,
      "Yes",
      FALSE,
      TRUE,
      "No",
      FALSE,
      "Yes",
      TRUE
    ),
    risk = c(
      "High",
      "High",
      "Moderate",
      "Moderate",
      "Low",
      "Low",
      "High",
      "Moderate",
      "Low",
      "Moderate"
    )
  )

  result <- traumar::seqic_indicator_8(
    data = test_data,
    level = trauma_level,
    unique_incident_id = id,
    mortality_indicator = mortality,
    risk_group = risk
  )

  # Check output structure
  testthat::expect_named(result, c("overall", "risk_group"))
  testthat::expect_s3_class(result$overall, "tbl_df")
  testthat::expect_s3_class(result$risk_group, "tbl_df")

  # Check correct numerators/denominators
  testthat::expect_equal(result$overall$numerator_8_all, 4)
  testthat::expect_equal(result$overall$denominator_8_all, 9)
  testthat::expect_equal(result$overall$seqic_8_all, 4 / 9)

  # Check stratified values
  risk_summary <- result$risk_group
  testthat::expect_true(all(
    risk_summary$denominator_8_risk >= risk_summary$numerator_8_risk
  ))
  testthat::expect_true(all(risk_summary$seqic_8_risk <= 1))
})

testthat::test_that("seqic_indicator_8 filters by included_levels", {
  data <- dplyr::tibble(
    id = 1:5,
    trauma_level = c("I", "V", "II", "III", "IV"),
    mortality = c("No", "Yes", FALSE, TRUE, "No"),
    risk = rep("Moderate", 5)
  )

  result <- traumar::seqic_indicator_8(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    mortality_indicator = mortality,
    risk_group = risk
  )

  # Should exclude trauma_level "V"
  testthat::expect_equal(result$overall$denominator_8_all, 4)
})

testthat::test_that("seqic_indicator_8 includes confidence intervals when calculate_ci is specified", {
  data <- dplyr::tibble(
    id = 1:6,
    trauma_level = rep("II", 6),
    mortality = c("No", "Yes", "No", "No", "Yes", FALSE),
    risk = c("Low", "Low", "Low", "High", "High", "High")
  )

  result <- traumar::seqic_indicator_8(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    mortality_indicator = mortality,
    risk_group = risk,
    calculate_ci = "wilson"
  )

  testthat::expect_true(all(
    c("lower_ci_8", "upper_ci_8") %in% names(result$overall)
  ))
  testthat::expect_true(all(
    c("lower_ci_8_risk", "upper_ci_8_risk") %in% names(result$risk_group)
  ))
})

testthat::test_that("seqic_indicator_8 handles groupings correctly", {
  data <- dplyr::tibble(
    id = 1:6,
    trauma_level = rep("III", 6),
    mortality = c(TRUE, FALSE, FALSE, "Yes", "No", "No"),
    risk = rep("Low", 6),
    region = c("East", "East", "West", "West", "East", "West")
  )

  result <- traumar::seqic_indicator_8(
    data = data,
    level = trauma_level,
    unique_incident_id = id,
    mortality_indicator = mortality,
    risk_group = risk,
    groups = "region"
  )

  testthat::expect_true("region" %in% names(result$overall))
  testthat::expect_equal(nrow(result$overall), 2)
})

#' @title SEQIC Indicator 11 – Overtriage for Minor Trauma Patients
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Calculates SEQIC Indicator 11, which estimates the proportion of minor trauma
#' patients who were transferred into a trauma center and remained in the
#' Emergency Department for less than 24 hours. This indicator is designed to
#' identify potential overtriage events within the trauma system. Minor trauma
#' patients are identified using the Injury Severity Score (ISS < 9). Patients
#' must not have been transferred out and must have been received at a trauma
#' center level included in `included_levels`.
#'
#' @inheritParams seqic_indicator_1
#' @inheritParams seqic_indicator_4
#' @inheritParams seqic_indicator_6
#' @inheritParams seqic_indicator_10
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @returns A tibble summarizing the numerator, denominator, and proportion of
#'   overtriaged patients (Indicator 11), with optional 95% confidence
#'   intervals.
#'
#' @details
#'
#' This function:
#' \itemize{
#'   \item Filters the dataset to include only patients treated at trauma
#'   centers designated Level I through IV.
#'   \item Excludes patients transferred out and retains only those received by
#'   the trauma center.
#'   \item Deduplicates incident-level records using `unique_incident_id`.
#'   \item Classifies patients as low-risk based on the  Injury Severity Score
#'   (ISS < 9).
#'   \item Flags low-risk patients who were discharged from the ED in under 24
#'   hours.
#'   \item Stratifies results by one or more user-defined grouping variables.
#'   \item Returns a summarized tibble with the number of eligible low-risk
#'   short-stay discharges (numerator), all received patients meeting inclusion
#'   criteria (denominator), and the resulting proportion.
#'   \item Optionally includes 95% confidence intervals if `calculate_ci` is
#'   specified.
#' }
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Simulated data for SEQIC Indicator 11
#' test_data <- tibble::tibble(
#'   id = as.character(1:10),
#'   trauma_level = c("I", "II", "III", "IV", "II", "I", "IV", "III", "II",
#'   "I"),
#'   transferred_out = c(FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE,
#'   FALSE, FALSE),
#'   received = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
#'   iss = c(4, 8, 10, 6, 5, 7, 6, 15, 3, 2),
#'   ed_LOS = c(6, 20, 30, 18, 8, 5, 22, 40, 2, 4),
#'   region = rep(c("East", "West"), each = 5)
#' )
#'
#' # Run the function
#' traumar::seqic_indicator_11(
#'   data = test_data,
#'   level = trauma_level,
#'   included_levels = c("I", "II", "III", "IV"),
#'   transfer_out_indicator = transferred_out,
#'   receiving_indicator = received,
#'   unique_incident_id = id,
#'   iss = iss,
#'   ed_LOS = ed_LOS,
#'   groups = "region",
#'   calculate_ci = "clopper-pearson"
#' )
#'
#' @references
#'
#' Roden-Foreman JW, Rapier NR, Yelverton L, Foreman ML. Asking a Better
#' Question: Development and Evaluation of the Need For Trauma Intervention
#' (NFTI) Metric as a Novel Indicator of Major Trauma. J Trauma Nurs. 2017
#' May/Jun;24(3):150-157. doi: 10.1097/JTN.0000000000000283. PMID: 28486318.
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export

seqic_indicator_11 <- function(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  transfer_out_indicator,
  receiving_indicator,
  unique_incident_id,
  iss,
  ed_LOS,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # Validate if `data` is a data frame or tibble. ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # Validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "integer", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # Ensure that `iss` can be validated ----
  iss_check <- validate_data_pull(
    input = data,
    col = {{ iss }},
    type = "error",
    var_name = "iss"
  )

  # Validation of `iss` ----
  validate_numeric(
    input = iss_check,
    min = 0,
    max = 75,
    type = "error",
    var_name = "iss"
  )

  # Ensure that `transfer_out_indicator` can be validated  ----
  transfer_out_indicator_check <- validate_data_pull(
    input = data,
    col = {{ transfer_out_indicator }},
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Validate `transfer_out_indicator` ----
  validate_class(
    input = transfer_out_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "transfer_out_indicator"
  )

  # Ensure that `receiving_indicator` can be validated
  receiving_indicator_check <- validate_data_pull(
    input = data,
    col = {{ receiving_indicator }},
    type = "error",
    var_name = "receiving_indicator"
  )

  # Validate that `receiving_indicator` is character, factor, or logical. ----
  validate_class(
    input = receiving_indicator_check,
    class_type = c("logical", "character", "factor"),
    logic = "or",
    type = "error",
    var_name = "receiving_indicator"
  )

  # Ensure that ed_LOS can be validated ----
  ed_los_check <- validate_data_pull(
    input = data,
    col = {{ ed_LOS }},
    type = "error",
    var_name = "ed_LOS"
  )

  # Validate `ed_LOS` ----
  validate_numeric(
    input = ed_los_check,
    min = NULL,
    max = NULL,
    type = "error",
    var_name = "ed_LOS"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all `groups` exist in the `data` ----
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE,
    var_name = "calculate_ci"
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor", "integer"),
    type = "error",
    logic = "or"
  )

  ###___________________________________________________________________________
  ### Data preparation ----
  ###___________________________________________________________________________

  # Dynamically classify patients using ISS logic ----
  data_prep <- data |>
    dplyr::filter(
      {{ level }} %in% included_levels,
      {{ transfer_out_indicator }} %in% c("No", FALSE),
      {{ receiving_indicator }} %in% c("Yes", TRUE)
    ) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::mutate(
      # Patients who clearly did not require activation (ISS < 9)
      minor_trauma = {{ iss }} < 9,
      los_less_24_hrs = {{ ed_LOS }} < 1440
    )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # state 11 ----
  seqic_11 <- data_prep |>
    dplyr::summarize(
      numerator_11 = sum(
        minor_trauma &
          los_less_24_hrs,
        na.rm = TRUE
      ),
      denominator_11 = dplyr::n(),
      seqic_11 = dplyr::if_else(
        denominator_11 > 0,
        numerator_11 / denominator_11,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # Optionally calculate confidence intervals for the proportions: ----
  # - If `calculate_ci` is provided, apply a binomial confidence interval method (Wilson or Clopper-Pearson).
  # - `nemsqa_binomial_confint()` calculates the confidence intervals for the proportion.
  # - Select only the relevant columns and rename the CI columns for clarity.
  if (!is.null(calculate_ci)) {
    seqic_11 <- seqic_11 |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_11,
          x = numerator_11, # Number of successes (numerator).
          n = denominator_11, # Number of trials (denominator).
          method = calculate_ci, # Confidence interval calculation method (e.g., "wilson").
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_11 = lower_ci, upper_ci_11 = upper_ci) # Rename CI columns.
      )
  }

  # Assign a label ----
  # to indicate whether the data represents population or sample-level results:
  # - If no grouping is applied, label the data as "Population/Sample".
  if (is.null(groups)) {
    seqic_11 <- seqic_11 |>
      tibble::add_column(data = "population/sample", .before = "numerator_11") # Add the label column.
  } else if (!is.null(groups)) {
    seqic_11 <- seqic_11 |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  return(seqic_11)
}

# tests/testthat/test-seqic_indicator_11.R

testthat::test_that("seqic_indicator_11() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "III", "IV", "II", "III"),
    transferred_out = c(FALSE, FALSE, TRUE, FALSE, FALSE, FALSE),
    received = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    iss = c(4, 8, 10, 6, 5, 3),
    ed_LOS = c(8, 22, 12, 5, 7, 3),
    region = c("East", "West", "East", "East", "West", "West")
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = fake,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      unique_incident_id = id,
      iss = iss,
      ed_LOS = ed_LOS,
      groups = "region"
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = blah,
      receiving_indicator = received,
      unique_incident_id = id,
      iss = iss,
      ed_LOS = ed_LOS,
      groups = "region"
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = transferred_out,
      receiving_indicator = error,
      unique_incident_id = id,
      iss = iss,
      ed_LOS = ed_LOS,
      groups = "region"
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      unique_incident_id = TRUE,
      iss = iss,
      ed_LOS = ed_LOS,
      groups = "region"
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      unique_incident_id = id,
      iss = FALSE,
      ed_LOS = ed_LOS,
      groups = "region"
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      unique_incident_id = id,
      iss = iss,
      ed_LOS = length_of_stay,
      groups = "region"
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_11 works with minimal valid input", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "III", "IV", "II", "III"),
    transferred_out = c(FALSE, FALSE, TRUE, FALSE, FALSE, FALSE),
    received = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    iss = c(4, 8, 10, 6, 5, 3),
    ed_LOS = c(8, 22, 12, 5, 7, 3),
    region = c("East", "West", "East", "East", "West", "West")
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS,
    groups = "region"
  )

  testthat::expect_s3_class(res, "tbl_df")

  expected_columns <- c("numerator_11", "denominator_11", "seqic_11")
  testthat::expect_true(all(expected_columns %in% names(res)))
})

testthat::test_that("seqic_indicator_11 correctly validates columns", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "III", "IV", "II", "III"),
    transferred_out = c(FALSE, FALSE, TRUE, FALSE, FALSE, FALSE),
    received = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
    iss = c(4, 8, 10, 6, 5, 3),
    ed_LOS = c(8, 22, 12, 5, 7, 3),
    region = c("East", "West", "East", "East", "West", "West")
  )

  testthat::expect_error(seqic_indicator_11(data = "not_a_data"))

  bad_level <- data |> dplyr::mutate(trauma_level = 1:6)
  testthat::expect_error(
    seqic_indicator_11(
      data = bad_level,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "level.*must be of class.*character.*factor"
  )

  bad_id <- data |> dplyr::mutate(id = rep(TRUE, 6))
  testthat::expect_error(
    seqic_indicator_11(
      data = bad_id,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "unique_incident_id.*must be of class.*numeric, integer, character, factor"
  )

  bad_transfer_indicator <- data |>
    dplyr::mutate(transferred_out = numeric(length = 6))
  testthat::expect_error(
    seqic_indicator_11(
      data = bad_transfer_indicator,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "transfer_out_indicator.*must be of class.*logical.*character.*factor"
  )

  bad_receiving_indicator <- data |>
    dplyr::mutate(received = numeric(length = 6))
  testthat::expect_error(
    seqic_indicator_11(
      data = bad_receiving_indicator,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "receiving_indicator.*must be of class.*logical, character, factor"
  )

  bad_los <- data |>
    dplyr::mutate(ed_LOS = character(length = 6))
  testthat::expect_error(
    seqic_indicator_11(
      data = bad_los,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "ed_LOS.*must be.*numeric"
  )

  testthat::expect_error(
    seqic_indicator_11(
      data = data,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS,
      groups = 1
    ),
    "groups.*must be of class.*character.*factor"
  )

  testthat::expect_error(
    seqic_indicator_11(
      data = data,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS,
      groups = "group"
    ),
    "groups.*contains invalid column names.*group.*Valid column names are.*id, trauma_level, transferred_out, received, iss, ed_LOS, region"
  )

  testthat::expect_error(
    seqic_indicator_11(
      data = data,
      unique_incident_id = id,
      level = trauma_level,
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS,
      calculate_ci = "z"
    ),
    "calculate_ci.*contains invalid values.*z.*Valid values are.*wilson, clopper-pearson"
  )

  testthat::expect_error(
    seqic_indicator_11(
      data = data,
      unique_incident_id = id,
      level = trauma_level,
      included_levels = c(T, F, NA),
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "included_levels.*must be of class.*numeric, character, factor, integer"
  )
})

testthat::test_that("seqic_indicator_11 includes CI columns when calculate_ci is specified", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "II", "III", "IV", "II", "III"),
    transferred_out = rep(FALSE, 6),
    received = rep(TRUE, 6),
    iss = c(4, 5, 3, 8, 7, 6),
    ed_LOS = c(5, 6, 7, 20, 21, 8)
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = c("I", "II", "III", "IV"),
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS,
    calculate_ci = "clopper-pearson"
  )

  testthat::expect_true(all(c("lower_ci_11", "upper_ci_11") %in% names(res)))
})

testthat::test_that("seqic_indicator_11 fails with both invalid and missing arguments", {
  data <- tibble::tibble(
    id = 1:3,
    trauma_level = c("I", "II", "III"),
    transferred_out = c(FALSE, FALSE, FALSE),
    received = c(TRUE, TRUE, TRUE),
    iss = c("low", "medium", "high"), # invalid
    ed_LOS = c(5, 6, 7)
  )

  testthat::expect_error(
    traumar::seqic_indicator_11(
      data = data,
      level = trauma_level,
      included_levels = c("I", "II", "III", "IV"),
      transfer_out_indicator = transferred_out,
      receiving_indicator = received,
      unique_incident_id = id,
      iss = iss,
      ed_LOS = ed_LOS
    ),
    "iss.*must be.*numeric"
  )
})

testthat::test_that("seqic_indicator_11 correctly filters and deduplicates input", {
  data <- tibble::tibble(
    id = c(1, 1, 2, 3),
    trauma_level = c("I", "I", "II", "III"),
    transferred_out = c(FALSE, FALSE, FALSE, TRUE),
    received = c(TRUE, TRUE, TRUE, TRUE),
    iss = c(4, 4, 8, 6),
    ed_LOS = c(10, 10, 15, 20)
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = c("I", "II", "III"),
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS
  )

  testthat::expect_equal(res$denominator_11, 2) # One duplicated ID, one transfer out excluded
})

testthat::test_that("seqic_indicator_11 calculates correct numerator and denominator", {
  data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("I", "II", "II", "III"),
    transferred_out = c(FALSE, FALSE, FALSE, FALSE),
    received = c(TRUE, TRUE, TRUE, TRUE),
    iss = c(5, 6, 10, 8),
    ed_LOS = c(10, 120, 30, 1441)
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = c("I", "II", "III"),
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS
  )

  testthat::expect_equal(res$numerator_11, 2) # Two patients with ISS<9 and LOS<1440
  testthat::expect_equal(res$denominator_11, 4)
  testthat::expect_equal(res$seqic_11, 0.5)
})

testthat::test_that("seqic_indicator_11 appends CI columns when requested", {
  data <- tibble::tibble(
    id = 1:10,
    trauma_level = rep("I", 10),
    transferred_out = rep(FALSE, 10),
    received = rep(TRUE, 10),
    iss = c(3, 4, 5, 6, 7, 10, 12, 4, 5, 8),
    ed_LOS = rep(120, 10)
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = "I",
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS,
    calculate_ci = "wilson"
  )

  testthat::expect_true(all(c("lower_ci_11", "upper_ci_11") %in% names(res)))
})

testthat::test_that("seqic_indicator_11 includes 'population/sample' label if no grouping", {
  data <- tibble::tibble(
    id = 1:3,
    trauma_level = c("I", "II", "III"),
    transferred_out = rep(FALSE, 3),
    received = rep(TRUE, 3),
    iss = c(4, 6, 5),
    ed_LOS = c(12, 14, 16)
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = c("I", "II", "III"),
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS
  )

  testthat::expect_true("data" %in% names(res))
  testthat::expect_equal(res$data, "population/sample")
})

testthat::test_that("seqic_indicator_11 orders by group variable if provided", {
  data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("II", "II", "I", "III"),
    transferred_out = rep(FALSE, 4),
    received = rep(TRUE, 4),
    iss = c(4, 4, 4, 4),
    ed_LOS = c(10, 20, 15, 10),
    region = c("Z", "A", "M", "B")
  )

  res <- traumar::seqic_indicator_11(
    data = data,
    level = trauma_level,
    included_levels = c("I", "II", "III"),
    transfer_out_indicator = transferred_out,
    receiving_indicator = received,
    unique_incident_id = id,
    iss = iss,
    ed_LOS = ed_LOS,
    groups = "region"
  )

  testthat::expect_equal(res$region, sort(data$region))
})

#' @title Validate Length of an Input
#'
#' @description
#' This function checks if the length of a vector or list is within a specified
#' range. Depending on the specified type, it will either throw an error, issue
#' a warning, or send a message. It also checks for NULL and NA values based on
#' the specified parameters.
#'
#' @inheritParams validate_numeric
#' @param exact_length The required length of the vector or list. If this
#' argument is used, then min_length and max_length are not required.
#' @param min_length The minimum length of the vector or list.
#' @param max_length The maximum length of the vector or list.
#' @inheritParams validate_data_pull
#'
#' @return NULL. The function is used for its side effects.
#'
#' @author
#' Nicolas Foss, Ed.D., MS
#' 
#' @keywords internal
#' @noRd
#'
validate_length <- function(
  input,
  exact_length = NULL,
  min_length = NULL,
  max_length = NULL,
  type = c("error", "warning", "message"),
  na_ok = TRUE,
  null_ok = TRUE,
  var_name = NULL,
  calls = NULL
) {
  # Validate the type argument
  type <- match.arg(arg = type, choices = c("error", "warning", "message"))

  # Define number of callers to go back
  calls <- ifelse(is.null(calls), 2, calls)

  # Get the input name, optionally using var_name
  if (is.null(var_name)) {
    input_name <- deparse(substitute(input))
  } else {
    # Validate var_name
    validate_character_factor(input = var_name, type = "error", calls = 1)

    # Initialize input_name using var_name
    input_name <- var_name
  }

  # Check if the input is NULL
  if (is.null(input)) {
    if (!null_ok) {
      validate_error_type(
        input = input_name,
        message = "must not be NULL.",
        type = "error",
        calls = calls
      )
    }
    return(NULL)
  }

  # Check for NA values if na_ok is FALSE
  if (!na_ok && any(is.na(input))) {
    validate_error_type(
      input = input_name,
      message = "must not contain NA values.",
      type = "error",
      calls = calls
    )
  }

  # Check if the exact length is specified
  if (!is.null(exact_length)) {
    if (length(input) != exact_length) {
      validate_error_type(
        input = input_name,
        message = glue::glue(
          "must have an exact length of {cli::col_blue(exact_length)}."
        ),
        type = type,
        calls = calls
      )
    }
  } else {
    # Handle nulls in min_length and max_length
    # Max goes to Inf
    max_length <- ifelse(
      all(is.null(exact_length), is.null(max_length), !is.null(min_length)),
      Inf,
      max_length
    )

    # Min goes to -Inf
    min_length <- ifelse(
      all(is.null(exact_length), !is.null(max_length), is.null(min_length)),
      -Inf,
      min_length
    )

    # Get required range
    required_range <- glue::glue("[{min_length}, {max_length}]")

    # Check if the length is within the specified range
    if (length(input) < min_length || length(input) > max_length) {
      validate_error_type(
        input = input_name,
        message = glue::glue(
          "must have a length within range {cli::col_blue(required_range)}."
        ),
        type = type,
        calls = calls
      )
    }
  }
}

#' @title SEQIC Indicator 12 - Timeliness of Data Entry Post-Discharge
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Calculates the proportion of trauma cases where data were entered into the
#' trauma registry within a defined number of days post-discharge. This measure
#' supports trauma system quality improvement by identifying facilities meeting
#' timely documentation expectations.
#'
#' @inheritParams seqic_indicator_1
#' @param facility_id Numeric, character, or factor. Column giving the unique
#'   facility identifiers in the trauma dataset.
#' @param exclude_facility_list Optional. Numeric, character, or factor. List of
#'   facilities to exclude from analysis due to known data quality issues or
#'   other justifiable reasons. Defaults to `NULL`.
#' @param data_entry_time Numeric. Column representing the time in days between
#'   patient discharge and trauma registry data entry.
#' @param data_entry_standard Numeric. The maximum allowable number of days
#'   between discharge and data entry. Records entered within this threshold are
#'   considered timely. Default is `60`.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details
#'
#' This function:
#' \itemize{
#'   \item Filters to include only patients treated at Level I–IV trauma
#'   centers.
#'   \item Excludes records from facilities specified by the user, if
#'   applicable.
#'   \item Deduplicates by `unique_incident_id` to ensure each incident is
#'   counted once.
#'   \item Flags records where data entry occurred within `data_entry_standard`
#'   days of discharge.
#'   \item Optionally calculates confidence intervals using methods from
#'   `nemsqa_binomial_confint()`.
#'   \item Returns a tibble with numerator,
#'   denominator, and proportion of timely entries, with optional confidence
#'   intervals and population/sample labels.
#' }
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @return A tibble summarizing SEQIC Indicator 12 results. Includes numerator,
#'   denominator, and performance rate. 95% confidence intervals are included if
#'   requested.
#'
#' @examples
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Simulated data for SEQIC Indicator 12
#' test_data <- tibble::tibble(
#'   id = as.character(1:10),
#'   trauma_level = c("I", "II", "III", "IV", "II", "I", "IV", "III", "II",
#'   "I"),
#'   facility = c("A", "B", "C", "D", "A", "C", "B", "A", "C", "D"),
#'   data_entry_delay = c(30, 65, 10, 70, 45, 20, 80, 15, 55, 90)
#' )
#'
#' # Run the function
#' traumar::seqic_indicator_12(
#'   data = test_data,
#'   level = trauma_level,
#'   included_levels = c("I", "II", "III", "IV"),
#'   facility_id = facility,
#'   unique_incident_id = id,
#'   exclude_facility_list = c("D"),
#'   data_entry_time = data_entry_delay,
#'   data_entry_standard = 60,
#'   calculate_ci = "wilson"
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_12 <-
  function(
    data,
    level,
    included_levels = c("I", "II", "III", "IV"),
    facility_id,
    exclude_facility_list = NULL,
    unique_incident_id,
    data_entry_time,
    data_entry_standard = 60,
    groups = NULL,
    calculate_ci = NULL,
    ...
  ) {
    ###___________________________________________________________________________
    ### Data validation ----
    ###___________________________________________________________________________

    # Validate if `data` is a data frame or tibble. ----
    validate_data_structure(
      input = data,
      structure_type = c("data.frame", "tbl", "tbl_df"),
      type = "error"
    )

    # make the `level` column accessible for validation ----
    level_check <- validate_data_pull(
      input = data,
      type = "error",
      col = {{ level }},
      var_name = "level"
    )

    # validate `level` ----
    validate_character_factor(
      input = level_check,
      type = "error",
      var_name = "level"
    )

    # make the `unique_incident_id` column accessible for validation ----
    unique_incident_id_check <- validate_data_pull(
      input = data,
      type = "error",
      col = {{ unique_incident_id }},
      var_name = "unique_incident_id"
    )

    # Validate `unique_incident_id` ----
    validate_class(
      input = unique_incident_id_check,
      class_type = c("numeric", "integer", "character", "factor"),
      logic = "or",
      type = "error",
      var_name = "unique_incident_id"
    )

    # Ensure `facility_id` can be validated ----
    facility_id_check <- validate_data_pull(
      input = data,
      col = {{ facility_id }},
      type = "error",
      var_name = "facility_id"
    )

    # Validate facility ID type ----
    validate_class(
      input = facility_id_check,
      class_type = c("numeric", "character", "factor"),
      type = "error",
      var_name = "facility_id"
    )

    # Ensure `data_entry_time` can be validated ----
    data_entry_time_check <- validate_data_pull(
      input = data,
      col = {{ data_entry_time }},
      type = "error",
      var_name = "data_entry_time"
    )

    # Validate that data_entry_time is numeric ----
    validate_numeric(
      input = data_entry_time_check,
      type = "error",
      var_name = "data_entry_time"
    )

    # Validate the cutoff standard for timeliness ----
    validate_numeric(input = data_entry_standard, type = "error")

    # Check if all elements in groups are strings (i.e., character vectors) ----
    validate_character_factor(input = groups, type = "error", null_ok = TRUE)

    # Check if all `groups` exist in the `data` ----
    validate_names(
      input = data,
      check_names = groups,
      type = "error",
      var_name = "groups",
      null_ok = TRUE
    )

    # Validate the `calculate_ci` argument ----
    calculate_ci <- validate_choice(
      input = calculate_ci,
      choices = c("wilson", "clopper-pearson"),
      several.ok = FALSE,
      type = "error",
      null_ok = TRUE,
      var_name = "calculate_ci"
    )

    # Validate the `included_levels` argument ----
    validate_class(
      input = included_levels,
      class_type = c("numeric", "character", "factor", "integer"),
      type = "error",
      logic = "or"
    )

    # Validate optional excluded facility list ----
    validate_class(
      input = exclude_facility_list,
      class_type = c("numeric", "character", "factor"),
      logic = "or",
      type = "error",
      null_ok = TRUE
    )

    ###_________________________________________________________________________
    ### Calculations ----
    ###_________________________________________________________________________

    seqic_12 <- data |>
      # Filter for designated trauma centers only (Levels defined by user)
      dplyr::filter({{ level }} %in% included_levels) |>

      # Exclude any user-specified facilities from the analysis
      dplyr::filter(!{{ facility_id }} %in% exclude_facility_list) |>

      # Deduplicate based on incident ID
      dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>

      # Compute numerator, denominator, and proportion of timely records ----
      dplyr::summarize(
        numerator_12 = sum(
          {{ data_entry_time }} <= data_entry_standard,
          na.rm = TRUE
        ),
        denominator_12 = dplyr::n(),
        seqic_12 = dplyr::if_else(
          denominator_12 > 0,
          numerator_12 / denominator_12,
          NA_real_
        ),
        .by = {{ groups }}
      )

    # Optionally compute confidence intervals using chosen method ----
    if (!is.null(calculate_ci)) {
      seqic_12 <- seqic_12 |>
        dplyr::bind_cols(
          nemsqar::nemsqa_binomial_confint(
            data = seqic_12,
            x = numerator_12,
            n = denominator_12,
            method = calculate_ci,
            ...
          ) |>
            dplyr::select(lower_ci, upper_ci) |>
            dplyr::rename(lower_ci_12 = lower_ci, upper_ci_12 = upper_ci)
        )
    }

    # Add population label or group if applicable ----
    if (is.null(groups)) {
      seqic_12 <- seqic_12 |>
        tibble::add_column(data = "population/sample", .before = "numerator_12")
    } else {
      seqic_12 <- seqic_12 |>
        dplyr::arrange(!!!rlang::syms(groups))
    }

    return(seqic_12)
  }

# tests/testthat/test-seqic_indicator_12.R

testthat::test_that("seqic_indicator_12() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("I", "II", "III", "IV"),
    facility = c("A", "B", "C", "D"),
    data_entry_delay = c(10, 20, 80, 70)
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = TRUE,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      data_entry_standard = 60
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = fake,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      data_entry_standard = 60
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = not_a_column,
      data_entry_time = data_entry_delay,
      data_entry_standard = 60
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = something,
      data_entry_standard = 60
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_12: input validation", {
  data <- tibble::tibble(
    id = 1:3,
    trauma_level = c("I", "II", "III"),
    facility = c("A", "B", "C"),
    data_entry_delay = c(10, 20, 30)
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = "not a data",
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay
    ),
    "data.*must be of class.*data\\.frame, tbl, tbl_df"
  )

  bad_id <- data |> dplyr::mutate(id = logical(3))
  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = bad_id,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay
    ),
    "unique_incident_id.*must be of class.*numeric.*integer.*character.*factor"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      data_entry_standard = "60"
    ),
    "data_entry_standard.*must be.*numeric"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      groups = 123
    ),
    "groups.*must be of class.*character.*factor"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      groups = "group"
    ),
    "groups.*contains invalid column names.*group.*Valid column names are.*id, trauma_level, facility, data_entry_delay"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      included_levels = c(T, F, NA),
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay
    ),
    "included_levels.*must be of class.*numeric, character, factor, integer"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      exclude_facility_list = c(T, F, NA)
    ),
    "exclude_facility_list.*must be of class.*numeric, character, factor"
  )

  bad_data <- data |> dplyr::mutate(trauma_level = as.numeric(1:3))
  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = bad_data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay
    ),
    "level.*must be of class.*character.*factor"
  )

  bad_data2 <- data |> dplyr::mutate(facility = list(1, 2, 3))
  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = bad_data2,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay
    ),
    "must be of class"
  )

  bad_delay <- data |> dplyr::mutate(data_entry_delay = character(3))
  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = bad_delay,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay
    ),
    "data_entry_time.*must be.*numeric"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      groups = c("nonexistent")
    ),
    "groups.*contains invalid column names.*nonexistent.*Valid column names are.*id, trauma_level, facility, data_entry_delay"
  )

  testthat::expect_error(
    traumar::seqic_indicator_12(
      data = data,
      level = trauma_level,
      facility_id = facility,
      unique_incident_id = id,
      data_entry_time = data_entry_delay,
      calculate_ci = "fake"
    ),
    "calculate_ci.*contains invalid values.*fake.*Valid values are.*wilson, clopper-pearson"
  )
})

testthat::test_that("seqic_indicator_12: basic calculation", {
  data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("I", "II", "III", "IV"),
    facility = c("A", "B", "C", "D"),
    data_entry_delay = c(10, 20, 80, 70)
  )

  res <- traumar::seqic_indicator_12(
    data = data,
    level = trauma_level,
    facility_id = facility,
    unique_incident_id = id,
    data_entry_time = data_entry_delay,
    data_entry_standard = 60
  )

  testthat::expect_equal(nrow(res), 1)
  testthat::expect_equal(res$numerator_12, 2)
  testthat::expect_equal(res$denominator_12, 4)
  testthat::expect_equal(res$seqic_12, 0.5)
})

testthat::test_that("seqic_indicator_12: excluded facilities are removed", {
  data <- tibble::tibble(
    id = 1:4,
    trauma_level = c("I", "II", "III", "IV"),
    facility = c("A", "B", "C", "D"),
    data_entry_delay = c(10, 20, 80, 70)
  )

  res <- traumar::seqic_indicator_12(
    data = data,
    level = trauma_level,
    facility_id = facility,
    unique_incident_id = id,
    exclude_facility_list = c("D"),
    data_entry_time = data_entry_delay,
    data_entry_standard = 60
  )

  testthat::expect_equal(res$numerator_12, 2)
  testthat::expect_equal(res$denominator_12, 3)
})

testthat::test_that("seqic_indicator_12: works with grouping", {
  data <- tibble::tibble(
    id = 1:6,
    trauma_level = c("I", "I", "II", "II", "III", "III"),
    facility = c("X", "X", "Y", "Y", "Z", "Z"),
    data_entry_delay = c(10, 90, 30, 70, 40, 65),
    region = c("A", "A", "A", "A", "B", "B")
  )

  res <- traumar::seqic_indicator_12(
    data = data,
    level = trauma_level,
    facility_id = facility,
    unique_incident_id = id,
    data_entry_time = data_entry_delay,
    groups = "region",
    data_entry_standard = 60
  )

  testthat::expect_equal(nrow(res), 2)
  testthat::expect_equal(
    res |> dplyr::filter(region == "A") |> dplyr::pull(seqic_12),
    0.5
  )
  testthat::expect_equal(
    res |> dplyr::filter(region == "B") |> dplyr::pull(seqic_12),
    0.5
  )
})

testthat::test_that("seqic_indicator_12: confidence intervals are added", {
  data <- tibble::tibble(
    id = 1:10,
    trauma_level = rep("I", 10),
    facility = rep("A", 10),
    data_entry_delay = c(rep(20, 6), rep(70, 4))
  )

  res <- traumar::seqic_indicator_12(
    data = data,
    level = trauma_level,
    facility_id = facility,
    unique_incident_id = id,
    data_entry_time = data_entry_delay,
    calculate_ci = "wilson"
  )

  testthat::expect_true("lower_ci_12" %in% names(res))
  testthat::expect_true("upper_ci_12" %in% names(res))
  testthat::expect_true(res$lower_ci_12 < res$seqic_12)
  testthat::expect_true(res$upper_ci_12 > res$seqic_12)
})

#' @title SEQIC Indicator 1 – Trauma Team Response Evaluation
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' This function calculates System Evaluation and Quality Improvement Committee
#' (SEQIC) Indicator 1 (subparts a through f). These indicators assess the
#' timeliness and type of provider response (e.g., surgeon, mid-level,
#' physician) to trauma alerts based on trauma team activation level, hospital
#' trauma level, and time to provider presence. Confidence intervals can
#' optionally be calculated for the proportion, using either the Wilson or
#' Clopper-Pearson method.
#'
#' @param data A data frame containing trauma incident records.
#' @param trauma_team_activation_level Column identifying trauma team activation
#'   level (e.g., Level 1, Level 2).
#' @param trauma_team_physician_service_type Column indicating the type of
#'   medical provider (e.g., Surgery/Trauma, Emergency Medicine). For indicators
#'   1a, 1b, and 1c, `seqic_indicator_1()` will only look for records with the
#'   trauma team member service type documented as marked as 'Surgery/Trauma'.
#'   For Indicators 1d, 1e, and 1f, `seqic_indicator_1()` will look for the
#'   following service types:
#'   \itemize{
#'    \item "Surgery/Trauma",
#'    \item "Emergency Medicine",
#'    \item "Family Practice",
#'    \item "Nurse Practitioner",
#'    \item "Physician Assistant",
#'    \item "Surgery Senior Resident",
#'    \item "Hospitalist",
#'    \item "Internal Medicine"
#'    }
#' @param level Column indicating the trauma center designation level (e.g., I,
#'   II, III, IV).
#' @param included_levels Character vector indicating what facility levels to
#' include in the analysis.  Defaults to `c("I", "II", "III", "IV")`.
#' @param unique_incident_id Unique identifier for each record.
#' @param response_time Numeric variable representing the time (in minutes)
#'   to provider response.
#' @param trauma_team_activation_provider Column identifying the responding
#'   provider for trauma activation.
#' @param groups Additional columns passed as a vector of strings to
#'   `dplyr::summarize()` via the `.by` argument for grouped summaries. Defaults
#'   to `NULL`.
#' @param calculate_ci If `NULL`, 95% confidence intervals will not be
#'   calculated for the performance estimates.  Otherwise, options of "wilson"
#'   or "clopper-pearson" can be supplied to utilize the corresponding methods
#'   to calculate the confidence intervals for the proportions. Defaults to
#'   `NULL`.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function filters and summarizes trauma records to calculate
#' SEQIC Indicators 1a through 1f:
#' \itemize{
#'   \item 1a: Proportion of Level 1 activations at Level I/II centers with
#'   surgical response <= 15 minutes.
#'   \item 1b: Same as 1a, but includes Level III centers and uses <= 30 minutes.
#'   \item 1c: Proportion of Level 1 activations with missing surgical response
#'   time.
#'   \item 1d/e: Response within 5 and 20 minutes, respectively, for specific
#'   provider types and activation levels, includes level I-IV trauma centers.
#'   \item 1f: Proportion of missing response times among the group in 1d/e,
#'   includes level I-IV trauma centers.
#' }
#'
#' @note This function:
#' \itemize{
#'   \item Filters trauma records to those with a trauma team activation level
#'   of "Level 1" and/or "Level 2" based on the indicator.
#'   \item Restricts provider type to surgical, physician, and mid-level
#'   provider roles.
#'   \item Filters trauma center levels to I–IV based on the measure.
#'   \item Calculates the proportion of cases where the response time is within
#'   5, 15, or 30 minutes, depending on the indicator.
#'   \item Computes proportions for trauma activation times, including missing
#'   times and within thresholds.
#' }
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @return A tibble summarizing SEQIC Indicator 1 results across sub-measures
#'   (1a–1f). Includes numerators, denominators, and performance rate for each
#'   indicator. 95% confidence intervals are provided optionally.
#'
#' @examples
#'
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Data
#' data <- tibble::tibble(
#'   incident_id = 1:6,
#'   activation_level = c("Level 1", "Level 1", "Level 2", "Level 1", "Level 2",
#'   "Level 1"),
#'   provider_type = c("Surgery/Trauma", "Emergency Medicine", "Physician
#'   Assistant", "Surgery/Trauma", "Surgery/Trauma", "Family Practice"),
#'   trauma_level = c("I", "II", "III", "I", "III", "IV"),
#'   response_minutes = c(12, 25, 6, NA, 18, 22),
#'   provider = c("Dr. A", "Dr. B", "PA C", "Dr. D", "Dr. E", "NP F")
#' )
#'
#' # Run the function
#' traumar::seqic_indicator_1(
#'   data = data,
#'   trauma_team_activation_level = activation_level,
#'   trauma_team_physician_service_type = provider_type,
#'   level = trauma_level,
#'   unique_incident_id = incident_id,
#'   response_time = response_minutes,
#'   trauma_team_activation_provider = provider,
#'   calculate_ci = "wilson"
#' ) |>
#' tidyr::pivot_longer(cols = -1,
#'                     names_to = "Indicator",
#'                     values_to = "Values"
#'                     )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_1 <- function(
  data,
  trauma_team_activation_level,
  trauma_team_physician_service_type,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  response_time,
  trauma_team_activation_provider,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation  ----
  ###___________________________________________________________________________

  # validate `data` ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error"
  )

  # make the `trauma_team_activation_level` column ----
  # accessible for validation
  trauma_team_activation_level_check <- validate_data_pull(
    input = data,
    col = {{ trauma_team_activation_level }},
    var_name = "trauma_team_activation_level"
  )

  # validate `trauma_team_activation_level` ----
  validate_character_factor(
    input = trauma_team_activation_level_check,
    type = "error",
    var_name = "trauma_team_activation_level"
  )

  # make the `trauma_team_physician_service_type` column ----
  # accessible for validation
  trauma_team_physician_service_type_check <- validate_data_pull(
    input = data,
    col = {{ trauma_team_physician_service_type }},
    var_name = "trauma_team_physician_service_type"
  )

  # validate `trauma_team_physician_service_type` ----
  validate_character_factor(
    input = trauma_team_physician_service_type_check,
    type = "error",
    var_name = "trauma_team_physician_service_type"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("character", "factor", "numeric"),
    logic = "or",
    type = "error",
    var_name = "unique_incident_id"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_class(
    input = level_check,
    class_type = c("character", "factor"),
    logic = "or",
    type = "error",
    var_name = "level"
  )

  # make the `response_time` column accessible for validation ----
  response_time_check <- validate_data_pull(
    input = data,
    col = {{ response_time }},
    var_name = "response_time"
  )

  # validate `response_time` ----
  validate_numeric(
    input = response_time_check,
    min = NULL,
    max = NULL,
    type = "error",
    var_name = "response_time"
  )

  # make the `trauma_team_activation_provider` column ----
  # accessible for validation
  trauma_team_activation_provider_check <- validate_data_pull(
    input = data,
    col = {{ trauma_team_activation_provider }},
    var_name = "trauma_team_activation_provider"
  )

  # validate `trauma_team_activation_provider` ----
  validate_character_factor(
    input = trauma_team_activation_provider_check,
    type = "error",
    var_name = "trauma_team_activation_provider"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(
    input = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Check if all groups exist in the `data` ----
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  # - If not NULL, must be either "wilson" or "clopper-pearson"
  # - Use match.arg() to enforce allowed values
  # - Catch invalid input silently and report cleanly with cli
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("character", "numeric", "factor"),
    logic = "or",
    type = "error",
    var_name = "included_levels"
  )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Indicator 1a – Proportion of Level 1 activations at Level I/II centers ----
  # where the first arriving Surgery/Trauma provider arrived within 15 minutes.
  seqic_1a <- data |>
    dplyr::filter(
      {{ trauma_team_activation_level }} == "Level 1",
      {{ trauma_team_physician_service_type }} == "Surgery/Trauma",
      {{ level }} %in% c("I", "II")
    ) |>
    dplyr::group_by({{ unique_incident_id }}) |>
    dplyr::slice_min({{ response_time }}, n = 1, with_ties = FALSE) |>
    dplyr::ungroup() |>
    dplyr::summarize(
      numerator_1a = sum({{ response_time }} <= 15, na.rm = TRUE),
      denominator_1a = sum(!is.na({{ response_time }})),
      seqic_1a = dplyr::if_else(
        denominator_1a > 0,
        numerator_1a / denominator_1a,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # optionally calculate the confidence intervals for 1a ----
  if (!is.null(calculate_ci)) {
    seqic_1a <- seqic_1a |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_1a,
          x = numerator_1a,
          n = denominator_1a,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_1a = lower_ci, upper_ci_1a = upper_ci)
      )
  }

  # Indicator 1b – Same as 1a but for Level I/II/III centers and 30-minute ----
  # threshold.
  seqic_1b <- data |>
    dplyr::filter(
      {{ trauma_team_activation_level }} == "Level 1",
      {{ trauma_team_physician_service_type }} == "Surgery/Trauma",
      {{ level }} %in% c("I", "II", "III"),
      !is.na({{ response_time }})
    ) |>
    dplyr::group_by({{ unique_incident_id }}) |>
    dplyr::slice_min({{ response_time }}, n = 1, with_ties = FALSE) |>
    dplyr::ungroup() |>
    dplyr::summarize(
      numerator_1b = sum({{ response_time }} <= 30, na.rm = TRUE),
      denominator_1b = sum(!is.na({{ response_time }})),
      seqic_1b = dplyr::if_else(
        denominator_1b > 0,
        numerator_1b / denominator_1b,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # optionally calculate the confidence intervals for 1b ----
  if (!is.null(calculate_ci)) {
    seqic_1b <- seqic_1b |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_1b,
          x = numerator_1b,
          n = denominator_1b,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_1b = lower_ci, upper_ci_1b = upper_ci)
      )
  }

  # Indicator 1c – Proportion of Level 1 activations where arrival time is ----
  # missing.
  seqic_1c <- data |>
    dplyr::filter(
      {{ trauma_team_activation_level }} == "Level 1",
      {{ trauma_team_physician_service_type }} == "Surgery/Trauma",
      {{ level }} %in% c("I", "II", "III")
    ) |>
    dplyr::distinct(
      {{ unique_incident_id }},
      {{ trauma_team_activation_provider }},
      .keep_all = TRUE
    ) |>
    dplyr::summarize(
      numerator_1c = sum(is.na({{ response_time }})),
      denominator_1c = dplyr::n(),
      seqic_1c = dplyr::if_else(
        denominator_1c > 0,
        numerator_1c / denominator_1c,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # optionally calculate the confidence intervals for 1c ----
  if (!is.null(calculate_ci)) {
    seqic_1c <- seqic_1c |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_1c,
          x = numerator_1c,
          n = denominator_1c,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_1c = lower_ci, upper_ci_1c = upper_ci)
      )
  }

  # Combine 1a, 1b, and 1c results; assign label for state-level reporting. ----
  if (is.null(groups)) {
    seqic_1abc <- dplyr::bind_cols(
      seqic_1a,
      seqic_1b,
      seqic_1c
    ) |>
      tibble::add_column(data = "population/sample", .before = "numerator_1a")
  } else {
    seqic_1abc <- seqic_1a |>
      dplyr::full_join(seqic_1b, by = dplyr::join_by(!!!rlang::syms(groups))) |>
      dplyr::full_join(seqic_1c, by = dplyr::join_by(!!!rlang::syms(groups)))
  }

  # Create a provider group string vector to clean up code ----
  provider_group_1de <- c(
    "Surgery/Trauma",
    "Emergency Medicine",
    "Family Practice",
    "Nurse Practitioner",
    "Physician Assistant",
    "Surgery Senior Resident",
    "Hospitalist",
    "Internal Medicine"
  )

  # Indicators 1d and 1e – Broader provider group, Level I-IV centers. ----
  # 1d: Arrival within 5 minutes; 1e: Arrival within 20 minutes.
  seqic_1de <- data |>
    dplyr::filter(
      {{ trauma_team_activation_level }} %in% c("Level 1", "Level 2"),
      {{ trauma_team_physician_service_type }} %in% provider_group_1de,
      {{ level }} %in% {{ included_levels }}
    ) |>
    dplyr::group_by({{ unique_incident_id }}) |>
    dplyr::slice_min({{ response_time }}, n = 1, with_ties = FALSE) |>
    dplyr::ungroup() |>
    dplyr::summarize(
      numerator_1d = sum({{ response_time }} <= 5, na.rm = TRUE),
      denominator_1d = sum(!is.na({{ response_time }})),
      seqic_1d = dplyr::if_else(
        denominator_1d > 0,
        numerator_1d / denominator_1d,
        NA_real_
      ),
      numerator_1e = sum({{ response_time }} <= 20, na.rm = TRUE),
      denominator_1e = sum(!is.na({{ response_time }})),
      seqic_1e = dplyr::if_else(
        denominator_1e > 0,
        numerator_1e / denominator_1e,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # optionally calculate the confidence intervals for 1de ----
  if (!is.null(calculate_ci)) {
    seqic_1de <- seqic_1de |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_1de,
          x = numerator_1d,
          n = denominator_1d,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_1d = lower_ci, upper_ci_1d = upper_ci),
        nemsqar::nemsqa_binomial_confint(
          data = seqic_1de,
          x = numerator_1e,
          n = denominator_1e,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_1e = lower_ci, upper_ci_1e = upper_ci)
      ) |>
      dplyr::relocate(lower_ci_1d, .after = seqic_1d) |>
      dplyr::relocate(upper_ci_1d, .after = lower_ci_1d)
  }

  # Indicator 1f – Proportion of activations in 1d/e where arrival time is ----
  # missing.
  seqic_1f <- data |>
    dplyr::filter(
      {{ trauma_team_activation_level }} %in% c("Level 1", "Level 2"),
      {{ trauma_team_physician_service_type }} %in% provider_group_1de,
      {{ level }} %in% {{ included_levels }}
    ) |>
    dplyr::distinct(
      {{ unique_incident_id }},
      {{ trauma_team_activation_provider }},
      .keep_all = TRUE
    ) |>
    dplyr::summarize(
      numerator_1f = sum(is.na({{ response_time }})),
      denominator_1f = dplyr::n(),
      seqic_1f = dplyr::if_else(
        denominator_1f > 0,
        numerator_1f / denominator_1f,
        NA_real_
      ),
      .by = {{ groups }}
    )

  # optionally calculate the confidence intervals for 1f ----
  if (!is.null(calculate_ci)) {
    seqic_1f <- seqic_1f |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_1f,
          x = numerator_1f,
          n = denominator_1f,
          method = calculate_ci,
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_1f = lower_ci, upper_ci_1f = upper_ci)
      )
  }

  # Combine 1d, 1e, and 1f results; assign label for state-level reporting. ----
  if (is.null(groups)) {
    seqic_1def <- dplyr::bind_cols(seqic_1de, seqic_1f) |>
      tibble::add_column(
        data = "population/sample",
        .before = "numerator_1d"
      )
  } else {
    seqic_1def <- seqic_1de |>
      dplyr::full_join(seqic_1f, by = dplyr::join_by(!!!rlang::syms(groups)))
  }

  # Final combination of all indicators into single summary. ----
  if (is.null(groups)) {
    seqic_1 <- dplyr::bind_cols(seqic_1abc, seqic_1def[, -1])
  } else {
    seqic_1 <- seqic_1abc |>
      dplyr::full_join(
        seqic_1def,
        by = dplyr::join_by(!!!rlang::syms(groups))
      ) |>
      dplyr::arrange(!!!rlang::syms(groups))
  }
  return(seqic_1)
}

# tests/testthat/test-seqic_indicator_1.R

testthat::test_that("seqic_indicator_1() correctly expects columns to be in the 'data'", {
  # Create minimal valid input
  valid_data <- tibble::tibble(
    activation_level = factor("Level 1"),
    provider_type = factor("Surgery/Trauma"),
    trauma_level = factor("I"),
    response_minutes = 10,
    provider = "Dr. A",
    incident_id = 1
  )

  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_leve,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = TRUE,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_leve,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = "thing",
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_seconds,
      trauma_team_activation_provider = provider
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_leve,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = "fake"
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_1() validates input types correctly", {
  # Create minimal valid input
  valid_data <- tibble::tibble(
    activation_level = factor("Level 1"),
    provider_type = factor("Surgery/Trauma"),
    trauma_level = factor("I"),
    response_minutes = 10,
    provider = "Dr. A",
    incident_id = 1
  )

  # Valid call should not error
  testthat::expect_silent(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    )
  )

  # data is not a data.frame
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = as.matrix(valid_data),
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "data.*must be of class.*data\\.frame, tbl, tbl_df"
  )

  # trauma_team_activation_level is not character or factor
  bad_data <- dplyr::mutate(valid_data, activation_level = 1)
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = bad_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "trauma_team_activation_level.*must be of class"
  )

  # trauma_team_physician_service_type is invalid
  bad_data <- dplyr::mutate(valid_data, provider_type = 42)
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = bad_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "trauma_team_physician_service_type.*must be of class"
  )

  # unique_incident_id invalid
  bad_data <- dplyr::mutate(valid_data, incident_id = list("A"))
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = bad_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "unique_incident_id.*must be of class"
  )

  # level is not character or factor
  bad_data <- dplyr::mutate(valid_data, trauma_level = 99)
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = bad_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "level.*must be of class"
  )

  # response_time is not numeric
  bad_data <- dplyr::mutate(valid_data, response_minutes = "soon")
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = bad_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "response_time.*must be.*numeric"
  )

  # trauma_team_activation_provider is not character or factor
  bad_data <- dplyr::mutate(valid_data, provider = 999)
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = bad_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider
    ),
    "trauma_team_activation_provider.*must be of class"
  )
})

testthat::test_that("seqic_indicator_1() validates groups and calculate_ci", {
  valid_data <- tibble::tibble(
    activation_level = "Level 1",
    provider_type = "Surgery/Trauma",
    trauma_level = "I",
    response_minutes = 10,
    provider = "Dr. A",
    incident_id = 1,
    group_var = "Test"
  )

  # groups contains a non-string
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider,
      groups = list(123)
    ),
    "groups.*must be of class.*character.*or.*factor"
  )

  # groups contains a string not found in data
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider,
      groups = c("not_a_col")
    ),
    "groups.*contains invalid column names.*not_a_col.*Valid column names are.*"
  )

  # calculate_ci is invalid
  testthat::expect_error(
    traumar::seqic_indicator_1(
      data = valid_data,
      trauma_team_activation_level = activation_level,
      trauma_team_physician_service_type = provider_type,
      level = trauma_level,
      unique_incident_id = incident_id,
      response_time = response_minutes,
      trauma_team_activation_provider = provider,
      calculate_ci = "wrong"
    ),
    "calculate_ci.*contains invalid values.*wrong.*Valid values are.*wilson, clopper-pearson"
  )
})

testthat::test_that("seqic_indicator_1 calculates indicators 1a–1c correctly", {
  # Sample test data
  test_data <- tibble::tibble(
    incident_id = c(1, 2, 3, 4, 5, 6),
    activation_level = c(
      "Level 1",
      "Level 1",
      "Level 1",
      "Level 1",
      "Level 1",
      "Level 1"
    ),
    provider_type = c(
      "Surgery/Trauma",
      "Surgery/Trauma",
      "Surgery/Trauma",
      "Surgery/Trauma",
      "Surgery/Trauma",
      "Surgery/Trauma"
    ),
    trauma_level = c("I", "II", "III", "III", "II", "I"),
    response_minutes = c(10, 20, 25, NA, 40, NA),
    provider = paste("Dr", LETTERS[1:6])
  )

  # Run without CI
  result <- traumar::seqic_indicator_1(
    data = test_data,
    trauma_team_activation_level = activation_level,
    trauma_team_physician_service_type = provider_type,
    level = trauma_level,
    unique_incident_id = incident_id,
    response_time = response_minutes,
    trauma_team_activation_provider = provider,
    calculate_ci = NULL
  )

  # Expectations for 1a (I and II only, within 15 minutes)
  testthat::expect_equal(result$numerator_1a, 1)
  testthat::expect_equal(result$denominator_1a, 3)
  testthat::expect_equal(result$seqic_1a, 1 / 3)

  # Expectations for 1b (I, II, III, within 30 minutes)
  testthat::expect_equal(result$numerator_1b, 3)
  testthat::expect_equal(result$denominator_1b, 4)
  testthat::expect_equal(result$seqic_1b, 0.75)

  # Expectations for 1c (missing response time among eligible)
  testthat::expect_equal(result$numerator_1c, 2)
  testthat::expect_equal(result$denominator_1c, 6)
  testthat::expect_equal(result$seqic_1c, 1 / 3)
})

testthat::test_that("seqic_indicator_1 returns confidence intervals when requested", {
  test_data <- tibble::tibble(
    incident_id = c(1, 2, 3),
    activation_level = rep("Level 1", 3),
    provider_type = rep("Surgery/Trauma", 3),
    trauma_level = c("I", "II", "III"),
    response_minutes = c(10, 30, NA),
    provider = c("Dr. X", "Dr. Y", "Dr. Z")
  )

  result <- traumar::seqic_indicator_1(
    data = test_data,
    trauma_team_activation_level = activation_level,
    trauma_team_physician_service_type = provider_type,
    level = trauma_level,
    unique_incident_id = incident_id,
    response_time = response_minutes,
    trauma_team_activation_provider = provider,
    calculate_ci = "wilson"
  )

  testthat::expect_true("lower_ci_1a" %in% names(result))
  testthat::expect_true("upper_ci_1a" %in% names(result))
  testthat::expect_true("lower_ci_1b" %in% names(result))
  testthat::expect_true("upper_ci_1b" %in% names(result))
  testthat::expect_true("lower_ci_1c" %in% names(result))
  testthat::expect_true("upper_ci_1c" %in% names(result))
})

#' @title Classify Dates as Weekday or Weekend
#'
#' @description
#'
#' This function classifies each date in a vector of dates as either "Weekday"
#' or "Weekend". The function returns "Weekday" for Monday to Friday and
#' "Weekend" for Saturday and Sunday.
#'
#' @param input_date A vector of `Date` or `POSIXct` objects to classify.
#'
#' @return A character vector with the classification for each date: either
#'   "Weekday" or "Weekend".
#'
#' @details The function checks if the `input_date` is a valid `Date` or
#'   `POSIXct` object. It returns "Weekday" for dates that fall on Monday
#'   through Friday and "Weekend" for dates that fall on Saturday or Sunday. If
#'   the input is not of the correct class, the function will throw an error.
#'
#' @examples
#' # Example 1: Date of a weekend
#' weekend(as.Date("2025-01-18"))
#'
#' # Example 2: Date of a weekday
#' weekend(as.Date("2025-01-15"))
#'
#' # Example 3: Date of an invalid object
#' try(
#' weekend("2025-01-18") # This will throw an error
#' )
#'
#' @export
#'
#' @author Nicolas Foss, Ed.D., MS
#'
weekend <- function(input_date) {
  #check if the value supplied is in fact Date or POSIXct
  if (!lubridate::is.Date(input_date) & !lubridate::is.POSIXct(input_date)) {
    cli::cli_abort(
      paste0(
        "The input to {.var input_date} must be an object of class {.cls Date} or {.cls POSIXct}, but you supplied an object of class {.cls {class(input_date)}}.",
        "i" = "Supply a {.cls Date} object to {.fn weekend}."
      )
    )
  }

  # Get the day of the week
  day_of_week <- base::weekdays(input_date, abbreviate = F)

  # Classify as Weekday or Weekend
  classification <- ifelse(
    day_of_week %in% c("Saturday", "Sunday"),
    "Weekend",
    base::ifelse(
      day_of_week %in%
        c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday"),
      "Weekday",
      "Undetermined"
    )
  )

  return(classification)
}

testthat::test_that("weekend classifies correctly for valid dates", {

  # Test for a weekend date (Saturday)
  testthat::expect_equal(weekend(as.Date("2025-01-18")), "Weekend")

  # Test for a weekend date (Sunday)
  testthat::expect_equal(weekend(as.Date("2025-01-19")), "Weekend")

  # Test for a weekday date (Monday)
  testthat::expect_equal(weekend(as.Date("2025-01-15")), "Weekday")

  # Test for a weekday date (Friday)
  testthat::expect_equal(weekend(as.Date("2025-01-17")), "Weekday")
})

testthat::test_that("weekend handles POSIXct correctly", {

  # Test for a weekend date (Saturday) as POSIXct
  testthat::expect_equal(weekend(as.POSIXct("2025-01-18 00:00:00")), "Weekend")

  # Test for a weekday date (Monday) as POSIXct
  testthat::expect_equal(weekend(as.POSIXct("2025-01-15 00:00:00")), "Weekday")
})

testthat::test_that("weekend throws error for invalid input", {

  # Test for invalid input: character string
  testthat::expect_error(weekend("2025-01-18"), "must be an object of class")

  # Test for invalid input: numeric value
  testthat::expect_error(weekend(20250118),
               "must be an object of class")

  # Test for invalid input: NULL
  testthat::expect_error(weekend(NULL),
               "must be an object of class")
})

testthat::test_that("weekend works with vectorized input", {

  # Test for a mix of weekend and weekday dates
  input_dates <- as.Date(c("2025-01-18", "2025-01-15", "2025-01-19", "2025-01-17"))
  expected_output <- c("Weekend", "Weekday", "Weekend", "Weekday")

  testthat::expect_equal(weekend(input_dates), expected_output)
})

testthat::test_that("weekend returns correct classification for all weekdays", {

  weekdays_dates <- as.Date(c("2025-01-13", "2025-01-14", "2025-01-15", "2025-01-16", "2025-01-17"))
  weekdays_classification <- rep("Weekday", 5)

  testthat::expect_equal(weekend(weekdays_dates), weekdays_classification)
})

testthat::test_that("weekend returns correct classification for all weekend days", {

  weekend_dates <- as.Date(c("2025-01-18", "2025-01-19"))
  weekend_classification <- rep("Weekend", 2)

  testthat::expect_equal(weekend(weekend_dates), weekend_classification)
})

#' @title SEQIC Indicator 2 – Missing Incident Time
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' This function calculates System Evaluation and Quality Improvement Committee
#' (SEQIC) Indicator 2. This indicator evaluates the proportion of trauma
#' incidents with missing documented incident time across Level I–IV trauma
#' centers.
#'
#' @inheritParams seqic_indicator_1
#' @param incident_time The time the patient's injury occurred.
#' @inheritDotParams nemsqar::nemsqa_binomial_confint conf.level correct
#'
#' @details This function:
#' \itemize{
#'   \item Filters trauma records to those with a trauma center level of I–IV.
#'   \item Deduplicates by `unique_incident_id` to ensure one record per
#'   incident.
#'   \item Calculates the proportion of cases missing `incident_time`.
#' }
#'
#' @note
#'
#' Users must ensure appropriate column names are passed and data is
#' pre-processed to include the necessary fields without missing critical
#' identifiers or timestamps.
#'
#' @return A tibble summarizing SEQIC Indicator 2 results. Includes numerator,
#'   denominator, and performance rate for the indicator. 95% confidence
#'   intervals are provided optionally.
#'
#' @examples
#'
#' # Packages
#' library(dplyr)
#' library(traumar)
#'
#' # Data
#' data <- tibble::tibble(
#'   incident_id = as.character(101:106),
#'   trauma_level = c("I", "II", "III", "IV", "II", "I"),
#'   incident_time = as.POSIXct(c("2023-01-01 12:00", NA, "2023-01-02 14:15",
#'                                NA, "2023-01-03 09:30", "2023-01-04 16:45"))
#' )
#'
#' # Run the function
#' traumar::seqic_indicator_2(
#'   data = data,
#'   unique_incident_id = incident_id,
#'   level = trauma_level,
#'   incident_time = incident_time,
#'   calculate_ci = "clopper-pearson"
#' )
#'
#' @author Nicolas Foss, Ed.D., MS
#'
#' @export
#'
seqic_indicator_2 <- function(
  data,
  unique_incident_id,
  level,
  included_levels = c("I", "II", "III", "IV"),
  incident_time,
  groups = NULL,
  calculate_ci = NULL,
  ...
) {
  ###___________________________________________________________________________
  ### Data validation ----
  ###___________________________________________________________________________

  # validate `data` ----
  validate_data_structure(
    input = data,
    structure_type = c("data.frame", "tbl", "tbl_df"),
    logic = "or",
    type = "error"
  )

  # make the `unique_incident_id` column accessible for validation ----
  unique_incident_id_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ unique_incident_id }},
    var_name = "unique_incident_id"
  )

  # validate `unique_incident_id` ----
  validate_class(
    input = unique_incident_id_check,
    class_type = c("numeric", "factor", "character"),
    logic = "or",
    type = "error"
  )

  # make the `incident_time` column accessible for validation ----
  incident_time_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ incident_time }},
    var_name = "incident_time"
  )

  # validate `incident_time` ----
  validate_class(
    input = incident_time_check,
    class_type = c("date", "date-time", "hms", "character"),
    logic = "or",
    type = "error"
  )

  # make the `level` column accessible for validation ----
  level_check <- validate_data_pull(
    input = data,
    type = "error",
    col = {{ level }},
    var_name = "level"
  )

  # validate `level` ----
  validate_character_factor(
    input = level_check,
    type = "error",
    var_name = "level"
  )

  # Check if all elements in groups are strings (i.e., character vectors) ----
  validate_character_factor(input = groups, type = "error", null_ok = TRUE)

  # Check if all groups exist in the `data` ----
  validate_names(
    input = data,
    check_names = groups,
    type = "error",
    var_name = "groups",
    null_ok = TRUE
  )

  # Validate the `calculate_ci` argument ----
  # - If not NULL, must be either "wilson" or "clopper-pearson"
  # - Use match.arg() to enforce allowed values
  # - Catch invalid input silently and report cleanly with cli
  calculate_ci <- validate_choice(
    input = calculate_ci,
    choices = c("wilson", "clopper-pearson"),
    several.ok = FALSE,
    type = "error",
    null_ok = TRUE
  )

  # Validate the `included_levels` argument ----
  validate_class(
    input = included_levels,
    class_type = c("numeric", "character", "factor"),
    logic = "or",
    type = "error"
  )

  ###___________________________________________________________________________
  ### Calculations ----
  ###___________________________________________________________________________

  # Summarize the data for Indicator 2: ----
  # - Filter records to include only Level I–IV trauma centers.
  # - Remove duplicate incidents, keeping the first occurrence of each unique
  #   `unique_incident_id`.
  # - Summarize the data by calculating:
  #   - `numerator_2`: The number of incidents with missing `incident_time`.
  #   - `denominator_2`: The total number of unique incidents.
  #   - `seqic_2`: The proportion of incidents with missing `incident_time`
  #     (rounded to 3 decimal places).
  #   - Optionally, group results by columns specified in the `groups` argument.
  seqic_2 <- data |>
    dplyr::filter({{ level }} %in% included_levels) |>
    dplyr::distinct({{ unique_incident_id }}, .keep_all = TRUE) |>
    dplyr::summarize(
      numerator_2 = sum(is.na({{ incident_time }})), # Calculate the number of missing incident_time values.
      denominator_2 = dplyr::n(), # Count the total number of unique incidents.
      seqic_2 = dplyr::if_else(
        denominator_2 > 0,
        numerator_2 / denominator_2,
        NA_real_
      ), # Calculate the proportion for Indicator 2.
      .by = {{ groups }} # Group by the specified columns (if any).
    )

  # Optionally calculate confidence intervals for the proportions: ----
  # - If `calculate_ci` is provided, apply a binomial confidence interval method
  #   (Wilson or Clopper-Pearson).
  # - `nemsqa_binomial_confint()` calculates the confidence intervals for the
  #   proportion.
  # - Select only the relevant columns and rename the CI columns for clarity.
  if (!is.null(calculate_ci)) {
    seqic_2 <- seqic_2 |>
      dplyr::bind_cols(
        nemsqar::nemsqa_binomial_confint(
          data = seqic_2,
          x = numerator_2, # Number of successes (numerator).
          n = denominator_2, # Number of trials (denominator).
          method = calculate_ci, # Confidence interval calculation method (e.g., "wilson").
          ...
        ) |>
          dplyr::select(lower_ci, upper_ci) |>
          dplyr::rename(lower_ci_2 = lower_ci, upper_ci_2 = upper_ci) # Rename CI columns.
      )
  }

  # Assign a label to indicate whether the data represents population or ----
  # sample-level results:
  # - If no grouping is applied, label the data as "Population/Sample".
  if (is.null(groups)) {
    seqic_2 <- seqic_2 |>
      tibble::add_column(data = "population/sample", .before = "numerator_2") # Add the label column.
  } else if (!is.null(groups)) {
    seqic_2 <- seqic_2 |>
      dplyr::arrange(!!!rlang::syms(groups))
  }

  # Return the final summary dataframe for Indicator 2. ----
  return(seqic_2)
}

# tests/testthat/test-seqic_indicator_2.R

testthat::test_that("seqic_indicator_2() correctly expects columns to be in the 'data'", {
  # Minimal valid data
  data <- tibble::tibble(
    incident_id = c("A", "B", "C", "D", "E"),
    trauma_level = c("I", "II", "III", "V", "II"),
    incident_time = as.POSIXct(c(
      NA,
      "2023-01-02 12:00",
      NA,
      "2023-01-03 14:00",
      "2023-01-04 15:00"
    ))
  )

  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = TRUE,
      level = trauma_level,
      incident_time = incident_time
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = "fake",
      incident_time = incident_time
    ),
    regexp = "It was not possible to validate"
  )

  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = "false"
    ),
    regexp = "It was not possible to validate"
  )
})

testthat::test_that("seqic_indicator_2() validates `data` input type", {
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = "not_a_data",
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time
    ),
    "data.*must be of class.*data.frame, tbl, tbl_df"
  )
})

testthat::test_that("seqic_indicator_2() validates `unique_incident_id` column type", {
  data <- tibble::tibble(
    incident_id = logical(length = 1),
    trauma_level = "I",
    incident_time = Sys.time() # invalid type
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time
    ),
    "unique_incident_id.*must be of class"
  )
})

testthat::test_that("seqic_indicator_2() validates `incident_time` column type", {
  data <- tibble::tibble(
    incident_id = 1,
    trauma_level = "I",
    incident_time = 12345 # invalid type
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time
    ),
    "incident_time.*must be of class"
  )
})

testthat::test_that("seqic_indicator_2() validates `level` column type", {
  data <- tibble::tibble(
    incident_id = 1,
    trauma_level = 3, # numeric, invalid
    incident_time = Sys.time()
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time
    ),
    "level.*must be of class"
  )
})

testthat::test_that("seqic_indicator_2() validates `groups` is character vector", {
  data <- tibble::tibble(
    incident_id = 1,
    trauma_level = "I",
    incident_time = Sys.time()
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time,
      groups = list(123)
    ),
    "groups.*must be of class.*character.*factor"
  )
})

testthat::test_that("seqic_indicator_2() validates `groups` exist in data", {
  data <- tibble::tibble(
    incident_id = 1,
    trauma_level = "I",
    incident_time = Sys.time()
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time,
      groups = c("nonexistent_column")
    ),
    "groups.*contains invalid column names.*nonexistent_column.*Valid column names are.*incident_id, trauma_level, incident_time"
  )
})

testthat::test_that("seqic_indicator_2() validates `calculate_ci` options", {
  data <- tibble::tibble(
    incident_id = 1,
    trauma_level = "I",
    incident_time = Sys.time()
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time,
      calculate_ci = "invalid_method"
    ),
    "calculate_ci.*contains invalid values.*invalid_method.*Valid values are.*wilson, clopper-pearson"
  )
})

testthat::test_that("seqic_indicator_2() validates `included_levels` class", {
  data <- tibble::tibble(
    incident_id = 1,
    trauma_level = "I",
    incident_time = Sys.time()
  )
  testthat::expect_error(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time,
      included_levels = list("I", "II")
    ),
    "included_levels.*must be of class"
  )
})

testthat::test_that("seqic_indicator_2 basic calculations work", {
  data <- tibble::tibble(
    incident_id = c("A", "B", "C", "D", "E"),
    trauma_level = c("I", "II", "III", "V", "II"),
    incident_time = as.POSIXct(c(
      NA,
      "2023-01-02 12:00",
      NA,
      "2023-01-03 14:00",
      "2023-01-04 15:00"
    ))
  )

  result <- traumar::seqic_indicator_2(
    data = data,
    unique_incident_id = incident_id,
    level = trauma_level,
    incident_time = incident_time
  )

  testthat::expect_true(tibble::is_tibble(result))
  testthat::expect_equal(result$numerator_2, 2)
  testthat::expect_equal(result$denominator_2, 4) # Level V excluded
  testthat::expect_equal(result$seqic_2, 0.5)
})

testthat::test_that("seqic_indicator_2 handles confidence intervals", {
  data <- tibble::tibble(
    incident_id = c("A", "B", "C", "D"),
    trauma_level = c("I", "II", "III", "IV"),
    incident_time = as.POSIXct(c(
      NA,
      "2023-01-01 12:00",
      "2023-01-01 13:00",
      NA
    ))
  )

  result <- traumar::seqic_indicator_2(
    data = data,
    unique_incident_id = incident_id,
    level = trauma_level,
    incident_time = incident_time,
    calculate_ci = "clopper-pearson"
  )

  testthat::expect_true("lower_ci_2" %in% names(result))
  testthat::expect_true("upper_ci_2" %in% names(result))
  testthat::expect_true(result$lower_ci_2 <= result$seqic_2)
  testthat::expect_true(result$upper_ci_2 >= result$seqic_2)
})

testthat::test_that("seqic_indicator_2 respects grouping", {
  data <- tibble::tibble(
    incident_id = c("A", "B", "C", "D", "E", "F"),
    trauma_level = c("I", "I", "II", "II", "III", "III"),
    region = c("East", "East", "West", "West", "East", "East"),
    incident_time = as.POSIXct(c(NA, NA, "2023-01-01", NA, NA, "2023-01-01"))
  )

  result <- traumar::seqic_indicator_2(
    data = data,
    unique_incident_id = incident_id,
    level = trauma_level,
    incident_time = incident_time,
    groups = "region"
  )

  testthat::expect_true("region" %in% names(result))
  testthat::expect_equal(nrow(result), 2)
  testthat::expect_true(all(result$seqic_2 >= 0 & result$seqic_2 <= 1))
})

testthat::test_that("seqic_indicator_2 handles character incident_time (invalid)", {
  data <- tibble::tibble(
    incident_id = c("A", "B"),
    trauma_level = c("I", "II"),
    incident_time = c(NA, "2023-01-01 12:00") # Not POSIXct
  )

  testthat::expect_warning(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time
    ),
    regexp = NA # no warning expected even though type is not POSIXct
  )

  result <- suppressWarnings(
    traumar::seqic_indicator_2(
      data = data,
      unique_incident_id = incident_id,
      level = trauma_level,
      incident_time = incident_time
    )
  )

  testthat::expect_equal(result$numerator_2, 1)
})

testthat::test_that("seqic_indicator_2 returns label if not grouped", {
  data <- tibble::tibble(
    incident_id = c("A", "B"),
    trauma_level = c("I", "II"),
    incident_time = as.POSIXct(c(NA, "2023-01-01 12:00"))
  )

  result <- traumar::seqic_indicator_2(
    data = data,
    unique_incident_id = incident_id,
    level = trauma_level,
    incident_time = incident_time
  )

  testthat::expect_true("data" %in% names(result))
  testthat::expect_equal(result$data, "population/sample")
})

test_that("`%not_in%` works with character vectors", {
  x <- c("apple", "banana", "cherry")
  y <- c("banana", "grape")

  result <- x %not_in% y
  expected <- c(TRUE, FALSE, TRUE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with numeric vectors", {
  x <- c(1, 2, 3, 4, 5)
  y <- c(2, 4, 6)

  result <- x %not_in% y
  expected <- c(TRUE, FALSE, TRUE, FALSE, TRUE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with logical vectors", {
  x <- c(TRUE, FALSE, TRUE)
  y <- c(FALSE)

  result <- x %not_in% y
  expected <- c(TRUE, FALSE, TRUE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with mixed types", {
  x <- c(1, "banana", TRUE)
  y <- c("banana", 2, FALSE)

  result <- x %not_in% y
  expected <- c(TRUE, FALSE, TRUE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with empty vectors", {
  x <- character(0)
  y <- c("apple", "banana")

  result <- x %not_in% y
  expected <- logical(0)

  expect_equal(result, expected)

  x <- c("apple", "banana")
  y <- character(0)

  result <- x %not_in% y
  expected <- c(TRUE, TRUE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with NA values in `x` and `y`", {
  x <- c(NA, "apple", NA)
  y <- c("banana", NA)

  result <- x %not_in% y
  expected <- c(FALSE, TRUE, FALSE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with identical `x` and `y`", {
  x <- c("apple", "banana", "cherry")
  y <- c("apple", "banana", "cherry")

  result <- x %not_in% y
  expected <- c(FALSE, FALSE, FALSE)

  expect_equal(result, expected)
})

test_that("`%not_in%` handles duplicates in `x` and `y`", {
  x <- c("apple", "banana", "banana", "cherry")
  y <- c("banana", "banana", "grape")

  result <- x %not_in% y
  expected <- c(TRUE, FALSE, FALSE, TRUE)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with NULL values in `x` or `y`", {
  x <- c("apple", "banana", "cherry")
  y <- NULL

  result <- x %not_in% y
  expected <- c(TRUE, TRUE, TRUE)

  expect_equal(result, expected)

  x <- NULL
  y <- c("apple", "banana")

  result <- x %not_in% y
  expected <- logical(0)

  expect_equal(result, expected)
})

test_that("`%not_in%` works with numeric edge cases", {
  x <- c(-1, 0, 1, Inf, -Inf, NA)
  y <- c(0, NA, Inf)

  result <- x %not_in% y
  expected <- c(TRUE, FALSE, TRUE, FALSE, TRUE, FALSE)

  expect_equal(result, expected)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_2.R
\name{seqic_indicator_2}
\alias{seqic_indicator_2}
\title{SEQIC Indicator 2 – Missing Incident Time}
\usage{
seqic_indicator_2(
  data,
  unique_incident_id,
  level,
  included_levels = c("I", "II", "III", "IV"),
  incident_time,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{incident_time}{The time the patient's injury occurred.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 2 results. Includes numerator,
denominator, and performance rate for the indicator. 95\% confidence
intervals are provided optionally.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This function calculates System Evaluation and Quality Improvement Committee
(SEQIC) Indicator 2. This indicator evaluates the proportion of trauma
incidents with missing documented incident time across Level I–IV trauma
centers.
}
\details{
This function:
\itemize{
\item Filters trauma records to those with a trauma center level of I–IV.
\item Deduplicates by \code{unique_incident_id} to ensure one record per
incident.
\item Calculates the proportion of cases missing \code{incident_time}.
}
}
\note{
Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{

# Packages
library(dplyr)
library(traumar)

# Data
data <- tibble::tibble(
  incident_id = as.character(101:106),
  trauma_level = c("I", "II", "III", "IV", "II", "I"),
  incident_time = as.POSIXct(c("2023-01-01 12:00", NA, "2023-01-02 14:15",
                               NA, "2023-01-03 09:30", "2023-01-04 16:45"))
)

# Run the function
traumar::seqic_indicator_2(
  data = data,
  unique_incident_id = incident_id,
  level = trauma_level,
  incident_time = incident_time,
  calculate_ci = "clopper-pearson"
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traumar-package.R
\docType{package}
\name{traumar-package}
\alias{traumar}
\alias{traumar-package}
\title{traumar: Calculate Metrics for Trauma System Performance}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Hospitals, hospital systems, and even trauma systems that provide care to injured patients may not be aware of robust metrics that can help gauge the efficacy of their programs in saving the lives of injured patients. 'traumar' provides robust functions driven by the academic literature to automate the calculation of relevant metrics to individuals desiring to measure the performance of their trauma center or even a trauma system. 'traumar' also provides some helper functions for the data analysis journey. Users can refer to the following publications for descriptions of the methods used in 'traumar'. TRISS methodology, including probability of survival, and the W, M, and Z Scores - Flora (1978) \doi{10.1097/00005373-197810000-00003}, Boyd et al. (1987, PMID:3106646), Llullaku et al. (2009) \doi{10.1186/1749-7922-4-2}, Singh et al. (2011) \doi{10.4103/0974-2700.86626}, Baker et al. (1974, PMID:4814394), and Champion et al. (1989) \doi{10.1097/00005373-198905000-00017}. For the Relative Mortality Metric, see Napoli et al. (2017) \doi{10.1080/24725579.2017.1325948}, Schroeder et al. (2019) \doi{10.1080/10903127.2018.1489021}, and Kassar et al. (2016) \doi{10.1177/00031348221093563}. For more information about methods to calculate over- and under-triage in trauma hospital populations and samples, please see the following publications - Peng & Xiang (2016) \doi{10.1016/j.ajem.2016.08.061}, Beam et al. (2022) \doi{10.23937/2474-3674/1510136}, Roden-Foreman et al. (2017) \doi{10.1097/JTN.0000000000000283}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://bemts-hhs.github.io/traumar/}
  \item \url{https://github.com/bemts-hhs/traumar}
  \item Report bugs at \url{https://github.com/bemts-hhs/traumar/issues}
}

}
\author{
\strong{Maintainer}: Nicolas Foss \email{nicolas.foss@hhs.iowa.gov}

Other contributors:
\itemize{
  \item Iowa Department of Health and Human Services [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_5.R
\name{seqic_indicator_5}
\alias{seqic_indicator_5}
\title{SEQIC Indicator 5 - Alcohol and Drug Screening}
\usage{
seqic_indicator_5(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  blood_alcohol_content,
  drug_screen,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{blood_alcohol_content}{Unquoted column name for blood alcohol
concentration. Numeric. A non-missing value indicates a test was performed.
Values greater than zero are considered positive results.}

\item{drug_screen}{Unquoted column name for the drug screen result. Character
or factor. May contain keywords (e.g., "opioid", "cocaine", "none"). The
keywords used in this function correspond to the National Trauma Data Bank
(NTDB) field values for the corresponding data element.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 5a–5d results. Includes
numerator, denominator, and calculated proportion for each measure.
Optionally includes 95\% confidence intervals.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Computes SEQIC Indicator 5a–5d for trauma system quality monitoring. These
indicators measure alcohol and drug screening rates among trauma patients at
trauma level I–IV facilities.
}
\details{
This function:
\itemize{
\item Filters to trauma records at trauma levels I–IV.
\item Deduplicates by \code{unique_incident_id} to ensure one record per
incident.
\item Calculates four sub-measures:
\itemize{
\item {Indicator 5a:} Proportion of patients with a blood
alcohol test performed.
\item {Indicator 5b:} Among those tested, the proportion with
BAC > 0.
\item {Indicator 5c:} Proportion of patients with any recorded
drug screen result.
\item {Indicator 5d:} Among those with a drug result, the
proportion that included a known positive drug (e.g., opioids,
cocaine, THC).
}
\item Matches drug-related terms using regular expressions for a broad set
of known substances. Matching is case-insensitive.
}
}
\note{
Users must ensure input columns are correctly named and contain standardized
values where applicable. Drug screen values should ideally use consistent
naming or be mapped to recognizable substance terms prior to function use.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Create synthetic test data for Indicators 5a–5d
test_data <- tibble::tibble(
  id = as.character(1:10),
  trauma_level = rep(c("I", "II", "III", "IV", "V"), each = 2),
  bac = c(0.08, NA, 0, 0.02, NA, 0.15, NA, NA, 0, 0),
  drug = c(
    "opioid", "none", "cocaine", "none", NA,
    "benzodiazepine", "alcohol", "thc", "none", NA
  )
)

# Run the indicator function
traumar::seqic_indicator_5(
  data = test_data,
  level = trauma_level,
  unique_incident_id = id,
  blood_alcohol_content = bac,
  drug_screen = drug
) |>
  tidyr::pivot_longer(cols = -1, names_to = "Indicator", values_to =
  "Values")

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_3.R
\name{seqic_indicator_3}
\alias{seqic_indicator_3}
\title{SEQIC Indicator 3 - Presence of Probability of Survival Calculations}
\usage{
seqic_indicator_3(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  trauma_type,
  unique_incident_id,
  probability_of_survival,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{trauma_type}{A column name indicating the type of trauma. The function
filters out "Burn" cases.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{probability_of_survival}{A column name for the probability of survival
for each incident.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 3 results. Includes numerator,
denominator, and performance rate for the indicator. 95\% confidence
intervals are provided optionally.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This function calculates Indicator 3, a measure of the proportion of trauma
incidents where the probability of survival is recorded. It filters the data
by trauma center level (I-IV), excluding burn cases, and computes the
proportion of incidents with a valid probability of survival value.
}
\details{
This function:
\itemize{
\item Filters trauma records to those with a trauma center level of I–IV.
\item Excludes records with a trauma type of "Burn".
\item Deduplicates by \code{unique_incident_id} to ensure one record per
incident.
\item Calculates the proportion of records with a non-missing
\code{probability_of_survival}.
}
}
\note{
Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Create a synthetic test dataset
test_data <- tibble::tibble(
  unique_id = as.character(1:10),
  trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV", "I",
  "II"),
  trauma_category = c("Blunt", "Penetrating", "Burn", "Blunt", "Penetrating",
                      "Burn", "Blunt", "Penetrating", "Blunt", "Blunt"),
  survival_prob = c(0.95, 0.89, NA, 0.76, NA, 0.92, 0.88, NA, 0.97, 0.91)
)

# Run the indicator function
traumar::seqic_indicator_3(
  data = test_data,
  level = trauma_level,
  trauma_type = trauma_category,
  unique_incident_id = unique_id,
  probability_of_survival = survival_prob,
  groups = "trauma_level"
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_it_normal.R
\name{is_it_normal}
\alias{is_it_normal}
\title{Exploratory Data Analysis, Normality Testing, and
Visualization}
\usage{
is_it_normal(
  df,
  ...,
  group_vars = NULL,
  seed = 10232015,
  normality_test = NULL,
  include_plots = FALSE,
  plot_theme = traumar::theme_cleaner
)
}
\arguments{
\item{df}{A \code{data.frame} or \code{tibble} containing the variables to assess.}

\item{...}{One or more unquoted column names from \code{df} to be analyzed.}

\item{group_vars}{Optional. A character vector of column names in \code{df} to
group results by (e.g., \code{c("year", "hospital_level")}). If \code{NULL}, no
grouping is applied. Grouped summaries and normality tests are computed
within each unique combination of values across these variables.}

\item{seed}{A numeric value passed to \code{set.seed()} to ensure reproducibility.
Default is \code{10232015}.}

\item{normality_test}{A character string specifying the statistical test to
use. Must be one of: \verb{"shapiro-wilk" or "shapiro" or "sw"},
\verb{"kolmogorov-smirnov" or "ks"}, \verb{"anderson-darling" or "ad"}, \verb{"lilliefors" or "lilli"}, \verb{"cramer-von-mises" or "cvm"}, \verb{"pearson" or "p"}, or
\verb{"shapiro-francia" or "sf"}. If \code{NULL}, no normality test is performed,
which is the default.}

\item{include_plots}{Logical. If \code{TRUE}, plots are generated for a single
variable. Plotting is disabled if multiple variables are passed.}

\item{plot_theme}{A \code{ggplot2::theme} function to apply to all plots. Default
is \code{traumar::theme_cleaner}.}
}
\value{
A named list with the following elements:
\describe{
\item{descriptive_statistics}{A \code{tibble} of summary statistics for each
variable.}
\item{normality_test}{A \code{tibble} of test statistics and p-values
(if \code{normality_test == TRUE}).}
\item{plots}{A patchwork object containing four plots (if \code{include_plots = TRUE} and one variable supplied).}
}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{is_it_normal()} calculates descriptive statistics and conducts univariate
normality testing on one or more numeric variables in a dataset using a
selected statistical test. Optional plots are included for one variable at a
time, only. Results are returned as a named list containing summaries and,
optionally, normality tests and/or diagnostic plots.
}
\details{
\itemize{
\item If the data do not meet the test requirements for a chosen test of
normality, \code{is_it_normal()} will not run the tests.
\item Normality tests may yield differing results. Each test has distinct
assumptions and sensitivity. Users should verify assumptions and consult
test-specific guidance to ensure appropriate use.
\item The function will abort with helpful CLI messages if input types or
structures are incorrect.
\item If plotting is enabled, and \code{nrow(df) > 10000}, a warning is issued
as plotting may become computationally expensive.
}
}
\note{
Supported normality tests are below. Please check the specifications of
these tests in the corresponding documentation.
\itemize{
\item Shapiro-Wilk (\code{stats::shapiro.test()})
\item Kolmogorov-Smirnov (\code{stats::ks.test()})
\item Anderson-Darling (\code{nortest::ad.test()})
\item Lilliefors (\code{nortest::lillie.test()})
\item Cramer-von Mises (\code{nortest::cvm.test()})
\item Pearson (\code{norest::pearson.test()})
\item Shapiro-Francia (\code{nortest::sf.test()})
}

Please note that if grouped plotting is enabled, each group will generate its
own set of plots. This may flood your IDE or console. Plan your use of this
functionality with care to avoid lags or unwanted outputs.
}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/non_linear_bins.R
\name{nonlinear_bins}
\alias{nonlinear_bins}
\title{Create Nonlinear Probability of Survival Bins}
\usage{
nonlinear_bins(
  data,
  Ps_col,
  outcome_col,
  group_vars = NULL,
  divisor1 = 5,
  divisor2 = 5,
  threshold_1 = 0.9,
  threshold_2 = 0.99
)
}
\arguments{
\item{data}{A \code{data.frame} or \code{tibble} containing the probability of survival
data for a set of patients.}

\item{Ps_col}{The name of the column containing the survival probabilities
(Ps). Should be numeric on a scale from 0 to 1.}

\item{outcome_col}{The name of the column containing the outcome data. It
should be binary, with values indicating patient survival. A value of \code{1}
should represent "alive" (survived), while \code{0} should represent "dead" (did
not survive).  \code{TRUE/FALSE} are accepted as well. Ensure the column
contains only these possible values.}

\item{group_vars}{Optional grouping variables for bin statistics
calculations. These should be specified as quoted column names.}

\item{divisor1}{A positive numeric value controlling the coarseness of bins
for Ps values below \code{threshold_1}. It scales the number of steps from the
start of the dataset up to the \code{threshold_1} cut point. Larger values
produce fewer, broader bins; smaller values produce more, narrower bins.
Defaults to \code{5}.}

\item{divisor2}{A positive numeric value controlling the coarseness of bins
for Ps values between \code{threshold_1} and \code{threshold_2}. Larger values yield
wider bins, and smaller values yield narrower bins in this range. Defaults
to \code{5}.}

\item{threshold_1}{A numeric value that defines the lower bound of the
high-survival probability range in \code{Ps_col}. The function identifies the
first index where \code{Ps_col} exceeds this value and begins applying smaller
bin widths from that point onward. Defaults to \code{0.9}, meaning binning
changes once Ps > 0.90.}

\item{threshold_2}{A numeric value that defines the upper bound of the
high-survival probability range in \code{Ps_col}. The function identifies the
first index where \code{Ps_col} exceeds this value. Between \code{threshold_1} and
\code{threshold_2}, finer binning is applied; above \code{threshold_2}, binning may
again change. Defaults to \code{0.99}, meaning the special binning range is
between Ps values of 0.90 and 0.99.}
}
\value{
A list with two elements:
\itemize{
\item \code{intervals}: A vector defining bin boundaries for probability of
survival.
\item \code{bin_stats}: A \code{tibble} containing:
\itemize{
\item \code{bin_number}: Bin index.
\item \code{bin_start}, \code{bin_end}: Bin range.
\item \code{mean}, \code{sd}: Mean and standard deviation of \code{Ps_col} within the bin.
\item \code{Pred_Survivors_b}, \code{Pred_Deaths_b}: Predicted counts of survivors and
decedents, respectively.
\item \code{AntiS_b}, \code{AntiM_b}: Anticipated proportion survived, and deceased,
respectively.
\item \code{alive}, \code{dead}: Count of observed survivors and non-survivors.
\item \code{count}: Total records in the bin.
\item \code{percent}: Percentage of total records within each bin.
}
}
}
\description{
This function generates nonlinear bins for probability of survival data based
on specified thresholds and divisors as specified in Napoli et al. (2017),
Schroeder et al. (2019), and Kassar et al. (2016). This function calculates
bin statistics, including mean, standard deviation, total alive, total dead,
count, and percentage for each bin.
}
\details{
Like other statistical computing functions, \code{nonlinear_bins()} is happiest
without missing data.  It is best to pass complete probability of survival
and outcome data to the function for optimal performance. With smaller
datasets, this is especially helpful.  However, \code{nonlinear_bins()} will throw
a warning about missing values, if any exist in \code{Ps_col} and/or
\code{outcome_col}.

\code{nonlinear_bins()} assumes \code{Ps_col} contains probabilities derived from
real-world inputs for the Trauma Injury Severity Score (TRISS) model.
Synthetic or low-variability data (especially with small sample sizes) may
not reflect the distribution of TRISS-derived survival probabilities. This
can result in unstable estimates or function failure due to insufficient
dispersion. With small sample sizes, it may be important to use smaller
values with the divisor arguments and adjust the thresholds (based on the
distribution of the \code{Ps_col} values) to create bins that better accommodate
the data.

By default, \code{nonlinear_bins()} derives bin cut points from the full dataset’s
distribution. This ensures comparability across groups when \code{group_vars} is
used. To tailor binning to a specific group (e.g., a single hospital), filter
the dataset to that subgroup before calling \code{nonlinear_bins()}. The function
will then compute bins and related statistics using only that subset’s
\code{Ps_col} distribution. When \code{group_vars} is used, and ff a group lacks
observations within one or more bins, \code{rm_bin_summary()} will compute
statistics only for the bins that contain data. Bins with no observations are
excluded from the summary for that group.

The \code{threshold_1} and \code{threshold_2} arguments set probability cut points
that define the start and end of a high-survival range where bin widths are
adjusted for finer resolution. The \code{divisor1} and \code{divisor2} arguments are
scaling factors that determine how many bins are created before and within
this high-survival range, respectively. Lower divisors yield narrower bins,
capturing more detail, while higher divisors yield broader bins, smoothing
the distribution.
}
\note{
This function will produce the most reliable and interpretable results when
using a dataset that has one row per patient, with each column being a
feature.

The \code{mean} and \code{AntiS_b} are approximately equivalent in this context.  They
are kept in the output for clarity.
}
\examples{
# Generate example data
set.seed(123)

# Parameters
# Total number of patients
n_patients <- 5000

# Arbitrary group labels
groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

# Trauma types
trauma_type_values <- sample(
  x = c("Blunt", "Penetrating"),
  size = n_patients,
  replace = TRUE
)

# RTS values
rts_values <- sample(
  x = seq(from = 0, to = 7.8408, by = 0.005),
  size = n_patients,
  replace = TRUE
)

# patient ages
ages <- sample(
  x = seq(from = 0, to = 100, by = 1),
  size = n_patients,
  replace = TRUE
)

# ISS scores
iss_scores <- sample(
  x = seq(from = 0, to = 75, by = 1),
  size = n_patients,
  replace = TRUE
)

# Generate survival probabilities (Ps)
Ps <- traumar::probability_of_survival(
  trauma_type = trauma_type_values,
  age = ages,
  rts = rts_values,
  iss = iss_scores
)

# Simulate survival outcomes based on Ps
survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

# Create data frame
data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Apply the nonlinear_bins function
results <- nonlinear_bins(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  divisor1 = 4,
  divisor2 = 4,
  threshold_1 = 0.9,
  threshold_2 = 0.99
)

# View results
results$intervals
results$bin_stats

# Example with grouping by a categorical variable

# Run the function using a single grouping variable
results_grouped <- nonlinear_bins(
  data,
  Ps_col = Ps,
  outcome_col = survival,
  group_vars = "groups"
)

# View grouped results
results_grouped$bin_stats

}
\references{
Kassar, O.M., Eklund, E.A., Barnhardt, W.F., Napoli, N.J., Barnes, L.E.,
Young, J.S. (2016). Trauma survival margin analysis: A dissection of trauma
center performance through initial lactate. The American Surgeon, 82(7),
649-653. \url{doi:10.1177/000313481608200733}

Napoli, N. J., Barnhardt, W., Kotoriy, M. E., Young, J. S., & Barnes, L. E.
(2017). Relative mortality analysis: A new tool to evaluate clinical
performance in trauma centers. IISE Transactions on Healthcare Systems
Engineering, 7(3), 181–191. \url{doi:10.1080/24725579.2017.1325948}

Schroeder, P. H., Napoli, N. J., Barnhardt, W. F., Barnes, L. E., & Young, J.
S. (2018). Relative mortality analysis of the “golden hour”: A comprehensive
acuity stratification approach to address disagreement in current literature.
Prehospital Emergency Care, 23(2), 254–262.
\url{doi:10.1080/10903127.2018.1489021}
}
\seealso{
\code{\link[=probability_of_survival]{probability_of_survival()}}, \code{\link[=rmm]{rmm()}}, and \code{\link[=rm_bin_summary]{rm_bin_summary()}}
}
\author{
Nicolas Foss, Ed.D, MS, original implementation in MATLAB by Nicholas
J. Napoli, Ph.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_cleaner.R
\name{theme_cleaner}
\alias{theme_cleaner}
\title{Customizable Minimalistic ggplot2 Theme}
\usage{
theme_cleaner(
  base_size = 12,
  base_family = "sans",
  base_color = "#70C8B8",
  base_color_title = "#03617A",
  title_text_size = ceiling(base_size * 1.1),
  subtitle_text_size = ceiling(base_size * 1.05),
  caption_color = "#19405B",
  legend_position = "top",
  vjust_title = 0,
  vjust_subtitle = 0,
  hjust_title = 0,
  hjust_subtitle = 0,
  axis_lines = FALSE,
  facets = FALSE,
  facet_text_size = base_size,
  draw_panel_border = FALSE,
  ...
)
}
\arguments{
\item{base_size}{Numeric. Default font size for plot elements. Defaults to
\code{12}.}

\item{base_family}{Character. Font family used for text in the plot. Defaults
to \code{"Work Sans"}.}

\item{base_color}{Character. Hex color code for primary plot elements (e.g.,
axis text, legend text). Defaults to \code{"#70C8B8"}.}

\item{base_color_title}{Character. Hex color code for plot title and legend
title text. Defaults to \code{"#03617A"}.}

\item{title_text_size}{Numeric. Font size for plot title text. Defaults to
\code{base_size * 1.1}.}

\item{subtitle_text_size}{Numeric. Font size for plot subtitle text. Defaults
to \code{base_size * 1.05}.}

\item{caption_color}{Character. Hex color code for plot caption text.
Defaults to \code{"#19405B"}.}

\item{legend_position}{Character. Legend position on the plot. Accepts "top",
"bottom", "left", or "right". Defaults to \code{"top"}.}

\item{vjust_title}{Numeric. Vertical justification of the plot title.
Defaults to \code{0}.}

\item{vjust_subtitle}{Numeric. Vertical justification of the plot subtitle.
Defaults to \code{0}.}

\item{hjust_title}{Numeric. Horizontal justification of the plot title.
Defaults to \code{0}.}

\item{hjust_subtitle}{Numeric. Horizontal justification of the plot subtitle.
Defaults to \code{0}.}

\item{axis_lines}{Logical. If \code{TRUE}, axis lines are drawn in \code{base_color};
otherwise, they are hidden. Defaults to \code{FALSE}.}

\item{facets}{Logical. If \code{TRUE}, additional formatting for facet plots is
applied. Defaults to \code{FALSE}.}

\item{facet_text_size}{Numeric. If \code{facets = TRUE}, size formatting for facet
text (\code{strip.text}) is applied. Defaults to \code{base_size}.}

\item{draw_panel_border}{Logical. If \code{TRUE}, a border is drawn around panels
in facet plots. Defaults to \code{FALSE}.}

\item{...}{Additional arguments passed to \code{ggplot2::theme} for further
customization.}
}
\value{
A \code{ggplot2} theme object that can be applied to plots.
}
\description{
A flexible and customizable theme function for creating polished and
minimalistic plots using \code{ggplot2}. The \code{theme_cleaner} function provides
various options to control the appearance of plot elements, including font
styles, sizes, colors, axis lines, grid lines, legend, title, subtitle,
captions, and facet appearance. The theme is highly customizable, allowing
for the creation of visually appealing and clean plots.
}
\details{
The function customizes common plot elements like axis text, titles,
subtitles, captions, legend, and facet labels. It is designed to work with
\code{ggplot2} plots, providing a clean and professional look with minimal
styling. You can adjust various aesthetic features such as font size, color,
and legend position for maximum control over the appearance.
}
\examples{
# Create a ggplot2 plot with the theme_cleaner theme
library(ggplot2)
ggplot(mtcars, aes(x = mpg, y = wt)) +
  geom_point() +
  theme_cleaner(
    base_size = 14,
    title_text_size = 16,
    legend_position = "bottom"
  )

# Customize facet plots with theme_cleaner
ggplot(mtcars, aes(x = mpg, y = wt)) +
  geom_point() +
  facet_wrap(~cyl) +
  theme_cleaner(facets = TRUE,
  facet_text_size = 12,
  draw_panel_border = TRUE
  )

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/season.r
\name{season}
\alias{season}
\title{Get Season Based on a Date}
\usage{
season(input_date)
}
\arguments{
\item{input_date}{A Date or POSIXct object representing the date to determine
the season for. The input must be of class \code{Date} or \code{POSIXct}.}
}
\value{
A factor indicating the season corresponding to the input date. The
factor levels are:
\itemize{
\item "Winter" for December, January, and February.
\item "Spring" for March, April, and May.
\item "Summer" for June, July, and August.
\item "Fall" for September, October, and November.
\item "Undetermined" if the input is not a valid Date or POSIXct object or
if the month is missing.
}
}
\description{
This function determines the season (Winter, Spring, Summer, or Fall) based
on an input date.

The seasons are assigned based on geographic regions similar to how seasons
occur in the United States.

The seasons are determined using the month of the year and
the traditional meteorological definition of seasons (Winter: December,
January, February; Spring: March, April, May; Summer: June, July, August;
Fall: September, October, November).
}
\examples{
# Example usage of the season function
season(as.Date("2025-01-15"))
season(as.POSIXct("2025-07-01 12:00:00"))

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_7.R
\name{seqic_indicator_7}
\alias{seqic_indicator_7}
\title{SEQIC Indicator 7 - Delayed Arrival to Definitive Care}
\usage{
seqic_indicator_7(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  time_from_injury_to_arrival,
  transfer_out_indicator,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{time_from_injury_to_arrival}{Column name representing the time
in minutes from injury occurrence to arrival at the trauma center. Numeric
type.}

\item{transfer_out_indicator}{Column name indicating whether the patient was
transferred out of the initial trauma center to definitive care. Logical,
character, or factor type. Values representing "No" (e.g., FALSE, "No")
indicate no transfer out.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 7 results. Includes numerator,
denominator, and proportion. 95\% confidence intervals are included if
requested.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Computes SEQIC Indicator 7, which measures the proportion of trauma patients
arriving at the definitive care facility trauma centers (level I–IV) more
than 180 minutes after injury. This indicator identifies delays in definitive
care.
}
\details{
This function:
\itemize{
\item Filters the dataset to trauma center levels I through IV.
\item Deduplicates the dataset by \code{unique_incident_id}.
\item Creates a logical flag for arrivals occurring more than 180 minutes
after injury.
\item Identifies definitive care records where the patient arrived greater
than 180 minutes after the time of injury.
\item Returns a summarized tibble with the number of such cases
(numerator), total eligible records (denominator), and the proportion.
\item Optionally includes 95\% confidence intervals if \code{calculate_ci} is
specified.
}
}
\note{
The user must ensure all columns are correctly passed and that time values
are numeric and measured in minutes.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Create test data for Indicator 7
test_data <- tibble::tibble(
  id = as.character(1:10),
  trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
  time_to_arrival = c(200, 100, 220, 150, 400, 181, 90, 179, 240, 178),
  transfer_out = c("No", "No", "No", "No", "Yes", "No", "No", "No", "No",
  "No")
)

# Run the indicator function
traumar::seqic_indicator_7(
  data = test_data,
  level = trauma_level,
  unique_incident_id = id,
  time_from_injury_to_arrival = time_to_arrival,
  transfer_out_indicator = transfer_out
)

}
\author{
Nicolas Foss Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_13.R
\name{seqic_indicator_13}
\alias{seqic_indicator_13}
\title{SEQIC Indicator 13 – Validation of  Trauma Registry Records}
\usage{
seqic_indicator_13(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  validity_score,
  validity_threshold = 85,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{validity_score}{Numeric. The proportion of each trauma registry record
that is valid, expressed as a percentage (0–100). Typically calculated by
the registry system.}

\item{validity_threshold}{Numeric. The minimum acceptable validity percentage
threshold for records to be counted in the numerator. Defaults to \code{85}.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 13 results. Includes numerator,
denominator, and performance rate 95\% confidence intervals are included if
requested.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calculates the proportion of trauma records that meet or exceed
a threshold for data validity among facilities at the specified trauma center
levels. Optionally computes confidence intervals.
}
\details{
This function:
\itemize{
\item Filters to include only patients treated at trauma centers with
levels specified in \code{included_levels} (default: Levels I–IV).
\item Deduplicates the dataset using \code{unique_incident_id} to ensure each
incident is counted only once.
\item Flags records with a \code{validity_score} greater than or equal to the
specified \code{validity_threshold} threshold (default: 85).
\item Calculates the proportion of valid records among all included
records.
\item Optionally calculates binomial confidence intervals using the method
specified in \code{calculate_ci} via \code{nemsqa_binomial_confint()}.
\item Adds a "Population/Sample" label unless grouping is applied via
\code{groups}.
}

Users must ensure that appropriate column names are passed using tidy
evaluation (bare column names) and that the input data has been cleaned and
includes no missing or malformed identifiers, trauma level classifications,
or validity scores.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Simulated data for SEQIC Indicator 13
test_data <- tibble::tibble(
  id = as.character(1:12),
  trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV", "I", "II",
  "III", "IV"),
  validity = c(90, 80, 88, 92, 86, 75, 89, 70, 95, 85, 83, 87)
)

# Run the function
traumar::seqic_indicator_13(
  data = test_data,
  level = trauma_level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id = id,
  validity_score = validity,
  validity_threshold = 85,
  calculate_ci = "wilson"
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trauma_case_mix.r
\name{trauma_case_mix}
\alias{trauma_case_mix}
\title{View the Current Patient Population Case Mix Compared to the Major
Trauma Study Case Mix}
\usage{
trauma_case_mix(df, Ps_col, outcome_col)
}
\arguments{
\item{df}{A data frame containing patient data.}

\item{Ps_col}{The name of the column containing the probability of survival
(Ps) values.}

\item{outcome_col}{The name of the column containing the binary outcome data
(valid values are 1 or TRUE for alive, 0 or FALSE for dead).}
}
\value{
A data frame containing:
\itemize{
\item \code{Ps_range}: The probability of survival range category.
\item \code{current_fraction}: The fraction of patients in the current
dataset within each Ps range.
\item \code{MTOS_distribution}: The reference distribution of patients in
each Ps range based on the MTOS study.
\item \code{survivals}: The number of observed survivors (outcome = 1) in
each Ps range.
\item \code{predicted_survivals}: The sum of predicted survivals (sum of
Ps values) in each Ps range.
\item \code{deaths}: The number of observed deaths (outcome = 0) in each
Ps range.
\item \code{predicted_deaths}: The sum of predicted deaths (sum of 1 - Ps
values) in each Ps range.
\item \code{count}: The total number of patients in each Ps range.
}
}
\description{
This function compares the current patient population's case mix (based on
probability of survival, Ps) to the MTOS case mix by binning patients into
specific Ps ranges. It returns the fraction of patients in each range and
compares it to the MTOS distribution. For more information on the methods
used in these calculations, please see Flora (1978) and Boyd et al. (1987).
}
\details{
The function checks whether the \code{outcome_col} contains values representing a
binary outcome. It also ensures that \code{Ps_col} contains numeric values within
the range 0 to 1. If any values exceed 1, a warning is issued. The patients
are then grouped into predefined Ps ranges, and the function compares the
fraction of patients in each range with the MTOS case mix distribution.

Like other statistical computing functions, \code{trauma_case_mix()} is happiest
without missing data.  It is best to pass complete probability of survival
and outcome data to the function for optimal performance. With smaller
datasets, this is especially helpful.  However, \code{trauma_case_mix()} will
throw a warning about missing values, if any exist in \code{Ps_col} and/or
\code{outcome_col}.
}
\note{
This function will produce the most reliable and interpretable results when
using a dataset that has one row per patient, with each column being a
feature.
}
\examples{
# Generate example data
set.seed(123)

# Parameters
# Total number of patients
n_patients <- 5000

# Arbitrary group labels
groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

# Trauma types
trauma_type_values <- sample(
  x = c("Blunt", "Penetrating"),
  size = n_patients,
  replace = TRUE
)

# RTS values
rts_values <- sample(
  x = seq(from = 0, to = 7.8408, by = 0.005),
  size = n_patients,
  replace = TRUE
)

# patient ages
ages <- sample(
  x = seq(from = 0, to = 100, by = 1),
  size = n_patients,
  replace = TRUE
)

# ISS scores
iss_scores <- sample(
  x = seq(from = 0, to = 75, by = 1),
  size = n_patients,
  replace = TRUE
)

# Generate survival probabilities (Ps)
Ps <- traumar::probability_of_survival(
  trauma_type = trauma_type_values,
  age = ages,
  rts = rts_values,
  iss = iss_scores
)

# Simulate survival outcomes based on Ps
survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

# Create data frame
data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Compare the current case mix with the MTOS case mix
trauma_case_mix(data, Ps_col = Ps, outcome_col = death)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalize.r
\name{normalize}
\alias{normalize}
\title{Normalize a Numeric Vector}
\usage{
normalize(x, method = c("min_max", "z_score"))
}
\arguments{
\item{x}{A numeric or integer vector to be normalized.}

\item{method}{A character string specifying the normalization method. Options
are \code{"min_max"} for min-max normalization or \code{"z_score"} for z-score
normalization. If no method is provided, the default is \code{"min_max"}.}
}
\value{
A numeric vector of the same length as \code{x}, containing the
normalized values.
}
\description{
This function normalizes a numeric or integer vector using one of two
methods: min-max normalization (scales data to the range (0, 1)) or z-score
normalization (centers data around 0 with a standard deviation of 1).
}
\examples{
# Example data
data <- c(10, 20, 30, 40, 50, NA)

# Min-max normalization
normalize(data, method = "min_max")

# Z-score normalization
normalize(data, method = "z_score")

# Default behavior (min-max normalization)
normalize(data)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_11.R
\name{seqic_indicator_11}
\alias{seqic_indicator_11}
\title{SEQIC Indicator 11 – Overtriage for Minor Trauma Patients}
\usage{
seqic_indicator_11(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  transfer_out_indicator,
  receiving_indicator,
  unique_incident_id,
  iss,
  ed_LOS,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{transfer_out_indicator}{Column name indicating whether the patient was
transferred out of the initial trauma center to definitive care. Logical,
character, or factor type. Values representing "No" (e.g., FALSE, "No")
indicate no transfer out.}

\item{receiving_indicator}{Column name indicating whether the
patient was transferred into the trauma center. Logical, character, or
factor type. Values representing "Yes" (e.g., TRUE, "Yes") indicate
transfer in.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{iss}{Optional numeric column representing the Injury Severity Score.}

\item{ed_LOS}{Column for the calculated ED length of stay, measured in
minutes.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing the numerator, denominator, and proportion of
overtriaged patients (Indicator 11), with optional 95\% confidence
intervals.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calculates SEQIC Indicator 11, which estimates the proportion of minor trauma
patients who were transferred into a trauma center and remained in the
Emergency Department for less than 24 hours. This indicator is designed to
identify potential overtriage events within the trauma system. Minor trauma
patients are identified using the Injury Severity Score (ISS < 9). Patients
must not have been transferred out and must have been received at a trauma
center level included in \code{included_levels}.
}
\details{
This function:
\itemize{
\item Filters the dataset to include only patients treated at trauma
centers designated Level I through IV.
\item Excludes patients transferred out and retains only those received by
the trauma center.
\item Deduplicates incident-level records using \code{unique_incident_id}.
\item Classifies patients as low-risk based on the  Injury Severity Score
(ISS < 9).
\item Flags low-risk patients who were discharged from the ED in under 24
hours.
\item Stratifies results by one or more user-defined grouping variables.
\item Returns a summarized tibble with the number of eligible low-risk
short-stay discharges (numerator), all received patients meeting inclusion
criteria (denominator), and the resulting proportion.
\item Optionally includes 95\% confidence intervals if \code{calculate_ci} is
specified.
}

Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Simulated data for SEQIC Indicator 11
test_data <- tibble::tibble(
  id = as.character(1:10),
  trauma_level = c("I", "II", "III", "IV", "II", "I", "IV", "III", "II",
  "I"),
  transferred_out = c(FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE,
  FALSE, FALSE),
  received = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
  iss = c(4, 8, 10, 6, 5, 7, 6, 15, 3, 2),
  ed_LOS = c(6, 20, 30, 18, 8, 5, 22, 40, 2, 4),
  region = rep(c("East", "West"), each = 5)
)

# Run the function
traumar::seqic_indicator_11(
  data = test_data,
  level = trauma_level,
  included_levels = c("I", "II", "III", "IV"),
  transfer_out_indicator = transferred_out,
  receiving_indicator = received,
  unique_incident_id = id,
  iss = iss,
  ed_LOS = ed_LOS,
  groups = "region",
  calculate_ci = "clopper-pearson"
)

}
\references{
Roden-Foreman JW, Rapier NR, Yelverton L, Foreman ML. Asking a Better
Question: Development and Evaluation of the Need For Trauma Intervention
(NFTI) Metric as a Novel Indicator of Major Trauma. J Trauma Nurs. 2017
May/Jun;24(3):150-157. doi: 10.1097/JTN.0000000000000283. PMID: 28486318.
}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_10.R
\name{seqic_indicator_10}
\alias{seqic_indicator_10}
\title{SEQIC Indicator 10 – Trauma Team Activation Appropriateness}
\usage{
seqic_indicator_10(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  transfer_out_indicator,
  trauma_team_activation_level,
  iss,
  nfti,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{transfer_out_indicator}{Column name indicating whether the patient was
transferred out of the initial trauma center to definitive care. Logical,
character, or factor type. Values representing "No" (e.g., FALSE, "No")
indicate no transfer out.}

\item{trauma_team_activation_level}{Column indicating the trauma team
activation level (e.g., \code{"Level 1"}, \code{"Level 2"}, \code{"Level 3"},
\code{"Consultation"}). Must be character or factor.}

\item{iss}{Optional numeric column representing the Injury Severity Score.}

\item{nfti}{Optional column indicating Need For Trauma Intervention
classification of positive or negative. Should be character, factor, or
logical.}

\item{groups}{Optional character vector of column names used for grouping
results.}

\item{calculate_ci}{Optional; if not \code{NULL}, must be \code{"wilson"} or
\code{"clopper-pearson"} to compute confidence intervals.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A list of two tibbles with counts and proportions for SEQIC
Indicators 10a, 10b, and 10c, along with model diagnostics for the Cribari
or NFTI ouputs.  The proportions in 10a, 10b, and 10c will optionally
include 95\% confidence intervals.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calculates three trauma system quality indicators related to trauma team
activations where the patient was kept at the facility:
\itemize{
\item 10a: Proportion of patients meeting triage criteria (based on Injury
Severity Score or Need For Trauma Intervention) who received low-level
or no activation (undertriage).
\item 10b: Proportion of patients not meeting triage criteria who received
highest-level trauma activation (overtriage).
\item 10c: Proportion of major trauma patients receiving a full activation
(undertriage via Peng & Xiang, 2019).

(10a, 10b, 10c can be based on Injury Severity Score or Need For Trauma
Intervention based on user choice)
}

Users may stratify results by one or more grouping variables and optionally
compute confidence intervals.
}
\details{
This function:
\itemize{
\item Restricts analysis to Level I–IV trauma centers.
\item Removes duplicate incidents using \code{unique_incident_id}.
\item Classifies each record as meeting or not meeting triage criteria
based on ISS or NFTI logic.
\item Optionally computes 95\% confidence intervals for each indicator.
}

Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Simulated data for SEQIC Indicator 10
test_data <- tibble::tibble(
  id = as.character(1:12),
  trauma_level = c("I", "II", "III", "IV", "II", "I", "IV", "III", "II", "I",
  "III", "IV"),
  activation = c("Level 1", "Level 2", "None", "Consultation", "Level 1",
  "Level 1", "None", "Level 3", "Level 1", "Consultation", "None", "Level
  2"),
  acute_transfer = rep("No", 12),
  iss = c(25, 10, 16, 8, 30, 45, 12, 9, 28, 6, 17, 14),
  nfti = c(TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, FALSE, FALSE, TRUE, FALSE,
  TRUE, TRUE),
  region = rep(c("East", "West"), each = 6)
)

# Run the function, this will succeed
traumar::seqic_indicator_10(
  data = test_data,
  level = trauma_level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id = id,
  transfer_out_indicator = acute_transfer,
  trauma_team_activation_level = activation,
  iss = iss,
  nfti = NULL,
  groups = "region",
  calculate_ci = "wilson"
)

# Run the function, this will fail
# Only one of `iss` or `nfti` arguments can be passed, not both
try(
  traumar::seqic_indicator_10(
  data = test_data,
  level = trauma_level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id = id,
  transfer_out_indicator = acute_transfer,
  trauma_team_activation_level = activation,
  iss = iss,
  nfti = nfti,
  groups = "region",
  calculate_ci = "wilson"
))

}
\references{
Beam G, Gorman K, Nannapaneni S, Zipf J, Simunich T, et al. (2022) Need for
Trauma Intervention and Improving Under-Triaging in Geriatric Trauma
Patients: undertriaged or Misclassified. Int J Crit Care Emerg Med 8:136.
doi.org/10.23937/2474-3674/1510136

Peng J, Xiang H. Trauma undertriage and overtriage rates: are we using the
wrong formulas? Am J Emerg Med. 2016 Nov;34(11):2191-2192. doi:
10.1016/j.ajem.2016.08.061. Epub 2016 Aug 31. PMID: 27615156; PMCID:
PMC6469681.

Roden-Foreman JW, Rapier NR, Yelverton L, Foreman ML. Asking a Better
Question: Development and Evaluation of the Need For Trauma Intervention
(NFTI) Metric as a Novel Indicator of Major Trauma. J Trauma Nurs. 2017
May/Jun;24(3):150-157. doi: 10.1097/JTN.0000000000000283. PMID: 28486318.
}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_9.R
\name{seqic_indicator_9}
\alias{seqic_indicator_9}
\title{SEQIC Indicator 9 - Emergency Department Transfer Timeliness}
\usage{
seqic_indicator_9(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  transfer_out_indicator,
  transport_method,
  unique_incident_id,
  trauma_team_activated,
  risk_group,
  ed_LOS,
  ed_decision_LOS,
  ed_decision_discharge_LOS,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{transfer_out_indicator}{Column name indicating whether the patient was
transferred out of the initial trauma center to definitive care. Logical,
character, or factor type. Values representing "No" (e.g., FALSE, "No")
indicate no transfer out.}

\item{transport_method}{Column identifying the EMS transport method (e.g.,
ambulance, private vehicle). Used to exclude non-qualified modes of
arrival.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{trauma_team_activated}{Column indicating whether the trauma team was
activated (character, factor, or logical).}

\item{risk_group}{A character or factor column indicating the patient's risk
group (e.g., "High", "Moderate", "Low"). See risk definitions below.}

\item{ed_LOS}{Column for the calculated ED length of stay, measured in
minutes.}

\item{ed_decision_LOS}{Numeric column representing minutes from ED arrival to
decision to transfer.}

\item{ed_decision_discharge_LOS}{Numeric column representing minutes from ED
decision to discharge to physical discharge.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A list of four tibbles, with optional 95\% confidence intervals:
\itemize{
\item{\code{seqic_9_all}}: Proportion of transferred trauma patients with ED
discharge or decision delays >2 or >3 hours, grouped by optional
variables.
\item{\code{seqic_9_activations}}: Same proportions as above, further stratified
by trauma team activation status.
\item{\code{seqic_9_risk}}: Same proportions as above, further stratified by
risk groups.
\item{\code{seqic_9_activations_risk}}: Same proportions as above, further
stratified by risk groups and trauma team activation status.
}

Each tibble includes numerators, denominators, proportions, and (optionally)
confidence intervals for:
\itemize{
\item{9a}: Delayed discharge >2 hours
\item{9b}: Delayed discharge >3 hours
\item{9c}: Delayed decision >1 hours
\item{9d}: Delayed decision >2 hours
\item{9e}: Delayed decision to discharge >1 hour
\item{9f}: Delayed decision to discharge >2 hours
}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calculates the proportion of EMS-transferred trauma patients who experienced
delayed transfer from the emergency department (ED) based on disposition and
decision-to-transfer time frames. This includes both overall rates and
stratified results by trauma team activation status, with optional confidence
intervals.
}
\details{
This function:
\itemize{
\item Filters the dataset to include only transfers out from trauma centers
designated Level I through IV.
\item Deduplicates records using \code{unique_incident_id}.
\item Flags records where emergency department decision to discharge
occurred more than 60 or 120 minutes after ED arrival.
\item Flags records where physical departure from the ED occurred more than
120 or 180 minutes after ED arrival.
\item Flags records where physical discharge occurred more than 60 or 120
minutes after ED decision to discharge.
\item Stratifies results by trauma team activation status and
one or more grouping variables.
\item Stratifies results by risk groups and one or more grouping variables.
\item Returns a summarized tibble with the number of delayed cases
(numerator), eligible records (denominator), and the proportion for each
delay threshold.
\item Optionally includes 95\% confidence intervals if \code{calculate_ci = TRUE}.
}
}
\note{
This function calculates discharge timeliness outcomes for patients
transported to trauma centers, stratified by risk of mortality. Risk
groups—low, moderate, and high— are defined by the Iowa System Evaluation and
Quality Improvement Committee (SEQIC) as described below. Users may also
apply alternative risk stratification methods if preferred.

\itemize{
\item Abnormal Physiology Criteria: GCS 3–5; Respirations <5 or >30 per
minute; Systolic BP <60 mm Hg
\item High Risk: Probability of Survival < 0.2; ISS > 41; ISS > 24 with
abnormal physiology
\item Moderate Risk: Probability of Survival 0.2–0.5; ISS 16–41
\item Low Risk: Probability of Survival > 0.5; ISS < 16; Normal physiology
}

Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Simulated dataset for SEQIC Indicator 9
test_data <- tibble::tibble(
  id = as.character(1:10),
  trauma_level = c("I", "II", "III", "IV", "V", "II", "III", "IV", "I",
  "II"),
  transport = c("Ambulance", "Ambulance", "Private Vehicle", "Ambulance",
  "Helicopter", "Ambulance", "Ambulance", "Ambulance", "Ambulance",
  "Ambulance"),
  activated = c(TRUE, FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, FALSE,
  FALSE),
  ed_LOS = c(120, 180, 90, 60, 200, 130, 110, 160, 95, 220),
  ed_decision = c(55, 125, 65, 30, 190, 80, 70, 45, 61, 130),
  ed_discharge = c(130, 185, 110, 65, 150, 160, 95, 180, 70, 210),
  transfer_out = c(TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, FALSE, TRUE,
  TRUE),
  risk = c("High", "High", "Moderate", "Low", "Moderate", "Low",
           "High", "Low", "Moderate", "High")
)

# Run the function, and store as a list object
seqic_9_result <- traumar::seqic_indicator_9(
  data = test_data,
  level = trauma_level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id = id,
  transport_method = transport,
  transfer_out_indicator = transfer_out,
  ed_LOS = ed_LOS,
  ed_decision_LOS = ed_decision,
  ed_decision_discharge_LOS = ed_discharge,
  trauma_team_activated = activated,
  risk_group = risk
)

# Take a look at the overall output of the function
seqic_9_result$overall |>
tidyr::pivot_longer(cols = -1,
                    names_to = "Indicator",
                    values_to = "Values"
                    )

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probability_of_survival.R
\name{probability_of_survival}
\alias{probability_of_survival}
\title{Calculate Probability of Survival Using TRISS Method}
\usage{
probability_of_survival(trauma_type, age, rts, iss)
}
\arguments{
\item{trauma_type}{Character vector indicating the type of trauma ("Blunt" or
"Penetrating"). Different methods exist for calculating probability of
survival for burn patients, and so these records are excluded here.}

\item{age}{Numeric vector indicating the patient's age in years.}

\item{rts}{Numeric vector indicating the patient's Revised Trauma Score
(RTS).}

\item{iss}{Numeric vector indicating the patient's Injury Severity Score
(ISS).}
}
\value{
Numeric vector of predicted probabilities of survival on a scale
from 0 to 1.
}
\description{
This function calculates the probability of survival (Ps) for trauma patients
based on the Trauma and Injury Severity Score (TRISS) methodology. TRISS
combines physiological and anatomical data to predict survival likelihood
using a logistic regression model. The function incorporates trauma type,
patient age, Revised Trauma Score (RTS), and Injury Severity Score (ISS) into
the calculation. Probability of survival is expressed as a percentage.
}
\details{
The methodology used in the calculation of survival probabilities
aligns with the coefficients published in Norouzi et al. (2013) and Merchant
et al. (2023). Consistent with Boyd et al. (1987),
\code{probability_of_survival()} does not treat patients under 15 years of age
differently and accounts for penetrating injuries similarly to other age
groups. Norouzi et al. (2013) and Merchant et al. (2023) use the updated
TRISS coefficients to calculate survival probabilities for penetrating
traumas with the same coefficients as for blunt traumas. If this approach is
preferred, please take note and adjust accordingly.
}
\examples{
# Example usage:
trauma_data <- data.frame(
  Trauma_Type = c("Blunt", "Penetrating"),
  Patient_Age_Years = c(30, 60),
  RTS = c(7.84, 6.90),
  ISS = c(10, 25)
)

# Run the function on example data
result <- trauma_data |>
  dplyr::mutate(Ps = probability_of_survival(
    trauma_type = Trauma_Type,
    age = Patient_Age_Years,
    rts = RTS,
    iss = ISS
  ))

# Print the result
result

}
\references{
Boyd CR, Tolson MA, Copes WS. (1987). Evaluating trauma care: the TRISS
method. Trauma Score and the Injury Severity Score. J Trauma. 1987
Apr;27(4):370-8. PMID: 3106646.

Merchant AAH, Shaukat N, Ashraf N, Hassan S, Jarrar Z, Abbasi A, et al.
(2023). Which curve is better? A comparative analysis of trauma scoring
systems in a South Asian country. Trauma Surgery & Acute Care Open.
2023;8:e001171. \url{doi:10.1136/tsaco-2023-001171}

Norouzi V, Feizi I, Vatankhah S, Pourshaikhian M. (2013). Calculation of the
probability of survival for trauma patients based on trauma score and the
injury severity score model in fatemi hospital in ardabil. Arch Trauma Res.
2013 Spring;2(1):30-5. \url{doi:10.5812/atr.9411}. Epub 2013 Jun 1. PMID:
24396787; PMCID: PMC3876517.
}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relative_mortality.R
\name{rm_bin_summary}
\alias{rm_bin_summary}
\title{Bin-Level Summary for Relative Mortality Metric (RMM)}
\usage{
rm_bin_summary(
  data,
  Ps_col,
  outcome_col,
  group_vars = NULL,
  n_samples = 100,
  Divisor1 = 5,
  Divisor2 = 5,
  Threshold_1 = 0.9,
  Threshold_2 = 0.99,
  bootstrap_ci = TRUE,
  seed = NULL
)
}
\arguments{
\item{data}{A data frame or tibble containing the data.}

\item{Ps_col}{The name of the column containing the survival probabilities
(Ps). Should be numeric on a scale from 0 to 1.}

\item{outcome_col}{The name of the column containing the outcome data. It
should be binary, with values indicating patient survival. A value of \code{1}
should represent "alive" (survived), while \code{0} should represent "dead" (did
not survive).  \code{TRUE/FALSE} are accepted as well. Ensure the column
contains only these possible values.}

\item{group_vars}{Optional character vector specifying grouping variables for
stratified analysis. If \code{NULL}, the calculation is performed on the entire
dataset.}

\item{n_samples}{A numeric value indicating the number of bootstrap samples
to take from the data source.}

\item{Divisor1}{A positive numeric value controlling the coarseness of bins
for Ps values below \code{Threshold_1}. It scales the number of steps from the
start of the dataset up to the \code{Threshold_1} cut point. Larger values
produce fewer, broader bins; smaller values produce more, narrower bins.
Defaults to \code{5}.}

\item{Divisor2}{A positive numeric value controlling the coarseness of bins
for Ps values between \code{Threshold_1} and \code{Threshold_2}. Larger values yield
wider bins, and smaller values yield narrower bins in this range. Defaults
to \code{5}.}

\item{Threshold_1}{A numeric value that defines the lower bound of the
high-survival probability range in \code{Ps_col}. The function identifies the
first index where \code{Ps_col} exceeds this value and begins applying smaller
bin widths from that point onward. Defaults to \code{0.9}, meaning binning
changes once Ps > 0.90.}

\item{Threshold_2}{A numeric value that defines the upper bound of the
high-survival probability range in \code{Ps_col}. The function identifies the
first index where \code{Ps_col} exceeds this value. Between \code{Threshold_1} and
\code{Threshold_2}, finer binning is applied; above \code{Threshold_2}, binning may
again change. Defaults to \code{0.99}, meaning the special binning range is
between Ps values of 0.90 and 0.99.}

\item{bootstrap_ci}{A logical indicating whether to return the relative
mortality metric estimate and 95\% confidence intervals using bootstrap
sampling. Default is \code{TRUE}.}

\item{seed}{Optional numeric value to set a random seed for reproducibility.
If \code{NULL} (default), no seed is set.}
}
\value{
A tibble containing bin-level statistics including:
\itemize{
\item \code{bin_number}: The bin to which each record was assigned.
\item \code{TA_b}: Total alive in each bin (number of patients who survived).
\item \code{TD_b}: Total dead in each bin (number of patients who did not survive).
\item \code{N_b}: Total number of patients in each bin.
\item \code{EM_b}: Estimated mortality rate for each bin (TD_b / (TA_b + TD_b)).
\item \code{AntiS_b}: The anticipated survival rate for each bin.
\item \code{AntiM_b}: The anticipated mortality rate for each bin.
\item \code{bin_start}: The lower bound of the survival probability range for each
bin.
\item \code{bin_end}: The upper bound of the survival probability range for each
bin.
\item \code{midpoint}: The midpoint of the bin range (calculated as
(bin_start + bin_end) / 2).
\item \code{R_b}: The width of each bin (bin_end - bin_start).
\item \code{population_RMM_LL}: The lower bound of the 95\% confidence interval for
the population RMM.
\item \code{population_RMM}: The final calculated Relative Mortality Metric for the
population existing in \code{data}.
\item \code{population_RMM_UL}: The upper bound of the 95\% confidence interval for
the population RMM.
\item \code{population_CI}: The confidence interval width for the population RMM.
\item \code{bootstrap_RMM_LL}: The lower bound of the 95\% confidence interval for
the bootstrap RMM. (optional, if \code{bootstrap_ci = TRUE})
\item \code{bootstrap_RMM}: The average RMM value calculated for the bootstrap
sample. (optional, if \code{bootstrap_ci = TRUE})
\item \code{bootstrap_RMM_UL}: The upper bound of the 95\% confidence interval for
the bootstrap RMM. (optional, if \code{bootstrap_ci = TRUE})
\item \code{bootstrap_CI}: The width of the 95\% confidence interval for the
bootstrap RMM. (optional, if \code{bootstrap_ci = TRUE})
}
}
\description{
Calculates a bin-level summary for the Relative Mortality Metric (RMM) from
Napoli et al. (2017) by grouping data into bins based on survival
probabilities (Ps) and summarizing outcomes within each bin. This function
returns statistics such as total alive, total dead, estimated mortality,
anticipated mortality, and confidence intervals for each bin. For more
information on the methods used in this function, see as well Schroeder et
al. (2019), and Kassar et al. (2016).

The Relative Mortality Metric (RMM) quantifies the performance of a center in
comparison to the anticipated mortality based on the TRISS national
benchmark. The RMM measures the difference between observed and expected
mortality, with a range from -1 to 1.
\itemize{
\item An RMM of 0 indicates that the observed mortality aligns with the expected
national benchmark across all acuity levels.
\item An RMM greater than 0 indicates better-than-expected performance, where
the center is outperforming the national benchmark.
\item An RMM less than 0 indicates under-performance, where the center’s observed
mortality is higher than the expected benchmark.
}

This metric helps assess how a center's mortality compares to the national
standards, guiding quality improvement efforts.\code{rm_bin_summary()} utilizes
bootstrap sampling to calculate the confidence intervals via the standard
error method.
}
\details{
Like other statistical computing functions, \code{rm_bin_summary()} is happiest
without missing data.  It is best to pass complete probability of survival
and mortality outcome data to the function for optimal performance. With
smaller datasets, this is especially helpful.  However, \code{rm_bin_summary()}
will throw a warning about missing values, if any exist in \code{Ps_col} and/or
\code{outcome_col}.

\code{rm_bin_summary()} assumes \code{Ps_col} contains probabilities derived from
real-world inputs for the Trauma Injury Severity Score (TRISS) model.
Synthetic or low-variability data (especially with small sample sizes) may
not reflect the distribution of TRISS-derived survival probabilities. This
can result in unstable estimates or function failure due to insufficient
dispersion. With small sample sizes, it may be important to use smaller
values with the divisor arguments and adjust the thresholds (based on the
distribution of the \code{Ps_col} values) to create bins that better accommodate
the data.

Due to the use of bootstrap sampling within the function, users should
consider setting the random number seed within \code{rm_bin_summary()} using the
\code{seed} argument for reproducibility.
}
\note{
This function will produce the most reliable and interpretable results when
using a dataset that has one row per patient, with each column being a
feature.

By default, \code{rm_bin_summary()} derives bin cut points from the full dataset’s
distribution. This ensures comparability across groups when \code{group_vars} is
used. To tailor results to a specific group (e.g., a single hospital), filter
the dataset to that subgroup before calling \code{rm_bin_summary()}. The function
will then compute bins and related statistics using only that subset’s
\code{Ps_col} distribution. When \code{group_vars} is used, and ff a group lacks
observations within one or more bins, \code{rm_bin_summary()} will compute
statistics only for the bins that contain data. Bins with no observations are
excluded from the summary for that group.
}
\examples{
# Generate example data
set.seed(123)

# Parameters
# Total number of patients
n_patients <- 5000

# Arbitrary group labels
groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

# Trauma types
trauma_type_values <- sample(
  x = c("Blunt", "Penetrating"),
  size = n_patients,
  replace = TRUE
)

# RTS values
rts_values <- sample(
  x = seq(from = 0, to = 7.8408, by = 0.005),
  size = n_patients,
  replace = TRUE
)

# patient ages
ages <- sample(
  x = seq(from = 0, to = 100, by = 1),
  size = n_patients,
  replace = TRUE
)

# ISS scores
iss_scores <- sample(
  x = seq(from = 0, to = 75, by = 1),
  size = n_patients,
  replace = TRUE
)

# Generate survival probabilities (Ps)
Ps <- traumar::probability_of_survival(
  trauma_type = trauma_type_values,
  age = ages,
  rts = rts_values,
  iss = iss_scores
)

# Simulate survival outcomes based on Ps
survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

# Create data frame
data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Example usage of the `rm_bin_summary()` function
rm_bin_summary(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  n_samples = 10,
  Divisor1 = 4,
  Divisor2 = 4
)

# Create example grouping variable (e.g., hospital)
hospital <- sample(c("Hospital A", "Hospital B"), n_patients, replace = TRUE)

# Create data frame
data <- data.frame(
  Ps = Ps,
  survival = survival_outcomes,
  hospital = hospital
) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Example usage of the `rm_bin_summary()` function with grouping
rm_bin_summary(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  group_vars = "hospital", # Stratifies by hospital
  n_samples = 10,
  Divisor1 = 4,
  Divisor2 = 4
)

}
\references{
Kassar, O.M., Eklund, E.A., Barnhardt, W.F., Napoli, N.J., Barnes, L.E.,
Young, J.S. (2016). Trauma survival margin analysis: A dissection of trauma
center performance through initial lactate. The American Surgeon, 82(7),
649-653. \url{doi:10.1177/000313481608200733}

Napoli, N. J., Barnhardt, W., Kotoriy, M. E., Young, J. S., & Barnes, L. E.
(2017). Relative mortality analysis: A new tool to evaluate clinical
performance in trauma centers. IISE Transactions on Healthcare Systems
Engineering, 7(3), 181–191. \url{doi:10.1080/24725579.2017.1325948}

Schroeder, P. H., Napoli, N. J., Barnhardt, W. F., Barnes, L. E., & Young, J.
S. (2018). Relative mortality analysis of the “golden hour”: A comprehensive
acuity stratification approach to address disagreement in current literature.
Prehospital Emergency Care, 23(2), 254–262.
\url{doi:10.1080/10903127.2018.1489021}
}
\seealso{
\code{\link[=probability_of_survival]{probability_of_survival()}}, \code{\link[=rmm]{rmm()}}, and \code{\link[=nonlinear_bins]{nonlinear_bins()}}
}
\author{
Nicolas Foss, Ed.D, MS, original implementation in MATLAB by Nicholas
J. Napoli, Ph.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pretty_number.r
\name{pretty_number}
\alias{pretty_number}
\title{Convert Numbers into Readable Abbreviated Formats}
\usage{
pretty_number(
  x,
  digits = 2,
  n_decimal = deprecated(),
  prefix = NULL,
  truncate = FALSE
)
}
\arguments{
\item{x}{A numeric value or vector to be converted into a readable format.}

\item{digits}{Number of decimal places to display. Defaults to 2.}

\item{n_decimal}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use \code{digits} instead.}

\item{prefix}{An optional character string to prepend to the formatted number
(e.g., "$"). Defaults to \code{NULL}.}

\item{truncate}{A logical value indicating whether to truncate the numbers
before formatting. When \code{TRUE}, the function uses \code{base::signif()} to
truncate the numbers to the specified number of significant digits, making
the output more concise. When \code{FALSE}, the function uses \code{base::round()} to
round the numbers to the specified number of decimal places, preserving the
original scale of the number. Defaults to \code{FALSE}.}
}
\value{
A character vector with the numbers formatted as abbreviated
strings. If \code{prefix} is provided, it prepends the formatted numbers.
}
\description{
This function converts large numeric values into readable abbreviated formats
(e.g., 1,000 becomes "1k") with options for rounding, decimal precision, and
a custom prefix. It supports numbers up to the decillion range.
}
\examples{
# Basic usage
pretty_number(1234)               # "1.23k"
pretty_number(1234567)            # "1.23m"
pretty_number(1234567890)         # "1.23b"

# Adjusting decimal places
pretty_number(1234, digits = 1) # "1.2k"

# Adding a prefix
pretty_number(1234, prefix = "$")  # "$1.23k"

# Without rounding
pretty_number(1250, truncate = TRUE) # "1.2k"

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weekend.r
\name{weekend}
\alias{weekend}
\title{Classify Dates as Weekday or Weekend}
\usage{
weekend(input_date)
}
\arguments{
\item{input_date}{A vector of \code{Date} or \code{POSIXct} objects to classify.}
}
\value{
A character vector with the classification for each date: either
"Weekday" or "Weekend".
}
\description{
This function classifies each date in a vector of dates as either "Weekday"
or "Weekend". The function returns "Weekday" for Monday to Friday and
"Weekend" for Saturday and Sunday.
}
\details{
The function checks if the \code{input_date} is a valid \code{Date} or
\code{POSIXct} object. It returns "Weekday" for dates that fall on Monday
through Friday and "Weekend" for dates that fall on Saturday or Sunday. If
the input is not of the correct class, the function will throw an error.
}
\examples{
# Example 1: Date of a weekend
weekend(as.Date("2025-01-18"))

# Example 2: Date of a weekday
weekend(as.Date("2025-01-15"))

# Example 3: Date of an invalid object
try(
weekend("2025-01-18") # This will throw an error
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_1.R
\name{seqic_indicator_1}
\alias{seqic_indicator_1}
\title{SEQIC Indicator 1 – Trauma Team Response Evaluation}
\usage{
seqic_indicator_1(
  data,
  trauma_team_activation_level,
  trauma_team_physician_service_type,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  response_time,
  trauma_team_activation_provider,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{trauma_team_activation_level}{Column identifying trauma team activation
level (e.g., Level 1, Level 2).}

\item{trauma_team_physician_service_type}{Column indicating the type of
medical provider (e.g., Surgery/Trauma, Emergency Medicine). For indicators
1a, 1b, and 1c, \code{seqic_indicator_1()} will only look for records with the
trauma team member service type documented as marked as 'Surgery/Trauma'.
For Indicators 1d, 1e, and 1f, \code{seqic_indicator_1()} will look for the
following service types:
\itemize{
\item "Surgery/Trauma",
\item "Emergency Medicine",
\item "Family Practice",
\item "Nurse Practitioner",
\item "Physician Assistant",
\item "Surgery Senior Resident",
\item "Hospitalist",
\item "Internal Medicine"
}}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{response_time}{Numeric variable representing the time (in minutes)
to provider response.}

\item{trauma_team_activation_provider}{Column identifying the responding
provider for trauma activation.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 1 results across sub-measures
(1a–1f). Includes numerators, denominators, and performance rate for each
indicator. 95\% confidence intervals are provided optionally.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This function calculates System Evaluation and Quality Improvement Committee
(SEQIC) Indicator 1 (subparts a through f). These indicators assess the
timeliness and type of provider response (e.g., surgeon, mid-level,
physician) to trauma alerts based on trauma team activation level, hospital
trauma level, and time to provider presence. Confidence intervals can
optionally be calculated for the proportion, using either the Wilson or
Clopper-Pearson method.
}
\details{
This function filters and summarizes trauma records to calculate
SEQIC Indicators 1a through 1f:
\itemize{
\item 1a: Proportion of Level 1 activations at Level I/II centers with
surgical response <= 15 minutes.
\item 1b: Same as 1a, but includes Level III centers and uses <= 30 minutes.
\item 1c: Proportion of Level 1 activations with missing surgical response
time.
\item 1d/e: Response within 5 and 20 minutes, respectively, for specific
provider types and activation levels, includes level I-IV trauma centers.
\item 1f: Proportion of missing response times among the group in 1d/e,
includes level I-IV trauma centers.
}
}
\note{
This function:
\itemize{
\item Filters trauma records to those with a trauma team activation level
of "Level 1" and/or "Level 2" based on the indicator.
\item Restricts provider type to surgical, physician, and mid-level
provider roles.
\item Filters trauma center levels to I–IV based on the measure.
\item Calculates the proportion of cases where the response time is within
5, 15, or 30 minutes, depending on the indicator.
\item Computes proportions for trauma activation times, including missing
times and within thresholds.
}

Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{

# Packages
library(dplyr)
library(traumar)

# Data
data <- tibble::tibble(
  incident_id = 1:6,
  activation_level = c("Level 1", "Level 1", "Level 2", "Level 1", "Level 2",
  "Level 1"),
  provider_type = c("Surgery/Trauma", "Emergency Medicine", "Physician
  Assistant", "Surgery/Trauma", "Surgery/Trauma", "Family Practice"),
  trauma_level = c("I", "II", "III", "I", "III", "IV"),
  response_minutes = c(12, 25, 6, NA, 18, 22),
  provider = c("Dr. A", "Dr. B", "PA C", "Dr. D", "Dr. E", "NP F")
)

# Run the function
traumar::seqic_indicator_1(
  data = data,
  trauma_team_activation_level = activation_level,
  trauma_team_physician_service_type = provider_type,
  level = trauma_level,
  unique_incident_id = incident_id,
  response_time = response_minutes,
  trauma_team_activation_provider = provider,
  calculate_ci = "wilson"
) |>
tidyr::pivot_longer(cols = -1,
                    names_to = "Indicator",
                    values_to = "Values"
                    )

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_12.R
\name{seqic_indicator_12}
\alias{seqic_indicator_12}
\title{SEQIC Indicator 12 - Timeliness of Data Entry Post-Discharge}
\usage{
seqic_indicator_12(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  facility_id,
  exclude_facility_list = NULL,
  unique_incident_id,
  data_entry_time,
  data_entry_standard = 60,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{facility_id}{Numeric, character, or factor. Column giving the unique
facility identifiers in the trauma dataset.}

\item{exclude_facility_list}{Optional. Numeric, character, or factor. List of
facilities to exclude from analysis due to known data quality issues or
other justifiable reasons. Defaults to \code{NULL}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{data_entry_time}{Numeric. Column representing the time in days between
patient discharge and trauma registry data entry.}

\item{data_entry_standard}{Numeric. The maximum allowable number of days
between discharge and data entry. Records entered within this threshold are
considered timely. Default is \code{60}.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 12 results. Includes numerator,
denominator, and performance rate. 95\% confidence intervals are included if
requested.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calculates the proportion of trauma cases where data were entered into the
trauma registry within a defined number of days post-discharge. This measure
supports trauma system quality improvement by identifying facilities meeting
timely documentation expectations.
}
\details{
This function:
\itemize{
\item Filters to include only patients treated at Level I–IV trauma
centers.
\item Excludes records from facilities specified by the user, if
applicable.
\item Deduplicates by \code{unique_incident_id} to ensure each incident is
counted once.
\item Flags records where data entry occurred within \code{data_entry_standard}
days of discharge.
\item Optionally calculates confidence intervals using methods from
\code{nemsqa_binomial_confint()}.
\item Returns a tibble with numerator,
denominator, and proportion of timely entries, with optional confidence
intervals and population/sample labels.
}

Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Simulated data for SEQIC Indicator 12
test_data <- tibble::tibble(
  id = as.character(1:10),
  trauma_level = c("I", "II", "III", "IV", "II", "I", "IV", "III", "II",
  "I"),
  facility = c("A", "B", "C", "D", "A", "C", "B", "A", "C", "D"),
  data_entry_delay = c(30, 65, 10, 70, 45, 20, 80, 15, 55, 90)
)

# Run the function
traumar::seqic_indicator_12(
  data = test_data,
  level = trauma_level,
  included_levels = c("I", "II", "III", "IV"),
  facility_id = facility,
  unique_incident_id = id,
  exclude_facility_list = c("D"),
  data_entry_time = data_entry_delay,
  data_entry_standard = 60,
  calculate_ci = "wilson"
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_6.R
\name{seqic_indicator_6}
\alias{seqic_indicator_6}
\title{SEQIC Indicator 6 - Delayed Arrival Following Low GCS}
\usage{
seqic_indicator_6(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  transfer_out_indicator,
  receiving_indicator,
  low_GCS_indicator,
  time_from_injury_to_arrival,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{transfer_out_indicator}{Column name indicating whether the patient was
transferred out of the initial trauma center to definitive care. Logical,
character, or factor type. Values representing "No" (e.g., FALSE, "No")
indicate no transfer out.}

\item{receiving_indicator}{Column name indicating whether the
patient was transferred into the trauma center. Logical, character, or
factor type. Values representing "Yes" (e.g., TRUE, "Yes") indicate
transfer in.}

\item{low_GCS_indicator}{Column name for identifying patients with a
Glasgow Coma Scale score less than 9. Logical, character, or factor type.}

\item{time_from_injury_to_arrival}{Column name representing the time
in minutes from injury occurrence to arrival at the trauma center. Numeric
type.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 6 results. Includes numerator,
denominator, calculated proportion, and optionally 95\% confidence
intervals.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Computes SEQIC Indicator 6 for trauma system quality monitoring. This
indicator measures the proportion of patients presenting with a Glasgow Coma
Scale (GCS) score < 9 who arrive at a trauma level I–IV center more than 180
minutes after injury. It excludes patients transferred out of the facility
and focuses on those transferred into a facility.
}
\details{
This function:
\itemize{
\item Filters to trauma center records from facilities at trauma levels
I–IV.
\item Deduplicates records using \code{unique_incident_id}.
\item Calculates:
\itemize{
\item {Numerator:} Patients with low GCS (< 9) who arrived more than
180 minutes after injury, were transferred in, and not transferred
out.
\item {Denominator:} All patients with low GCS (< 9) who were
transferred in and not transferred out.
}
\item Optionally calculates Wilson or Clopper-Pearson confidence intervals
for the resulting proportion if \code{calculate_ci} is specified.
}
}
\note{
Users must ensure input columns are appropriately coded and standardized.
Transfer and GCS indicators should use consistent logical or textual
representations.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Create test data for Indicator 6
test_data <- tibble::tibble(
  id = as.character(1:10),
  trauma_level = rep(c("I", "II", "III", "IV", "V"), times = 2),
  transfer_out = c("No", "No", "Yes", "No", "No", "No", "No", "No", "No",
  "No"),
  transfer_in = c("Yes", "Yes", "No", "Yes", "No", "Yes", "Yes", "Yes",
  "Yes", "Yes"),
  gcs_low = c(TRUE, TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
  time_to_arrival = c(200, 100, 300, 190, 400, 181, 100, 179, 240, 178)
)

# Run the indicator function
traumar::seqic_indicator_6(
  data = test_data,
  level = trauma_level,
  unique_incident_id = id,
  transfer_out_indicator = transfer_out,
  receiving_indicator = transfer_in,
  low_GCS_indicator = gcs_low,
  time_from_injury_to_arrival = time_to_arrival
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_sig.r
\name{stat_sig}
\alias{stat_sig}
\title{Assign Significance Codes Based on P-Values}
\usage{
stat_sig(p_val_data)
}
\arguments{
\item{p_val_data}{A numeric vector representing the p-values to be
categorized. The vector should contain p-values between 0 and 1.}
}
\value{
A character vector with the assigned significance codes for each
p-value.
}
\description{
This function assigns significance codes to a p-value vector based on commonly
accepted significance thresholds. The significance codes are:
\itemize{
\item \code{"***"} for p-values <= 0.001
\item \code{"**"} for p-values <= 0.01 and > 0.001
\item \code{"*"} for p-values <= 0.05 and > 0.01
\item \code{"."} for p-values <= 0.1 and > 0.05
\item \code{"<>"} for p-values > 0.1
}
}
\examples{
# Example usage of the stat_sig function
data <- data.frame(p_value = c(0.001, 0.03, 0.12, 0.05, 0.07))

data |>
  dplyr::mutate(significance = stat_sig(p_val_data = p_value))

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_8.R
\name{seqic_indicator_8}
\alias{seqic_indicator_8}
\title{SEQIC Indicator 8 - Survival by Risk Group}
\usage{
seqic_indicator_8(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  unique_incident_id,
  mortality_indicator,
  risk_group,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{mortality_indicator}{A logical, character, or factor variable
indicating whether the patient died at the trauma center. Accepts values
like \code{TRUE}/\code{FALSE} or \code{"Yes"}/\code{"No"}.}

\item{risk_group}{A character or factor column indicating the patient's risk
group (e.g., "High", "Moderate", "Low"). See risk definitions below.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A named list with two tibbles:

\code{overall}: A tibble summarizing overall mortality among trauma patients,
grouped by the variables specified in \code{groups}. Columns include:

\itemize{
\item \code{numerator_8_all} (number of survivors),
\item \code{denominator_8_all} (total number
of unique trauma incidents),
\item \code{seqic_8_all} (survival proportion), and optionally
\item \code{lower_ci_8},
\item \code{upper_ci_8} (confidence interval bounds if \code{calculate_ci} is
specified).
}

\code{risk_group}: A tibble summarizing mortality stratified by risk group and any
additional grouping variables. Columns include:

\itemize{
\item \code{risk_group} (used for stratification),
\item \code{numerator_8_risk} (survivors per group),
\item \code{denominator_8_risk} (total incidents per group),
\item \code{seqic_8_risk} (survival proportion per group), and optionally
\item \code{lower_ci_8_risk},
\item \code{upper_ci_8_risk} (confidence interval bounds if \code{calculate_ci} is
specified).
}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calculates the proportion of patients who survived based on risk groups
existing in the data among trauma patients transported to Level I–IV trauma
centers.
}
\details{
\itemize{
\item Filters the dataset to include only trauma center levels I through
IV.
\item Deduplicates the dataset using \code{unique_incident_id} to ensure one
record per incident.
\item Accepts a mortality indicator that may be logical, character, or
factor, and identifies survivors as those with values of \code{FALSE} or \code{"No"}.
\item Requires a predefined \code{risk_group} variable representing categories
such as "Low", "Moderate", or "High" risk.
\item Calculates overall survival proportions and survival proportions
stratified by risk group.
\item Optionally includes 95\% confidence intervals using binomial methods
if \code{calculate_ci} is specified.
}
}
\note{
This function calculates survival outcomes for patients transported
to trauma centers, stratified by risk of mortality. Risk groups—low,
moderate, and high— are defined by the Iowa System Evaluation and Quality
Improvement Committee (SEQIC) as described below. Users may also apply
alternative risk stratification methods if preferred.

\itemize{
\item Abnormal Physiology Criteria: GCS 3–5; Respirations <5 or >30 per
minute; Systolic BP <60 mm Hg
\item High Risk: Probability of Survival < 0.2; ISS > 41; ISS > 24 with
abnormal physiology
\item Moderate Risk: Probability of Survival 0.2–0.5; ISS 16–41
\item Low Risk: Probability of Survival > 0.5; ISS < 16; Normal physiology
}

Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Simulated dataset for SEQIC Indicator 8
test_data <- tibble::tibble(
  id = as.character(1:12),
  trauma_level = c("I", "II", "III", "IV", "V", "II", "I", "III", "IV", "II",
  "I", "III"),
  mortality = c(FALSE, "No", TRUE, "Yes", FALSE, TRUE, "No", FALSE, "Yes",
  FALSE, TRUE, "No"),
  risk = c("High", "High", "Moderate", "Moderate", "Low", "Low", "High",
  "Moderate", "Low", "Moderate", "High", "Low")
)

# Run indicator 8 function
traumar::seqic_indicator_8(
  data = test_data,
  level = trauma_level,
  unique_incident_id = id,
  mortality_indicator = mortality,
  risk_group = risk
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/small_count_label.r
\name{small_count_label}
\alias{small_count_label}
\title{Label Small Counts Based on a Cutoff}
\usage{
small_count_label(var, cutoff, replacement)
}
\arguments{
\item{var}{A numeric vector. This represents the variable to be checked
against the cutoff.}

\item{cutoff}{A numeric value representing the threshold. Values in \code{var}
smaller than this value will be replaced.}

\item{replacement}{A string or a numeric value. If the value in \code{var} is
smaller than the \code{cutoff}, this value will replace it. If a string is
provided, it will replace the numeric values with the string. If a numeric
value is provided, the replacement will also be numeric.}
}
\value{
A vector with values from \code{var}. Values smaller than the \code{cutoff}
will be replaced by the \code{replacement}. If \code{replacement} is a string, the
return type will be character, otherwise, it will remain numeric.
}
\description{
This function labels values in a vector as a replacement string if they are
smaller than a specified cutoff. The input can be numeric, and the function
will return either a modified version of the input vector with small values
replaced by a given label, or it will keep the original values otherwise.
}
\examples{
# Example usage of the small_count_label function
small_count_label(c(1, 5, 10), 5, "Below Cutoff")
small_count_label(c(1, 5, 10), 5, 0)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqic_4.R
\name{seqic_indicator_4}
\alias{seqic_indicator_4}
\title{SEQIC Indicator 4 - Autopsy and Long LOS Without Autopsy}
\usage{
seqic_indicator_4(
  data,
  level,
  included_levels = c("I", "II", "III", "IV"),
  ed_disposition,
  ed_LOS,
  hospital_disposition,
  hospital_LOS,
  unique_incident_id,
  autopsy,
  groups = NULL,
  calculate_ci = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing trauma incident records.}

\item{level}{Column indicating the trauma center designation level (e.g., I,
II, III, IV).}

\item{included_levels}{Character vector indicating what facility levels to
include in the analysis.  Defaults to \code{c("I", "II", "III", "IV")}.}

\item{ed_disposition}{Column representing the emergency department
disposition. For a record to be picked up in this function, the ED
dispostion must be documented as "Deceased/Expired".}

\item{ed_LOS}{Column for the calculated ED length of stay, measured in
minutes.}

\item{hospital_disposition}{Column representing the hospital disposition. For
a record to be picked up in this function, the hospital dispostion must be
documented as "Deceased/Expired".}

\item{hospital_LOS}{Column for the calculated hospital length of stay,
measured in minutes.}

\item{unique_incident_id}{Unique identifier for each record.}

\item{autopsy}{Unquoted column name indicating whether an autopsy was
performed. Expected values: \code{"Yes"} or \code{NA}.}

\item{groups}{Additional columns passed as a vector of strings to
\code{dplyr::summarize()} via the \code{.by} argument for grouped summaries. Defaults
to \code{NULL}.}

\item{calculate_ci}{If \code{NULL}, 95\% confidence intervals will not be
calculated for the performance estimates.  Otherwise, options of "wilson"
or "clopper-pearson" can be supplied to utilize the corresponding methods
to calculate the confidence intervals for the proportions. Defaults to
\code{NULL}.}

\item{...}{
  Arguments passed on to \code{\link[nemsqar:nemsqa_binomial_confint]{nemsqar::nemsqa_binomial_confint}}
  \describe{
    \item{\code{conf.level}}{Numeric value between 0 and 1 indicating the confidence
level. Defaults to 0.95 (95\% confidence interval).}
    \item{\code{correct}}{Logical, indicating whether to apply continuity correction for
Wilson intervals. Defaults to \code{TRUE}.}
  }}
}
\value{
A tibble summarizing SEQIC Indicator 4a and 4b results. Includes
numerator, denominator, and performance rate for the indicator. 95\%
confidence intervals are provided optionally.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Computes SEQIC Indicator 4a and 4b for trauma center performance. Indicator
4a captures the proportion of deceased trauma patients at trauma level I–IV
facilities who had an autopsy performed. Indicator 4b identifies deceased
trauma patients with a prolonged length of stay (LOS > 3 days) but without an
autopsy.
}
\details{
This function:
\itemize{
\item Filters trauma records to those with a trauma center level of I–IV.
\item Identifies records where the patient died, based on ED or hospital
disposition.
\item Deduplicates by \code{unique_incident_id} to ensure one record per
incident.
\item For Indicator 4a, calculates the proportion of deceased patients who
received an autopsy.
\item For Indicator 4b, calculates the proportion of deceased patients with
a hospital or ED length of stay greater than 72 hours (4320 minutes) and no
autopsy performed.
}
}
\note{
Users must ensure appropriate column names are passed and data is
pre-processed to include the necessary fields without missing critical
identifiers or timestamps.
}
\examples{
# Packages
library(dplyr)
library(traumar)

# Create a synthetic test dataset
test_data <- tibble::tibble(
  id = as.character(1:8),
  trauma_level = c("I", "II", "III", "IV", "I", "II", "III", "IV"),
  ed_disp = c(
    "Operating Room",
    "Admitted",
    "Deceased/Expired",
    "Transferred",
    "Deceased/Expired",
    "Deceased/Expired",
    "Admitted",
    "Deceased/Expired"
  ),
  ed_los = c(120, 200, 5000, 180, 3000, 4321, 60, 4000),
  hosp_disp = c(
    "Deceased/Expired",
    "Deceased/Expired",
    "Deceased/Expired",
    "Discharged",
    "Deceased/Expired",
    "Deceased/Expired",
    "Discharged",
    "Deceased/Expired"
  ),
  hosp_los = c(3000, 4500, 1000, 200, 5000, 4400, 150, 3000),
  autopsy_done = c("Yes", "No", "No", NA, "Yes", "No", NA, "Yes")
)

# Run the indicator function
traumar::seqic_indicator_4(
  data = test_data,
  level = trauma_level,
  ed_disposition = ed_disp,
  ed_LOS = ed_los,
  hospital_disposition = hosp_disp,
  hospital_LOS = hosp_los,
  unique_incident_id = id,
  autopsy = autopsy_done
)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trauma_performance.r
\name{trauma_performance}
\alias{trauma_performance}
\title{Calculate Trauma Hospital Performance Based on Robust and Validated
Measures}
\usage{
trauma_performance(
  df,
  Ps_col,
  outcome_col,
  z_method = c("survival", "mortality")
)
}
\arguments{
\item{df}{A data frame containing patient data.}

\item{Ps_col}{The name of the column containing the probability of survival
(Ps). The values should be numeric and between 0 and 1. Values greater than
1 will be automatically converted to decimal format by dividing by 100.}

\item{outcome_col}{The name of the column containing the binary outcome data.

The column should contain binary values indicating the patient outcome.
Valid values include 1 (dead) and 0 (alive), or TRUE (dead) and FALSE
(alive). The function will check values in this column and expects them to
represent the outcome in a binary form.}

\item{z_method}{A character vector indicating which method to use for
calculating the Z-score. Must be one of "survival" or "mortality". The
default is "survival".}
}
\value{
A tibble containing the following calculations:
\itemize{
\item \code{N_Patients}: The total number of patients included in the analysis.
\item \code{N_Survivors}: The total number of patients who survived, based on the
provided outcome data.
\item \code{N_Deaths}: The total number of patients who died, based on the
provided outcome data.
\item \code{Predicted_Survivors}: The total predicted number of survivors based on
the survival probability (\code{Ps}) for all patients.
\item \code{Predicted_Deaths}: The total predicted number of deaths, calculated as
\code{1 - Ps} for all patients.
\item \code{Patient_Estimate}: The estimated number of patients who survived that
were predicted to die, calculated based on the W-score. This value reflects
the difference between the actual and predicted number of deceased patients.
\item \code{W_Score}: The W-score, representing the difference between the
observed and expected number of survivors per 100 patients. A positive
W-score indicates that more patients survived than expected, while a negative
score indicates that fewer patients survived than expected.
\item \code{M_Score}: The M-score, which compares the observed patient case mix to
the Major Trauma Outcomes Study (MTOS) case mix. A higher score indicates
that the patient mix is more similar to MTOS, while a lower score indicates a
dissimilar mix. Based on the MTOS literature, an M_Score >= 0.88 indicates
that the Z_Score comes from distribution similar enough to the MTOS Ps
distribution.
\item \code{Z_Score}: The Z-score, which quantifies the difference between the
actual and predicted mortality (if \code{z_method = "mortality"}) or survival (if
\code{z_method = "survival"}). A Z-score > 1.96 is considered to point to the
statistical significance of the W-Score at alpha = 0.05 level for survival.
The positive Z_Score indicates that more patients survived than predicted,
while a negative Z-score indicates fewer survivors than predicted.
}
}
\description{
This function calculates trauma hospital performance based on the M, W, and Z
scores, which are derived from survival probability and mortality data, using
established methods. It computes the W-score, M-score, and Z-score based on
the provided dataset and calculates performance metrics for trauma programs.
For more information on the methods used in this function, please see
Champion et al. (1990) on the W score, and Flora (1978) and Boyd et al.
(1987) on the M and Z scores.
}
\details{
The function checks whether the \code{outcome_col} contains values representing a
binary outcome. It also ensures that \code{Ps_col} contains numeric values within
the range 0 to 1. If any values exceed 1, a warning is issued. The patients
are then grouped into predefined Ps ranges, and the function compares the
fraction of patients in each range with the MTOS case mix distribution.

Like other statistical computing functions, \code{trauma_performance()} is
happiest without missing data.  It is best to pass complete probability of
survival and outcome data to the function for optimal performance. With
smaller datasets, this is especially helpful.  However,
\code{trauma_performance()} will throw a warning about missing values, if any
exist in \code{Ps_col} and/or \code{outcome_col}.
}
\note{
This function will produce the most reliable and interpretable results when
using a dataset that has one row per patient, with each column being a
feature.
}
\examples{
# Generate example data
set.seed(123)

# Parameters
# Total number of patients
n_patients <- 5000

# Arbitrary group labels
groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

# Trauma types
trauma_type_values <- sample(
  x = c("Blunt", "Penetrating"),
  size = n_patients,
  replace = TRUE
)

# RTS values
rts_values <- sample(
  x = seq(from = 0, to = 7.8408, by = 0.005),
  size = n_patients,
  replace = TRUE
)

# patient ages
ages <- sample(
  x = seq(from = 0, to = 100, by = 1),
  size = n_patients,
  replace = TRUE
)

# ISS scores
iss_scores <- sample(
  x = seq(from = 0, to = 75, by = 1),
  size = n_patients,
  replace = TRUE
)

# Generate survival probabilities (Ps)
Ps <- traumar::probability_of_survival(
  trauma_type = trauma_type_values,
  age = ages,
  rts = rts_values,
  iss = iss_scores
)

# Simulate survival outcomes based on Ps
survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

# Create data frame
data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Calculate trauma performance (W, M, Z scores)
trauma_performance(data, Ps_col = Ps, outcome_col = death)

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relative_mortality.R
\name{rmm}
\alias{rmm}
\title{Relative Mortality Metric (RMM) Calculation}
\usage{
rmm(
  data,
  Ps_col,
  outcome_col,
  group_vars = NULL,
  n_samples = 100,
  Divisor1 = 5,
  Divisor2 = 5,
  Threshold_1 = 0.9,
  Threshold_2 = 0.99,
  bootstrap_ci = TRUE,
  pivot = FALSE,
  seed = NULL
)
}
\arguments{
\item{data}{A data frame or tibble containing the data.}

\item{Ps_col}{The name of the column containing the survival probabilities
(Ps). Should be numeric on a scale from 0 to 1.}

\item{outcome_col}{The name of the column containing the outcome data. It
should be binary, with values indicating patient survival. A value of \code{1}
should represent "alive" (survived), while \code{0} should represent "dead" (did
not survive).  \code{TRUE/FALSE} are accepted as well. Ensure the column
contains only these possible values.}

\item{group_vars}{Optional character vector specifying grouping variables for
stratified analysis. If \code{NULL}, the calculation is performed on the entire
dataset.}

\item{n_samples}{A numeric value indicating the number of bootstrap samples
to take from the data source.}

\item{Divisor1}{A positive numeric value controlling the coarseness of bins
for Ps values below \code{Threshold_1}. It scales the number of steps from the
start of the dataset up to the \code{Threshold_1} cut point. Larger values
produce fewer, broader bins; smaller values produce more, narrower bins.
Defaults to \code{5}.}

\item{Divisor2}{A positive numeric value controlling the coarseness of bins
for Ps values between \code{Threshold_1} and \code{Threshold_2}. Larger values yield
wider bins, and smaller values yield narrower bins in this range. Defaults
to \code{5}.}

\item{Threshold_1}{A numeric value that defines the lower bound of the
high-survival probability range in \code{Ps_col}. The function identifies the
first index where \code{Ps_col} exceeds this value and begins applying smaller
bin widths from that point onward. Defaults to \code{0.9}, meaning binning
changes once Ps > 0.90.}

\item{Threshold_2}{A numeric value that defines the upper bound of the
high-survival probability range in \code{Ps_col}. The function identifies the
first index where \code{Ps_col} exceeds this value. Between \code{Threshold_1} and
\code{Threshold_2}, finer binning is applied; above \code{Threshold_2}, binning may
again change. Defaults to \code{0.99}, meaning the special binning range is
between Ps values of 0.90 and 0.99.}

\item{bootstrap_ci}{A logical indicating whether to return the relative
mortality metric estimate and 95\% confidence intervals using bootstrap
sampling. Default is \code{TRUE}.}

\item{pivot}{A logical indicating whether to return the results in a long
format (\code{pivot = TRUE}) or wide format (\code{pivot = FALSE}, default). Use with
caution in tandem with \code{group_vars} if the grouping variable is of a
different class than \code{rmm()}'s outputs, such as \code{factor} or \code{character}
grouping variables.}

\item{seed}{Optional numeric value to set a random seed for reproducibility.
If \code{NULL} (default), no seed is set.}
}
\value{
A tibble containing the Relative Mortality Metric (RMM) and related
statistics:
\itemize{
\item \code{population_RMM_LL}: The lower bound of the 95\% confidence interval for
the population RMM.
\item \code{population_RMM}: The final calculated Relative Mortality Metric for the
population existing in \code{data}.
\item \code{population_RMM_UL}: The upper bound of the 95\% confidence interval for
the population RMM.
\item \code{population_CI}: The confidence interval width for the population RMM.
\item \code{bootstrap_RMM_LL}: The lower bound of the 95\% confidence interval for
the bootstrap RMM. (optional, if \code{bootstrap_ci = TRUE})
\item \code{bootstrap_RMM}: The average RMM value calculated for the bootstrap
sample. (optional, if \code{bootstrap_ci = TRUE})
\item \code{bootstrap_RMM_UL}: The upper bound of the 95\% confidence interval for
the bootstrap RMM. (optional, if \code{bootstrap_ci = TRUE})
\item \code{bootstrap_CI}: The width of the 95\% confidence interval for the
bootstrap RMM. (optional, if \code{bootstrap_ci = TRUE})
\item If \code{pivot = TRUE}, the results will be in long format with two columns:
\code{stat} and \code{value}, where each row corresponds to one of the calculated
statistics.
\item If \code{pivot = FALSE} (default), the results will be returned in wide
format, with each statistic as a separate column.
}
}
\description{
Calculates the Relative Mortality Metric (RMM) from Napoli et al. (2017)
based on patient survival probabilities (Ps) and actual outcomes. The
function groups patients into bins based on their survival probability scores
(Ps) and computes a weighted mortality metric along with confidence
intervals. For more information on the methods used in this function, see as
well Schroeder et al. (2019), and Kassar et al. (2016).

The Relative Mortality Metric (RMM) quantifies the performance of a center in
comparison to the anticipated mortality based on the TRISS national
benchmark. The RMM measures the difference between observed and expected
mortality, with a range from -1 to 1.
\itemize{
\item An RMM of 0 indicates that the observed mortality aligns with the expected
national benchmark across all acuity levels.
\item An RMM greater than 0 indicates better-than-expected performance, where
the center is outperforming the national benchmark.
\item An RMM less than 0 indicates under-performance, where the center’s observed
mortality is higher than the expected benchmark.
}

This metric helps assess how a center's mortality compares to the national
standards, guiding quality improvement efforts. \code{rmm()} utilizes bootstrap
sampling to calculate the confidence intervals via the standard error method.
}
\details{
Like other statistical computing functions, \code{rmm()} is happiest without
missing data.  It is best to pass complete probability of survival and
mortality outcome data to the function for optimal performance. With smaller
datasets, this is especially helpful.  However, \code{rmm()} will throw a warning
about missing values, if any exist in \code{Ps_col} and/or \code{outcome_col}.

\code{rmm()} assumes \code{Ps_col} contains probabilities derived from
real-world inputs for the Trauma Injury Severity Score (TRISS) model.
Synthetic or low-variability data (especially with small sample sizes) may
not reflect the distribution of TRISS-derived survival probabilities. This
can result in unstable estimates or function failure due to insufficient
dispersion. With small sample sizes, it may be important to use smaller
values with the divisor arguments and adjust the thresholds (based on the
distribution of the \code{Ps_col} values) to create bins that better accommodate
the data.

Due to the use of bootstrap sampling within the function, users should
consider setting the random number \code{seed} within \code{rmm()} for reproducibility.
}
\note{
This function will produce the most reliable and interpretable results when
using a dataset that has one row per patient, with each column being a
feature.

By default, \code{rmm()} derives bin cut points from the full dataset’s
distribution. This ensures comparability across groups when \code{group_vars} is
used. To tailor results to a specific group (e.g., a single hospital), filter
the dataset to that subgroup before calling \code{rmm()}. The function will then
compute bins and related statistics using only that subset’s \code{Ps_col}
distribution. When \code{group_vars} is used, and ff a group lacks
observations within one or more bins, \code{rm_bin_summary()} will compute
statistics only for the bins that contain data. Bins with no observations are
excluded from the summary for that group.
}
\examples{
# Generate example data
set.seed(123)

# Parameters
# Total number of patients
n_patients <- 5000

# Arbitrary group labels
groups <- sample(x = LETTERS[1:2], size = n_patients, replace = TRUE)

# Trauma types
trauma_type_values <- sample(
  x = c("Blunt", "Penetrating"),
  size = n_patients,
  replace = TRUE
)

# RTS values
rts_values <- sample(
  x = seq(from = 0, to = 7.8408, by = 0.005),
  size = n_patients,
  replace = TRUE
)

# patient ages
ages <- sample(
  x = seq(from = 0, to = 100, by = 1),
  size = n_patients,
  replace = TRUE
)

# ISS scores
iss_scores <- sample(
  x = seq(from = 0, to = 75, by = 1),
  size = n_patients,
  replace = TRUE
)

# Generate survival probabilities (Ps)
Ps <- traumar::probability_of_survival(
  trauma_type = trauma_type_values,
  age = ages,
  rts = rts_values,
  iss = iss_scores
)

# Simulate survival outcomes based on Ps
survival_outcomes <- rbinom(n_patients, size = 1, prob = Ps)

# Create data frame
data <- data.frame(Ps = Ps, survival = survival_outcomes, groups = groups) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Example usage of the `rmm` function
rmm(data = data, Ps_col = Ps,
    outcome_col = survival,
    Divisor1 = 4,
    Divisor2 = 4,
    n_samples = 10
    )

# pivot!
rmm(data = data, Ps_col = Ps,
    outcome_col = survival,
    Divisor1 = 4,
    Divisor2 = 4,
    n_samples = 10,
    pivot = TRUE
    )

# Create example grouping variable (e.g., hospital)
hospital <- sample(c("Hospital A", "Hospital B"), n_patients, replace = TRUE)

# Create data frame
data <- data.frame(
  Ps = Ps,
  survival = survival_outcomes,
  hospital = hospital
) |>
  dplyr::mutate(death = dplyr::if_else(survival == 1, 0, 1))

# Example usage of the `rmm` function with grouping by hospital
rmm(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  group_vars = "hospital",
  Divisor1 = 4,
  Divisor2 = 4,
  n_samples = 10
)

# Pivoted output for easier visualization
rmm(
  data = data,
  Ps_col = Ps,
  outcome_col = survival,
  group_vars = "hospital",
  Divisor1 = 4,
  Divisor2 = 4,
  n_samples = 10,
  pivot = TRUE
)

}
\references{
Kassar, O.M., Eklund, E.A., Barnhardt, W.F., Napoli, N.J., Barnes, L.E.,
Young, J.S. (2016). Trauma survival margin analysis: A dissection of trauma
center performance through initial lactate. The American Surgeon, 82(7),
649-653. \url{doi:10.1177/000313481608200733}

Napoli, N. J., Barnhardt, W., Kotoriy, M. E., Young, J. S., & Barnes, L. E.
(2017). Relative mortality analysis: A new tool to evaluate clinical
performance in trauma centers. IISE Transactions on Healthcare Systems
Engineering, 7(3), 181–191. \url{doi:10.1080/24725579.2017.1325948}

Schroeder, P. H., Napoli, N. J., Barnhardt, W. F., Barnes, L. E., & Young, J.
S. (2018). Relative mortality analysis of the “golden hour”: A comprehensive
acuity stratification approach to address disagreement in current literature.
Prehospital Emergency Care, 23(2), 254–262.
\url{doi:10.1080/10903127.2018.1489021}
}
\seealso{
\code{\link[=probability_of_survival]{probability_of_survival()}}, \code{\link[=rm_bin_summary]{rm_bin_summary()}}, and
\code{\link[=nonlinear_bins]{nonlinear_bins()}}
}
\author{
Nicolas Foss, Ed.D, MS, original implementation in MATLAB by Nicholas
J. Napoli, Ph.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not_in.r
\name{\%not_in\%}
\alias{\%not_in\%}
\title{Check if Elements Are Not in a Vector}
\usage{
x \%not_in\% y
}
\arguments{
\item{x}{A vector of values to be checked.}

\item{y}{A vector of values to check against.}
}
\value{
A logical vector of the same length as \code{x}, where \code{TRUE} indicates
the corresponding element in \code{x} is not found in \code{y}, and \code{FALSE} indicates
it is found in \code{y}.
}
\description{
This function returns a logical vector indicating whether each
element of \code{x} is not in \code{y}.
}
\examples{

# Example vectors
x <- c("apple", "banana", "cherry")
y <- c("banana", "grape")

# Check which elements in `x` are not in `y`
x \%not_in\% y

# Example with numeric values
a <- c(1, 2, 3, 4, 5)
b <- c(2, 4, 6)

a \%not_in\% b

}
\author{
Nicolas Foss, Ed.D., MS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute.r
\name{impute}
\alias{impute}
\title{Impute Numeric Column Values}
\usage{
impute(
  x,
  focus = c("skew", "missing"),
  method = c("winsorize", "iqr", "mean", "median"),
  percentile = NULL
)
}
\arguments{
\item{x}{A numeric vector to be cleaned.}

\item{focus}{A character string indicating the focus. Options are:
\itemize{
\item \code{"skew"}: Handle extreme values using percentile or
IQR methods (default).
\item \code{"missing"}: Impute missing values.
}}

\item{method}{A character string specifying the method:
\itemize{
\item For \code{focus = "skew"}:
\itemize{
\item \code{"winsorize"}: Replace values outside specified percentiles (default).
\item \code{"iqr"}: Use IQR to limit extreme values.
}
\item For \code{focus = "missing"}:
\itemize{
\item \code{"mean"}: Replace missing values with the mean.
\item \code{"median"}: Replace missing values with the median.
}
}}

\item{percentile}{A numeric value (percentile > 0) for winsorization.
If not provided, defaults to 0.01 and 0.99.}
}
\value{
A numeric vector with cleaned or imputed values.
}
\description{
Cleans numeric columns by handling extreme values or imputing
missing values. The function supports two main focuses: handling skewed
distributions or imputing missing data.
}
\examples{
x <- c(1, 2, 3, 100, 200, NA)
# Winsorize to 1\% and 99\%
impute(x, focus = "skew", method = "winsorize")

# Replace missing values with the mean
impute(x, focus = "missing", method = "mean")

}
